function outputdata=ep_windowData(EPdata, chanGrp, inputCells, outCellNames, subjectSpecs, startwin, endwin, measure, fname, factor);

%  outputdata=ep_windowData(EPdata, chanGrp, inputCells, outCellNames, subjectSpecs, startwin, endwin, measure, fname, factor);
%
%  Outputs text file suitable for ANOVA.  Each line is a subject.  As for columns, cells will vary slowest while chan groups will
%  vary fastest.  Chan groups will be generated for spatial as well as temporal PCAs even though chan effects will not occur for spatial PCAs.
%  Results will be in microvolts (or msec) for a specific channel (or mean of group of channels) for a specific window to make results exactly comparable
%  to conventional analyses.  Strips EPdata of adds (combined channels, cells, subjects, and factors) first.
%  Starts with six header lines: name of dataset, window, channel group, factor if any, cell names, and channel area
%  names.
%
%Inputs:
%   EPdata         : Structured array with the data and accompanying information.  See readData.
%
% chanGrp   : Cell array of channels going into each output channel grouping.
%   .name       : Name of the channel grouping
%   .threshold  : Threshold used for factor groups (not used)
%   .areaname   : Cell array of the channel area names
%   .activeFac  : Active factor (not used)
%   .channel    : Vector of area each channel is assigned to
%
% inputCells : Cell array of conditions included in analysis (output cells, each containing array of input cells)
% outCellNames: Cell array of the output cell names
% subjectSpecs : Logical vector array of which specs to include with the output measures
% startwin   : Start of time window in samples (1-based).
% endwin	 : End of time window in samples.
% measure	 : Type of measure for windows: mean, minpeak, maxpeak, minlatency, maxlatency, mincentroid, maxcentroid.
% fname		 : name of output file
% factor     : Factor if a PCA dataset, otherwise set to 1.
%
%Outputs:
%   outputdata  : The resulting windowed measures.  Also written out to disk.
%
%  Centroid measure of latency:
%  Dien, J., Spencer, K. M., & Donchin, E. (2004). Parsing the "Late Positive Complex": Mental chronometry and the ERP
%  components that inhabit the neighborhood of the P300. Psychophysiology, 41(5), 665-678.
%  The present equation is corrected from that in the published article.
%
%  General introductory treatment of ANOVA of ERP data:
%  Dien, J., & Santuzzi, A. M. (2005). Application of repeated measures ANOVA to high-density ERP datasets: A review and
%  tutorial. In T. Handy (Ed.), Event-Related Potentials: A Methods Handbook. Cambridge, Mass: MIT Press.

%History:
%  by Joseph Dien (7/17/09)
%  jdien07@mac.com
%
% bugfix 10/10/10 JD
% Fixed centroid measures too large due to addition of the left side of the window (e.g., +200 ms for a window of
% 200-300 ms).

%     Copyright (C) 1999-2011  Joseph Dien
%
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
%
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
%
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

[EPdata]=ep_stripAdds(EPdata);
if isempty(EPdata.data)
    msg{1}='Error: The file had no data left after additions were removed.';
    [msg]=ep_errorMsg(msg);
    return
end;

outputdata=[];
numPoints=length(EPdata.timeNames);
numChans=length(EPdata.chanNames);
numCells=length(unique(EPdata.cellNames));
numWaves=length(EPdata.cellNames);
numSubs=length(EPdata.subNames);
numFacs=length(EPdata.facNames);

if endwin < startwin
    msg{1}='Endwindow sample is smaller than startwindow sample.';
    [msg]=ep_errorMsg(msg);
    return
end;

if endwin > numPoints
    msg{1}='Endwindow sample is larger than number of time points.';
    [msg]=ep_errorMsg(msg);
    return
end;

if factor == 0
    factor =1;
end;

if exist(fname,'file')
    msg{1}=[fname ' already exists!'];
    [msg]=ep_errorMsg(msg);
    return
end;

%determine number of channel areas and which channels are assigned to them.
numAreas=0;
for theArea=1:length(chanGrp.areaName)-1
    chanArray=find(chanGrp.channel == theArea);
    if ~isempty(chanArray)
        numAreas=numAreas+1;
        chanArea{numAreas}=chanArray;
        chanAreaName{numAreas}=chanGrp.areaName{theArea};
    end;
end;
    
%construct array of windowed measures
outputdata = zeros(numSubs,(numAreas*length(inputCells)));
test=[];
for theSub = 1:numSubs
    for theOutCell = 1:length(inputCells)
        for theArea = 1:numAreas
            theData = 0;
            count = 0;
            for theInCell = 1:length(inputCells{theOutCell})
                for theChan = 1:length(chanArea{theArea})
                    epoch=squeeze(ep_expandFacs(EPdata,chanArea{theArea}(theChan),startwin:endwin,inputCells{theOutCell}(theInCell),theSub,factor));
                    switch measure
                        case 'mean', theData = theData + mean(epoch,2);
                        case 'minpeak', theData = theData + min(epoch,[],2);
                        case 'maxpeak', theData = theData + max(epoch,[],2);
                        case 'minlatency', [dummy, latency] = min(epoch,[],2);
                            theData = theData + (latency+startwin-1-EPdata.baseline)*(1000/EPdata.Fs);
                        case 'maxlatency', [dummy, latency] = max(epoch,[],2);
                            theData = theData + (latency+startwin-1-EPdata.baseline)*(1000/EPdata.Fs);
                            test(end+1)=(latency+startwin-1-EPdata.baseline)*(1000/EPdata.Fs);
                        case 'mincentroid',
                            maxvalue=max(epoch,[],2);
                            theData = theData + (mean(([startwin:endwin].*(epoch-maxvalue))/mean(epoch-maxvalue))- EPdata.baseline)*(1000/EPdata.Fs);
                        case 'maxcentroid',
                            minvalue=min(epoch,[],2);
                            theData = theData + (mean(([startwin:endwin].*(epoch-minvalue))/mean(epoch-minvalue))- EPdata.baseline)*(1000/EPdata.Fs);
                        otherwise,
                            msg{1}='Measure should be one of the following options: mean, minpeak, maxpeak, minlatency, maxlatency, mincentroid, maxcentroid';
                            [msg]=ep_errorMsg(msg);
                            return
                    end;
                    count = count +1;
                end;
            end;
            theData = theData /count;
            outputdata(theSub,(theOutCell-1)*numAreas+theArea) = theData;
        end;
    end;
end;

outFID=fopen(fname,'w');
if (outFID == -1)
    msg{1}='Error creating output file!';
    [msg]=ep_errorMsg(msg);
    return
end;

%file header
fprintf(outFID,'%s\r',EPdata.dataName);
fprintf(outFID,'%d/%d ms\r',EPdata.timeNames(startwin),EPdata.timeNames(endwin));
fprintf(outFID,'%s\r',measure);
fprintf(outFID,'%s\r',chanGrp.name);
if isempty(EPdata.facNames)
    fprintf(outFID,'\r');
else
    fprintf(outFID,'Factor: %d\r',factor);    
end;
for theCell = 1:length(outCellNames)
    for theArea = 1:length(chanArea)
        fprintf(outFID,'%s\t',outCellNames{theCell});
    end
end;

for theSpec = 1:length(subjectSpecs)
    fprintf(outFID,'%s',EPdata.subjectSpecNames{subjectSpecs(theSpec)});
    if theSpec < length(subjectSpecs)
        fprintf(outFID,'\t');
    end;
end;

fprintf(outFID,'\r');

for theCell = 1:length(outCellNames)
    for theArea = 1:length(chanArea)
        fprintf(outFID,'%s\t',chanAreaName{theArea});
    end
end;

for theSpec = 1:length(subjectSpecs)
    fprintf(outFID,'spec\t');
end;

fprintf(outFID,'\r');

%file data
for theSub=1:numSubs
    for theCell = 1:size(outputdata,2)
        fprintf(outFID,'%f\t',outputdata(theSub,theCell));
    end;
    for theSpec = 1:length(subjectSpecs)
        theSpec=EPdata.subjectSpecs{theSub,subjectSpecs(theSpec)};
        if isnumeric(theSpec)
            theSpec=num2str(theSpec);
        end;
        fprintf(outFID,'%s\t',theSpec);
    end;
    fprintf(outFID,[EPdata.subNames{theSub} '\r']);
end;

fclose(outFID);
