function [MUHAT, SIGMA, RESULTS]=ep_WJGLMml(Y, NX, C, U, OPT1, PER, OPT2, NUMSIM, SEED, MISSING);
%function [MUHAT, SIGMA, RESULTS]=ep_WJGLMml(Y, NX, C, U, OPT1, PER, OPT2, NUMSIM, SEED, MISSING);
%Function for calculating robust statistics using Welch-James boostrapping and trimmed
%means.
%ADF algorithms made available by Harvey Keselman (kesel@Ms.UManitoba.CA)
%Department of Psychology, University of Manitoba, Winnipeg, Canada.
%http://www.umanitoba.ca/cgi-bin/psychology/hpg_main.cgi?data/Keselman.txt
%
%Converted to Matlab 1/13/04 by Joseph Dien (jdien@ku.edu)
%Department of Psychology, University of Kansas, Lawrence, USA.
%
%Keselman, H. J., Wilcox, R. R., & Lix, L. M. (2003). A generally robust approach
%to hypothesis testing in independent and correlated groups designs
%Psychophysiology, 40, 586-596.

%Inputs
%  Y	    : Input data (rows=subjects, columns = cells).  The first NX rows will be assigned to the first group and so forth.
%  NX   	: Number of subjects in each group.  If empty set then will assume a single group.
%  C    	: Contrast row vector for between factors.  Set to 1 in the case where there is only one group.
%  U    	: Contrast column vector for within factors.  Numbers should sum to zero.  Set to empty set [] for analyses with no within-factors.
%  OPT1     : Activate trimming option (rounded down).  0 = no and 1 = yes.
%  PER  	: Percentage to trim the means.  .05 is the recommended number.
%  OPT2 	: Activate Welch-James statistic.  0 = no and 1 = yes.  "No" option is not currently available.
%  NUMSIM	: Number of simulations used to generate bootstrapping statistic.  p-values will be unstable if too low.  50000 informally recommended.
%  SEED 	: Seed for random number generation.  0 specifies random seed. 1000 arbitrarily suggested as seed to ensure results are replicable.
%  MISSING  : Number to be treated as a missing value.  Observations with missing values are dropped from the analysis.

%Outputs
%  MUHAT	: Vector of trimmed means used in calculations. 
%             For between group analyses, within group conditions vary fastest and between group conditions vary slowest.
%  SIGMA	: Winsorized covariance matrix
%  RESULTS	: (1) = test statistic
%  RESULTS	: (2) = Numerator DF
%  RESULTS	: (3) = Denominator DF
%  RESULTS	: (4) = Significance

%****DEFINE MODULES TO PERFORM ALL CALCULATIONS****;
%****COMPUTE WELCH-JAMES STATISTIC****;

% modified 10/7/07 JD
% Missing number feature added.
%
% modified 2/11/09 JD
% Warning messages only output once.

errorflag.TestSingularMatrix=0;
errorflag.TestIllConditionedMatrix=0;
errorflag.TooFewSubjects=0;

if isempty(NX)
    NX = size(Y,1);
end;
if isempty(C)
    C = [1];
end;

if size(C,2) ~= length(NX)
    msg{1}=['Number of between group cells (' num2str(length(NX)) ') must equal number of terms in contrast C (' num2str(size(C,2)) ').'];
    [msg]=ep_errorMsg(msg);
    return
end;

if size(U,1) ~= size(Y,2)
    msg{1}=['Number of within group cells (' num2str(size(Y,2)) ') must equal number of terms in contrast U (' num2str(size(U,1)) ').'];
    [msg]=ep_errorMsg(msg);
    return
end;

if OPT2 ~= 1
    msg{1}='OPT2 must equal one at present.';
    [msg]=ep_errorMsg(msg);
    return
end;

if OPT1 ~=0 && OPT1 ~= 1
    msg{1}='OPT1 must equal zero or one.';
    [msg]=ep_errorMsg(msg);
    return
end;

if ~isempty(MISSING) %drop observations with missing data points
    good = [];
    newNX= zeros(1,length(NX));
    count=0;
    for group = 1:length(NX)
        for obs = 1:NX(group)
            count=count+1;
            if isempty(find(Y(count,:)==MISSING))
                good=[good count];
                newNX(group)=newNX(group)+1;
            end;
        end;
    end;
    if ~isempty(find(newNX==0))
        msg{1}='A group has zero members after dropping missing data points';
        [msg]=ep_errorMsg(msg);
        return
    end;
    NX=newNX;
    Y=Y(good,:);
end;

[U, R, X, NTOT, BOBS, WOBS, WOBS1] = INITIAL(C,U,Y, NX); %Initialize variables
if SEED == 0
    rand('state',sum(100*clock));
else
    rand('state',SEED);
end;
[MUHAT, BHAT, BHATW, YT, DF, errorflag] = MNMOD(Y, OPT1, BOBS, WOBS, WOBS1, NTOT, NX, PER, X, errorflag); %betas and trimmed means
if errorflag.TooFewSubjects
    MUHAT=[];
    SIGMA=[];
    RESULTS=[];
    return;
end;
[SIGMA] = SIGMOD(YT,BHATW,DF,BOBS, WOBS, WOBS1, NTOT, NX, PER, X); %sigma matrix
[FSTAT, DF1, DF2, errorflag] = TESTMOD(SIGMA,MUHAT,R,DF,BOBS, WOBS, WOBS1, NTOT, NX, PER, X, errorflag); %test statistic
if OPT2==1 %bootstrap procedure to set alpha threshold
    for SIMLOOP=1:NUMSIM;
        [YB] = BOOTDAT(Y, BHAT, BOBS, WOBS, WOBS1, NTOT, NX, PER, X, SEED);
        [FSTATB, errorflag] = BOOTSTAT(YB, OPT1, R, BOBS, WOBS, WOBS1, NTOT, NX, PER, X, SEED, errorflag);
        if SIMLOOP==1
            FMAT=FSTATB;
        else
            FMAT=[FMAT; FSTATB];
        end;
    end;
    [sorted index] = sort(FMAT);
    FMAT=sorted;
end;
%***CALCULATE SIGNIFICANCE LEVEL****;
RESULTS=zeros(4,1);
RESULTS(1)=FSTAT;
RESULTS(2)=DF1;
RESULTS(3)=DF2;
if OPT2==0
    msg{1}='No F-statistics available.';
    [msg]=ep_errorMsg(msg);
    return
    %RESULTS(4)=1-PROBF(RESULTS[1],DF1,DF2);%???????
end;
if OPT2==1
    AVEC=(FSTAT<=FMAT);
    PVAL=sum(sum(AVEC))/NUMSIM;
    RESULTS(4)=PVAL;
end;

if errorflag.TestSingularMatrix==1
    msg='Warning: The matrix was singular to working precision when computing the test statistic.';
    disp(msg);
end;
if errorflag.TestIllConditionedMatrix==1
    msg='Warning: Matrix is singular, close to singular or badly scaled. Results may be inaccurate. RCOND = NaN.';
    disp(msg);
end;

%**DEFINE MODULE FOR INITIAL SPECIFICATIONS****;
function [U, R, X, NTOT, BOBS, WOBS, WOBS1] = INITIAL(C, U, Y, NX);
if size(U,1)==0
    U=eye(size(Y,2));
end;

if size(U,2)>size(U,1)
    msg{1}='NUMBER OF COLUMNS OF U EXCEEDS NUMBER OF ROWS';
    [msg]=ep_errorMsg(msg);
    return
end;
for I=1:size(NX,2);
    X1=ones(NX(I),1)*I;
    if I==1
        X=X1;
    else
        X=[X; X1];
    end;
end;
X=design(X); %full design matrix
NTOT=size(Y,1); %number of subjects
WOBS=size(Y,2); %total number of within cells
BOBS=size(X,2); %number of between groups
WOBS1=WOBS-1; %one less than total number of within cells
R=kron(C,U'); %contrast vector combining within and between contrasts

function a = design(x);
a = zeros(size(x,1),length(unique(x)));
for i=1:length(x);
    a(i,x(i)) = 1;
end;

%****DEFINE MODULE TO COMPUTE LEAST SQUARES OR TRIMMED MEANS****;
function [MUHAT, BHAT, BHATW, YT, DF, errorflag] = MNMOD(Y, OPT1, BOBS, WOBS, WOBS1, NTOT, NX, PER, X, errorflag);
if OPT1==0
    BHAT=inv(X'*X)*X'*Y;
    BHATW=BHAT;
    YT=Y;
    DF=NX-1;
end;
if OPT1==1
    BHAT=zeros(BOBS,WOBS);
    BHATW=BHAT;
    YT=zeros(NTOT,WOBS);
    DF=zeros(1,BOBS);
    F=1;
    M=0;
    for J=1:size(NX,2);
        SAMP=NX(J);
        L=M+SAMP;
        G=floor(PER*SAMP);
        DF(J)=SAMP-2*G-1;
        for K=1:size(Y,2)
            TEMP=Y(F:L,K);
            NV=TEMP;
            [TEMP2 index] = sort(NV);
            TRIMY=TEMP2(G+1:SAMP-G,:);
            TRIMMN=sum(sum(TRIMY))/(DF(J)+1);
            BHAT(J,K)=TRIMMN;
            MINT=min(min(TRIMY));
            MAXT=max(max(TRIMY));
            for P=1:size(NV,1)
                if NV(P)<=MINT
                    NV(P)=MINT;
                end;
                if NV(P)>=MAXT
                    NV(P)=MAXT;
                end;
            end;
            YT(F:L,K)=NV;
            WINMN=sum(sum(NV))/SAMP;
            BHATW(J,K)=WINMN;
        end;
        M=L;
        F=F+NX(J);
    end;
end;

if DF == 0
    errorflag.TooFewSubjects=1;
    MUHAT=0;
    BHAT=0;
    BHATW=0;
    YT=0;
    msg{1}='Error, too few subjects.  Degrees of freedom is zero.';
    [msg]=ep_errorMsg(msg);
    return
end;

rows=kron(BOBS,WOBS);
columns=(size(BHAT,1)*size(BHAT,2))/rows;
MUHAT=reshape(BHAT',rows,columns);

%***DEFINE MODULE TO COMPUTE SIGMA MATRIX****;
function [SIGMA] = SIGMOD(YT, BHATW, DF, BOBS, WOBS, WOBS1, NTOT, NX, PER, X);
SIGMA=zeros(WOBS.*BOBS,WOBS.*BOBS);
for I=1:BOBS
    SIGB=(diag(X(:,I))*YT-X(:,I)*BHATW(I,:))'*(diag(X(:,I))*YT-X(:,I)*BHATW(I,:))/((DF(I)+1)*DF(I));
    F=I*WOBS-WOBS1;
    L=I*WOBS;
    SIGMA(F:L,F:L)=SIGB;
end;

%****DEFINE MODULE TO COMPUTE TEST STATISTIC****;
function [FSTAT, DF1, DF2, errorflag] = TESTMOD(SIGMA, MUHAT, R, DF, BOBS, WOBS, WOBS1, NTOT, NX, PER, X, errorflag);
warning off MATLAB:singularMatrix %turn off display of warning otherwise one gets dozens of messages
warning off MATLAB:illConditionedMatrix
lastwarn('');
T=(R*MUHAT)'*inv(R*SIGMA*R')*(R*MUHAT);
A=0;
IMAT=eye(WOBS);
for I=1:BOBS;
    QMAT=zeros(BOBS*WOBS,BOBS*WOBS);
    F=I*WOBS-WOBS1;
    L=I*WOBS;
    QMAT(F:L,F:L)=IMAT;
    PROD=(SIGMA*R')*inv(R*SIGMA*R')*R*QMAT;
    A=A+(trace(PROD*PROD)+trace(PROD)^2)/DF(I);
end;
A=A/2;
DF1=size(R,1);
DF2=DF1*(DF1+2)/(3*A);
CVAL=DF1+2*A-6*A/(DF1+2);
FSTAT=T/CVAL;
[errmsg errID]=lastwarn; %detect if a warning occurred.  The warning will be displayed at the end of the WJGLM routine.
if strcmp(errID, 'MATLAB:singularMatrix')
    errorflag.TestSingularMatrix=1;
end
if strcmp(errID, 'MATLAB:illConditionedMatrix')
    errorflag.TestIllConditionedMatrix=1;
end

warning on MATLAB:singularMatrix
warning on MATLAB:illConditionedMatrix

%****DEFINE MODULES TO PERFORM BOOTSTRAP****;
%***DEFINE MODULE TO GENERATE BOOTSTRAP DATA AND CENTRE DATA****;
function [YB] = BOOTDAT(Y, BHAT, BOBS, WOBS, WOBS1, NTOT, NX, PER, X, SEED); 
F=1;
M=0;
temp=[];
for J=1:BOBS
    L=M+NX(J);
    TEMP=Y(F:L,:);
    BVAL=TEMP;
    for P=1:size(TEMP,1);
        RVAL=rand(1);
        BVAL(P,:)=TEMP(ceil(size(TEMP,1)*RVAL),:);
    end;
    if J==1
        YB=BVAL;
    else
        YB=[YB; BVAL];
    end;
    M=L;
    F=F+NX(J);
end;
%****CENTRE THE BOOTSTRAP DATA****;
M=0;
F=1;
for I=1:BOBS
    L=M+NX(I);
    MVAL=BHAT(I,:);
    for Q=F:1:L
        YB(Q,:)=YB(Q,:)-MVAL;
    end;
    M=L;
    F=F+NX(I);
end;

%****DEFINE MODULE TO COMPUTE BOOTSTRAP STATISTIC****;
function [FSTATB, errorflag] = BOOTSTAT(YB,OPT1,R, BOBS, WOBS, WOBS1, NTOT, NX, PER, X, SEED, errorflag);
[MUHATB, BHATB, BHATBW, YTB, DFB] = MNMOD(YB, OPT1, BOBS, WOBS, WOBS1, NTOT, NX, PER, X);
[SIGMAB] = SIGMOD(YTB,BHATBW,DFB,BOBS, WOBS, WOBS1, NTOT, NX, PER, X);
[FSTATB,DF1B,DF2B] = TESTMOD(SIGMAB,MUHATB,R,DFB, BOBS, WOBS, WOBS1, NTOT, NX, PER, X, errorflag);

%****DEFINE MODULE TO COMPUTE BOOTSTRAP RESULTS WITH FWR CONTROL****;
function BOOTCOM;
[U, R, X, NTOT, BOBS, WOBS, WOBS1] = INITIAL(C,U,Y,NX);
[MUHAT, BHAT, BHATW, YT, DF] = MNMOD(Y, OPT1, BOBS, WOBS, WOBS1, NTOT, NX, PER, X);
[SIGMA] = SIGMOD(YT,BHATW,DF,BOBS, WOBS, WOBS1, NTOT, NX, PER, X);
for I=1:size(R,1);
    CM=R(I,:);
    [FSTAT,DF1,DF2] = TESTMOD(SIGMA,MUHAT,R,DF, BOBS, WOBS, WOBS1, NTOT, NX, PER, X);
    if I==1
        CMMAT=FSTAT;
        DF1MAT=DF1;
        DF2MAT=DF2;
    end;
    if I>1
        CMMAT=[CMMAT FSTAT];
        DF1MAT=[DF1MAT DF1];
        DF2MAT=[DF2MAT DF2];
    end;
end;
for SIMLOOP=1:NUMSIM;
    [YB] = BOOTDAT(Y,BHAT,BOBS, WOBS, WOBS1, NTOT, NX, PER, X, SEED);
    for Q=1:size(R,1);
        CM=R(Q,:);
        [FSTATB] = BOOTSTAT(YB,OPT1,CM, BOBS, WOBS, WOBS1, NTOT, NX, PER, X, SEED);
        if Q==1
            FROW=FSTATB;
        else
            FROW=[FROW FSTATB];
        end;
    end;
    if SIMLOOP==1
        FMAT=FROW;
    else
        FMAT=[FMAT; FROW];
    end;
end;
FMAX=zeros(NUMSIM,1);
for K=1:NUMSIM;
    FMAX(K)=max(FMAT(K,:));
end;
[sorted index] = sort(FMAX);
FMAX=sorted;
RESULTS=zeros(3,size(R,1));
RESULTS(1,:)=CMMAT;
RESULTS(2,:)=DF1MAT;
RESULTS(3,:)=DF2MAT;
QCRIT=round((1-ALPHA).*NUMSIM);%%%%round
CRITV=FMAX(QCRIT);

function S=formatMatrix(A, format);
S=[];
for i = 1:size(A,1)
    S = char(S, sprintf(format,A(i,:)));
end;
