function ep_template
% ep_template
% Manual creation of blink and saccade templates.
%
%Input:
%  EPdataset (via global variable)
%
%Output: (via save file)
%   EPblink: structured file with information about blink template
%      .template   : the averaged voltage values for the template
%      .eloc       : the electrode coordinates
%      .num        : the number of blinks averaged into the template
%   EPsaccade: structured file with information about saccade template
%      .template   : the averaged voltage values for the template
%      .eloc       : the electrode coordinates
%      .num        : the number of saccades averaged into the template

%History
%  by Joseph Dien (7/13/09)
%  jdien07@mac.com
%
%  bugfix 8/27/09 JD
%  Location of windows appearing partly off screen on some systems.
%
%  modified 8/30/09 JD
%  Added support for continuous data.
%
%  bugfix 9/5/09 JD
%  Crash when loading or saving blink template with spaces in the path name.
%  Maximum number of trials/segments not updating when switching between datasets.
%
%  bugfix 12/3/09 JD
%  Fixed crash when loading in blink template for a file or pathname with a space in it.
%
%  bugfix 12/8/09 JD
%  Bad channels incorrectly interpolated when adding a new blink to the template.
%
%  modified 10/28/10 JD
%  Added support for saccade template.  Added undo and clear buttons.

%     Copyright (C) 1999-2011  Joseph Dien
%
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
%
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
%
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

global EPmain EPdataset EPtemplate

scrsz = get(0,'ScreenSize');
EPtemplate.windowHeight=500;

templateFigure=findobj('Name', 'Eye Artifact Templates Creation');

set(EPmain.handles.preprocess.template,'enable','off');
set(EPmain.handles.preprocess.preprocess,'enable','off');
set(EPmain.handles.preprocess.done,'enable','off');

if ~isempty(templateFigure)
    close(templateFigure)
end;
EPtemplate.handles.window = figure('Name', 'Eye Artifact Templates Creation', 'NumberTitle', 'off', 'Position',[201 scrsz(4)-EPtemplate.windowHeight 700 EPtemplate.windowHeight]);

EPtemplate.handles.load = uicontrol('Style', 'pushbutton', 'String', 'Load',...
    'Position', [20 EPtemplate.windowHeight-100 60 30], 'Callback', @loadTemplate);

EPtemplate.handles.save = uicontrol('Style', 'pushbutton', 'String', 'Save',...
    'Position', [20 EPtemplate.windowHeight-130 60 30], 'Callback', @saveTemplate);

EPtemplate.handles.done = uicontrol('Style', 'pushbutton', 'String', 'Clear',...
    'Position', [20 EPtemplate.windowHeight-160 60 30], 'Callback', @clearTemplates);

EPtemplate.handles.done = uicontrol('Style', 'pushbutton', 'String', 'Undo',...
    'Position', [20 EPtemplate.windowHeight-190 60 30], 'Callback', @undo);

EPtemplate.handles.done = uicontrol('Style', 'pushbutton', 'String', 'Done',...
    'Position', [20 EPtemplate.windowHeight-220 60 30], 'Callback', @done);

EPtemplate.sessionFiles=[];
for theData=1:length(EPdataset.dataset)
    if any(strcmp(EPdataset.dataset(theData).dataType,{'single_trial','continuous'})) && ~isempty(EPdataset.dataset(theData).eloc)
        EPtemplate.sessionFiles=[EPtemplate.sessionFiles theData];
    end;
end;

if ~isempty(EPtemplate.sessionFiles)
    EPtemplate.dataset=EPtemplate.sessionFiles(1); %the dataset currently being examined
else
    EPtemplate.sessionFiles=[];
end;

EPtemplate.trial=1; %the trial of the dataset currently being examined
EPtemplate.dataset=1; %the dataset currently being examined
EPtemplate.saved=1; %flag that the templates have just been saved

EPtemplate.EPdata=ep_loadEPdataset(EPdataset,EPtemplate.sessionFiles(EPtemplate.dataset));
if strcmp(EPtemplate.EPdata.dataType,'continuous')
    EPtemplate.segLength=min(EPtemplate.EPdata.Fs,length(EPtemplate.EPdata.timeNames));
    EPtemplate.maxSegs=floor(length(EPtemplate.EPdata.timeNames)/EPtemplate.segLength);
else
    EPtemplate.maxSegs=length(EPtemplate.EPdata.cellNames);
end;

%remove bad channels
EPtemplate.criteria.neighbors=EPmain.preferences.preprocess.neighbors;
EPtemplate.criteria.badchan=EPmain.preferences.preprocess.badchan;
EPtemplate.badChans=ep_detectBadChans(EPtemplate.EPdata, EPtemplate.criteria,1);

eog=ep_findEOGchans(EPtemplate.EPdata.eloc);
EPtemplate.VEOG=[eog.LUVEOG eog.RUVEOG eog.LLVEOG eog.RLVEOG];
EPtemplate.HEOG=[eog.LHEOG eog.RHEOG];

EPtemplate.minVolt=min(EPdataset.dataset(EPtemplate.dataset).plotMVmin);
EPtemplate.maxVolt=max(EPdataset.dataset(EPtemplate.dataset).plotMVmax);

EPtemplate.blinks.template=zeros(length(EPtemplate.EPdata.chanNames),1);
EPtemplate.undo.blinks.template=EPtemplate.blinks.template;
EPtemplate.blinks.eloc=EPtemplate.EPdata.eloc;
EPtemplate.blinks.num=0;
EPtemplate.undo.blinks.num=EPtemplate.blinks.num;

EPtemplate.saccades.template=zeros(length(EPtemplate.EPdata.chanNames),1);
EPtemplate.undo.saccades.template=EPtemplate.saccades.template;
EPtemplate.saccades.eloc=EPtemplate.EPdata.eloc;
EPtemplate.saccades.num=0;
EPtemplate.undo.saccades.num=EPtemplate.saccades.num;

EPtemplate.handles.leftTrial = uicontrol('Style', 'pushbutton', 'String', '<--',...
    'Position', [150 EPtemplate.windowHeight-400 50 30], 'Callback', @leftTrial);
if EPtemplate.trial ==1
    set(EPtemplate.handles.leftTrial,'enable','off');
end;

EPtemplate.handles.rightTrial = uicontrol('Style', 'pushbutton', 'String', '-->',...
    'Position', [300 EPtemplate.windowHeight-400 50 30], 'Callback', @rightTrial);
if EPtemplate.trial == EPtemplate.maxSegs
    set(EPtemplate.handles.rightTrial,'enable','off');
end;

EPtemplate.handles.trialNum=uicontrol('Style','text',...
    'String',sprintf('%d of %d',EPtemplate.trial,EPtemplate.maxSegs),'HorizontalAlignment','left',...
    'Position',[220 EPtemplate.windowHeight-400 60 30]);

EPtemplate.handles.leftDataset = uicontrol('Style', 'pushbutton', 'String', '<--',...
    'Position', [150 EPtemplate.windowHeight-450 50 30], 'Callback', @leftDataset);
if EPtemplate.dataset ==1
    set(EPtemplate.handles.leftDataset,'enable','off');
end;

EPtemplate.handles.rightDataset = uicontrol('Style', 'pushbutton', 'String', '-->',...
    'Position', [300 EPtemplate.windowHeight-450 50 30], 'Callback', @rightDataset);
if EPtemplate.dataset == length(EPtemplate.sessionFiles)
    set(EPtemplate.handles.rightDataset,'enable','off');
end;

EPtemplate.handles.dataset=uicontrol('Style','text',...
    'String',EPdataset.dataset(EPtemplate.sessionFiles(EPtemplate.dataset)).dataName,'HorizontalAlignment','left',...
    'Position',[220 EPtemplate.windowHeight-450 60 30]);

if strcmp(EPtemplate.EPdata.dataType,'continuous')
    EPtemplate.theTrial=squeeze(EPtemplate.EPdata.data(:,(EPtemplate.trial-1)*EPtemplate.segLength+1:EPtemplate.trial*EPtemplate.segLength,:,1,1));
else
    EPtemplate.theTrial=squeeze(EPtemplate.EPdata.data(:,:,EPtemplate.trial,1,1));
end
%baseline correct
baseline=max(EPtemplate.EPdata.baseline,1);
EPtemplate.theTrial=EPtemplate.theTrial-diag(mean(EPtemplate.theTrial(:,1:baseline),2))*ones(size(EPtemplate.theTrial));
[C EPtemplate.blinkMarker]=max(max(abs(EPtemplate.theTrial(EPtemplate.VEOG,:))));
[C EPtemplate.saccadeMarker]=max(abs(EPtemplate.theTrial(EPtemplate.HEOG(1),:)-EPtemplate.theTrial(EPtemplate.HEOG(2),:)));

%Waveform plot of blink channels
EPtemplate.handles.blinkPlot = axes('units','pixels','position',[150 EPtemplate.windowHeight-200 200 100]);
EPtemplate.handles.blinkWaves = plot([1:size(EPtemplate.theTrial,2)],EPtemplate.theTrial(EPtemplate.VEOG,:));
axis([1 size(EPtemplate.theTrial,2) EPtemplate.minVolt EPtemplate.maxVolt]);
EPtemplate.handles.blinkMarker=line(repmat(EPtemplate.blinkMarker,length([EPtemplate.minVolt:EPtemplate.maxVolt]),1),[EPtemplate.minVolt:EPtemplate.maxVolt],'Color','red','LineWidth',1); %marker
for i=1:length(EPtemplate.handles.blinkWaves)
    set(EPtemplate.handles.blinkWaves(i),'YDataSource',['EPtemplate.theTrial(' num2str(EPtemplate.VEOG(i)) ',:)'])
end;

%2D plot of blink channels
maxRad=0.5;
EPtemplate.gridSize=67;
[y,x] = pol2cart(([EPtemplate.EPdata.eloc.theta]/360)*2*pi,[EPtemplate.EPdata.eloc.radius]);  % transform electrode locations from polar to cartesian coordinates
y=-y; %flip y-coordinate so that nose is upwards.
plotrad = min(1.0,max([EPtemplate.EPdata.eloc.radius])*1.02);            % default: just outside the outermost electrode location
plotrad = max(plotrad,0.5);                 % default: plot out to the 0.5 head boundary
x = x*(maxRad/plotrad);
y = y*(maxRad/plotrad);

xmin = min(-maxRad,min(x));
xmax = max(maxRad,max(x));
ymin = min(-maxRad,min(y));
ymax = max(maxRad,max(y));

EPtemplate.x=round(((x/(xmax-xmin))*EPtemplate.gridSize)+ceil(EPtemplate.gridSize/2));
EPtemplate.y=round(((y/(ymax-ymin))*EPtemplate.gridSize)+ceil(EPtemplate.gridSize/2));

trialGoodChans=setdiff([1:length(EPtemplate.EPdata.chanNames)],EPtemplate.badChans);
[Xi,Yi,Zi] = griddata(EPtemplate.x(trialGoodChans),EPtemplate.y(trialGoodChans),EPtemplate.theTrial(trialGoodChans,EPtemplate.blinkMarker),[1:EPtemplate.gridSize]',[1:EPtemplate.gridSize],'linear');

EPtemplate.handles.blinkTopo = axes('units','pixels','position',[400 EPtemplate.windowHeight-200 100 100]);
EPtemplate.handles.blinkTopoImage = imagesc(Zi);

%2D plot of blink template
[Xi,Yi,Zi] = griddata(EPtemplate.x,EPtemplate.y,EPtemplate.blinks.template,[1:EPtemplate.gridSize]',[1:EPtemplate.gridSize],'linear');

EPtemplate.handles.blink = axes('units','pixels','position',[550 EPtemplate.windowHeight-200 100 100]);
EPtemplate.handles.blinkImage = imagesc(Zi);

uicontrol('Style','text',...
    'String','Templates','HorizontalAlignment','left',...
    'Position',[570 EPtemplate.windowHeight-100 65 20]);

EPtemplate.handles.addBlink = uicontrol('Style', 'pushbutton', 'String', 'Add Blink',...
    'Position', [550 EPtemplate.windowHeight-250 100 30], 'Callback', @addBlink);

EPtemplate.handles.blinkNum=uicontrol('Style','text',...
    'String',sprintf('%d',EPtemplate.blinks.num),'HorizontalAlignment','left',...
    'Position',[660 EPtemplate.windowHeight-245 30 20]);

%Waveform plot of saccade channels
EPtemplate.handles.saccadePlot = axes('units','pixels','position',[150 EPtemplate.windowHeight-350 200 100]);
EPtemplate.handles.saccadeWaves = plot([1:size(EPtemplate.theTrial,2)],EPtemplate.theTrial(EPtemplate.HEOG,:));
axis([1 size(EPtemplate.theTrial,2) EPtemplate.minVolt/10 EPtemplate.maxVolt/10]);
EPtemplate.handles.saccadeMarker=line(repmat(EPtemplate.saccadeMarker,length([EPtemplate.minVolt/10:EPtemplate.maxVolt/10]),1),[EPtemplate.minVolt/10:EPtemplate.maxVolt/10],'Color','red','LineWidth',1); %marker
for i=1:length(EPtemplate.handles.saccadeWaves)
    set(EPtemplate.handles.saccadeWaves(i),'YDataSource',['EPtemplate.theTrial(' num2str(EPtemplate.HEOG(i)) ',:)'])
end;

%2D plot of saccade channels
maxRad=0.5;
EPtemplate.gridSize=67;
[y,x] = pol2cart(([EPtemplate.EPdata.eloc.theta]/360)*2*pi,[EPtemplate.EPdata.eloc.radius]);  % transform electrode locations from polar to cartesian coordinates
y=-y; %flip y-coordinate so that nose is upwards.
plotrad = min(1.0,max([EPtemplate.EPdata.eloc.radius])*1.02);            % default: just outside the outermost electrode location
plotrad = max(plotrad,0.5);                 % default: plot out to the 0.5 head boundary
x = x*(maxRad/plotrad);
y = y*(maxRad/plotrad);

xmin = min(-maxRad,min(x));
xmax = max(maxRad,max(x));
ymin = min(-maxRad,min(y));
ymax = max(maxRad,max(y));

EPtemplate.x=round(((x/(xmax-xmin))*EPtemplate.gridSize)+ceil(EPtemplate.gridSize/2));
EPtemplate.y=round(((y/(ymax-ymin))*EPtemplate.gridSize)+ceil(EPtemplate.gridSize/2));

trialGoodChans=setdiff([1:length(EPtemplate.EPdata.chanNames)],EPtemplate.badChans);
[Xi,Yi,Zi] = griddata(EPtemplate.x(trialGoodChans),EPtemplate.y(trialGoodChans),EPtemplate.theTrial(trialGoodChans,EPtemplate.saccadeMarker),[1:EPtemplate.gridSize]',[1:EPtemplate.gridSize],'linear');

EPtemplate.handles.saccadeTopo = axes('units','pixels','position',[400 EPtemplate.windowHeight-350 100 100]);
EPtemplate.handles.saccadeTopoImage = imagesc(Zi);

%2D plot of saccade template
[Xi,Yi,Zi] = griddata(EPtemplate.x,EPtemplate.y,EPtemplate.blinks.template,[1:EPtemplate.gridSize]',[1:EPtemplate.gridSize],'linear');

EPtemplate.handles.saccade = axes('units','pixels','position',[550 EPtemplate.windowHeight-350 100 100]);
EPtemplate.handles.saccadeImage = imagesc(Zi);

EPtemplate.handles.addSaccade = uicontrol('Style', 'pushbutton', 'String', 'Add Saccade',...
    'Position', [550 EPtemplate.windowHeight-400 100 30], 'Callback', @addSaccade);

EPtemplate.handles.saccadeNum=uicontrol('Style','text',...
    'String',sprintf('%d',EPtemplate.saccades.num),'HorizontalAlignment','left',...
    'Position',[660 EPtemplate.windowHeight-395 30 20]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function done(src,eventdata)
%quit from template window

global EPtemplate EPmain

if ~EPtemplate.saved
    choice = questdlg('Would you like to save the template first, or else lose your work?', ...
        'Save Template?', ...
        'Yes','No','Cancel','Yes');
    % Handle response
    switch choice
        case 'Yes'
            saveTemplate
        case 'No'
        case 'Cancel'
            return
    end
end;

close(EPtemplate.handles.window);

set(EPmain.handles.preprocess.template,'enable','on');
set(EPmain.handles.preprocess.preprocess,'enable','on');
set(EPmain.handles.preprocess.done,'enable','on');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function leftTrial(src,eventdata)
%move to previous trial

global EPtemplate

if EPtemplate.trial > 1
    EPtemplate.trial=EPtemplate.trial-1;
end;

if EPtemplate.trial ==1
    set(EPtemplate.handles.leftTrial,'enable','off');
end;

set(EPtemplate.handles.rightTrial,'enable','on');

displayTrial

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function rightTrial(src,eventdata)
%move to next trial

global EPtemplate

if EPtemplate.trial < EPtemplate.maxSegs
    EPtemplate.trial=EPtemplate.trial+1;
end;

if EPtemplate.trial == EPtemplate.maxSegs
    set(EPtemplate.handles.rightTrial,'enable','off');
end;

set(EPtemplate.handles.leftTrial,'enable','on');

displayTrial

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function leftDataset(src,eventdata)
%move to previous dataset

global EPtemplate EPdataset EPmain

if EPtemplate.dataset > 1
    EPtemplate.dataset=EPtemplate.dataset-1;
end;

if EPtemplate.dataset ==1
    set(EPtemplate.handles.leftDataset,'enable','off');
end;

if strcmp(EPtemplate.EPdata.dataType,'continuous')
    EPtemplate.segLength=min(250,length(EPtemplate.EPdata.timeNames));
    EPtemplate.maxSegs=floor(length(EPtemplate.EPdata.timeNames)/EPtemplate.segLength);
else
    EPtemplate.maxSegs=length(EPtemplate.EPdata.cellNames);
end;

EPtemplate.trial=1;

set(EPtemplate.handles.rightDataset,'enable','on');

delete(EPtemplate.handles.dataset)
EPtemplate.handles.dataset=uicontrol('Style','text',...
    'String',EPdataset.dataset(EPtemplate.sessionFiles(EPtemplate.dataset)).dataName,'HorizontalAlignment','left',...
    'Position',[220 EPtemplate.windowHeight-450 60 30]);

EPtemplate.EPdata=ep_loadEPdataset(EPdataset,EPtemplate.sessionFiles(EPtemplate.dataset));
%remove bad channels
EPtemplate.criteria.neighbors=EPmain.preferences.preprocess.neighbors;
EPtemplate.criteria.badchan=EPmain.preferences.preprocess.badchan;
EPtemplate.badChans=ep_detectBadChans(EPtemplate.EPdata, EPtemplate.criteria,1);

set(EPtemplate.handles.leftTrial,'enable','off');
set(EPtemplate.handles.rightTrial,'enable','on');

displayTrial

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function rightDataset(src,eventdata)
%move to next dataset

global EPtemplate EPdataset EPmain

if EPtemplate.dataset < length(EPtemplate.sessionFiles)
    EPtemplate.dataset=EPtemplate.dataset+1;
end;

if EPtemplate.dataset == length(EPtemplate.sessionFiles)
    set(EPtemplate.handles.rightDataset,'enable','off');
end;

if strcmp(EPtemplate.EPdata.dataType,'continuous')
    EPtemplate.segLength=min(250,length(EPtemplate.EPdata.timeNames));
    EPtemplate.maxSegs=floor(length(EPtemplate.EPdata.timeNames)/EPtemplate.segLength);
else
    EPtemplate.maxSegs=length(EPtemplate.EPdata.cellNames);
end;

EPtemplate.trial=1;

set(EPtemplate.handles.leftDataset,'enable','on');

delete(EPtemplate.handles.dataset)
EPtemplate.handles.dataset=uicontrol('Style','text',...
    'String',EPdataset.dataset(EPtemplate.sessionFiles(EPtemplate.dataset)).dataName,'HorizontalAlignment','left',...
    'Position',[220 EPtemplate.windowHeight-450 60 30]);

EPtemplate.EPdata=ep_loadEPdataset(EPdataset,EPtemplate.sessionFiles(EPtemplate.dataset));
%remove bad channels
EPtemplate.criteria.neighbors=EPmain.preferences.preprocess.neighbors;
EPtemplate.criteria.badchan=EPmain.preferences.preprocess.badchan;
EPtemplate.badChans=ep_detectBadChans(EPtemplate.EPdata, EPtemplate.criteria,1);

set(EPtemplate.handles.leftTrial,'enable','off');
set(EPtemplate.handles.rightTrial,'enable','on');

displayTrial

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function displayTrial
%display the updated trial data

global EPtemplate

if strcmp(EPtemplate.EPdata.dataType,'continuous')
    EPtemplate.theTrial=squeeze(EPtemplate.EPdata.data(:,(EPtemplate.trial-1)*EPtemplate.segLength+1:EPtemplate.trial*EPtemplate.segLength,:,1,1));
else
    EPtemplate.theTrial=squeeze(EPtemplate.EPdata.data(:,:,EPtemplate.trial,1,1));
end
%baseline correct
baseline=max(EPtemplate.EPdata.baseline,1);
EPtemplate.theTrial=EPtemplate.theTrial-diag(mean(EPtemplate.theTrial(:,1:baseline),2))*ones(size(EPtemplate.theTrial));
[C EPtemplate.blinkMarker]=max(max(abs(EPtemplate.theTrial(EPtemplate.VEOG,:))));
[C EPtemplate.saccadeMarker]=max(abs(EPtemplate.theTrial(EPtemplate.HEOG(1),:)-EPtemplate.theTrial(EPtemplate.HEOG(2),:)));
delete(EPtemplate.handles.blinkMarker);
delete(EPtemplate.handles.saccadeMarker);
set(EPtemplate.handles.window,'CurrentAxes',EPtemplate.handles.blinkPlot)
EPtemplate.handles.blinkMarker=line(repmat(EPtemplate.blinkMarker,length([EPtemplate.minVolt:EPtemplate.maxVolt]),1),[EPtemplate.minVolt:EPtemplate.maxVolt],'Color','red','LineWidth',1); %marker
set(EPtemplate.handles.window,'CurrentAxes',EPtemplate.handles.saccadePlot)
EPtemplate.handles.saccadeMarker=line(repmat(EPtemplate.saccadeMarker,length([EPtemplate.minVolt/10:EPtemplate.maxVolt/10]),1),[EPtemplate.minVolt/10:EPtemplate.maxVolt/10],'Color','red','LineWidth',1); %marker

refreshdata(EPtemplate.handles.blinkWaves)
refreshdata(EPtemplate.handles.saccadeWaves)

%2D blink plot
trialGoodChans=setdiff([1:length(EPtemplate.EPdata.chanNames)],EPtemplate.badChans);
[Xi,Yi,Zi] = griddata(EPtemplate.x(trialGoodChans),EPtemplate.y(trialGoodChans),EPtemplate.theTrial(trialGoodChans,EPtemplate.blinkMarker),[1:EPtemplate.gridSize]',[1:EPtemplate.gridSize],'linear');

delete(EPtemplate.handles.blinkTopoImage);
set(EPtemplate.handles.window,'CurrentAxes',EPtemplate.handles.blinkTopo)
EPtemplate.handles.blinkTopoImage = imagesc(Zi);

%2D saccade plot
trialGoodChans=setdiff([1:length(EPtemplate.EPdata.chanNames)],EPtemplate.badChans);
[Xi,Yi,Zi] = griddata(EPtemplate.x(trialGoodChans),EPtemplate.y(trialGoodChans),EPtemplate.theTrial(trialGoodChans,EPtemplate.saccadeMarker),[1:EPtemplate.gridSize]',[1:EPtemplate.gridSize],'linear');

delete(EPtemplate.handles.saccadeTopoImage);
set(EPtemplate.handles.window,'CurrentAxes',EPtemplate.handles.saccadeTopo)
EPtemplate.handles.saccadeTopoImage = imagesc(Zi);

delete(EPtemplate.handles.trialNum);
EPtemplate.handles.trialNum=uicontrol('Style','text',...
    'String',sprintf('%d of %d',EPtemplate.trial,EPtemplate.maxSegs),'HorizontalAlignment','left',...
    'Position',[220 EPtemplate.windowHeight-400 60 30]);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function addBlink(src,eventdata)
%add data at marker to the blink template

global EPtemplate

if strcmp(EPtemplate.EPdata.dataType,'continuous')
    theTrial=squeeze(EPtemplate.EPdata.data(:,(EPtemplate.trial-1)*EPtemplate.segLength+1:EPtemplate.trial*EPtemplate.segLength,:,1,1));
else
    theTrial=squeeze(EPtemplate.EPdata.data(:,:,EPtemplate.trial,1,1));
end

%baseline correct
baseline=max(EPtemplate.EPdata.baseline,1);
theTrial=theTrial-diag(mean(theTrial(:,1:baseline),2))*ones(size(theTrial));

theBlink=theTrial(:,EPtemplate.blinkMarker);

%if there are bad channels, then first interpolate them
if ~isempty(EPtemplate.badChans)
    trialGoodChans=setdiff([1:length(EPtemplate.EPdata.chanNames)],EPtemplate.badChans);
    [Xi,Yi,Zi] = griddata(EPtemplate.x(trialGoodChans),EPtemplate.y(trialGoodChans),theBlink(trialGoodChans),[1:EPtemplate.gridSize]',[1:EPtemplate.gridSize],'v4');
    for theBadchan=1:length(EPtemplate.badChans)
        badchan=EPtemplate.badChans(theBadchan);
        theBlink(badchan)=Zi(EPtemplate.y(badchan),EPtemplate.x(badchan));
    end;
end;

EPtemplate.undo.blinks.template=EPtemplate.blinks.template;
EPtemplate.undo.blinks.num=EPtemplate.blinks.num;
EPtemplate.undo.saccades.template=EPtemplate.saccades.template;
EPtemplate.undo.saccades.num=EPtemplate.saccades.num;

EPtemplate.blinks.template=EPtemplate.blinks.template*(EPtemplate.blinks.num/(EPtemplate.blinks.num+1))+theBlink/(EPtemplate.blinks.num+1);
EPtemplate.blinks.num=EPtemplate.blinks.num+1;

[Xi,Yi,Zi] = griddata(EPtemplate.x,EPtemplate.y,EPtemplate.blinks.template,[1:EPtemplate.gridSize]',[1:EPtemplate.gridSize],'linear');

delete(EPtemplate.handles.blinkImage);
set(EPtemplate.handles.window,'CurrentAxes',EPtemplate.handles.blink)
EPtemplate.handles.blinkImage = imagesc(Zi);

delete(EPtemplate.handles.blinkNum);
EPtemplate.handles.blinkNum=uicontrol('Style','text',...
    'String',sprintf('%d',EPtemplate.blinks.num),'HorizontalAlignment','left',...
    'Position',[660 EPtemplate.windowHeight-245 30 20]);

EPtemplate.saved=0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function addSaccade(src,eventdata)
%add data at marker to the saccade template

global EPtemplate

if strcmp(EPtemplate.EPdata.dataType,'continuous')
    theTrial=squeeze(EPtemplate.EPdata.data(:,(EPtemplate.trial-1)*EPtemplate.segLength+1:EPtemplate.trial*EPtemplate.segLength,:,1,1));
else
    theTrial=squeeze(EPtemplate.EPdata.data(:,:,EPtemplate.trial,1,1));
end

%baseline correct
baseline=max(EPtemplate.EPdata.baseline,1);
theTrial=theTrial-diag(mean(theTrial(:,1:baseline),2))*ones(size(theTrial));

theSaccade=theTrial(:,EPtemplate.saccadeMarker);

%if there are bad channels, then first interpolate them
if ~isempty(EPtemplate.badChans)
    trialGoodChans=setdiff([1:length(EPtemplate.EPdata.chanNames)],EPtemplate.badChans);
    [Xi,Yi,Zi] = griddata(EPtemplate.x(trialGoodChans),EPtemplate.y(trialGoodChans),theSaccade(trialGoodChans),[1:EPtemplate.gridSize]',[1:EPtemplate.gridSize],'v4');
    for theBadchan=1:length(EPtemplate.badChans)
        badchan=EPtemplate.badChans(theBadchan);
        theSaccade(badchan)=Zi(EPtemplate.y(badchan),EPtemplate.x(badchan));
    end;
end;


if (theSaccade(EPtemplate.HEOG(2))-theSaccade(EPtemplate.HEOG(1))) > 0 %if rightward saccade
%     [Xi,Yi,Zi] = griddata(EPtemplate.x,EPtemplate.y,theSaccade,[1:EPtemplate.gridSize]',[1:EPtemplate.gridSize],'v4');
%     for theChan=1:length(theSaccade)
%         leftSaccade(theChan,1)=Zi(EPtemplate.y(theChan),EPtemplate.gridSize-EPtemplate.x(theChan)+1); %flip the scalp topography
%     end;
    leftSaccade=-theSaccade;
else
    leftSaccade=theSaccade;
end

EPtemplate.undo.blinks.template=EPtemplate.blinks.template;
EPtemplate.undo.blinks.num=EPtemplate.blinks.num;
EPtemplate.undo.saccades.template=EPtemplate.saccades.template;
EPtemplate.undo.saccades.num=EPtemplate.saccades.num;

EPtemplate.saccades.template=EPtemplate.saccades.template*(EPtemplate.saccades.num/(EPtemplate.saccades.num+1))+leftSaccade/(EPtemplate.saccades.num+1);
EPtemplate.saccades.num=EPtemplate.saccades.num+1;

[Xi,Yi,Zi] = griddata(EPtemplate.x,EPtemplate.y,EPtemplate.saccades.template,[1:EPtemplate.gridSize]',[1:EPtemplate.gridSize],'linear');

delete(EPtemplate.handles.saccadeImage);
set(EPtemplate.handles.window,'CurrentAxes',EPtemplate.handles.saccade)
EPtemplate.handles.saccadeImage = imagesc(Zi);

delete(EPtemplate.handles.saccadeNum);
EPtemplate.handles.saccadeNum=uicontrol('Style','text',...
    'String',sprintf('%d',EPtemplate.saccades.num),'HorizontalAlignment','left',...
    'Position',[660 EPtemplate.windowHeight-395 30 20]);

EPtemplate.saved=0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function loadTemplate(src,eventdata)
%load blink template

global EPtemplate EPdataset

EPdata=ep_loadEPdataset(EPdataset,EPtemplate.dataset);

[FileName,PathName,FilterIndex] = uigetfile('','Load Templates','blinks');
if FileName ~= 0
    eval(['load ''' PathName FileName '''']);
    if ~exist('EPblink','var')
        msg{1}='Not a blink template.';
        [msg]=ep_errorMsg(msg);
        return
    end;
    
    if length(EPblink.eloc) ~= length(EPtemplate.blinks.eloc)
        msg{1}='Number of blink electrodes different from current dataset.';
        [msg]=ep_errorMsg(msg);
        return
    end;
    
    if any([EPtemplate.blinks.eloc.theta]-[EPdata.eloc.theta])
        msg{1}='Blink electrode locations not consistent with current data.';
        [msg]=ep_errorMsg(msg);
        return
    end;
    
    if ~exist('EPblink','var')
        EPblink.template=zeros(length(EPtemplate.EPdata.chanNames),1);
        EPblink.num=0;
    end;

    EPtemplate.undo.blinks.template=EPtemplate.blinks.template;
    EPtemplate.undo.blinks.num=EPtemplate.blinks.num;
    
    EPtemplate.blinks=EPblink;
end;

[FileName,PathName,FilterIndex] = uigetfile('','Load Templates','saccades');
if FileName ~= 0
    eval(['load ''' PathName FileName '''']);
    if ~exist('EPsaccade','var')
        msg{1}='Not a saccade template.';
        [msg]=ep_errorMsg(msg);
        return
    end;
    
    if length(EPsaccade.eloc) ~= length(EPtemplate.saccades.eloc)
        msg{1}='Number of saccade electrodes different from current dataset.';
        [msg]=ep_errorMsg(msg);
        return
    end;
    
    if any([EPtemplate.saccades.eloc.theta]-[EPdata.eloc.theta])
        msg{1}='Saccade electrode locations not consistent with current data.';
        [msg]=ep_errorMsg(msg);
        return
    end;
    
    if ~exist('EPsaccade','var')
        EPsaccade.template=zeros(length(EPtemplate.EPdata.chanNames),1);
        EPsaccade.num=0;
    end;

    EPtemplate.undo.saccades.template=EPtemplate.saccades.template;
    EPtemplate.undo.saccades.num=EPtemplate.saccades.num;
    
    EPtemplate.saccades=EPsaccade;
end;

[Xi,Yi,Zi] = griddata(EPtemplate.x,EPtemplate.y,EPtemplate.blinks.template,[1:EPtemplate.gridSize]',[1:EPtemplate.gridSize],'linear');

delete(EPtemplate.handles.blinkImage);
set(EPtemplate.handles.window,'CurrentAxes',EPtemplate.handles.blink)
EPtemplate.handles.blinkImage = imagesc(Zi);

delete(EPtemplate.handles.blinkNum);
EPtemplate.handles.blinkNum=uicontrol('Style','text',...
    'String',sprintf('%d',EPtemplate.blinks.num),'HorizontalAlignment','left',...
    'Position',[660 EPtemplate.windowHeight-245 30 20]);

[Xi,Yi,Zi] = griddata(EPtemplate.x,EPtemplate.y,EPtemplate.saccades.template,[1:EPtemplate.gridSize]',[1:EPtemplate.gridSize],'linear');

delete(EPtemplate.handles.saccadeImage);
set(EPtemplate.handles.window,'CurrentAxes',EPtemplate.handles.saccade)
EPtemplate.handles.saccadeImage = imagesc(Zi);

delete(EPtemplate.handles.saccadeNum);
EPtemplate.handles.saccadeNum=uicontrol('Style','text',...
    'String',sprintf('%d',EPtemplate.saccades.num),'HorizontalAlignment','left',...
    'Position',[660 EPtemplate.windowHeight-395 30 20]);

EPtemplate.saved=1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function saveTemplate(src,eventdata)
%save blink template

global EPtemplate

[FileName,PathName,FilterIndex] = uiputfile('','Save Templates','blinks');
EPblink=EPtemplate.blinks;
eval(['save ''' PathName FileName ''' EPblink']);

[FileName,PathName,FilterIndex] = uiputfile('','Save Templates','saccades');
EPsaccade=EPtemplate.saccades;
eval(['save ''' PathName FileName ''' EPsaccade']);

EPtemplate.saved=1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function undo(src,eventdata)
%undo last change to templates

global EPtemplate

temp.blinks.template=EPtemplate.blinks.template;
temp.blinks.num=EPtemplate.blinks.num;
temp.saccades.template=EPtemplate.saccades.template;
temp.saccades.num=EPtemplate.saccades.num;

EPtemplate.blinks.template=EPtemplate.undo.blinks.template;
EPtemplate.blinks.num=EPtemplate.undo.blinks.num;
EPtemplate.saccades.template=EPtemplate.undo.saccades.template;
EPtemplate.saccades.num=EPtemplate.undo.saccades.num;

EPtemplate.undo.blinks.template=temp.blinks.template;
EPtemplate.undo.blinks.num=temp.blinks.num;
EPtemplate.undo.saccades.template=temp.saccades.template;
EPtemplate.undo.saccades.num=temp.saccades.num;

delete(EPtemplate.handles.blinkImage);
set(EPtemplate.handles.window,'CurrentAxes',EPtemplate.handles.blink)
[Xi,Yi,Zi] = griddata(EPtemplate.x,EPtemplate.y,EPtemplate.blinks.template,[1:EPtemplate.gridSize]',[1:EPtemplate.gridSize],'linear');
EPtemplate.handles.blinkImage = imagesc(Zi);

delete(EPtemplate.handles.blinkNum);
EPtemplate.handles.blinkNum=uicontrol('Style','text',...
    'String',sprintf('%d',EPtemplate.blinks.num),'HorizontalAlignment','left',...
    'Position',[660 EPtemplate.windowHeight-245 30 20]);

delete(EPtemplate.handles.saccadeImage);
set(EPtemplate.handles.window,'CurrentAxes',EPtemplate.handles.saccade)
[Xi,Yi,Zi] = griddata(EPtemplate.x,EPtemplate.y,EPtemplate.saccades.template,[1:EPtemplate.gridSize]',[1:EPtemplate.gridSize],'linear');
EPtemplate.handles.saccadeImage = imagesc(Zi);

delete(EPtemplate.handles.saccadeNum);
EPtemplate.handles.saccadeNum=uicontrol('Style','text',...
    'String',sprintf('%d',EPtemplate.saccades.num),'HorizontalAlignment','left',...
    'Position',[660 EPtemplate.windowHeight-395 30 20]);

EPtemplate.saved=0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function clearTemplates(src,eventdata)
%clear templates

global EPtemplate

EPtemplate.undo.blinks.template=EPtemplate.blinks.template;
EPtemplate.undo.blinks.num=EPtemplate.blinks.num;
EPtemplate.undo.saccades.template=EPtemplate.saccades.template;
EPtemplate.undo.saccades.num=EPtemplate.saccades.num;

EPtemplate.blinks.template=zeros(length(EPtemplate.EPdata.chanNames),1);
EPtemplate.blinks.num=0;
EPtemplate.saccades.template=zeros(length(EPtemplate.EPdata.chanNames),1);
EPtemplate.saccades.num=0;

delete(EPtemplate.handles.blinkImage);
set(EPtemplate.handles.window,'CurrentAxes',EPtemplate.handles.blink)
[Xi,Yi,Zi] = griddata(EPtemplate.x,EPtemplate.y,EPtemplate.blinks.template,[1:EPtemplate.gridSize]',[1:EPtemplate.gridSize],'linear');
EPtemplate.handles.blinkImage = imagesc(Zi);

delete(EPtemplate.handles.blinkNum);
EPtemplate.handles.blinkNum=uicontrol('Style','text',...
    'String',sprintf('%d',EPtemplate.blinks.num),'HorizontalAlignment','left',...
    'Position',[660 EPtemplate.windowHeight-245 30 20]);

delete(EPtemplate.handles.saccadeImage);
set(EPtemplate.handles.window,'CurrentAxes',EPtemplate.handles.saccade)
[Xi,Yi,Zi] = griddata(EPtemplate.x,EPtemplate.y,EPtemplate.saccades.template,[1:EPtemplate.gridSize]',[1:EPtemplate.gridSize],'linear');
EPtemplate.handles.saccadeImage = imagesc(Zi);

delete(EPtemplate.handles.saccadeNum);
EPtemplate.handles.saccadeNum=uicontrol('Style','text',...
    'String',sprintf('%d',EPtemplate.saccades.num),'HorizontalAlignment','left',...
    'Position',[660 EPtemplate.windowHeight-395 30 20]);

EPtemplate.saved=0;