function [totsaccadeTrialNum, outputLog] = ep_fixBlink(inFile, startChunk, endChunk, badDataCriteria, badChans, eog, sacctemplate, saccadeFile, baseline, saturation, refChan, butterflyFig, theSubject)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% [totsaccadeTrialNum, outputLog] = ep_fixBlink(inFile, startChunk, endChunk, badDataCriteria, badChans, eog, sacctemplate, saccadeFile, baseline, saturation, refChan, butterflyFig, theSubject)
%
%	Reads in file chunks generated by chunkInputFile function
%	Function runs through all the files given and outputs them as fixed chunks.
%	chunks will still need to be rejoined.
%
%Inputs
%	inFile:     filename (not including the .mat suffix or chunk number.  e.g., "NT5") and sourcepath.
%	startChunk: starting chunk (usually 1)
%   endChunk:   ending chunk
%   badDataCriteria:  Criteria for detecting bad data.
%       .window:    moving average window for smoothing
%       .minmax:    difference from minimum to maximum for bad channel
%       .trialminmax:  difference from minimum to maximum for bad trial
%       .badnum:    percent of bad channels exceeded to declare bad trial, rounding down
%       .hminmax:   difference from minimum to maximum for bad horizontal EOG
%       .neighbors: number of electrodes considered to be neighbors
%       .badchan:   maximum microvolt difference allowed from best matching neighbor
%       .maxneighbor:   maximum microvolt difference allowed from best matching neighbor
%       .blink:     threshold correlation with blink template, 0 to 1
%       .saccade:     threshold correlation with saccade template, 0 to 1
%       .saccademin:  v Saccade Fac is the minimum HEOG voltage difference required to constitute a possible saccade.
%       .detrend:   1 to detrend
%       .badtrials: percentage of good trials chan is bad to declare a channel globally bad
%       .replace:   1 to interpolate bad channels from neighbors.
%       .noadjacent:1 to not allow adjacent bad channels (trial or subject declared bad)
%       .movefacs  : number of factors to retain during movement correction.
%       .channelMode: 'replace' to interpolate bad channels, 'mark' to mark them with a spike, and 'none' to do nothing.
%       .trialMode: 'fix' to fix bad trial data and 'none' to do nothing.
%   badChans:   list of bad channels to exclude from blink detection process.
%   eog:        EOG channels.
%   sacctemplate:   source of saccade template (fileTemplate: load 'saccades' file)
%   saccadeFile:  file with saccade template.  Assumed to be in the same directory as the data file.
%   baseline:   array of sample numbers to permanently baseline correct the trials with.  [] means don't correct.
%   saturation: followed by range of non-saturated data values
%   refChan:    Array of reference channels.
%   butterflyFig:  the handle for the output figure.  Otherwise, will open a new figure.
%   theSubject: which subject of the file is being processed.
%
%   The input chunks are EP format data files.
%
%Outputs
%   totsaccadeTrialNum: Total list of saccade trials.
%	Saves files with blinks removed, replacing the original chunked files.
%   If baseline option is used (provides better results, particularly when
%   there are large DC swings in the recordings), then the output will be
%   baselined.
%   outputLog: output messages from saccade fixing process
%
% History:
%
% by Joseph Dien (10/00)
% jdien07@mac.com
%
% modified 6/22/01 JD
% added bad channel and threshold parameters.  Also changed chunkNum to startChunk and endChunk.
%
% modified 5/27/02 JD
% added baseline correct feature.  Fix to bad chans so they are zeroed even if no blink factors.
%
% bugfix 10/11/02 JD
% added fix to baseline feature for when a blink occurs during the baseline
% period.  Output files will now be baselined.  Also, added check for chans
% with zero variability.
%
% modified 10/20/02 JD
% since ICA removal sometimes degrades trials without blinks, will now
% remove blink components only for trials where the largest feature has the
% topography of a blink (correlates .9 with raw EEG).
%
% modified 12/06/02 JD
% Can automatically generate individualized blink template.  Maxed out VEOG time
% points dropped from correction and baseline process and left intact (to trigger
% eyeblink rejection in averaging program) since they can't be corrected
% properly.  Added output log.
%
% modified 02/04/03  JD
% Can now specify blink template file name.  Saves topoplots of blink facs.
%  Added bothTemplate option.
%
% bugfix 4/1/04 JD
% Allow for spaces in file names.
%
% bugfix 11/15/05 JD
% Fixed bug that crashed program when a customized blink template was
% specified.
%
% bugfix 7/24/06 JD
% Contents of blink template no longer prints out on screen.  Added some error checking.
%
% modified 3/22/07 JD
% Finished autotemplate feature.  Also now takes into account possibility
% of blink recoveries (with an inverse topography).  Also now handles
% different montages through EEG_hardware function.
%
% modified 2/17/08 JD
% Checks to see if binary version of runica is installed and uses it if available.
%
% modified 11/17/08 JD
% Eliminated topoplot of blink factors. Made baseline specification more flexible.
%
% modified 12/13/08 JD
% Increased scaling for blink plots to make them more visible.
%
% modified 2/11/09 JD
% Dropped trialsize input field.  No longer zeroes bad channels.  Replaced
% threshold field with full badDataCriteria field.
%
% modified 3/14/09 JD
% Changed to use EP format data to provide more flexibility with I/O functions.
%
% modified 3/17/09 JD
% Only subtracted blink activity (that met variance reduced criterion) is shown for blink data figure.
%
% bugfix 4/16/09 JD
% Fixed crash when overall blink direction equaled exactly zero.
%
% bugfix &  modified 7/14/09 JD
% Fixed bothTemplate option, which was functioning the same as autoTemplate, ignoring the file template.
% Changed output from list of blink facs to list of blink trials.  No longer zeroes out bad channels.
% Includes list of blink trials in the data chunk.  Output figure can now be passed from function call.
% blink file assumed to be in the directory of the data file.  Uses updated blink template format.
%
% modified 8/28/09 JD
% Modified to use new option for binary runICA to control the output file names so that they can be cleaned up
% and so that unpredictable crash from bug causing strange file names can be circumvented.
%
% modified 9/4/09 JD
% Added refChan input to deal with mean mastoid data where the presence of the two reference channels (correlated -1)
% was causing ICA problems.
%
% bugfix 10/17/09 JD
% Crash when only one VEOG channel.
%
% modified 10/28/09 JD
% Added option to disable preprocessing figure for low memory situations.
%
% bugfix 11/20/09 JD
% Replaced "union" commands with "unique" commands because certain situations caused the "union" command to crash in
% Matlab 2007.
%
% bugfix 12/3/09 JD
% Additional check for ICA failure (denoted by imaginary numbers for weights).
%
% modified 2/11/10 JD
% Will now work with subject average files with multiple subjects.
% BlinkTrial no longer initialized to zero.
%
% modified & bugfix 5/15/10 JD
% Made more memory efficient by reducing resolution of the blink plots to no more than 10000 points regardless of size of data.
% Fixed when data is mean mastoid, not setting second reference channel to be inverse of the first reference channel.
% Fixed crashes when explicit reference channel is present.
%
%  modified 10/13/10 JD
%  For continuous files, data now divided into one second epochs and can be artifact rejected in an epochwise fashion
%  in same fashion as segmented data.
%
%  modified 10/28/10 JD
%  Added support for HEOG saccade correction.
% 
%  bugfix 11/3/10 JD
%  Now handles mismatch where file template or data has implicit reference and the other one has explicit reference.
% 
%  modified 1/20/11 JD
%  Added support for manually specifying EOG channels in the preferences.  Missing EOG channel marked as a -1.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%     Copyright (C) 1999-2011  Joseph Dien
%
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
%
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
%
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

msg='Fixing saccades.';
disp(msg);
outputLog{1}=msg;
totsaccadeTrialNum=[];

if nargin < 12
    butterflyFig=figure('Name','Artifact Correction','NumberTitle','off');
end;

% VEOG(1) = eog.LUVEOG;
% VEOG(2) = eog.RUVEOG;
% VEOG(3) = eog.LLVEOG;
% VEOG(4) = eog.RLVEOG;
% for i=1:4
%     goodVEOG(i)=i*~any(VEOG(i) == badChans);
% end;
% goodVEOG=goodVEOG(find(goodVEOG));
% VEOG=VEOG(goodVEOG);

theEOG(1) = eog.LHEOG;
theEOG(2) = eog.RHEOG;

HEOG=[];
for i=1:length(theEOG)
    if isempty(intersect(theEOG(i),badChans)) && (theEOG(i) ~= -1)
        HEOG=[HEOG theEOG(i)];
    end;
end;

if any(strcmp(sacctemplate, {'fileTemplate'}))
    [fileDir, name, ext] = fileparts(inFile);
    eval(['load ''' saccadeFile '''']);
    
    if ~exist('EPsaccade','var')
        msg='Not a saccade template.';
        [msg]=ep_errorMsg({msg});
        return
    end;
    
    if ~exist('EPsaccade','var')
        msg='No saccade template so file template saccade correction disabled.';
        outputLog{end+1}=msg;
        disp(' ');
        disp('**************************************************************');
        disp(msg);
        disp('**************************************************************');
        disp(' ');
        saccadeFileCorrect=0;
    else
        saccadeFileCorrect=1;
    end;
    
    eval(['load ''' deblank(inFile) '''-' num2str(1) '.mat']);
    
    if saccadeFileCorrect
        saccades=EPsaccade.template;
        if size(saccades,1) > 1
            saccades = saccades';
        end;
        if size(saccades,1) > 1
            msg='Something is wrong with the saccade template.  It has the wrong dimensions.';
            outputLog{end+1}=msg;
            disp(' ');
            disp('**************************************************************');
            disp(msg);
            disp('**************************************************************');
            disp(' ');
            return;
        end;
        
        if length(EPsaccade.eloc) ~= length(dataChunk.eloc)
            numSaccEEG=length(find(strcmp('EEG',{EPsaccade.eloc.type})));
            numSaccREF=length(find(strcmp('REF',{EPsaccade.eloc.type})));
            numDataEEG=length(find(strcmp('EEG',{dataChunk.eloc.type})));
            numDataREF=length(find(strcmp('REF',{dataChunk.eloc.type})));
            if numSaccEEG == numDataEEG
                if numSaccREF > numDataREF
                    saccades = saccades(find(strcmp('EEG',{EPsaccade.eloc.type}))); %drop reference channels in saccade template
                elseif numSaccREF < numDataREF
                    badChans = unique([badChans find(strcmp('REF',{dataChunk.eloc.type}))]); %drop reference channels in data temporarily
                end;
            else
                msg=['Number of saccade template electrodes (' num2str(length(EPsaccade.eloc)) ') different from the data (' num2str(length(dataChunk.eloc)) ').'];
                outputLog{end+1}=msg;
                disp(' ');
                disp('**************************************************************');
                disp(msg);
                disp('**************************************************************');
                disp(' ');
                return;
            end;
        end;
        
        if any([EPsaccade.eloc.theta]-[dataChunk.eloc.theta])
            msg='Saccade template electrode locations not consistent with the data.';
            outputLog{end+1}=msg;
            disp(' ');
            disp('**************************************************************');
            disp(msg);
            disp('**************************************************************');
            disp(' ');
            return;
        end;        
    end;
end;

% if any(strcmp(template, {'autoTemplate','bothTemplate'}))
%     saccadeSigns = [1 1 1 -1 -1 -1; 1 1 1 -1 -1 -1; 1 1 1 -1 -1 -1; -1 -1 -1 1 1 1; -1 -1 -1 1 1 1; -1 -1 -1 1 1 1];
%     saccadeSigns=saccadeSigns(goodHEOG,:);
%     saccadeSigns=saccadeSigns(:,goodHEOG);
%     saccadeSign = [1 1 1 -1 -1 -1]'; %left saccades
%     saccadeSign=saccadeSign(goodHEOG,1);
% end;

if ~any(strcmp(sacctemplate, {'fileTemplate'}))
    msg='Saccade template variable must be set to fileTemplate.';
    outputLog{end+1}=msg;
    disp(' ');
    disp('**************************************************************');
    disp(msg);
    disp('**************************************************************');
    disp(' ');
    return;
end;

facsdone = [];
badChans = badChans(find(badChans));	%drop out bad channels set as being zero
if length(refChan) == 2
    badChans = unique([badChans,refChan(2)]);     %temporarily take second reference site out of ICA
end;

outputLog=[];

warning off GLREN:doubleBufferSlow; %Turn off warning message about doublebuffer not available.

for chunk = startChunk:endChunk
    if endChunk > startChunk
        msg=[deblank(inFile) '-' num2str(chunk)];
        disp(msg);
        outputLog{end+1}=msg;
    end;
    eval(['load ''' deblank(inFile) '''-' num2str(chunk) '.mat']);
    
    if strcmp(dataChunk.dataType,'continuous')
        theSegment = 'one second epoch';
    else
        theSegment = 'trial';
    end;
    
    numChans=length(dataChunk.chanNames);
    if length(dataChunk.facNames) > 1
        msg='This function is not intended for application to factor data.';
        outputLog{end+1}=msg;
        disp(' ');
        disp('**************************************************************');
        disp(msg);
        disp('**************************************************************');
        disp(' ');
        return;
    end;
    if length(setdiff(HEOG,badChans)) < 2
        msg='At least one HEOG channel is a bad channel so no saccade correction can be made.';
        outputLog{end+1}=msg;
        disp(' ');
        disp('**************************************************************');
        disp(msg);
        disp('**************************************************************');
        disp(' ');
        return;
    end;
    
    trialdata=reshape(dataChunk.data(:,:,:,theSubject),numChans,[]);
    
    displayPeriod=size(trialdata,2);    %Number of timepoints to graph in display.
    decimateSamples=ceil(max(1,displayPeriod/10000));
    
    if displayPeriod == 1
        msg='There is only one time point and so the data cannot be blink/saccade corrected.';
        outputLog{end+1}=msg;
        disp(' ');
        disp('**************************************************************');
        disp(msg);
        disp('**************************************************************');
        disp(' ');
        return;
    end;
    
    if strcmp(dataChunk.dataType,'continuous')
        numTrials=floor(size(trialdata,2)/dataChunk.Fs); %excess time points are tacked onto final epoch
        trialSize = min(dataChunk.Fs,size(trialdata,2)); %one second epochs
    else
        trialSize = length(dataChunk.timeNames);
        numTrials = length(dataChunk.cellNames);
    end;
    
    if (length(refChan) == 1) && std(trialdata(refChan(1),:)') == 0
        badChans = unique([badChans,refChan(1)]);     %temporarily take first reference site out of ICA
    end;
    
    chans = [1:numChans];
    chans(badChans)=0;
    chans = chans(find(chans));
    
    for i=1:length(HEOG)
        goodHEOG(i)=find(chans==HEOG(i));
    end;
    
%     if length(VEOG) ==1
%         goodPoints = find(trialdata(VEOG,:) < saturation(2) & trialdata(VEOG,:) > saturation(1));
%     else
%         goodPoints = find((max(trialdata(VEOG,:)) < saturation(2)) & (min(trialdata(VEOG,:)) > saturation(1)));
%     end;
    
    %     if ~isempty(butterflyFig)
    %         figure(butterflyFig(chunk));
    %         [pathstr, fileName, ext]=fileparts(inFile);
    %         subplot(10,1,1), plot([1:decimateSamples:displayPeriod],trialdata([badChans chans],1:decimateSamples:displayPeriod));
    %         axis([1 displayPeriod -200 200])
    %         title([deblank(fileName) '-' num2str(chunk)],'Interpreter','none');
    %     end;
    
    %     if ~isempty(baseline)			%subtract out baselines before performing ICA
    %         for i = 1:numTrials
    %             epoch=goodPoints(find((goodPoints>((i-1)*trialSize)) & (goodPoints<=(i*trialSize))));
    %             base=goodPoints((i-1)*trialSize+baseline);
    %             if ~isempty(epoch) && ~isempty(base)
    %                 baseMeans = mean(trialdata(:,base),2);
    %                 trialdata(:,epoch)=trialdata(:,epoch)-diag(baseMeans)*ones(size(trialdata,1),length(epoch));
    %             end;
    %         end;
    %         if ~isempty(butterflyFig)
    %             subplot(10,1,2), plot([1:decimateSamples:displayPeriod],trialdata([badChans chans],1:decimateSamples:displayPeriod));
    %             axis([1 displayPeriod -200 200])
    %             title('with baselines subtracted','Interpreter','none');
    %         end;
    %     else
    %         if ~isempty(butterflyFig)
    %             subplot(10,1,2), plot([1:decimateSamples:displayPeriod],trialdata([badChans chans],1:decimateSamples:displayPeriod));
    %             axis([1 displayPeriod -200 200])
    %             title('no baseline subtraction specified','Interpreter','none');
    %         end;
    %     end;
    
    %     %generate automatic blink and saccade template
    %     if strcmp(template,'autoTemplate') || strcmp(template,'bothTemplate')
    %         saccadeTrials = zeros(numTrials,1);
    %         saccadeDirs = zeros(numTrials,1);
    %
    %         for i = 1:numTrials %check each trial for presence of blink or saccade
    %             if strcmp(dataChunk.dataType,'continuous') && (i == numTrials)
    %                 epoch=goodPoints(find(goodPoints>((i-1)*trialSize))); %excess time points are tacked onto final epoch
    %             else
    %                 epoch=goodPoints(find((goodPoints>((i-1)*trialSize)) & (goodPoints<=(i*trialSize))));
    %             end;
    %             if ~isempty(epoch)
    %                 if ~any((corrcoef(trialdata(HEOG,epoch)').*saccadeSigns) < 0)%if all 6 EOG channels correlate in the right direction for a saccade or its recovery
    %                     trial = diag(saccadeSign)*trialdata(HEOG,epoch);
    %                     [maxamp maxpoint] = max(abs(sum(trial)));
    %                     saccadeDirs(i)=sign(maxamp); %direction of saccade activity
    %                     if sign(maxamp) ~= 0
    %                         saccadeTrials(i)=epoch(maxpoint); %amplitude of saccade activity
    %                     end;
    %                 end;
    %             end;
    %         end;
    %
    %         saccadeTrials=saccadeTrials(find(saccadeTrials)); %compact array to eliminate trials with no saccades
    %         saccadeDirs=saccadeDirs(find(saccadeDirs));
    %         if ~isempty(saccadeTrials)
    %             if length(saccadeTrials) == 1
    %                 msg=['1 saccade ' theSegment ' used for template generation.'];
    %             else
    %                 msg=[num2str(length(saccadeTrials)) ' saccade ' theSegment 's used for template generation.'];
    %             end
    %             disp(msg);
    %             outputLog{end+1}=msg;
    %             if length(saccadeTrials) == 1
    %                 autoSaccades=trialdata(:,saccadeTrials)';
    %             else
    %                 autoSaccades=mean(diag(saccadeDirs)*trialdata(:,saccadeTrials)');
    %             end;
    %             corrs = corrcoef([autoSaccades(chans)' winv]);
    %             saccadefacs = find(abs(corrs(1,2:size(corrs,1))) >= badDataCriteria.blink);
    %         else
    %             msg='No Saccades auto-detected.';
    %             disp(msg);
    %             outputLog{end+1}=msg;
    %             autoSaccades=zeros(1,size(trialdata,1));
    %             saccadefacs=[];
    %         end;
    %     end;
    %
    %     if template == 'fileTemplate'
    %         if saccadeFileCorrect
    %             corrs = corrcoef([saccades(chans)' winv]);
    %             saccadefacs = find(abs(corrs(1,2:size(corrs,1))) >= badDataCriteria.blink);
    %         end;
    %     end;
    %
    %     if template == 'bothTemplate'
    %         if saccadeFileCorrect
    %             corrs = corrcoef([saccades(chans)' winv]);
    %             saccadefacs2 = find(abs(corrs(1,2:size(corrs,1))) >= badDataCriteria.blink);
    %             saccadefacs = unique([saccadefacs, saccadefacs2]);
    %         end;
    %     end;
    
    maxRad=0.5;
    EPtemplate.gridSize=67;
    [y,x] = pol2cart(([EPsaccade.eloc.theta]/360)*2*pi,[EPsaccade.eloc.radius]);  % transform electrode locations from polar to cartesian coordinates
    y=-y; %flip y-coordinate so that nose is upwards.
    plotrad = min(1.0,max([EPsaccade.eloc.radius])*1.02);            % default: just outside the outermost electrode location
    plotrad = max(plotrad,0.5);                 % default: plot out to the 0.5 head boundary
    x = x*(maxRad/plotrad);
    y = y*(maxRad/plotrad);
    
    xmin = min(-maxRad,min(x));
    xmax = max(maxRad,max(x));
    ymin = min(-maxRad,min(y));
    ymax = max(maxRad,max(y));
    
    EPtemplate.x=round(((x/(xmax-xmin))*EPtemplate.gridSize)+ceil(EPtemplate.gridSize/2));
    EPtemplate.y=round(((y/(ymax-ymin))*EPtemplate.gridSize)+ceil(EPtemplate.gridSize/2));
    [Xi,Yi,Zi] = griddata(EPtemplate.x,EPtemplate.y,saccades,[1:EPtemplate.gridSize]',[1:EPtemplate.gridSize],'v4');
    for theChan=1:length(saccades)
        rightSaccade(theChan,1)=Zi(EPtemplate.y(theChan),EPtemplate.gridSize-EPtemplate.x(theChan)+1); %flip the scalp topography
    end;
    
    saccadeTrialNum{chunk}=dataChunk.analysis.saccadeTrial;
    saccadeOnsetNum{chunk}=dataChunk.analysis.saccadeOnset;
    subtractedSaccades=zeros(size(dataChunk.data,1),displayPeriod);
    count=0;
    fprintf('%60s\n',' ' );
    for trial = 1:numTrials %correct trials for saccade if it does not increase overall variance
        fprintf('%s%-60s',repmat(sprintf('\b'),1,60),sprintf('%s%4d of %4d',['Working on ' theSegment '# '], trial, numTrials))
        epoch=[(trial-1)*trialSize+1:trial*trialSize];
        if ~isempty(epoch)
            theData = trialdata(chans,epoch);
            if max(abs(theData(goodHEOG(1),:)-theData(goodHEOG(2),:))) > badDataCriteria.saccademin %don't bother with PCA if no large amplitude voltages
                S = cov(theData'); %covariance matrix for a spatial PCA
                if ~any(~diag(S)) %if any time points have zero variance, may not be factorable and definitely not fixable
                    Sd = diag(sqrt(diag(S)));  %diagonal matrix of standard deviations of variables as used to generate relationship matrix
                    [V,L] = eig(S);
                    
                    V = fliplr(V);
                    numFac=badDataCriteria.movefacs;
                    V = V(:,1:numFac);  %truncated eigenvector matrix
                    FacScr = (theData') * V;    %factor scores, not mean corrected.
                    ScrDiag = diag(std(FacScr));
                    A = inv(Sd) * (V * ScrDiag);  %unrotated factor loading matrix
                    C = sum((A.^2),2)';
                    A = (diag(sqrt(C).^-1)) * A;  %factor loadings Kaiser-normalized by communalities
                    [FacPat]= ep_doVarimax(A);
                    FacPat = diag(sqrt(C)) * FacPat;  %renormalize factor loadings by original communalities
                    [FacPat, FacCor] = ep_doPromax(FacPat, 3); %Only apply loading weighting to the Varimax step
                    %to match SAS output and to avoid rounding errors.
                    FacStr = FacPat * FacCor;	%factor structure matrix (Harman, eq. 12.19, p. 268)
                    invR=pinv(corrcoef(theData'));
                    FacCof=invR*FacStr;
                    FacScr=(theData')*FacCof;
                    FacScr=(FacScr)*inv(diag(std(FacScr)));
                    
                    %HEOGdiff=(Sd(goodHEOG(1),goodHEOG(1))*FacPat(goodHEOG(1),:)-Sd(goodHEOG(2),goodHEOG(2))*FacPat(goodHEOG(2),:)).*max(abs(FacScr));
                    
                    %noiseFacs=find(abs(HEOGdiff) > 10);
                    
                    corrs=corrcoef([saccades(chans)' rightSaccade(chans) Sd*FacPat]);
                    
                    noiseFacs=unique([find(abs(corrs(3:end,1)) > badDataCriteria.saccade) find(abs(corrs(3:end,2)) > badDataCriteria.saccade)]);
                    
                    if ~isempty(noiseFacs) %if there are noise factors, subtract them out
                        %                         FacScrFull = zeros(numChans,numFac);
                        %                         for i = 1:length(chans)
                        %                             FacScrFull(chans(i),:)=FacScr(i,:);
                        %                         end;
                        temp = zeros(numFac,1);
                        temp(noiseFacs) = 1;
                        saccadedata = (Sd*FacPat* diag(temp)) * FacScr';
                        nosaccadedata= theData - saccadedata;
                        
                        if sum(var(theData)) >= sum(var(nosaccadedata))
                            peakSaccade=min(find(abs(saccadedata(goodHEOG(1),:)-saccadedata(goodHEOG(2),:))>badDataCriteria.saccademin)); %find the first peak point (over the criterion)
                            
                            minSamp=find(abs(saccadedata(goodHEOG(1),:)-saccadedata(goodHEOG(2),:))<2); %find the points below 2 microvolt onset threshold
                            if isempty(minSamp)
                                minSamp=1;
                            end;
                            if ~isempty(peakSaccade)
                                
                                temp=corrcoef([saccades(chans)' saccadedata(:,peakSaccade)]);
                                leftSaccCorr(trial)=temp(1,2);
                                
                                temp=corrcoef([rightSaccade(chans) saccadedata(:,peakSaccade)]);
                                rightSaccCorr(trial)=temp(1,2);
                                
                                thePeak(trial)=peakSaccade;
                                
                                theSd{trial}=Sd;
                                theFacPat{trial}=FacPat;
                                theNoiseFacs{trial}=noiseFacs;
                                theFacScr{trial}=FacScr;
                                
                                onsetSamp = minSamp(max(find(minSamp < peakSaccade)));
                                if isempty(onsetSamp)
                                    onsetSamp = peakSaccade;
                                else
                                    onsetSamp=onsetSamp+1;
                                end;
                                saccadeOnsetNum{chunk}(theSubject,trial)=onsetSamp;
                                count=count+1;
                                trialdata(chans,epoch) = nosaccadedata;
                                subtractedSaccades(chans,epoch) = saccadedata;
                                saccadeTrialNum{chunk}(theSubject,trial)=1;
                            end;
                        end;
                    end;
                end;
            end;
        end;
    end;
    fprintf('%60s\n',' ' );
    
    if count == 1
        msg=['1 ' theSegment ' corrected for saccade activity.'];
    else
        msg=[num2str(count) ' ' theSegment 's corrected for saccade activity.'];
    end
    disp(msg);
    outputLog{end+1}=msg;
    
    %    facsdone = [facsdone; winvFull(:,saccadefacs)'];
    if count > 0
        if ~isempty(butterflyFig)
            figure(butterflyFig(chunk));
            subplot(10,1,9), plot([1:decimateSamples:displayPeriod],subtractedSaccades([badChans chans],1:decimateSamples:displayPeriod));
            title('subtracted saccades','Interpreter','none');
            axis([1 displayPeriod -200 200])
            
            subplot(10,1,10), plot([1:decimateSamples:displayPeriod],trialdata([badChans chans],1:decimateSamples:displayPeriod));
            title('with saccades subtracted','Interpreter','none');
            axis([1 displayPeriod -200 200])
        end;
    else
        msg='No components match saccade template so no correction performed.';
        disp(msg);
        outputLog{end+1}=msg;
        
        if ~isempty(butterflyFig)
            figure(butterflyFig(chunk));
            subplot(10,1,9), plot([1:decimateSamples:displayPeriod],ones(1,ceil(displayPeriod/decimateSamples)));
            axis([1 displayPeriod -200 200])
            title('no saccades to subtract that were detected','Interpreter','none');
            subplot(10,1,10), plot([1:decimateSamples:displayPeriod],trialdata([badChans chans],1:decimateSamples:displayPeriod));
            title('no saccades subtracted','Interpreter','none');
            axis([1 displayPeriod -200 200])
        end;
    end;
    
    drawnow
    
    if ~isempty(baseline)			%baseline again in case there were blinks or saccades in the baseline period.
        for i = 1:numTrials
            epoch=[(trial-1)*trialSize+1:trial*trialSize];
            base=(i-1)*trialSize+baseline;
            if ~isempty(epoch) && ~isempty(base)
                baseMeans = mean(trialdata(:,base),2);
                trialdata(:,epoch)=trialdata(:,epoch)-diag(baseMeans)*ones(size(trialdata,1),length(epoch));
            end;
        end;
    end;
    
    if length(refChan) == 2
        trialdata(refChan(2),:)=-trialdata(refChan(1),:);     %put second reference channel back in.
    end;
    
    dataChunk.data(:,:,:,theSubject)=reshape(trialdata,size(dataChunk.data));
    %     dataChunk.analysis.blinkTrial(theSubject,:)=blinkTrialNum{chunk};
    dataChunk.analysis.saccadeTrial(theSubject,:)=saccadeTrialNum{chunk};
    dataChunk.analysis.saccadeOnset(theSubject,:)=saccadeOnsetNum{chunk};
    
    eval (['save ''' inFile '''-' num2str(chunk) '.mat dataChunk;']);
    if nargin < 12
        try
            eval (['print -f' num2str(butterflyFig(chunk)) ' -djpeg ''' inFile '''-' num2str(chunk) 'EOG.jpg']);
        catch
            disp('Couldn''t save a copy of the artifact correction figure.  Perhaps your version of Matlab is not current.');
        end;
    end;
    
    if nargin < 12
        close(butterflyFig);
    end;
    
    % totblinkTrialNum=[];
    totsaccadeTrialNum=[];
    for chunk = startChunk:endChunk
        %     totblinkTrialNum=[totblinkTrialNum blinkTrialNum{chunk}];
        totsaccadeTrialNum=[totsaccadeTrialNum saccadeTrialNum{chunk}];
    end;
end;
