function EPdata=ep_averageData(inputFiles,inputFormat,fileType,averagingMethod,trimLevel);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% EPdata=ep_averageData(inputFiles,inputFormat,fileType,averagingMethod,trimLevel);
% Averages single trial data into subject averages or subjects averages into grand averages.
% Will refuse to operate on a factor file as it would be meaningless.  The PCA procedure already generates
% a grand average and it can also be generated via the Edit Data function.
%
%	Reads in specified data files, averages them together using the specified method, dropping bad marked bad trials.
%   Returns an EPdata format structured variable.
%
%Inputs
%   inputFiles: cell array of file names to correct.  Assumes active directory if path not specified.  (default: will ask for files)
%   inputFormat: input file format.
%   fileType: input file type.
%   averagingMethod: output file format.
%   trimLevel: proportion to drop from each tail of the distribution if using trimmed means averaging.
%
%Outputs
%	Returns EPdata format structured variable.
%   EPdata         : Structured array with the data and accompanying information.  See readData.
%

%  Leonowicz, Z., Karvanen, J., & Shishkin, S. L. (2005). Trimmed estimators for robust averaging of event-related
%  potentials. J Neurosci Methods, 142(1), 17-26.
%
% History:
%
% by Joseph Dien (7/20/09)
% jdien07@mac.com
%
% bugfix 9/10/09 JD
% Crash when generating grand average from file with analysis fields.
%
% modified & bugfix 2/14/10 JD
% Changed bad channel field to negative numbers for still bad channels.  Modified averaging to deal with the possibility of
% these uncorrected bad channels.  Counts of total blink trials and movement trials for averaged waveforms include only
% good trials.  Averaging function handles possibility of no good trials for an average.
% Moved check for flat epochs to the readData function.  AvgNum and SubNum fields now calculated for grand averages.
% analysis fields no longer optional.
% Make the assumption that all the files averaged together will use the same ced file.
%
% bugfix 5/3/10 JD
% Fixed crash when generating grand average from files with more than one cell by using either mean or median methods.
%
% bugfix 5/25/10 JD
% Fixed cell names being associated with wrong waveforms when averaging data.
%
% bugfix 6/12/10 JD
% Fixed average files not being generated with .dataName field.
%
% modified 10/4/10 JD
% Now assumes that all the session files going into an average is using the same montage so doesn't keep asking for it.
% 
%  modified 10/17/10 JD
%  Added support for saccadeTrials and saccadeOnset fields.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%     Copyright (C) 1999-2011  Joseph Dien
%
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
%
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
%
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

EPdata=[];

if nargin < 3
    averagingMethod='Trimmed_Mean';
end;

if nargin < 2
    inputFormat='egi_egis';
end;

if nargin < 1
    [inputFiles, pathname] = uigetfile('*.*','Open:','MultiSelect','on');
    activeDirectory=pathname;
    if ~iscell(inputFiles)
        temp=inputFiles;
        inputFiles=[];
        inputFiles{1}=temp;
    end;
    if inputFiles{1}==0
        msg{1}='No filenames selected. You have to click on a name';
        [msg]=ep_errorMsg(msg);
        return
    end
    if ~iscell(inputFiles)
        temp=inputFiles;
        inputFiles=[];
        inputFiles{1}=temp;
    end;
    for theFile=1:size(inputFiles,2)
        inputFiles{theFile}=[activeDirectory inputFiles{theFile}];
    end;
else
    files=dir;
    for theFile=1:size(inputFiles,2)
        [activeDirectory, name, ext] = fileparts(inputFiles{theFile});
        if isempty(activeDirectory)
            activeDirectory=pwd;
            inputFiles{theFile}=[activeDirectory filesep inputFiles{theFile}];
        end;
        if ~exist(inputFiles{theFile},'file')
            msg{1}=['Error: The file ' inputFiles{theFile} ' is not in the directory.'];
            [msg]=ep_errorMsg(msg);
            return
        end;
    end;
end;

inputFiles=inputFiles';
sessionNum = size(inputFiles,1);

readArg{1}='format';
readArg{2}=inputFormat;
readArg{3}='type';
readArg{4}=fileType;

%assume the first session file is representative of the rest.
Name=deblank(inputFiles{1});
thisReadArg=readArg;
thisReadArg{end+1}='file';
[pathstr, fileName, ext]=fileparts(Name);
thisReadArg{end+1}=Name;
inputData=ep_readData(thisReadArg);
if isempty(inputData)
    return;
end;

[inputData]=ep_stripAdds(inputData);

if isempty(inputData.data)
    msg{1}=['Error: The file ' fileName ' had no data left to correct after additions were removed.'];
    [msg]=ep_errorMsg(msg);
    return
end;

if length(inputData.facNames) > 1
    msg{1}=['Error: The file ' fileName ' is a factor file.'];
    [msg]=ep_errorMsg(msg);
    return
end;

numChans=length(inputData.chanNames);
numPoints=length(inputData.timeNames);
numCells=length(unique(inputData.cellNames));
numWaves=length(inputData.cellNames);
numFacs=size(inputData.data,5); %dimensionality of the fifth dimension.  One even if no factors.
numCMBfacs=size(inputData.facData,5);
dataType=inputData.dataType;

EPdata.fileFormat = inputData.fileFormat;
EPdata.montage = inputData.montage;
EPdata.chanNames = inputData.chanNames;
EPdata.timeNames = inputData.timeNames;
EPdata.chanTypes= inputData.chanTypes;
EPdata.facNames= inputData.facNames;
EPdata.facTypes = inputData.facTypes;
EPdata.facVecT = inputData.facVecT;
EPdata.facVecS = inputData.facVecS;
EPdata.facData = inputData.facData;
try
    EPver=ver('EP_Toolkit');
catch
    EPver='unavailable'; %workaround for bug in earlier version of Matlab
end;
EPdata.EPver = EPver;
EPdata.ver = ver;
EPdata.date = date;
EPdata.Fs = inputData.Fs;
EPdata.baseline = inputData.baseline;
EPdata.ename = inputData.ename;
EPdata.dataName = inputData.dataName;
EPdata.trialSpecNames = inputData.trialSpecNames;
EPdata.trialNames = [];
EPdata.trialSpecs = [];
EPdata.ced = inputData.ced;
EPdata.eloc = inputData.eloc;
EPdata.implicit = inputData.implicit;
EPdata.history = inputData.history;
EPdata.history{end+1} = {'ep_averageData',inputFiles,inputFormat,fileType,averagingMethod,trimLevel};
if length(inputFiles) == 1
    EPdata.fileName = inputData.fileName;
else
    EPdata.fileName = 'average';
end;
EPdata.dataType='average';

for theFile=2:length(inputFiles)
    Name2=deblank(inputFiles{theFile});
    thisReadArg2=readArg;
    thisReadArg2{end+1}='file';
    %[pathstr, fileName, ext]=fileparts(Name2);
    thisReadArg2{end+1}=Name2;
    thisReadArg2{end+1}='silent';
    thisReadArg2{end+1}='on';
    thisReadArg2{end+1}='ced';
    thisReadArg2{end+1}=inputData.ced; %assume all the files averaged together will be using the same ced file.
    thisReadArg2{end+1}='montage';
    thisReadArg2{end+1}=inputData.montage; %assume all the files averaged together will be using the same montage.
    data2=ep_readData(thisReadArg2);
    [data2]=ep_stripAdds(data2);
    if (numChans ~= length(data2.chanNames))
        msg{1}=['Error: The file ' inputFiles{theFile} ' has a different set of electrodes.'];
        [msg]=ep_errorMsg(msg);
        return
    end;
    if (numPoints ~= length(data2.timeNames))
        msg{1}=['Error: The file ' inputFiles{theFile} ' has a different length epoch.'];
        [msg]=ep_errorMsg(msg);
        return
    end;
    if (numCells ~= length(unique(data2.cellNames)))
        msg{1}=['Error: The file ' inputFiles{theFile} ' has a different set of cells.'];
        [msg]=ep_errorMsg(msg);
        return
    end;
    if length(EPdata.facNames) > 1
        msg{1}=['Error: The file ' fileName ' is a factor file.'];
        [msg]=ep_errorMsg(msg);
        return
    end;
    if (length(EPdata.facNames) ~= length(data2.facNames))
        msg{1}=['Error: The file ' inputFiles{theFile} ' has a different set of factors.'];
        [msg]=ep_errorMsg(msg);
        return
    end;
    if isempty(data2.data)
        msg{1}=['Error: The file ' inputFiles{theFile} ' had no data left to correct after additions were removed.'];
        [msg]=ep_errorMsg(msg);
        return
    end;
end;

if any(strcmp(dataType,{'continuous','single_trial'})) %form subject averages
    EPdata.events = cell(sessionNum,numCells);
    EPdata.subjectSpecNames = inputData.subjectSpecNames;
    EPdata.subjectSpecNames{end+1}='fileName';
    
    for theSession = 1:sessionNum
        Name=deblank(inputFiles{theSession});
        [pathstr, fileName, ext]=fileparts(Name);
        
        disp(['Working on #' num2str(theSession) ': ' fileName '.']);
        
        if theSession > 1
            thisReadArg=readArg;
            thisReadArg{end+1}='file';
            thisReadArg{end+1}=Name;
            thisReadArg{end+1}='silent';
            thisReadArg{end+1}='on';
            thisReadArg{end+1}='ced';
            thisReadArg{end+1}=inputData.ced; %assume all the files averaged together will be using the same ced file.
            thisReadArg{end+1}='montage';
            thisReadArg{end+1}=inputData.montage; %assume all the files averaged together will be using the same montage.
            inputData=ep_readData(thisReadArg);
            [inputData]=ep_stripAdds(inputData);
        end;
        
        EPdata.subNames(theSession)=inputData.subNames;
        EPdata.subTypes{theSession}='AVG';
        [pathstr, name, ext] = fileparts(inputData.fileName);
        inputData.subjectSpecs{end+1}=[name ext];
        EPdata.subjectSpecs(theSession,:)=inputData.subjectSpecs;
        
        [uniqueCells waveOrder m]=unique(inputData.cellNames,'first'); %waveOrder is the true order of the unique cells (wave numbers of the first occurences)
        [a b cellOrder]=unique(waveOrder,'first'); %cellOrder is the true order of the unique cells (numbered by cells)
        
        for theCell=1:numCells
            theTrials=find(strcmp(uniqueCells(cellOrder==theCell),inputData.cellNames));
            badTrials=zeros(1,length(theTrials));
            theGoodTrials=theTrials;
            
            badTrials=inputData.analysis.badTrials(1,theTrials);
            EPdata.analysis.badTrials(theSession,theCell)= sum(badTrials);
            theGoodTrials=theGoodTrials(logical(~badTrials));
            EPdata.analysis.blinkTrial(theSession,theCell)= sum(inputData.analysis.blinkTrial(1,theGoodTrials));
            EPdata.analysis.saccadeTrial(theSession,theCell)= sum(inputData.analysis.saccadeTrial(1,theGoodTrials));
            EPdata.analysis.saccadeOnset(theSession,theCell)= sum(inputData.analysis.saccadeOnset(1,theGoodTrials));
            EPdata.analysis.moveTrial(theSession,theCell)= sum(inputData.analysis.moveTrial(1,theGoodTrials));
            for theChan=1:numChans
                if all(inputData.analysis.badChans(1,theGoodTrials,theChan) < 0) %if the channel is bad for all the available good trials
                    EPdata.analysis.badChans(theSession,theCell,theChan) = -1;
                else
                    EPdata.analysis.badChans(theSession,theCell,theChan)= sum(inputData.analysis.badChans(1,theGoodTrials,theChan).*(inputData.analysis.badChans(1,theGoodTrials,theChan)>0),2); %only good channels
                end;
            end;
            
            if isempty(theGoodTrials) %if no good trials for this cell, fill out values and move on
                EPdata.analysis.blinkTrial(theSession,theCell)=0;
                EPdata.analysis.saccadeTrial(theSession,theCell)=0;
                EPdata.analysis.saccadeOnset(theSession,theCell)=0;
                EPdata.analysis.moveTrial(theSession,theCell)=0;
                EPdata.analysis.badTrials(theSession,theCell)=length(theTrials);
                EPdata.analysis.badChans(theSession,theCell,1:numChans)=0;
                EPdata.avgNum(theSession,theCell)=-1;
                EPdata.subNum(theSession,theCell)=-1;
                EPdata.cellNames(theCell)=uniqueCells(cellOrder==theCell);
                EPdata.cellTypes(theCell)=inputData.cellTypes(theTrials(1));
                EPdata.data(:,:,theCell,theSession,:)=zeros(numChans,numPoints,1,1,numFacs);
                EPdata.noise(:,:,theCell,theSession,:)=zeros(numChans,numPoints,1,1,numFacs);
                EPdata.std(:,:,theCell,theSession,:)=zeros(numChans,numPoints,1,1,numFacs);
                continue
            end;
            
            numTrials=length(theGoodTrials);
            EPdata.avgNum(theSession,theCell)=numTrials;
            EPdata.subNum(theSession,theCell)=1;
            EPdata.cellNames(theCell)=uniqueCells(cellOrder==theCell);
            EPdata.cellTypes(theCell)=inputData.cellTypes(theGoodTrials(1));
            
            switch averagingMethod
                case 'Mean'
                    for theChan = 1:numChans
                        chanGoodTrials=theGoodTrials;
                        chanGoodTrials=theGoodTrials(find(inputData.analysis.badChans(1,theGoodTrials,theChan) > -1));
                        numChanGoodTrials=length(chanGoodTrials);
                        if numChanGoodTrials==0 %if the channel is bad for all the available good trials
                            EPdata.data(theChan,:,theCell,theSession,:)=zeros(1,numPoints,1,1,numFacs);
                            EPdata.std(theChan,:,theCell,theSession,:)=zeros(1,numPoints,1,1,numFacs);
                            EPdata.noise(theChan,:,theCell,theSession,:)=zeros(1,numPoints,1,1,numFacs);
                        else
                            for theFactor = 1:numFacs
                                EPdata.data(theChan,:,theCell,theSession,theFactor)=mean(inputData.data(theChan,:,chanGoodTrials,:,theFactor),3);
                                for theSample = 1:numPoints
                                    EPdata.std(theChan,theSample,theCell,theSession,theFactor)=std(squeeze(inputData.data(theChan,theSample,chanGoodTrials,:,theFactor)));
                                end
                                noiseData=zeros(1,numPoints,1,1,1);
                                theSigns=ones(numChanGoodTrials,1);
                                if numChanGoodTrials > 1
                                    theSigns(2:2:end)=-1;
                                end;
                                for theTrial=1:numChanGoodTrials
                                    noiseData=noiseData+mean(theSigns(theTrial)*inputData.data(theChan,:,chanGoodTrials(theTrial),:,theFactor),3);
                                end;
                                EPdata.noise(theChan,:,theCell,theSession,theFactor)=noiseData/numChanGoodTrials;
                            end;
                        end;
                    end;
                case 'Median'
                    for theChan = 1:numChans
                        chanGoodTrials=theGoodTrials;
                        chanGoodTrials=theGoodTrials(find(inputData.analysis.badChans(1,theGoodTrials,theChan) > -1));
                        numChanGoodTrials=length(chanGoodTrials);
                        if numChanGoodTrials==0 %if the channel is bad for all the available good trials
                            EPdata.data(theChan,:,theCell,theSession,:)=zeros(1,numPoints,1,1,numFacs);
                            EPdata.std(theChan,:,theCell,theSession,:)=zeros(1,numPoints,1,1,numFacs);
                            EPdata.noise(theChan,:,theCell,theSession,:)=zeros(1,numPoints,1,1,numFacs);
                        else
                            for theFactor = 1:numFacs
                                EPdata.data(theChan,:,theCell,theSession,theFactor)=median(inputData.data(theChan,:,chanGoodTrials,:,theFactor),3);
                                for theSample = 1:numPoints
                                    EPdata.std(theChan,theSample,theCell,theSession,theFactor)=std(squeeze(inputData.data(theChan,theSample,chanGoodTrials,:,theFactor)));
                                end
                                noiseData=zeros(1,numPoints,1,1,1);
                                theSigns=ones(numChanGoodTrials,1);
                                if numChanGoodTrials > 1
                                    theSigns(2:2:end)=-1;
                                end;
                                for theTrial=1:numChanGoodTrials
                                    noiseData=noiseData+mean(theSigns(theTrial)*inputData.data(theChan,:,chanGoodTrials(theTrial),:,theFactor),3);
                                end;
                                EPdata.noise(theChan,:,theCell,theSession,theFactor)=noiseData/numChanGoodTrials;
                            end;
                        end;
                    end;
                case 'Trimmed_Mean'
                    for theChan = 1:numChans
                        chanGoodTrials=theGoodTrials;
                        chanGoodTrials=theGoodTrials(find(inputData.analysis.badChans(1,theGoodTrials,theChan) > -1));
                        numChanGoodTrials=length(chanGoodTrials);
                        if numChanGoodTrials==0 %if the channel is bad for all the available good trials
                            EPdata.data(theChan,:,theCell,theSession,:)=zeros(1,numPoints,1,1,numFacs);
                            EPdata.std(theChan,:,theCell,theSession,:)=zeros(1,numPoints,1,1,numFacs);
                            EPdata.noise(theChan,:,theCell,theSession,:)=zeros(1,numPoints,1,1,numFacs);
                        else
                            trimTrials=floor(numChanGoodTrials*trimLevel);
                            numTrimmedTrials=length(trimTrials+1:numChanGoodTrials-trimTrials);
                            theSigns=ones(numTrimmedTrials,1);
                            
                            if numTrimmedTrials > 1
                                if rand() > .5
                                    theSigns(2:2:end)=-1;
                                else
                                    theSigns(1:2:end)=-1;
                                end;
                            end;
                            noiseData=zeros(1,numPoints,1,1,numFacs);
                            for theSample = 1:numPoints
                                for theFactor = 1:numFacs
                                    sortedValues=sort(squeeze(inputData.data(theChan,theSample,chanGoodTrials,:,theFactor)));
                                    EPdata.data(theChan,theSample,theCell,theSession,theFactor)=mean(sortedValues(trimTrials+1:numChanGoodTrials-trimTrials));
                                    EPdata.noise(theChan,theSample,theCell,theSession,theFactor)=mean(theSigns.*sortedValues(trimTrials+1:numChanGoodTrials-trimTrials));
                                    EPdata.std(theChan,theSample,theCell,theSession,theFactor)=std(squeeze(inputData.data(theChan,theSample,chanGoodTrials,:,theFactor)));
                                end;
                            end
                        end;
                    end;
                otherwise
                    msg{1}=['Error: The averaging method ' averagingMethod ' is not recognized.'];
                    [msg]=ep_errorMsg(msg);
                    return
            end;
        end;
    end;
elseif strcmp(dataType,'average') %form grand average
    
    EPdata.subNames{1}='grand average';
    EPdata.subTypes{1}='GAV';
    EPdata.subjectSpecs={};
    EPdata.subjectSpecNames={};
    EPdata.cellNames=inputData.cellNames;
    EPdata.cellTypes=inputData.cellTypes;
    EPdata.events = cell(1,numCells);
    avgNum=[];
    subNum=[];
    
    %glue together the various subject average files
    totNumSubs=0;
    for theSession = 1:sessionNum
        Name=deblank(inputFiles{theSession});
        [pathstr, fileName, ext]=fileparts(Name);
        
        disp(['Working on: ' Name '.']);
        
        if theSession > 1
            thisReadArg=readArg;
            thisReadArg{end+1}='file';
            thisReadArg{end+1}=Name;
            thisReadArg{end+1}='silent';
            thisReadArg{end+1}='on';
            thisReadArg{end+1}='ced';
            thisReadArg{end+1}=inputData.ced; %assume all the files averaged together will be using the same ced file.
            inputData=ep_readData(thisReadArg);
            [inputData]=ep_stripAdds(inputData);
        end;
        
        numSubs=length(inputData.subNames);
        totNumSubs=totNumSubs+numSubs;
        
        EPdata.analysis.blinkTrial(totNumSubs-numSubs+1:totNumSubs,:)= inputData.analysis.blinkTrial;
        EPdata.analysis.saccadeTrial(totNumSubs-numSubs+1:totNumSubs,:)= inputData.analysis.saccadeTrial;
        EPdata.analysis.saccadeOnset(totNumSubs-numSubs+1:totNumSubs,:)= inputData.analysis.saccadeOnset;
        EPdata.analysis.moveTrial(totNumSubs-numSubs+1:totNumSubs,:)= inputData.analysis.moveTrial;
        EPdata.analysis.badTrials(totNumSubs-numSubs+1:totNumSubs,:)= inputData.analysis.badTrials;
        for theCell=1:length(inputData.cellNames)
            EPdata.analysis.badChans(totNumSubs-numSubs+1:totNumSubs,theCell,:)= inputData.analysis.badChans(1:numSubs,theCell,:);
        end;
        
        avgNum(end+1:end+numSubs,:)=inputData.avgNum;
        subNum(end+1:end+numSubs,:)=inputData.subNum;
        if theSession ==1
            gavData=inputData.data;
        else
            gavData(:,:,:,end+1:end+numSubs,:)=inputData.data;
        end;
    end;
    numSubs=totNumSubs;
    
    %compute the grand average
    
    for theCell = 1:numCells
        theSubs=[1:numSubs];
        badSubs=(avgNum(:,theCell)<0);
        theGoodSubs=theSubs(~badSubs);
        if isempty(theGoodSubs) %if no good subjects for this cell, fill out values and move on
            EPdata.avgNum(1,theCell)=-1;
            EPdata.subNum(1,theCell)=-1;
            EPdata.data(:,:,theCell,1,:)=zeros(numChans,numPoints,1,1,numFacs);
            EPdata.noise(:,:,theCell,1,:)=zeros(numChans,numPoints,1,1,numFacs);
            EPdata.std(:,:,theCell,1,:)=zeros(numChans,numPoints,1,1,numFacs);
            continue
        end;
        EPdata.avgNum(1,theCell)=sum(avgNum(theGoodSubs,theCell));
        EPdata.subNum(1,theCell)=sum(subNum(theGoodSubs,theCell));
        switch averagingMethod
            case 'Mean'
                for theChan = 1:numChans
                    chanGoodSubs=[1:length(theGoodSubs)];
                    chanGoodSubs=theGoodSubs(find(EPdata.analysis.badChans(theGoodSubs,theCell,theChan) > -1));
                    numChanGoodSubs=length(chanGoodSubs);
                    if numChanGoodSubs==0 %if the channel is bad for all the available good subjects
                        EPdata.data(theChan,:,theCell,1,:)=zeros(1,numPoints,1,1,numFacs);
                        EPdata.std(theChan,:,theCell,1,:)=zeros(1,numPoints,1,1,numFacs);
                        EPdata.noise(theChan,:,theCell,1,:)=zeros(1,numPoints,1,1,numFacs);
                    else
                        for theFactor = 1:numFacs
                            EPdata.data(theChan,:,theCell,1,theFactor)=mean(gavData(theChan,:,theCell,chanGoodSubs,theFactor),4);
                            for theSample = 1:numPoints
                                EPdata.std(theChan,theSample,theCell,1,theFactor)=std(squeeze(gavData(theChan,theSample,theCell,chanGoodSubs,theFactor)));
                            end
                        end;
                        noiseData=zeros(1,numPoints,1,1,1);
                        theSigns=ones(numChanGoodSubs,1);
                        if numChanGoodSubs > 1
                            theSigns(2:2:end)=-1;
                        end;
                        for theSub=1:numChanGoodSubs
                            noiseData=noiseData+mean(theSigns(theSub)*gavData(theChan,:,theCell,chanGoodSubs(theSub),theFactor),4);
                        end;
                        EPdata.noise(theChan,:,theCell,1,theFactor)=noiseData/numChanGoodSubs;
                    end;
                end;
            case 'Median'
                for theChan = 1:numChans
                    chanGoodSubs=[1:length(theGoodSubs)];
                    chanGoodSubs=theGoodSubs(find(inputData.analysis.badChans(theGoodSubs,theCell,theChan) > -1));
                    numChanGoodSubs=length(chanGoodSubs);
                    if numChanGoodSubs==0 %if the channel is bad for all the available good subjects
                        EPdata.data(theChan,:,theCell,1,:)=zeros(1,numPoints,1,1,numFacs);
                        EPdata.std(theChan,:,theCell,1,:)=zeros(1,numPoints,1,1,numFacs);
                        EPdata.noise(theChan,:,theCell,1,:)=zeros(1,numPoints,1,1,numFacs);
                    else
                        for theFactor = 1:numFacs
                            EPdata.data(theChan,:,theCell,1,theFactor)=median(gavData(theChan,:,theCell,chanGoodSubs,theFactor),4);
                            for theSample = 1:numPoints
                                EPdata.std(theChan,theSample,theCell,1,theFactor)=std(squeeze(gavData(theChan,theSample,theCell,chanGoodSubs,theFactor)));
                            end
                        end;
                        noiseData=zeros(1,numPoints,1,1,1);
                        theSigns=ones(numChanGoodSubs,1);
                        if numChanGoodSubs > 1
                            theSigns(2:2:end)=-1;
                        end;
                        for theSub=1:numChanGoodSubs
                            noiseData=noiseData+mean(theSigns(theSub)*gavData(theChan,:,theCell,chanGoodSubs(theSub),theFactor),4);
                        end;
                        EPdata.noise(theChan,:,theCell,1,theFactor)=noiseData/numChanGoodSubs;
                    end;
                end;
            case 'Trimmed_Mean'
                for theChan = 1:numChans
                    chanGoodSubs=[1:length(theGoodSubs)];
                    chanGoodSubs=theGoodSubs(find(inputData.analysis.badChans(theGoodSubs,theCell,theChan) > -1));
                    numChanGoodSubs=length(chanGoodSubs);
                    if numChanGoodSubs==0 %if the channel is bad for all the available good subjects
                        EPdata.data(theChan,:,theCell,1,:)=zeros(1,numPoints,1,1,numFacs);
                        EPdata.std(theChan,:,theCell,1,:)=zeros(1,numPoints,1,1,numFacs);
                        EPdata.noise(theChan,:,theCell,1,:)=zeros(1,numPoints,1,1,numFacs);
                    else
                        trimSubs=floor(numChanGoodSubs*trimLevel);
                        numTrimmedSubs=length(trimSubs+1:numChanGoodSubs-trimSubs);
                        theSigns=ones(numTrimmedSubs,1);
                        if numTrimmedSubs > 1
                            if rand() > .5
                                theSigns(2:2:end)=-1;
                            else
                                theSigns(1:2:end)=-1;
                            end;
                        end;
                        noiseData=zeros(1,numPoints,1,1,numFacs);
                        for theSample = 1:numPoints
                            for theCell = 1:numCells
                                for theFactor = 1:numFacs
                                    sortedValues=sort(squeeze(gavData(theChan,theSample,theCell,chanGoodSubs,theFactor)));
                                    EPdata.data(theChan,theSample,theCell,1,theFactor)=mean(sortedValues(trimTrials+1:numTrials-trimTrials));
                                    EPdata.noise=mean(theSigns.*sortedValues(trimTrials+1:numTrials-trimTrials));
                                    EPdata.std=std(gavData(theChan,theSample,theCell,1,theFactor));
                                end;
                            end;
                        end
                    end;
                end;
            otherwise
                msg{1}=['Error: The averaging method ' averagingMethod ' is not recognized.'];
                [msg]=ep_errorMsg(msg);
                return
        end;
    end;
    
    EPdata.analysis.blinkTrial=sum(EPdata.analysis.blinkTrial,1);
    EPdata.analysis.saccadeTrial=sum(EPdata.analysis.saccadeTrial,1);
    EPdata.analysis.saccadeOnset=sum(EPdata.analysis.saccadeOnset,1)/EPdata.analysis.saccadeTrial; %compute average saccade onset
    EPdata.analysis.moveTrial=sum(EPdata.analysis.moveTrial,1);
    EPdata.analysis.badTrials=sum(EPdata.analysis.badTrials,1);
    EPdata.analysis.badChans=sum(EPdata.analysis.badChans,1);
    
else
    msg{1}=['Error: The data type ' dataType ' is not recognized.'];
    [msg]=ep_errorMsg(msg);
    return
end;

[err]=ep_checkEPfile(EPdata);

disp('Finished averaging.');

