function ep_editData(varargin)
% ep_editData - ep_editData(varargin) -
% Allows one to examine and edit the structure of the data.
%
%Input:
%  EPdata         : Structured array with the data and accompanying information.  See readData.
%
%   EPoverview     : Structured array with the information for the current overview.
%     .lastData: data prior to last change.  To permit undo.  Structure same as for data.
%     .workData: current working copy of data.  Only made permanent if "save" is chosen.
%     .handles: handles for GUI objects
%        cells.hCellName    :Names of cells (cells)
%        cells.hDeleteCell  :delete button for each cell (cells)
%        cells.hPlusCell    :plus button for each cell (cells)
%        cells.hSubtractCell  :subtract button for each cell (cells)
%        cells.hCombinedCells :The text window for listing the cells to be combined.
%     .combineWeight   :The +/- for each unique cell.
%     .newCellName     :The name for the new cell.
%     .subjects.select  :The subjects that are currently selected.
%
%Output:
%  EPdataset      : Structured array with the data and accompanying information.
%                   See Input.
%  EPoverview     : Structured array with the information for the current analysis.
%                   See Input.

%History
%  by Joseph Dien (3/23/09)
%  jdien07@mac.com
%
%  modified 7/20/09 JD
%  Added ability to reorder cells and to rename cells.
%  Added pages for file overview, subjects, trials, channels, samples, event, and factors.  Most of the data can be edited.
%  Changed global data name from data to EPdataset
%
%  bugfix 9/5/09 JD
%  Location of windows appearing partly off screen on some systems due to offset from having Dock (on Mac) or Task Bar
%  (on PC) at the bottom of the screen.
%
%  bugfix 10/29/09 JD
%  Crash when editing single_trial data with the cells subpane.
%
%  modified 11/6/09 JD
%  Shortened the edit window so that it'll it into smaller monitors.
%
%  bugfix 11/9/09 JD
%  Crash when exporting data from QC and PCA subpanes when using a Matlab version predating 2008.
%
%  bugfix 1/15/10 JD
%  Error when changing ced field in overview subpane of edit pane.
%
%  bugfix 1/29/10 JD
%  Fixed crash when examining QC data on bad channels and trials.
%  Refreshes Edit pane when Done button is pressed so when Data Name is changed, the name in the list of datasets is updated.
%  Fixed crash when adding a cell in the edit pane whose type is 'SGL'.
%
% modified & bugfix 2/27/10 JD
% Changed bad channel field to negative numbers for still bad channels.  Divided QC subpane badChan into badChan and repChan
% subpanes.  Bad channel and rep channel QC figures no longer include bad trials.
% BadChans numbers of QC subpane will now use number of subjects to calculate proportion for grand averages.
% analysis fields no longer optional.
% Fixed crash when adding subject spec.
% Eliminated chantype field for implicit channels.
% Export file dialogs now correctly indicate that the file with be of type .txt.
% Fixed crash when examining factors that include a combined factor add.
%
% modified 5/16/10 JD
% Added summary page to the PCA subpane of the Edit function, including the total variance accounted for information.
%
% bugfix 5/17/10 JD
% Fixed location of the variance accounted for table for old versions of Matlab for two-step PCAs in the Summary
% subpane of the PCA pane.
% Fixed crash when reordering cells.
%
% bugfix 5/23/10 JD
% Fixed assumption that appended files will be in EGIS format.
%
% bugfix 5/27/10 JD
% Fixed baseline control on Samples subpane failing to change baseline value.
%
% modified 6/15/10 JD
% Marks when a file isn't saved yet.
%
% bugfix 7/2/10 JD
% In order fix crash due to Matlab bug on Windows, put in brief pause after spec name requestor diaglog box.
%
% bugfix 8/27/10 JD
% Fixed append cells and append subjects functions not working due to fallacious error messages.
% Fixed append cells and append subjects and append chans crashing when averaged data (typically ept files) contain noise or std information from having run
% the averaging with the Toolkit and the new file is not a factor file.
% Fixed append cells and append subjects and append chans not adding std and noise information from the new file.
% Fixed append cells failing if the files contained trial specs, as in EGIS files.
% Fixed not allowing subject rows to be changeable even when exceeding limits of subject subpane when using a Matlab
% version predating 2008.
%
% bugfix 10/5/10 JD
% Fixed not able to append subjects or cells to data with same names by generating new name.
% Fixed crash when appending cells to non-single trial data.
%
%  modified 10/12/10 JD
%  For continuous files, data now divided into one second epochs and can be artifact rejected in an epochwise fashion
%  in same fashion as segmented data.  Analysis fields reset to zero if sampling rate is changed.
% 
%  modified 10/17/10 JD
%  Added support for saccadeTrials and saccadeOnset fields.
% 
%  modified 1/17/11 JD
%  When adding combinations of cells or subjects, the name of the new addition describes what went into it.

%     Copyright (C) 1999-2011  Joseph Dien
%
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
%
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
%
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

global EPdataset EPoverview EPmain EPtable

if isempty(EPoverview.mode)
    EPoverview.lastData=[];
    EPoverview.workData=ep_loadEPdataset(EPdataset,EPoverview.dataset);
    EPoverview.handles = [];
    EPoverview.handles.hEditWindow =[];
    EPoverview.mode='overview';
    EPoverview.QC.mode='avgNum';
    EPoverview.PCA.mode='Summary';
    EPtable.edit=[];
end;

numPoints=length(EPoverview.workData.timeNames);
numChans=length(EPoverview.workData.chanNames);
numCells=length(unique(EPoverview.workData.cellNames));
numWaves=length(EPoverview.workData.cellNames);
numSubs=length(EPoverview.workData.subNames);
numSpecs=length(EPoverview.workData.subjectSpecNames);
numImplicit=length(EPoverview.workData.implicit);
numFacs=length(EPoverview.workData.facNames);
if ~isempty(EPoverview.workData.facData)
    numCMBfacs=size(EPoverview.workData.facData,5);
else
    numCMBfacs=0;
end;
numSGLfacs=numFacs-numCMBfacs;

numEvents=0;
for subject=1:numSubs
    for wave=1:numWaves
        numEvents=numEvents+length(EPoverview.workData.events{subject,wave});
    end;
end;

if nargin < 1
    EPoverview.subjects.select=repmat(false,numSubs,1);
    EPoverview.subjects.weights=repmat(0,numSubs,1);
    EPoverview.channels.select=repmat(false,numChans+numImplicit,1);
    EPoverview.channels.weights=repmat(0,numChans+numImplicit,1);
    EPoverview.cells.select=repmat(false,numCells,1);
    EPoverview.cells.weights=repmat(0,numCells,1);
    EPoverview.events.select=repmat(false,numEvents,1);
    EPoverview.events.weights=repmat(0,numEvents,1);
    EPoverview.factors.select=repmat(false,numFacs,1);
    EPoverview.factors.weights=repmat(0,numFacs,1);
end;

if isempty(varargin)
    varargin{1}='start';
end;

scrsz = get(0,'ScreenSize');
windowHeight=700;


switch varargin{1}
    case 'start'
        if ~isempty(EPoverview.handles.hEditWindow)
            clf(EPoverview.handles.hEditWindow)
        else
            [err]=ep_checkEPfile(EPoverview.workData);
            if any(findobj('name','editData'))
                h=findobj('name','EditData');
                for i=1:length(h)
                    close(h(i))
                end;
            end;
            EPoverview.handles.hEditWindow = figure('Name', 'EditData', 'NumberTitle', 'off', 'Position',[201 scrsz(4)-windowHeight 700 windowHeight], 'MenuBar', 'none');
        end;
        
        
        EPoverview.handles.hUndo = uicontrol('Style', 'pushbutton', 'String', 'Undo',...
            'Position', [20 windowHeight-100 60 30], 'Callback', 'ep_editData(''undo'');');
        
        EPoverview.handles.hDone = uicontrol('Style', 'pushbutton', 'String', 'Done',...
            'Position', [20 windowHeight-130 60 30], 'Callback', 'ep_editData(''done'');');
        
        EPoverview.handles.hCancel = uicontrol('Style', 'pushbutton', 'String', 'Cancel',...
            'Position', [20 windowHeight-160 60 30], 'Callback', 'ep_editData(''cancel'');');
        
        
        
        EPoverview.handles.hOverview = uicontrol('Style', 'pushbutton', 'String', 'Overview',...
            'Position', [100 windowHeight-80 60 30], 'Callback', ['global EPoverview;','EPoverview.mode=''overview'';','ep_editData(''start'');']);
        
        EPoverview.handles.hSubjects = uicontrol('Style', 'pushbutton', 'String', 'Subjects',...
            'Position', [160 windowHeight-80 60 30], 'Callback', ['global EPoverview;','EPoverview.mode=''subjects'';','ep_editData(''start'');']);
        
        EPoverview.handles.hCells = uicontrol('Style', 'pushbutton', 'String', 'Cells',...
            'Position', [220 windowHeight-80 60 30], 'Callback', ['global EPoverview;','EPoverview.mode=''cells'';','ep_editData(''start'');']);
        
        EPoverview.handles.hTrials = uicontrol('Style', 'pushbutton', 'String', 'Trials',...
            'Position', [280 windowHeight-80 60 30], 'Callback', ['global EPoverview;','EPoverview.mode=''trials'';','ep_editData(''start'');']);
        
        if ~strcmp(EPoverview.workData.dataType,'single_trial')
            set(EPoverview.handles.hTrials,'enable','off');
        end;
        
        EPoverview.handles.hChannels = uicontrol('Style', 'pushbutton', 'String', 'Channels',...
            'Position', [340 windowHeight-80 60 30], 'Callback', ['global EPoverview;','EPoverview.mode=''channels'';','ep_editData(''start'');']);
        
        EPoverview.handles.hSamples = uicontrol('Style', 'pushbutton', 'String', 'Samples',...
            'Position', [400 windowHeight-80 60 30], 'Callback', ['global EPoverview;','EPoverview.mode=''samples'';','ep_editData(''start'');']);
        
        EPoverview.handles.hEvents = uicontrol('Style', 'pushbutton', 'String', 'Events',...
            'Position', [460 windowHeight-80 60 30], 'Callback', ['global EPoverview;','EPoverview.mode=''events'';','ep_editData(''start'');']);
        
        EPoverview.handles.hFactors = uicontrol('Style', 'pushbutton', 'String', 'Factors',...
            'Position', [520 windowHeight-80 60 30], 'Callback', ['global EPoverview;','EPoverview.mode=''factors'';','ep_editData(''start'');']);
        
        if isempty(EPoverview.workData.facNames)
            set(EPoverview.handles.hFactors,'enable','off');
        end;
        
        EPoverview.handles.hQC = uicontrol('Style', 'pushbutton', 'String', 'QC',...
            'Position', [580 windowHeight-80 60 30], 'Callback', ['global EPoverview;','EPoverview.mode=''QC'';','ep_editData(''start'');']);
        
        EPoverview.handles.hPCA = uicontrol('Style', 'pushbutton', 'String', 'PCA',...
            'Position', [640 windowHeight-80 60 30], 'Callback', ['global EPoverview;','EPoverview.mode=''PCA'';','ep_editData(''start'');']);
        
        if ~isfield(EPoverview.workData,'pca')
            set(EPoverview.handles.hPCA,'enable','off');
        end;
        
        
        switch EPoverview.mode
            case 'overview'
                set(EPoverview.handles.hOverview,'ForegroundColor','blue');
                ep_editData('startOverview');
                
            case 'subjects'
                set(EPoverview.handles.hSubjects,'ForegroundColor','blue');
                ep_editData('startSubjects');
                
            case 'cells'
                set(EPoverview.handles.hCells,'ForegroundColor','blue');
                ep_editData('startCells');
                
            case 'trials'
                set(EPoverview.handles.hTrials,'ForegroundColor','blue');
                ep_editData('startTrials');
                
            case 'channels'
                set(EPoverview.handles.hChannels,'ForegroundColor','blue');
                ep_editData('startChannels');
                
            case 'samples'
                set(EPoverview.handles.hSamples,'ForegroundColor','blue');
                ep_editData('startSamples');
                
            case 'events'
                set(EPoverview.handles.hEvents,'ForegroundColor','blue');
                ep_editData('startEvents');
                
            case 'factors'
                set(EPoverview.handles.hFactors,'ForegroundColor','blue');
                ep_editData('startFactors');
                
            case 'QC'
                set(EPoverview.handles.hQC,'ForegroundColor','blue');
                ep_editData('startQC');
                
            case 'PCA'
                set(EPoverview.handles.hPCA,'ForegroundColor','blue');
                ep_editData('startPCA');
                
            otherwise
                error('Not a valid mode.');
        end;
        
    case 'startOverview'
        
        uicontrol('Style','text','HorizontalAlignment','left','String', sprintf('%s', 'Data Name'),...
            'Position',[100 windowHeight-120 150 20]);
        EPoverview.handles.overview.hDataName = uicontrol('Style','edit','HorizontalAlignment','left','String', sprintf('%s', EPoverview.workData.dataName),...
            'Position',[100 windowHeight-140 150 20],'Callback', ['ep_editData(''renameDataName'');']);
        
        uicontrol('Style','text','HorizontalAlignment','left','String', sprintf('%s', 'Experiment Name'),...
            'Position',[100 windowHeight-170 150 20]);
        EPoverview.handles.overview.hExpName = uicontrol('Style','edit','HorizontalAlignment','left','String', sprintf('%s', EPoverview.workData.ename),...
            'Position',[100 windowHeight-190 150 20],'Callback', ['ep_editData(''renameExp'');']);
        
        uicontrol('Style','text','HorizontalAlignment','left','String', sprintf('%s', 'Montage'),...
            'Position',[100 windowHeight-220 150 20]);
        EPoverview.handles.overview.hMontage = uicontrol('Style','edit','HorizontalAlignment','left','String', sprintf('%s', EPoverview.workData.montage),...
            'Position',[100 windowHeight-240 150 20],'Callback', ['ep_editData(''montage'');']);
        
        uicontrol('Style','text','HorizontalAlignment','left','String', sprintf('%s', 'File Format'),...
            'Position',[100 windowHeight-270 150 20]);
        EPoverview.handles.overview.hFileFormat = uicontrol('Style','text','HorizontalAlignment','left','String', sprintf('%s', EPoverview.workData.fileFormat),...
            'Position',[100 windowHeight-290 150 20]);
        
        uicontrol('Style','text','HorizontalAlignment','left','String', sprintf('%s', 'Sampling Rate (Hz)'),...
            'Position',[100 windowHeight-320 150 20]);
        EPoverview.handles.overview.hFs = uicontrol('Style','edit','HorizontalAlignment','left','String', sprintf('%d', EPoverview.workData.Fs),...
            'Position',[100 windowHeight-340 150 20],'Callback', ['ep_editData(''Fs'');']);
        
        uicontrol('Style','text','HorizontalAlignment','left','String', sprintf('%s', 'Prestim Period (msec)'),...
            'Position',[100 windowHeight-370 150 20]);
        EPoverview.handles.overview.hBaseline = uicontrol('Style','edit','HorizontalAlignment','left','String', sprintf('%d', (1000/EPoverview.workData.Fs)*EPoverview.workData.baseline),...
            'Position',[100 windowHeight-390 150 20],'Callback', ['ep_editData(''baseline'');']);
        
        [pathstr, name, ext] = fileparts(EPoverview.workData.fileName);
        
        uicontrol('Style','text','HorizontalAlignment','left','String', sprintf('%s', 'Original File Name'),...
            'Position',[100 windowHeight-420 150 20]);
        EPoverview.handles.overview.hFileName = uicontrol('Style','text','HorizontalAlignment','left','String', sprintf('%s', name),...
            'Position',[100 windowHeight-440 150 20]);
        
        uicontrol('Style','text','HorizontalAlignment','left','String', sprintf('%s', 'Channel Coordinates'),...
            'Position',[100 windowHeight-470 150 20]);
        EPoverview.handles.overview.hCed = uicontrol('Style','edit','HorizontalAlignment','left','String', sprintf('%s', EPoverview.workData.ced),...
            'Position',[100 windowHeight-490 150 20],'Callback', ['ep_editData(''ced'');']);
        
        uicontrol('Style','text','HorizontalAlignment','left','String', sprintf('%s', 'Number of Channels'),...
            'Position',[100 windowHeight-520 150 20]);
        EPoverview.handles.overview.hChannels= uicontrol('Style','text','HorizontalAlignment','left','String', sprintf('%d', length(EPoverview.workData.chanNames)),...
            'Position',[100 windowHeight-540 150 20]);
        
        uicontrol('Style','text','HorizontalAlignment','left','String', sprintf('%s', 'Number of Samples'),...
            'Position',[100 windowHeight-570 150 20]);
        EPoverview.handles.overview.hSamples = uicontrol('Style','text','HorizontalAlignment','left','String', sprintf('%d', length(EPoverview.workData.timeNames)),...
            'Position',[100 windowHeight-590 150 20]);
        
        uicontrol('Style','text','HorizontalAlignment','left','String', sprintf('%s', 'Number of Cells'),...
            'Position',[100 windowHeight-620 150 20]);
        EPoverview.handles.overview.hCells = uicontrol('Style','text','HorizontalAlignment','left','String', sprintf('%d', length(unique(EPoverview.workData.cellNames))),...
            'Position',[100 windowHeight-640 150 20]);
        
        uicontrol('Style','text','HorizontalAlignment','left','String', sprintf('%s', 'Number of Subjects'),...
            'Position',[100 windowHeight-670 150 20]);
        EPoverview.handles.overview.hSubjects = uicontrol('Style','text','HorizontalAlignment','left','String', sprintf('%d', length(EPoverview.workData.subNames)),...
            'Position',[100 windowHeight-690 150 20]);
        
        uicontrol('Style','text','HorizontalAlignment','left','String', sprintf('%s', 'Data Type'),...
            'Position',[300 windowHeight-120 150 20]);
        EPoverview.handles.overview.hDataType = uicontrol('Style','text','HorizontalAlignment','left','String', sprintf('%s', EPoverview.workData.dataType),...
            'Position',[300 windowHeight-140 150 20]);
        
        if ~isempty(EPoverview.workData.facNames)
            uicontrol('Style','text','HorizontalAlignment','left','String', sprintf('%s', 'Number of Factors'),...
                'Position',[300 windowHeight-170 150 20]);
            EPoverview.handles.overview.hSubjects = uicontrol('Style','text','HorizontalAlignment','left','String', sprintf('%d', length(EPoverview.workData.facNames)),...
                'Position',[300 windowHeight-190 150 20]);
        end;
        
        if strcmp(EPoverview.workData.dataType,'single_trial')
            uicontrol('Style','text','HorizontalAlignment','left','String', sprintf('%s', 'Number of Trials'),...
                'Position',[300 windowHeight-220 150 20]);
            EPoverview.handles.overview.hCells = uicontrol('Style','text','HorizontalAlignment','left','String', sprintf('%d', length(EPoverview.workData.cellNames)),...
                'Position',[300 windowHeight-240 150 20]);
        end;
        
        
    case 'startSubjects'
        
        for i=1:numSubs
            tableData{i,1}=i;
            tableData{i,2}=EPoverview.subjects.select(i);
            if ~strcmp(EPoverview.workData.dataType,'single_trial')
                tableData{i,3}=EPoverview.subjects.weights(i);
            end;
            tableData{i,4}=char(EPoverview.workData.subNames(i));
            tableData{i,5}=char(EPoverview.workData.subTypes(i));
            if numSpecs
                tableData(i,6:numSpecs+5)=EPoverview.workData.subjectSpecs(i,:);
            end;
        end;
        
        tableNames{1}='order';
        tableNames{2}='select';
        if ~strcmp(EPoverview.workData.dataType,'single_trial')
            tableNames{3}='weights';
        end;
        tableNames{4}='names';
        tableNames{5}='type';
        tableNames(6:length(EPoverview.workData.subjectSpecNames)+5)=EPoverview.workData.subjectSpecNames;
        numNames=size(tableNames,2);
        
        columnEditable =  repmat(true,1,length(EPoverview.workData.subjectSpecNames)+5);
        if strcmp(EPoverview.workData.dataType,'single_trial')
            columnEditable(3)=false;
        end;
        ColumnFormat{1}='numeric';
        ColumnFormat{2}='logical';
        ColumnFormat{3}='numeric';
        ColumnFormat{4}=[];
        ColumnFormat{5}={'RAW','AVE','GAV'};
        
        if ~EPmain.noTable
            try
                EPoverview.handles.subjects.hTable = uitable('Data',tableData,'ColumnName',tableNames,...
                    'ColumnEditable', columnEditable, 'ColumnFormat', ColumnFormat,...
                    'CellEditCallback','ep_editData(''editSubs'');','Position',[100 windowHeight-600 500 500]);
            catch
                EPmain.noTable=1;
            end;
        end;
        if EPmain.noTable
            initializeEPtable=0;
            if isfield(EPtable,'edit')
                if ~isfield(EPtable.edit,'subjects')
                    initializeEPtable=1;
                end
            else
                initializeEPtable=1;
            end;
            if initializeEPtable
                ColumnFormat{4}='char';
                ColumnFormat{5}='char';
                for i=6:length(tableNames)
                    ColumnFormat{i}='char';
                end;
                
                EPtable.edit.subjects.Xloc=100;
                EPtable.edit.subjects.Yloc=windowHeight-120;
                EPtable.edit.subjects.nRows=min(30,length(EPoverview.workData.subNames));
                EPtable.edit.subjects.nCols=min(6,size(tableData,2));
                EPtable.edit.subjects.colWidth=80;
                EPtable.edit.subjects.rowNames=[1:length(EPoverview.workData.subNames)];
                EPtable.edit.subjects.rowNames=cellstr(num2str(EPtable.edit.subjects.rowNames'));
                EPtable.edit.subjects.tableNames=tableNames;
                EPtable.edit.subjects.tableData=tableData;
                EPtable.edit.subjects.columnEditable=columnEditable;
                EPtable.edit.subjects.ColumnFormat=ColumnFormat;
                EPtable.edit.subjects.selRows=[1:EPtable.edit.subjects.nRows];
                EPtable.edit.subjects.selCols=[1:EPtable.edit.subjects.nCols];
                EPtable.edit.subjects.endHeaderCallback='ep_editData(''start'');';
                EPtable.edit.subjects.endTableCallback='ep_editData(''editSubs'');';
                if length(EPoverview.workData.subNames)-1 <= EPtable.edit.subjects.nRows
                    EPtable.edit.subjects.chgRows=repmat(false,1,EPtable.edit.subjects.nRows);
                else
                    EPtable.edit.subjects.chgRows=repmat(true,1,EPtable.edit.subjects.nRows);
                end;
                if length(tableNames) <= EPtable.edit.subjects.nCols
                    EPtable.edit.subjects.chgCols=repmat(false,1,EPtable.edit.subjects.nCols);
                else
                    EPtable.edit.subjects.chgCols=repmat(true,1,EPtable.edit.subjects.nCols);
                end;
                
                EPtable.edit.subjects.name='edit.subjects';
            end
            
            EPtable.edit.subjects=ep_noTable(EPtable.edit.subjects);
        end;
        
        EPoverview.handles.subjects.hAdd = uicontrol('Style', 'pushbutton', 'String', 'Delete',...
            'Position', [20 windowHeight-280 60 30], 'Callback', 'ep_editData(''deleteSubs'');');
        if ~strcmp(EPoverview.workData.dataType,'single_trial')
            EPoverview.handles.subjects.hAdd = uicontrol('Style', 'pushbutton', 'String', 'Add',...
                'Position', [20 windowHeight-310 60 30], 'Callback', 'ep_editData(''addSubs'');');
        end;
        if EPmain.noTable
            weights=EPtable.edit.subjects.tableData{:,3};
        else
            tableData=get(EPoverview.handles.subjects.hTable,'Data');
            weights=cell2mat(tableData(:,3));
        end;
        EPoverview.handles.subjects.hWeightsSum = uicontrol('Style', 'text', 'String', sprintf('%f',sum(weights)),...
            'Position', [20 windowHeight-340 60 30]);
        EPoverview.handles.subjects.hClear = uicontrol('Style', 'pushbutton', 'String', 'Clear',...
            'Position', [20 windowHeight-370 60 30], 'Callback', 'ep_editData(''clearSubWeights'');');
        EPoverview.handles.subjects.hAll = uicontrol('Style', 'pushbutton', 'String', 'All',...
            'Position', [20 windowHeight-400 60 30], 'Callback', 'ep_editData(''allSubs'');');
        EPoverview.handles.subjects.hAppend = uicontrol('Style', 'pushbutton', 'String', 'Append',...
            'Position', [20 windowHeight-430 60 30], 'Callback', 'ep_editData(''appendSubs'');');
        EPoverview.handles.subjects.hExport = uicontrol('Style', 'pushbutton', 'String', 'Export',...
            'Position', [20 windowHeight-460 60 30], 'Callback', 'ep_editData(''exportSubs'');');
        EPoverview.handles.subjects.addSpec = uicontrol('Style', 'pushbutton', 'String', '+ Spec',...
            'Position', [20 windowHeight-490 60 30], 'Callback', 'ep_editData(''addSubSpec'');');
        EPoverview.handles.subjects.minusSpec = uicontrol('Style', 'pushbutton', 'String', '- Spec',...
            'Position', [20 windowHeight-520 60 30], 'Callback', 'ep_editData(''minusSubSpec'');');
        if isempty(EPoverview.workData.subjectSpecNames)
            set(EPoverview.handles.subjects.minusSpec,'enable','off');
        end;
        
    case 'startCells'
        
        [uniqueCells waveOrder m]=unique(EPoverview.workData.cellNames,'first'); %waveOrder is the true order of the unique cells (wave numbers of the first occurences)
        [a b cellOrder]=unique(waveOrder,'first'); %cellOrder is the true order of the unique cells (numbered by cells)
        [n xout] = hist(m,length(uniqueCells));
        
        for i=1:numCells %where i is the display order
            tableData{i,1}=i;
            tableData{i,2}=EPoverview.cells.select(i);
            if ~strcmp(EPoverview.workData.dataType,'single_trial')
                tableData{i,3}=EPoverview.cells.weights(i);
            end;
            tableData{i,4}=char(uniqueCells(cellOrder==i));
            if strcmp(EPoverview.workData.dataType,'single_trial')
                tableData{i,5}=n(cellOrder==i);
            else
                tableData{i,5}=char(EPoverview.workData.cellTypes(i));
            end;
        end;
        
        tableNames{1}='order';
        tableNames{2}='select';
        if ~strcmp(EPoverview.workData.dataType,'single_trial')
            tableNames{3}='weights';
        end;
        tableNames{4}='names';
        if strcmp(EPoverview.workData.dataType,'single_trial')
            tableNames{5}='trials';
        else
            tableNames{5}='types';
        end
        
        columnEditable =  repmat(true,1,5);
        ColumnFormat{1}='numeric';
        ColumnFormat{2}='logical';
        ColumnFormat{3}='numeric';
        ColumnFormat{4}=[];
        if strcmp(EPoverview.workData.dataType,'single_trial')
            ColumnFormat{5}=[];
            columnEditable(5)=false;
        else
            ColumnFormat{5}={'SGL','CMB'};
            columnEditable(5)=true;
        end
        
        if ~EPmain.noTable
            try
                EPoverview.handles.cells.hTable = uitable('Data',tableData,'ColumnName',tableNames,...
                    'ColumnEditable', columnEditable, 'ColumnFormat', ColumnFormat,...
                    'CellEditCallback','ep_editData(''editCells'');','Position',[100 windowHeight-600 500 500]);
            catch
                EPmain.noTable=1;
            end;
        end;
        if EPmain.noTable
            initializeEPtable=0;
            if isfield(EPtable,'edit')
                if ~isfield(EPtable.edit,'cells')
                    initializeEPtable=1;
                end
            else
                initializeEPtable=1;
            end;
            if initializeEPtable
                ColumnFormat{4}='char';
                ColumnFormat{5}='char';
                
                EPtable.edit.cells.Xloc=100;
                EPtable.edit.cells.Yloc=windowHeight-120;
                EPtable.edit.cells.nRows=min(30,numCells);
                EPtable.edit.cells.nCols=min(6,size(tableData,2));
                EPtable.edit.cells.colWidth=80;
                EPtable.edit.cells.rowNames=[1:numCells];
                EPtable.edit.cells.rowNames=cellstr(num2str(EPtable.edit.cells.rowNames'));
                EPtable.edit.cells.tableNames=tableNames;
                EPtable.edit.cells.tableData=tableData;
                EPtable.edit.cells.columnEditable=columnEditable;
                EPtable.edit.cells.ColumnFormat=ColumnFormat;
                EPtable.edit.cells.selRows=[1:EPtable.edit.cells.nRows];
                EPtable.edit.cells.selCols=[1:EPtable.edit.cells.nCols];
                EPtable.edit.cells.endHeaderCallback='ep_editData(''start'');';
                EPtable.edit.cells.endTableCallback='ep_editData(''editCells'');';
                if length(EPoverview.workData.cellNames)-1 <= EPtable.edit.cells.nRows
                    EPtable.edit.cells.chgRows=repmat(false,1,EPtable.edit.cells.nRows);
                else
                    EPtable.edit.cells.chgRows=repmat(true,1,EPtable.edit.cells.nRows);
                end;
                if length(tableNames) <= EPtable.edit.cells.nCols
                    EPtable.edit.cells.chgCols=repmat(false,1,EPtable.edit.cells.nCols);
                else
                    EPtable.edit.cells.chgCols=repmat(true,1,EPtable.edit.cells.nCols);
                end;
                
                EPtable.edit.cells.name='edit.cells';
            end
            
            EPtable.edit.cells=ep_noTable(EPtable.edit.cells);
        end;
        
        EPoverview.handles.cells.hDelete = uicontrol('Style', 'pushbutton', 'String', 'Delete',...
            'Position', [20 windowHeight-280 60 30], 'Callback', 'ep_editData(''deleteCells'');');
        if ~strcmp(EPoverview.workData.dataType,'single_trial')
            EPoverview.handles.cells.hAdd = uicontrol('Style', 'pushbutton', 'String', 'Add',...
                'Position', [20 windowHeight-310 60 30], 'Callback', 'ep_editData(''addCells'');');
            if EPmain.noTable
                weights=EPtable.edit.cells.tableData{:,3};
            else
                tableData=get(EPoverview.handles.cells.hTable,'Data');
                weights=cell2mat(tableData(:,3));
            end;
            EPoverview.handles.cells.hWeights = uicontrol('Style', 'text', 'String', sprintf('%f',sum(weights)),...
                'Position', [20 windowHeight-340 60 30]);
        end;
        EPoverview.handles.cells.hDelete = uicontrol('Style', 'pushbutton', 'String', 'Clear',...
            'Position', [20 windowHeight-370 60 30], 'Callback', 'ep_editData(''clearCellWeights'');');
        EPoverview.handles.cells.hAll = uicontrol('Style', 'pushbutton', 'String', 'All',...
            'Position', [20 windowHeight-400 60 30], 'Callback', 'ep_editData(''allCells'');');
        EPoverview.handles.cells.hAppend = uicontrol('Style', 'pushbutton', 'String', 'Append',...
            'Position', [20 windowHeight-430 60 30], 'Callback', 'ep_editData(''appendCells'');');
        EPoverview.handles.cells.hExport = uicontrol('Style', 'pushbutton', 'String', 'Export',...
            'Position', [20 windowHeight-460 60 30], 'Callback', 'ep_editData(''exportCells'');');
        
        
    case 'startTrials'
        
        tableData(:,1)=EPoverview.workData.cellNames;
        tableData(:,2)=num2cell(EPoverview.workData.trialNames);
        tableData(:,3:length(EPoverview.workData.trialSpecNames)+2)=EPoverview.workData.trialSpecs;
        
        tableNames{1}='cell';
        tableNames{2}='trial';
        tableNames(3:length(EPoverview.workData.trialSpecNames)+2)=EPoverview.workData.trialSpecNames;
        
        columneditable =  repmat(true,1,length(EPoverview.workData.trialSpecNames));
        
        if ~EPmain.noTable
            try
                EPoverview.handles.trials.hTable = uitable('Data',tableData,'ColumnName',tableNames,...
                    'ColumnEditable', columneditable,...
                    'CellEditCallback','ep_editData(''trialSpecs'');','Position',[100 windowHeight-600 500 500]);
            catch
                EPmain.noTable=1;
            end;
        end;
        
        if EPmain.noTable
            initializeEPtable=0;
            if isfield(EPtable,'edit')
                if ~isfield(EPtable.edit,'trials')
                    initializeEPtable=1;
                end
            else
                initializeEPtable=1;
            end;
            if initializeEPtable
                for i=1:size(tableData,2)
                    ColumnFormat{i}='char';
                end
                
                EPtable.edit.trials.Xloc=100;
                EPtable.edit.trials.Yloc=windowHeight-120;
                EPtable.edit.trials.nRows=min(30,numWaves);
                EPtable.edit.trials.nCols=min(6,size(tableData,2));
                EPtable.edit.trials.colWidth=80;
                EPtable.edit.trials.rowNames=[1:numWaves];
                EPtable.edit.trials.rowNames=trialstr(num2str(EPtable.edit.trials.rowNames'));
                EPtable.edit.trials.tableNames=tableNames;
                EPtable.edit.trials.tableData=tableData;
                EPtable.edit.trials.columnEditable=columnEditable;
                EPtable.edit.trials.ColumnFormat=ColumnFormat;
                EPtable.edit.trials.selRows=[1:EPtable.edit.trials.nRows];
                EPtable.edit.trials.selCols=[1:EPtable.edit.trials.nCols];
                EPtable.edit.trials.endHeaderCallback='ep_editData(''start'');';
                EPtable.edit.trials.endTableCallback='ep_editData(''editTrials'');';
                if size(tableData,1) <= EPtable.edit.trials.nRows
                    EPtable.edit.trials.chgRows=repmat(false,1,EPtable.edit.trials.nRows);
                else
                    EPtable.edit.trials.chgRows=repmat(true,1,EPtable.edit.trials.nRows);
                end;
                if length(tableNames) <= EPtable.edit.trials.nCols
                    EPtable.edit.trials.chgCols=repmat(false,1,EPtable.edit.trials.nCols);
                else
                    EPtable.edit.trials.chgCols=repmat(true,1,EPtable.edit.trials.nCols);
                end;
                
                EPtable.edit.trials.name='edit.trials';
            end
            
            EPtable.edit.trials=ep_noTable(EPtable.edit.trials);
        end;
        
        EPoverview.handles.trials.hExport = uicontrol('Style', 'pushbutton', 'String', 'Export',...
            'Position', [20 windowHeight-460 60 30], 'Callback', 'ep_editData(''exportTrials'');');
        
    case 'startChannels'
        
        for i=1:numChans
            tableData{i,1}=i;
            tableData{i,2}=EPoverview.channels.select(i);
            tableData{i,3}=EPoverview.channels.weights(i);
            tableData{i,4}=char(EPoverview.workData.chanNames(i));
            tableData{i,5}=char(EPoverview.workData.chanTypes(i));
        end;
        
        for i=1:numImplicit
            tableData{i+numChans,1}=i+numChans;
            tableData{i+numChans,2}=EPoverview.channels.select(i+numChans);
            tableData{i+numChans,3}=EPoverview.channels.weights(i+numChans);
            tableData{i+numChans,4}=char(EPoverview.workData.implicit(i).labels);
            tableData{i+numChans,5}=char(EPoverview.workData.implicit(i).type);
        end;
        
        tableNames{1}='order';
        tableNames{2}='select';
        tableNames{3}='weights';
        tableNames{4}='Name';
        tableNames{5}='Type';
        
        columnEditable =  [repmat(true,1,4) false];
        ColumnFormat{1}='numeric';
        ColumnFormat{2}='logical';
        ColumnFormat{3}='numeric';
        ColumnFormat{4}=[];
        ColumnFormat{5}={'EEG','ANS','MEG','REF','FID','REG'};
        
        if ~EPmain.noTable
            try
                EPoverview.handles.channels.hTable = uitable('Data',tableData,'ColumnName',tableNames,...
                    'ColumnEditable', columnEditable, 'ColumnFormat', ColumnFormat,...
                    'CellEditCallback','ep_editData(''editChannels'');','Position',[100 windowHeight-600 500 500]);
            catch
                EPmain.noTable=1;
            end;
        end;
        
        if EPmain.noTable
            initializeEPtable=0;
            if isfield(EPtable,'edit')
                if ~isfield(EPtable.edit,'channels')
                    initializeEPtable=1;
                end
            else
                initializeEPtable=1;
            end;
            if initializeEPtable
                for i=4:size(tableData,2)
                    ColumnFormat{i}='char';
                end
                
                EPtable.edit.channels.Xloc=100;
                EPtable.edit.channels.Yloc=windowHeight-120;
                EPtable.edit.channels.nRows=min(30,numChans);
                EPtable.edit.channels.nCols=min(6,size(tableData,2));
                EPtable.edit.channels.colWidth=80;
                EPtable.edit.channels.rowNames=[1:numChans];
                EPtable.edit.channels.rowNames=cellstr(num2str(EPtable.edit.channels.rowNames'));
                EPtable.edit.channels.tableNames=tableNames;
                EPtable.edit.channels.tableData=tableData;
                EPtable.edit.channels.columnEditable=columnEditable;
                EPtable.edit.channels.ColumnFormat=ColumnFormat;
                EPtable.edit.channels.selRows=[1:EPtable.edit.channels.nRows];
                EPtable.edit.channels.selCols=[1:EPtable.edit.channels.nCols];
                EPtable.edit.channels.endHeaderCallback='ep_editData(''start'');';
                EPtable.edit.channels.endTableCallback='ep_editData(''editChannels'');';
                if size(tableData,1) <= EPtable.edit.channels.nRows
                    EPtable.edit.channels.chgRows=repmat(false,1,EPtable.edit.channels.nRows);
                else
                    EPtable.edit.channels.chgRows=repmat(true,1,EPtable.edit.channels.nRows);
                end;
                if length(tableNames) <= EPtable.edit.channels.nCols
                    EPtable.edit.channels.chgCols=repmat(false,1,EPtable.edit.channels.nCols);
                else
                    EPtable.edit.channels.chgCols=repmat(true,1,EPtable.edit.channels.nCols);
                end;
                
                EPtable.edit.channels.name='edit.channels';
            end
            
            EPtable.edit.channels=ep_noTable(EPtable.edit.channels);
        end;
        
        EPoverview.handles.channels.hDelete = uicontrol('Style', 'pushbutton', 'String', 'Delete',...
            'Position', [20 windowHeight-280 60 30], 'Callback', 'ep_editData(''deleteChans'');');
        EPoverview.handles.channels.hAdd = uicontrol('Style', 'pushbutton', 'String', 'Add',...
            'Position', [20 windowHeight-310 60 30], 'Callback', 'ep_editData(''addChans'');');
        if EPmain.noTable
            weights=EPtable.edit.channels.tableData{:,3};
        else
            tableData=get(EPoverview.handles.channels.hTable,'Data');
            weights=cell2mat(tableData(:,3));
        end;
        EPoverview.handles.channels.hWeights = uicontrol('Style', 'text', 'String', sprintf('%f',sum(weights)),...
            'Position', [20 windowHeight-340 60 30]);
        EPoverview.handles.channels.hClear = uicontrol('Style', 'pushbutton', 'String', 'Clear',...
            'Position', [20 windowHeight-370 60 30], 'Callback', 'ep_editData(''clearChanWeights'');');
        EPoverview.handles.channels.hAll = uicontrol('Style', 'pushbutton', 'String', 'All',...
            'Position', [20 windowHeight-400 60 30], 'Callback', 'ep_editData(''allChans'');');
        EPoverview.handles.channels.hAppend = uicontrol('Style', 'pushbutton', 'String', 'Append',...
            'Position', [20 windowHeight-430 60 30], 'Callback', 'ep_editData(''appendChans'');');
        EPoverview.handles.channels.hExport = uicontrol('Style', 'pushbutton', 'String', 'Export',...
            'Position', [20 windowHeight-460 60 30], 'Callback', 'ep_editData(''exportChans'');');
        
    case 'startSamples'
        
        sampleTime=1000/EPoverview.workData.Fs;
        uicontrol('Style', 'text', 'String', 'Ms',...
            'Position', [100 windowHeight-120 60 15],'HorizontalAlignment','left');
        uicontrol('Style', 'text', 'String', 'Samples',...
            'Position', [100 windowHeight-145 60 15],'HorizontalAlignment','left');
        uicontrol('Style', 'text', 'String', sprintf('%d', -EPoverview.workData.baseline*sampleTime),...
            'Position', [200 windowHeight-120 60 15],'HorizontalAlignment','left');
        uicontrol('Style', 'text', 'String', sprintf('%d', (length(EPoverview.workData.timeNames)-EPoverview.workData.baseline)*sampleTime),...
            'Position', [350 windowHeight-120 60 15],'HorizontalAlignment','left');
        EPoverview.handles.samples.startSamp = uicontrol('Style','edit','HorizontalAlignment','left','String', sprintf('%d', 1),...
            'Position',[200 windowHeight-145 100 25],'Callback', ['ep_editData(''changeStartSamp'');']);
        EPoverview.handles.samples.endSamp = uicontrol('Style','edit','HorizontalAlignment','left','String', sprintf('%d', (length(EPoverview.workData.timeNames))),...
            'Position',[350 windowHeight-145 100 25],'Callback', ['ep_editData(''changeEndSamp'');']);
        
        EPoverview.handles.samples.hFsLabel = uicontrol('Style','text','HorizontalAlignment','left','String', sprintf('%s', 'Sampling Rate (Hz)'),...
            'Position',[100 windowHeight-270 150 20]);
        EPoverview.handles.samples.hFs = uicontrol('Style','edit','HorizontalAlignment','left','String', sprintf('%d', EPoverview.workData.Fs),...
            'Position',[100 windowHeight-290 150 20],'Callback', ['ep_editData(''Fs'');']);
        
        EPoverview.handles.samples.hBaselineLabel = uicontrol('Style','text','HorizontalAlignment','left','String', sprintf('%s', 'Baseline (msec)'),...
            'Position',[100 windowHeight-320 150 20]);
        EPoverview.handles.samples.hBaseline = uicontrol('Style','edit','HorizontalAlignment','left','String', sprintf('%d', (1000/EPoverview.workData.Fs)*EPoverview.workData.baseline),...
            'Position',[100 windowHeight-340 150 20],'Callback', ['ep_editData(''baseline'');']);
        
        
        EPoverview.handles.samples.halfRate = uicontrol('Style', 'pushbutton', 'String', '1/2 Rate',...
            'Position', [20 windowHeight-280 60 30], 'Callback', 'ep_editData(''halfRate'');');
        
    case 'startEvents'
        
        [uniqueCells waveOrder m]=unique(EPoverview.workData.cellNames,'first'); %waveOrder is the true order of the unique cells (wave numbers of the first occurences)
        [a b cellOrder]=unique(waveOrder,'first'); %cellOrder is the true order of the unique cells (numbered by cells)
        [n xout] = hist(m,length(uniqueCells));
        
        tableData=[];
        eventCounter=0;
        for subject=1:numSubs
            for wave=1:numWaves
                for event=1:length(EPoverview.workData.events{subject,wave})
                    eventCounter=eventCounter+1;
                    tableData{eventCounter,1}=EPoverview.events.select(eventCounter);
                    tableData{eventCounter,2}=EPoverview.workData.cellNames{wave};
                    if strcmp(EPoverview.workData.dataType,'single_trial')
                        tableData{eventCounter,3}=EPoverview.workData.trialNames(wave);
                    end;
                    tableData{eventCounter,4}=EPoverview.workData.subNames{subject};
                    tableData{eventCounter,5}=event;
                    tableData{eventCounter,6}=EPoverview.workData.events{subject,wave}(event).type;
                    tableData{eventCounter,7}=EPoverview.workData.events{subject,wave}(event).sample;
                    tableData{eventCounter,8}=EPoverview.workData.events{subject,wave}(event).value;
                    tableData{eventCounter,9}=(-(EPoverview.workData.events{subject,wave}(event).offset)-EPoverview.workData.baseline)*(1000/EPoverview.workData.Fs);
                    tableData{eventCounter,10}=EPoverview.workData.events{subject,wave}(event).duration;
                end;
            end;
        end;
        
        tableNames{1}='select';
        tableNames{2}='cells';
        if strcmp(EPoverview.workData.dataType,'single_trial')
            tableNames{3}='trials';
        end;
        tableNames{4}='subjects';
        tableNames{5}='events';
        tableNames{6}='type';
        tableNames{7}='sample';
        tableNames{8}='value';
        tableNames{9}='latency';
        tableNames{10}='duration';
        
        columnEditable = [true repmat(false,1,4) repmat(true,1,5)];
        if ~strcmp(EPoverview.workData.dataType,'single_trial')
            columnEditable(3)=false;
        end;
        
        ColumnFormat = cell(1,10);
        ColumnFormat{1}='logical';
        for i=2:10
            ColumnFormat{i}=[];
        end;
        
        if ~isempty(tableData)
            try
                EPoverview.handles.events.hTable = uitable('Data',tableData,'ColumnName',tableNames,...
                    'ColumnEditable', columnEditable, 'ColumnFormat', ColumnFormat,...
                    'CellEditCallback','ep_editData(''editEvents'');','Position',[100 windowHeight-600 500 500]);
                EPoverview.handles.events.hAdd = uicontrol('Style', 'pushbutton', 'String', 'Delete',...
                    'Position', [20 windowHeight-280 60 30], 'Callback', 'ep_editData(''deleteEvents'');');
                EPoverview.handles.events.hAdd = uicontrol('Style', 'pushbutton', 'String', 'Clear',...
                    'Position', [20 windowHeight-370 60 30], 'Callback', 'ep_editData(''clearEventWeights'');');
                EPoverview.handles.events.hAdd = uicontrol('Style', 'pushbutton', 'String', 'All',...
                    'Position', [20 windowHeight-400 60 30], 'Callback', 'ep_editData(''allEvents'');');
                EPoverview.handles.trials.hExport = uicontrol('Style', 'pushbutton', 'String', 'Export',...
                    'Position', [20 windowHeight-460 60 30], 'Callback', 'ep_editData(''exportEvents'');');
            catch
                EPoverview.handles.events.hTable = uicontrol('Style','text','HorizontalAlignment','left','String', 'This function does not work with this version of Matlab',...
                    'Position',[130 windowHeight-200 200 30]);
            end;
        else
            EPoverview.handles.events.hTable = uicontrol('Style','text','HorizontalAlignment','left','String', 'No events in this data.',...
                'Position',[130 windowHeight-200 200 30]);
        end;
        
    case 'startFactors'
        
        [EPdata]=ep_stripAdds(EPoverview.workData,{'SGLfac','CMBfac'});
        
        numPoints=length(EPdata.timeNames);
        numChans=length(EPdata.chanNames);
        
        if ~isempty(EPdata.facVecT)
            [C peakLatency]=max(abs(EPdata.facVecT));
        else
            peakLatency=zeros(1,numSGLfacs);
            for theFactor=1:numSGLfacs
                [C peakLatency(theFactor)]=max(max(reshape(shiftdim(abs(mean(EPdata.data(:,:,:,:,theFactor),4)),1),numPoints,[])'));
            end;
        end;
        peakLatency=[peakLatency repmat(0,1,numCMBfacs)];
        
        if ~isempty(EPdata.facVecS)
            [C peakChan]=max(abs(EPdata.facVecS));
        else
            peakChan=zeros(1,numSGLfacs);
            for theFactor=1:numSGLfacs
                [C peakChan(theFactor)]=max(max(reshape(abs(mean(EPdata.data(:,:,:,:,theFactor),4)),numChans,[])'));
            end;
        end;
        peakChan=[peakChan repmat(0,1,numCMBfacs)];
        
        if ~isempty(EPdata.facVecT)
            timeSign=sign(EPdata.facVecT(peakLatency(1:numSGLfacs)));
        else
            timeSign=repmat(1,1,numSGLfacs);
        end;
        
        if ~isempty(EPdata.facVecS)
            chanSign=sign(EPdata.facVecS(peakChan(1:numSGLfacs)));
        else
            chanSign=repmat(1,1,numSGLfacs);
        end;
        
        peakPolarity=zeros(1,numSGLfacs);
        for theFactor=1:numSGLfacs
            theMax=max(max(reshape(shiftdim(mean(EPdata.data(:,:,:,:,theFactor),4),1),size(EPdata.data,2),[])')); %to get signs for factor data, multiply signs of loadings by scores.
            theMin=min(min(reshape(shiftdim(mean(EPdata.data(:,:,:,:,theFactor),4),1),size(EPdata.data,2),[])')); %to get signs for factor data, multiply signs of loadings by scores.
            if abs(theMax) >= abs(theMin)
                peakPolarity(theFactor)=1;
            else
                peakPolarity(theFactor)=-1;
            end;
            peakPolarity(theFactor)=peakPolarity(theFactor)*chanSign(theFactor)*timeSign(theFactor);
        end;
        peakPolarity=[peakPolarity repmat(0,1,numCMBfacs)];
        
        peakLatency=(peakLatency-EPdata.baseline)*(1000/EPdata.Fs);
        peakChan=EPdata.chanNames(peakChan(1:numSGLfacs))';
        peakChan=peakChan(:)';
        peakChan=[peakChan cellstr(repmat(' ',1,numCMBfacs))];
        
        for i=1:numFacs
            tableData{i,1}=i;
            tableData{i,2}=EPoverview.factors.select(i);
            tableData{i,3}=EPoverview.factors.weights(i);
            tableData{i,4}=char(EPdata.facNames(i));
            tableData{i,5}=char(EPdata.facTypes(i));
            tableData{i,6}=num2str(peakLatency(i));
            tableData{i,7}=char(peakChan(i));
            tableData{i,8}=peakPolarity(i);
        end;
        
        tableNames{1}='order';
        tableNames{2}='select';
        tableNames{3}='weights';
        tableNames{4}='names';
        tableNames{5}='type';
        tableNames{6}='peakLatency';
        tableNames{7}='peakChan';
        tableNames{8}='peakPolarity';
        
        columnEditable =  [repmat(true,1,5) repmat(false,1,3)];
        ColumnFormat{1}='numeric';
        ColumnFormat{2}='logical';
        ColumnFormat{3}='numeric';
        ColumnFormat{4}=[];
        ColumnFormat{5}={'SGL','CMB'};
        ColumnFormat{6}=[];
        ColumnFormat{7}=[];
        ColumnFormat{8}='+';
        
        if isfield(EPdata,'pca')
            singleFacs=find(ismember(EPdata.facTypes,'SGL'));
            singleFacsNum=length(singleFacs);
            newColumn=size(tableData,2)+1;
            
            if isfield(EPdata,'facVar')
                for i=1:singleFacsNum
                    tableData{singleFacs(i),newColumn}=EPdata.facVar(i);
                end;
                tableNames{newColumn}='facVar';
                ColumnFormat{newColumn}=[];
            end;
            
            newColumn=size(tableData,2)+1;
            if isfield(EPdata,'facVarQ')
                for i=1:singleFacsNum
                    tableData{singleFacs(i),newColumn}=EPdata.facVarQ(i);
                end;
                tableNames{newColumn}='facVarQ';
            end;
        end;
        
        if ~EPmain.noTable
            try
                EPoverview.handles.factors.hTable = uitable('Data',tableData,'ColumnName',tableNames,...
                    'ColumnEditable', columnEditable, 'ColumnFormat', ColumnFormat,...
                    'CellEditCallback','ep_editData(''editFacs'');','Position',[100 windowHeight-600 500 500]);
            catch
                EPmain.noTable=1;
            end;
        end;
        
        if EPmain.noTable
            initializeEPtable=0;
            if isfield(EPtable,'edit')
                if ~isfield(EPtable.edit,'factors')
                    initializeEPtable=1;
                end
            else
                initializeEPtable=1;
            end;
            if initializeEPtable
                for i=4:size(tableData,2)
                    ColumnFormat{i}='char';
                end
                
                EPtable.edit.factors.Xloc=100;
                EPtable.edit.factors.Yloc=windowHeight-120;
                EPtable.edit.factors.nRows=min(30,numFacs);
                EPtable.edit.factors.nCols=min(6,size(tableData,2));
                EPtable.edit.factors.colWidth=80;
                EPtable.edit.factors.rowNames=[1:numFacs];
                EPtable.edit.factors.rowNames=cellstr(num2str(EPtable.edit.factors.rowNames'));
                EPtable.edit.factors.tableNames=tableNames;
                EPtable.edit.factors.tableData=tableData;
                EPtable.edit.factors.columnEditable=columnEditable;
                EPtable.edit.factors.ColumnFormat=ColumnFormat;
                EPtable.edit.factors.selRows=[1:EPtable.edit.factors.nRows];
                EPtable.edit.factors.selCols=[1:EPtable.edit.factors.nCols];
                EPtable.edit.factors.endHeaderCallback='ep_editData(''start'');';
                EPtable.edit.factors.endTableCallback='ep_editData(''editFacs'');';
                if size(tableData,1) <= EPtable.edit.factors.nRows
                    EPtable.edit.factors.chgRows=repmat(false,1,EPtable.edit.factors.nRows);
                else
                    EPtable.edit.factors.chgRows=repmat(true,1,EPtable.edit.factors.nRows);
                end;
                if length(tableNames) <= EPtable.edit.factors.nCols
                    EPtable.edit.factors.chgCols=repmat(false,1,EPtable.edit.factors.nCols);
                else
                    EPtable.edit.factors.chgCols=repmat(true,1,EPtable.edit.factors.nCols);
                end;
                
                EPtable.edit.factors.name='edit.factors';
            end
            
            EPtable.edit.factors=ep_noTable(EPtable.edit.factors);
        end;
        
        EPoverview.handles.factors.hAdd = uicontrol('Style', 'pushbutton', 'String', 'Delete',...
            'Position', [20 windowHeight-280 60 30], 'Callback', 'ep_editData(''deleteFacs'');');
        EPoverview.handles.factors.hAdd = uicontrol('Style', 'pushbutton', 'String', 'Add',...
            'Position', [20 windowHeight-310 60 30], 'Callback', 'ep_editData(''addFacs'');');
        
        if EPmain.noTable
            weights=EPtable.edit.factors.tableData{:,3};
        else
            tableData=get(EPoverview.handles.factors.hTable,'Data');
            weights=cell2mat(tableData(:,3));
        end;
        
        EPoverview.handles.factors.hWeightSum = uicontrol('Style', 'text', 'String', sprintf('%f',sum(weights)),...
            'Position', [20 windowHeight-340 60 30]);
        EPoverview.handles.factors.hClear = uicontrol('Style', 'pushbutton', 'String', 'Clear',...
            'Position', [20 windowHeight-370 60 30], 'Callback', 'ep_editData(''clearFacWeights'');');
        EPoverview.handles.factors.hAll = uicontrol('Style', 'pushbutton', 'String', 'All',...
            'Position', [20 windowHeight-400 60 30], 'Callback', 'ep_editData(''allFacs'');');
        EPoverview.handles.factors.hAppend = uicontrol('Style', 'pushbutton', 'String', 'Append',...
            'Position', [20 windowHeight-430 60 30], 'Callback', 'ep_editData(''appendFacs'');');
        EPoverview.handles.factors.hExport = uicontrol('Style', 'pushbutton', 'String', 'Export',...
            'Position', [20 windowHeight-460 60 30], 'Callback', 'ep_editData(''exportFactors'');');
        
    case 'startQC'
        
        badTrials=EPoverview.workData.analysis.badTrials;
        
        %prepare contents of the table based on the QC setting
        switch EPoverview.QC.mode
            case 'avgNum'
                QCdata=EPoverview.workData.avgNum;
                if all(QCdata == 0)
                    QCdata=nan(size(QCdata));
                end;
            case 'subNum'
                QCdata=EPoverview.workData.subNum;
            case 'blinkTrial'
                QCdata=EPoverview.workData.analysis.blinkTrial;
            case 'saccadeTrial'
                QCdata=EPoverview.workData.analysis.saccadeTrial;
            case 'saccadeOnset'
                QCdata=EPoverview.workData.analysis.saccadeOnset;
            case 'moveTrial'
                QCdata=EPoverview.workData.analysis.moveTrial;
            case 'badTrials'
                QCdata=EPoverview.workData.analysis.badTrials;
            case 'badChans'
                if strcmp(EPoverview.workData.dataType,'single_trial')
                    theNumbers=EPoverview.workData.analysis.badChans.*repmat(~badTrials,[1,1,numChans]); %count only good trials
                else
                    theNumbers=EPoverview.workData.analysis.badChans;
                end;
                QCdata=-squeeze(sum((theNumbers < 0).*theNumbers,3));
            case 'repChans'
                if strcmp(EPoverview.workData.dataType,'single_trial')
                    theNumbers=EPoverview.workData.analysis.badChans.*repmat(~badTrials,[1,1,numChans]); %count only good trials
                else
                    theNumbers=EPoverview.workData.analysis.badChans;
                end;
                QCdata=squeeze(sum((theNumbers > 0).*theNumbers,3));
            case 'noise'
                QCdata=sqrt(squeeze(mean(mean(mean(EPoverview.workData.noise.^2,1),2),5)))'; %root mean square
            case 'std'
                QCdata=squeeze(mean(mean(mean(abs(EPoverview.workData.std),1),2),5))';
        end;
        
        avgNum=EPoverview.workData.avgNum;
        subNum=EPoverview.workData.subNum;
        if all(avgNum(:) == 0) && all(subNum(:) == 0)
            avgNum=nan(size(avgNum)); %if number of trials going into each waveform is unavailable, default to nan.
        elseif all(avgNum(:) == 0) && ~all(subNum(:) == 0)
            avgNum=subNum; %if number of trials going into each waveform is unavailable but subject numbers are available (as in grand averages) then default to subject numbers.
        end;
        
        %convert QC statistics to %if appropriate
        switch EPoverview.QC.mode
            case 'blinkTrial'
                QCdata=QCdata./(avgNum+EPoverview.workData.analysis.blinkTrial);
            case 'saccadeTrial'
                QCdata=QCdata./(avgNum+EPoverview.workData.analysis.saccadeTrial);
            case 'moveTrial'
                QCdata=QCdata./(avgNum+EPoverview.workData.analysis.moveTrial);
            case 'badTrials'
                QCdata=QCdata./(avgNum+EPoverview.workData.analysis.badTrials);
            case 'badChans'
                QCdata=QCdata./(numChans*avgNum);
            case 'repChans'
                QCdata=QCdata./(numChans*avgNum);
        end;
        
        [uniqueCells waveOrder m]=unique(EPoverview.workData.cellNames,'first'); %waveOrder is the true order of the unique cells (wave numbers of the first occurences)
        [a b cellOrder]=unique(waveOrder,'first'); %cellOrder is the true order of the unique cells (numbered by cells)
        
        %aggregate over trials for single trial data
        if strcmp(EPoverview.workData.dataType,'single_trial')
            for i=1:numCells
                theTrials=find(strcmp(uniqueCells(cellOrder==i),EPoverview.workData.cellNames(~badTrials)));
                
                switch EPoverview.QC.mode
                    case 'avgNum'
                        tempQC(i) = sum(QCdata(theTrials));
                    case 'subNum'
                        tempQC(i) = mean(QCdata(theTrials));
                    case 'blinkTrial'
                        tempQC(i) = mean(QCdata(theTrials));
                    case 'saccadeTrial'
                        tempQC(i) = mean(QCdata(theTrials));
                    case 'saccadeOnset'
                        tempQC(i) = mean(QCdata(theTrials));
                    case 'moveTrial'
                        tempQC(i) = mean(QCdata(theTrials));
                    case 'badTrials'
                        tempQC(i) = mean(QCdata(theTrials));
                    case 'badChans'
                        tempQC(i) = mean(QCdata(theTrials));
                    case 'repChans'
                        tempQC(i) = mean(QCdata(theTrials));
                    case 'noise'
                        tempQC(i) = mean(QCdata(theTrials));
                    case 'std'
                        tempQC(i) = mean(QCdata(theTrials));
                end;
                
            end;
            QCdata=tempQC;
        end;
        
        for i=1:numSubs
            tableData{i,1}=char(EPoverview.workData.subNames(i));
            tableData{i,2}=char(EPoverview.workData.subTypes(i));
            tableData(i,3:2+numCells)=num2cell(QCdata(i,:));
        end;
        
        tableNames{1}='names';
        tableNames{2}='type';
        for i=1:numCells
            tableNames(2+i)=uniqueCells(cellOrder==i);
        end;
        
        columnEditable =  repmat(false,1,2+numCells);
        
        ColumnFormat = repmat([],1,2+numCells);
        
        if ~EPmain.noTable
            try
                EPoverview.handles.QC.hTable = uitable('Data',tableData,'ColumnName',tableNames,...
                    'ColumnEditable', columnEditable, 'ColumnFormat', ColumnFormat,...
                    'Position',[100 windowHeight-600 500 500]);
            catch
                EPmain.noTable=1;
            end;
        end;
        
        if EPmain.noTable
            initializeEPtable=0;
            if isfield(EPtable,'edit')
                if ~isfield(EPtable.edit,'QC')
                    initializeEPtable=1;
                end
            else
                initializeEPtable=1;
            end;
            if initializeEPtable
                for i=4:size(tableData,2)
                    ColumnFormat{i}='char';
                end
                
                EPtable.edit.QC.Xloc=100;
                EPtable.edit.QC.Yloc=windowHeight-120;
                EPtable.edit.QC.nRows=min(30,size(tableData,1));
                EPtable.edit.QC.nCols=min(6,size(tableData,2));
                EPtable.edit.QC.colWidth=80;
                EPtable.edit.QC.rowNames=[1:size(tableData,1)];
                EPtable.edit.QC.rowNames=cellstr(num2str(EPtable.edit.QC.rowNames'));
                EPtable.edit.QC.tableNames=tableNames;
                EPtable.edit.QC.tableData=tableData;
                EPtable.edit.QC.columnEditable=columnEditable;
                EPtable.edit.QC.ColumnFormat=ColumnFormat;
                EPtable.edit.QC.selRows=[1:EPtable.edit.QC.nRows];
                EPtable.edit.QC.selCols=[1:EPtable.edit.QC.nCols];
                EPtable.edit.QC.endHeaderCallback='ep_editData(''start'');';
                EPtable.edit.QC.endTableCallback='ep_editData(''start'');';
                if size(tableData,1) <= EPtable.edit.QC.nRows
                    EPtable.edit.QC.chgRows=repmat(false,1,EPtable.edit.QC.nRows);
                else
                    EPtable.edit.QC.chgRows=repmat(true,1,EPtable.edit.QC.nRows);
                end;
                if length(tableNames) <= EPtable.edit.QC.nCols
                    EPtable.edit.QC.chgCols=repmat(false,1,EPtable.edit.QC.nCols);
                else
                    EPtable.edit.QC.chgCols=repmat(true,1,EPtable.edit.QC.nCols);
                end;
                
                EPtable.edit.QC.name='edit.QC';
            end
            
            EPtable.edit.QC=ep_noTable(EPtable.edit.QC);
        end;
        
        EPoverview.handles.QC.hExport = uicontrol('Style', 'pushbutton', 'String', 'Export',...
            'Position', [20 windowHeight-460 60 30], 'Callback', 'ep_editData(''exportQC'');');
        
        if EPmain.noTable
            callBackEnd=['EPtable.edit=rmfield(EPtable.edit,''QC'');','ep_editData(''start'');'];
        else
            callBackEnd=['ep_editData(''start'');'];
        end;
        
        if ~isempty(EPoverview.workData.avgNum)
            EPoverview.handles.QC.avgNum = uicontrol('Style', 'pushbutton', 'String', 'Trials',...
                'TooltipString','Number of trials going into each waveform.',...
                'Position', [100 windowHeight-650 60 30], 'Callback', ['global EPoverview EPtable;','EPoverview.QC.mode=''avgNum'';',callBackEnd]);
        end;
        
        if ~isempty(EPoverview.workData.subNum)
            EPoverview.handles.QC.subNum = uicontrol('Style', 'pushbutton', 'String', 'Subs',...
                'TooltipString','Number of subjects going into each waveform.',...
                'Position', [160 windowHeight-650 60 30], 'Callback', ['global EPoverview EPtable;','EPoverview.QC.mode=''subNum'';',callBackEnd]);
        end;
        
        EPoverview.handles.QC.blinkTrial = uicontrol('Style', 'pushbutton', 'String', 'Blinks',...
            'TooltipString','Number of blink-corrected trials going into each waveform.',...
            'Position', [220 windowHeight-650 60 30], 'Callback', ['global EPoverview EPtable;','EPoverview.QC.mode=''blinkTrial'';',callBackEnd]);
        EPoverview.handles.QC.saccadeTrial = uicontrol('Style', 'pushbutton', 'String', 'Saccades',...
            'TooltipString','Number of saccade-corrected trials going into each waveform.',...
            'Position', [280 windowHeight-650 60 30], 'Callback', ['global EPoverview EPtable;','EPoverview.QC.mode=''saccadeTrial'';',callBackEnd]);
        EPoverview.handles.QC.saccadeOnset = uicontrol('Style', 'pushbutton', 'String', 'Sacc Ms',...
            'TooltipString','Onset of saccades.',...
            'Position', [340 windowHeight-650 60 30], 'Callback', ['global EPoverview EPtable;','EPoverview.QC.mode=''saccadeOnset'';',callBackEnd]);
        EPoverview.handles.QC.moveTrial = uicontrol('Style', 'pushbutton', 'String', 'Move',...
            'TooltipString','Number of movement-corrected trials going into each waveform.',...
            'Position', [400 windowHeight-650 60 30], 'Callback', ['global EPoverview EPtable;','EPoverview.QC.mode=''moveTrial'';',callBackEnd]);
        EPoverview.handles.QC.badTrials = uicontrol('Style', 'pushbutton', 'String', 'BadTrials',...
            'TooltipString','Proportion of trials excluded from each averaged waveform due to being bad trials.',...
            'Position', [460 windowHeight-650 60 30], 'Callback', ['global EPoverview EPtable;','EPoverview.QC.mode=''badTrials'';',callBackEnd]);
        if any(any(any(EPoverview.workData.analysis.badChans(repmat(~badTrials,[1,1,numChans])) < 0)))
            EPoverview.handles.QC.badChans = uicontrol('Style', 'pushbutton', 'String', 'BadChan',...
                'TooltipString','Proportion of bad channels in each set of waveforms, not include bad trials.',...
                'Position', [520 windowHeight-650 60 30], 'Callback', ['global EPoverview EPtable;','EPoverview.QC.mode=''badChans'';',callBackEnd]);
        end;
        if any(any(any(EPoverview.workData.analysis.badChans(repmat(~badTrials,[1,1,numChans])) > 0)))
            EPoverview.handles.QC.repChans = uicontrol('Style', 'pushbutton', 'String', 'RepChan',...
                'TooltipString','Proportion of replaced bad channels in each set of waveforms, not include bad trials.',...
                'Position', [580 windowHeight-650 60 30], 'Callback', ['global EPoverview EPtable;','EPoverview.QC.mode=''repChans'';',callBackEnd]);
        end;
        
        if isfield(EPoverview.workData,'noise')
            EPoverview.handles.QC.noise = uicontrol('Style', 'pushbutton', 'String', 'Noise',...
                'TooltipString','Noise level in each averaged waveform, as computed by the +/- reference.',...
                'Position', [100 windowHeight-680 60 30], 'Callback', ['global EPoverview EPtable;','EPoverview.QC.mode=''noise'';',callBackEnd]);
        end;
        if isfield(EPoverview.workData,'std')
            EPoverview.handles.QC.std = uicontrol('Style', 'pushbutton', 'String', 'Std',...
                'TooltipString','Mean standard deviation in each averaged waveform.',...
                'Position', [160 windowHeight-680 60 30], 'Callback', ['global EPoverview EPtable;','EPoverview.QC.mode=''std'';',callBackEnd]);
        end;
        
        switch EPoverview.QC.mode
            case 'avgNum'
                set(EPoverview.handles.QC.avgNum,'ForegroundColor','blue');
            case 'subNum'
                set(EPoverview.handles.QC.subNum,'ForegroundColor','blue');
            case 'blinkTrial'
                set(EPoverview.handles.QC.blinkTrial,'ForegroundColor','blue');
            case 'saccadeTrial'
                set(EPoverview.handles.QC.saccadeTrial,'ForegroundColor','blue');
            case 'saccadeOnset'
                set(EPoverview.handles.QC.saccadeOnset,'ForegroundColor','blue');
            case 'moveTrial'
                set(EPoverview.handles.QC.moveTrial,'ForegroundColor','blue');
            case 'badTrials'
                set(EPoverview.handles.QC.badTrials,'ForegroundColor','blue');
            case 'badChans'
                set(EPoverview.handles.QC.badChans,'ForegroundColor','blue');
            case 'repChans'
                set(EPoverview.handles.QC.repChans,'ForegroundColor','blue');
            case 'noise'
                set(EPoverview.handles.QC.noise,'ForegroundColor','blue');
            case 'std'
                set(EPoverview.handles.QC.std,'ForegroundColor','blue');
        end;
        
    case 'startPCA'
        
        if strcmp(EPoverview.PCA.mode,'Summary')
            if isfield(EPoverview.workData.pca,'PCAmode2')
                title='First Rotation';
            else
                title='Rotation';
            end;
            uicontrol('Style','text','HorizontalAlignment','left','String', title,...
                'Position',[100 windowHeight-120 150 20]);
            switch EPoverview.workData.pca.PCAmode
                case 'spat'
                    theMode='spatial';
                case 'temp'
                    theMode='temporal';
                otherwise
                    beep()
                    disp('Oops')
                    theMode='Oops';
            end;
            
            uicontrol('Style','text','HorizontalAlignment','left','String', 'Mode',...
                'Position',[100 windowHeight-140 150 20]);
            uicontrol('Style','text','HorizontalAlignment','left','String', theMode,...
                'Position',[100 windowHeight-160 150 20]);
            
            switch EPoverview.workData.pca.ROTATION
                case 'QMIN'
                    theRotationName = 'Quartimin';
                case 'OMIN'
                    theRotationName = 'Oblimin';
                case 'CRFE'
                    theRotationName = 'Crawford-Ferguson';
                case 'MINE'
                    theRotationName = 'minimum entropy';
                case 'IPSC'
                    theRotationName = 'Bentler''s invariant pattern simplicity criterion';
                case 'TIIC'
                    theRotationName = 'Comrey''s tandem II criterion';
                case 'GMIN'
                    theRotationName = 'geomin';
                case 'IMAX'
                    theRotationName = 'Infomax';
                case 'MMER'
                    theRotationName = 'McCammon minimum entropy ratio';
                case 'UNRT'
                    theRotationName = 'none';
                case 'VMAX'
                    theRotationName = 'Varimax';
                case 'QMAX'
                    theRotationName = 'Quartimax';
                case 'PMAX'
                    theRotationName = 'Promax';
                case 'VOMN'
                    theRotationName = 'Variable Oblimin';
                otherwise
                    beep()
                    disp('Oops')
                    theRotationName = 'oops';
            end;
            
            uicontrol('Style','text','HorizontalAlignment','left','String', 'Rotation',...
                'Position',[100 windowHeight-180 150 20]);
            uicontrol('Style','text','HorizontalAlignment','left','String', theRotationName,...
                'Position',[100 windowHeight-200 150 20]);
            
            switch EPoverview.workData.pca.MAT_TYPE
                case 'SCP'
                    theMat='Cross-Product';
                case 'COV'
                    theMat='Covariance';
                case 'COR'
                    theMat='Correlation';
                otherwise
                    beep()
                    disp('Oops')
                    theMat='Oops';
            end;
            uicontrol('Style','text','HorizontalAlignment','left','String', 'Matrix Type',...
                'Position',[100 windowHeight-220 150 20]);
            uicontrol('Style','text','HorizontalAlignment','left','String', theMat,...
                'Position',[100 windowHeight-240 150 20]);
            
            switch EPoverview.workData.pca.LOADING
                case 'K'
                    theLoadings='Kaiser';
                case 'C'
                    theLoadings='Covariance';
                case 'N'
                    theLoadings='None';
                case 'W'
                    theLoadings='Cureton-Mulaik';
                otherwise
                    beep()
                    disp('Oops')
                    theLoadings='Oops';
            end;
            if strcmp(theRotationName,'Infomax')
                theLoadings='None';
            end;
            uicontrol('Style','text','HorizontalAlignment','left','String', 'Loading Weighting',...
                'Position',[100 windowHeight-260 150 20]);
            uicontrol('Style','text','HorizontalAlignment','left','String', theLoadings,...
                'Position',[100 windowHeight-280 150 20]);
            
            uicontrol('Style','text','HorizontalAlignment','left','String', 'Rotation Option',...
                'Position',[100 windowHeight-300 150 20]);
            uicontrol('Style','text','HorizontalAlignment','left','String', num2str(EPoverview.workData.pca.RotOpt),...
                'Position',[100 windowHeight-320 150 20]);
            
            uicontrol('Style','text','HorizontalAlignment','left','String', 'Factors Retained',...
                'Position',[100 windowHeight-340 150 20]);
            uicontrol('Style','text','HorizontalAlignment','left','String', num2str(EPoverview.workData.pca.numFacs),...
                'Position',[100 windowHeight-360 150 20]);
            
            uicontrol('Style','text','HorizontalAlignment','left','String', 'Total Variance',...
                'Position',[100 windowHeight-380 150 20]);
            uicontrol('Style','text','HorizontalAlignment','left','String', num2str(EPoverview.workData.pca.facVarTot),...
                'Position',[100 windowHeight-400 150 20]);
            
            if isfield(EPoverview.workData.pca,'PCAmode2')
                uicontrol('Style','text','HorizontalAlignment','left','String', 'Second Rotation',...
                    'Position',[300 windowHeight-120 150 20]);
                switch EPoverview.workData.pca.PCAmode2
                    case 'spat'
                        theMode='spatial';
                    case 'temp'
                        theMode='temporal';
                    otherwise
                        beep()
                        disp('Oops')
                        theMode='Oops';
                end;
                
                uicontrol('Style','text','HorizontalAlignment','left','String', 'Mode',...
                    'Position',[300 windowHeight-140 150 20]);
                uicontrol('Style','text','HorizontalAlignment','left','String', theMode,...
                    'Position',[300 windowHeight-160 150 20]);
                
                switch EPoverview.workData.pca.ROTATION2
                    case 'QMIN'
                        theRotationName = 'Quartimin';
                    case 'OMIN'
                        theRotationName = 'Oblimin';
                    case 'CRFE'
                        theRotationName = 'Crawford-Ferguson';
                    case 'MINE'
                        theRotationName = 'minimum entropy';
                    case 'IPSC'
                        theRotationName = 'Bentler''s invariant pattern simplicity criterion';
                    case 'TIIC'
                        theRotationName = 'Comrey''s tandem II criterion';
                    case 'GMIN'
                        theRotationName = 'geomin';
                    case 'IMAX'
                        theRotationName = 'Infomax';
                    case 'MMER'
                        theRotationName = 'McCammon minimum entropy ratio';
                    case 'UNRT'
                        theRotationName = 'none';
                    case 'VMAX'
                        theRotationName = 'Varimax';
                    case 'QMAX'
                        theRotationName = 'Quartimax';
                    case 'PMAX'
                        theRotationName = 'Promax';
                    case 'VOMN'
                        theRotationName = 'Variable Oblimin';
                    otherwise
                        beep()
                        disp('Oops')
                        theRotationName = 'oops';
                end;
                
                uicontrol('Style','text','HorizontalAlignment','left','String', 'Rotation',...
                    'Position',[300 windowHeight-180 150 20]);
                uicontrol('Style','text','HorizontalAlignment','left','String', theRotationName,...
                    'Position',[300 windowHeight-200 150 20]);
                
                switch EPoverview.workData.pca.MAT_TYPE2
                    case 'SCP'
                        theMat='Cross-Product';
                    case 'COV'
                        theMat='Covariance';
                    case 'COR'
                        theMat='Correlation';
                    otherwise
                        beep()
                        disp('Oops')
                        theMat='Oops';
                end;
                uicontrol('Style','text','HorizontalAlignment','left','String', 'Matrix Type',...
                    'Position',[300 windowHeight-220 150 20]);
                uicontrol('Style','text','HorizontalAlignment','left','String', theMat,...
                    'Position',[300 windowHeight-240 150 20]);
                
                switch EPoverview.workData.pca.LOADING2
                    case 'K'
                        theLoadings='Kaiser';
                    case 'C'
                        theLoadings='Covariance';
                    case 'N'
                        theLoadings='None';
                    case 'W'
                        theLoadings='Cureton-Mulaik';
                    otherwise
                        beep()
                        disp('Oops')
                        theLoadings='Oops';
                end;
                if strcmp(theRotationName,'Infomax')
                    theLoadings='None';
                end;
                uicontrol('Style','text','HorizontalAlignment','left','String', 'Loading Weighting',...
                    'Position',[300 windowHeight-260 150 20]);
                uicontrol('Style','text','HorizontalAlignment','left','String', theLoadings,...
                    'Position',[300 windowHeight-280 150 20]);
                
                uicontrol('Style','text','HorizontalAlignment','left','String', 'Rotation Option',...
                    'Position',[300 windowHeight-300 150 20]);
                uicontrol('Style','text','HorizontalAlignment','left','String', num2str(EPoverview.workData.pca.RotOpt2),...
                    'Position',[300 windowHeight-320 150 20]);
                
                uicontrol('Style','text','HorizontalAlignment','left','String', 'Factors Retained',...
                    'Position',[300 windowHeight-340 150 20]);
                uicontrol('Style','text','HorizontalAlignment','left','String', num2str(EPoverview.workData.pca.numFacs2),...
                    'Position',[300 windowHeight-360 150 20]);
                
                uicontrol('Style','text','HorizontalAlignment','left','String', 'Total Variance',...
                    'Position',[300 windowHeight-380 150 20]);
                uicontrol('Style','text','HorizontalAlignment','left','String', num2str(sum(EPoverview.workData.pca.facVar.*EPoverview.workData.pca.facVarTotST)),...
                    'Position',[300 windowHeight-400 150 20]);
                
                PCAdata=[EPoverview.workData.pca.facVarTotST; EPoverview.workData.pca.facVar.*EPoverview.workData.pca.facVarTotST]';
                tableNames={'Factor Var','Total Var'};
                
                tableData=num2cell(PCAdata);
                
                columnEditable =  repmat(false,1,size(PCAdata,2));
                
                ColumnFormat = repmat([],1,size(PCAdata,2));
                
                if ~EPmain.noTable
                    try
                        EPoverview.handles.PCA.hTable = uitable('Data',tableData,'ColumnName',tableNames,...
                            'ColumnEditable', columnEditable, 'ColumnFormat', ColumnFormat,...
                            'Position',[300 windowHeight-600 200 200]);
                    catch
                        EPmain.noTable=1;
                    end;
                end;
                
                if EPmain.noTable
                    initializeEPtable=0;
                    if isfield(EPtable,'edit')
                        if ~isfield(EPtable.edit,'PCA')
                            initializeEPtable=1;
                        end
                    else
                        initializeEPtable=1;
                    end;
                    if initializeEPtable
                        for i=4:size(tableData,2)
                            ColumnFormat{i}='char';
                        end
                        
                        EPtable.edit.PCA.Xloc=300;
                        EPtable.edit.PCA.Yloc=windowHeight-420;
                        EPtable.edit.PCA.nRows=min(25,size(tableData,1));
                        EPtable.edit.PCA.nCols=min(5,size(tableData,2));
                        EPtable.edit.PCA.colWidth=90;
                        EPtable.edit.PCA.rowNames=[1:size(tableData,1)];
                        EPtable.edit.PCA.rowNames=cellstr(num2str(EPtable.edit.PCA.rowNames'));
                        EPtable.edit.PCA.tableNames=tableNames;
                        EPtable.edit.PCA.tableData=tableData;
                        EPtable.edit.PCA.columnEditable=columnEditable;
                        EPtable.edit.PCA.ColumnFormat=ColumnFormat;
                        EPtable.edit.PCA.selRows=[1:EPtable.edit.PCA.nRows];
                        EPtable.edit.PCA.selCols=[1:EPtable.edit.PCA.nCols];
                        EPtable.edit.PCA.endHeaderCallback='ep_editData(''start'');';
                        EPtable.edit.PCA.endTableCallback='ep_editData(''start'');';
                        if size(tableData,1) <= EPtable.edit.PCA.nRows
                            EPtable.edit.PCA.chgRows=repmat(false,1,EPtable.edit.PCA.nRows);
                        else
                            EPtable.edit.PCA.chgRows=repmat(true,1,EPtable.edit.PCA.nRows);
                        end;
                        if length(tableNames) <= EPtable.edit.PCA.nCols
                            EPtable.edit.PCA.chgCols=repmat(false,1,EPtable.edit.PCA.nCols);
                        else
                            EPtable.edit.PCA.chgCols=repmat(true,1,EPtable.edit.PCA.nCols);
                        end;
                        
                        EPtable.edit.PCA.name='edit.PCA';
                    end
                    
                    EPtable.edit.PCA=ep_noTable(EPtable.edit.PCA);
                end;
                
            end;
            
        else
            
            %prepare contents of the table based on the PCA setting
            
            switch EPoverview.PCA.mode
                case 'FacPat'
                    PCAdata=EPoverview.workData.pca.FacPat;
                    tableNames=EPoverview.workData.pca.facNames;
                case 'FacStr'
                    PCAdata=EPoverview.workData.pca.FacStr;
                    tableNames=EPoverview.workData.pca.facNames;
                case 'FacScr'
                    PCAdata=EPoverview.workData.pca.FacScr;
                    tableNames=EPoverview.workData.pca.facNames;
                case 'FacCof'
                    PCAdata=EPoverview.workData.pca.FacCof;
                    tableNames=EPoverview.workData.pca.facNames;
                case 'FacCor'
                    PCAdata=EPoverview.workData.pca.FacCor;
                    tableNames=EPoverview.workData.pca.facNames;
                case 'FacPatST'
                    PCAdata=EPoverview.workData.pca.FacPatST;
                    tableNames=EPoverview.workData.pca.facNames2;
                case 'FacStrST'
                    PCAdata=EPoverview.workData.pca.FacStrST;
                    tableNames=EPoverview.workData.pca.facNames2;
                case 'FacScrST'
                    PCAdata=EPoverview.workData.pca.FacScrST;
                    tableNames=EPoverview.workData.pca.facNames2;
                case 'FacCofST'
                    PCAdata=EPoverview.workData.pca.FacCofST;
                    tableNames=EPoverview.workData.pca.facNames2;
                case 'FacCorST'
                    PCAdata=EPoverview.workData.pca.FacCorST;
                    tableNames=EPoverview.workData.pca.facNames2;
            end;
            
            
            tableData=num2cell(PCAdata);
            
            columnEditable =  repmat(false,1,size(PCAdata,2));
            
            ColumnFormat = repmat([],1,size(PCAdata,2));
            
            if ~EPmain.noTable
                try
                    EPoverview.handles.PCA.hTable = uitable('Data',tableData,'ColumnName',tableNames,...
                        'ColumnEditable', columnEditable, 'ColumnFormat', ColumnFormat,...
                        'Position',[100 windowHeight-600 500 500]);
                catch
                    EPmain.noTable=1;
                end;
            end;
            
            if EPmain.noTable
                initializeEPtable=0;
                if isfield(EPtable,'edit')
                    if ~isfield(EPtable.edit,'PCA')
                        initializeEPtable=1;
                    end
                else
                    initializeEPtable=1;
                end;
                if initializeEPtable
                    for i=4:size(tableData,2)
                        ColumnFormat{i}='char';
                    end
                    
                    EPtable.edit.PCA.Xloc=100;
                    EPtable.edit.PCA.Yloc=windowHeight-120;
                    EPtable.edit.PCA.nRows=min(25,size(tableData,1));
                    EPtable.edit.PCA.nCols=min(5,size(tableData,2));
                    EPtable.edit.PCA.colWidth=90;
                    EPtable.edit.PCA.rowNames=[1:size(tableData,1)];
                    EPtable.edit.PCA.rowNames=cellstr(num2str(EPtable.edit.PCA.rowNames'));
                    EPtable.edit.PCA.tableNames=tableNames;
                    EPtable.edit.PCA.tableData=tableData;
                    EPtable.edit.PCA.columnEditable=columnEditable;
                    EPtable.edit.PCA.ColumnFormat=ColumnFormat;
                    EPtable.edit.PCA.selRows=[1:EPtable.edit.PCA.nRows];
                    EPtable.edit.PCA.selCols=[1:EPtable.edit.PCA.nCols];
                    EPtable.edit.PCA.endHeaderCallback='ep_editData(''start'');';
                    EPtable.edit.PCA.endTableCallback='ep_editData(''start'');';
                    if size(tableData,1) <= EPtable.edit.PCA.nRows
                        EPtable.edit.PCA.chgRows=repmat(false,1,EPtable.edit.PCA.nRows);
                    else
                        EPtable.edit.PCA.chgRows=repmat(true,1,EPtable.edit.PCA.nRows);
                    end;
                    if length(tableNames) <= EPtable.edit.PCA.nCols
                        EPtable.edit.PCA.chgCols=repmat(false,1,EPtable.edit.PCA.nCols);
                    else
                        EPtable.edit.PCA.chgCols=repmat(true,1,EPtable.edit.PCA.nCols);
                    end;
                    
                    EPtable.edit.PCA.name='edit.PCA';
                end
                
                EPtable.edit.PCA=ep_noTable(EPtable.edit.PCA);
            end;
            
        end;
        
        EPoverview.handles.PCA.hExport = uicontrol('Style', 'pushbutton', 'String', 'Export',...
            'Position', [20 windowHeight-460 60 30], 'Callback', 'ep_editData(''exportPCA'');');
        
        if EPmain.noTable
            callBackEnd=['EPtable.edit=rmfield(EPtable.edit,''PCA'');','ep_editData(''start'');'];
        else
            callBackEnd=['ep_editData(''start'');'];
        end;
        
        
        if isfield(EPoverview.workData,'pca')
            EPoverview.handles.PCA.Summary = uicontrol('Style', 'pushbutton', 'String', 'Summary',...
                'Position', [100 windowHeight-650 65 30], 'Callback', ['global EPoverview EPtable;','EPoverview.PCA.mode=''Summary'';',callBackEnd]);
            if isfield(EPoverview.workData.pca,'FacPat')
                EPoverview.handles.PCA.FacPat = uicontrol('Style', 'pushbutton', 'String', 'FacPat',...
                    'Position', [165 windowHeight-650 65 30], 'Callback', ['global EPoverview EPtable;','EPoverview.PCA.mode=''FacPat'';',callBackEnd]);
            end;
            if isfield(EPoverview.workData.pca,'FacStr')
                EPoverview.handles.PCA.FacStr = uicontrol('Style', 'pushbutton', 'String', 'FacStr',...
                    'Position', [230 windowHeight-650 65 30], 'Callback', ['global EPoverview EPtable;','EPoverview.PCA.mode=''FacStr'';',callBackEnd]);
            end;
            if isfield(EPoverview.workData.pca,'FacScr')
                EPoverview.handles.PCA.FacScr = uicontrol('Style', 'pushbutton', 'String', 'FacScr',...
                    'Position', [295 windowHeight-650 65 30], 'Callback', ['global EPoverview EPtable;','EPoverview.PCA.mode=''FacScr'';',callBackEnd]);
            end;
            if isfield(EPoverview.workData.pca,'FacCof')
                EPoverview.handles.PCA.FacCof = uicontrol('Style', 'pushbutton', 'String', 'FacCof',...
                    'Position', [360 windowHeight-650 65 30], 'Callback', ['global EPoverview EPtable;','EPoverview.PCA.mode=''FacCof'';',callBackEnd]);
            end;
            if isfield(EPoverview.workData.pca,'FacCor')
                EPoverview.handles.PCA.FacCor = uicontrol('Style', 'pushbutton', 'String', 'FacCor',...
                    'Position', [425 windowHeight-650 65 30], 'Callback', ['global EPoverview EPtable;','EPoverview.PCA.mode=''FacCor'';',callBackEnd]);
            end;
            if isfield(EPoverview.workData.pca,'FacPatST')
                EPoverview.handles.PCA.FacPatST = uicontrol('Style', 'pushbutton', 'String', 'FacPatST',...
                    'Position', [165 windowHeight-680 65 30], 'Callback', ['global EPoverview EPtable;','EPoverview.PCA.mode=''FacPatST'';',callBackEnd]);
            end;
            if isfield(EPoverview.workData.pca,'FacStrST')
                EPoverview.handles.PCA.FacStrST = uicontrol('Style', 'pushbutton', 'String', 'FacStrST',...
                    'Position', [230 windowHeight-680 65 30], 'Callback', ['global EPoverview EPtable;','EPoverview.PCA.mode=''FacStrST'';',callBackEnd]);
            end;
            if isfield(EPoverview.workData.pca,'FacScrST')
                EPoverview.handles.PCA.FacScrST = uicontrol('Style', 'pushbutton', 'String', 'FacScrST',...
                    'Position', [295 windowHeight-680 65 30], 'Callback', ['global EPoverview EPtable;','EPoverview.PCA.mode=''FacScrST'';',callBackEnd]);
            end;
            if isfield(EPoverview.workData.pca,'FacCofST')
                EPoverview.handles.PCA.FacCofST = uicontrol('Style', 'pushbutton', 'String', 'FacCofST',...
                    'Position', [360 windowHeight-680 65 30], 'Callback', ['global EPoverview EPtable;','EPoverview.PCA.mode=''FacCofST'';',callBackEnd]);
            end;
            if isfield(EPoverview.workData.pca,'FacCorST')
                EPoverview.handles.PCA.FacCorST = uicontrol('Style', 'pushbutton', 'String', 'FacCorST',...
                    'Position', [425 windowHeight-680 65 30], 'Callback', ['global EPoverview EPtable;','EPoverview.PCA.mode=''FacCorST'';',callBackEnd]);
            end;
        end;
        
        switch EPoverview.PCA.mode
            case 'Summary'
                set(EPoverview.handles.PCA.Summary,'ForegroundColor','blue');
            case 'FacPat'
                set(EPoverview.handles.PCA.FacPat,'ForegroundColor','blue');
            case 'FacStr'
                set(EPoverview.handles.PCA.FacStr,'ForegroundColor','blue');
            case 'FacScr'
                set(EPoverview.handles.PCA.FacScr,'ForegroundColor','blue');
            case 'FacCof'
                set(EPoverview.handles.PCA.FacCof,'ForegroundColor','blue');
            case 'FacCor'
                set(EPoverview.handles.PCA.FacCor,'ForegroundColor','blue');
            case 'FacPatST'
                set(EPoverview.handles.PCA.FacPatST,'ForegroundColor','blue');
            case 'FacStrST'
                set(EPoverview.handles.PCA.FacStrST,'ForegroundColor','blue');
            case 'FacScrST'
                set(EPoverview.handles.PCA.FacScrST,'ForegroundColor','blue');
            case 'FacCofST'
                set(EPoverview.handles.PCA.FacCofST,'ForegroundColor','blue');
            case 'FacCorST'
                set(EPoverview.handles.PCA.FacCorST,'ForegroundColor','blue');
        end;
        
    case 'rename'
        if length(varargin) < 1
            msg{1}='No cell specified for renaming.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        if ~isa(varargin{2},'char')
            msg{1}='Old name needs to be specified.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        EPoverview.lastData=EPoverview.workData;
        
        oldName=varargin{2};
        
        [uniqueCells m order]=unique(EPoverview.workData.cellNames,'first');
        
        theCell=order(strcmp(oldName,uniqueCells)); %the cell number as ordered alphabetically
        
        newName=get(EPoverview.handles.cells.hCellName(theCell),'String');
        
        EPoverview.workData.cellNames(strcmp(oldName,EPoverview.workData.cellNames))={newName};
        
        [err]=ep_checkEPfile(EPoverview.workData);
        
        ep_editData('start');
        
    case 'renameDataName'
        
        EPoverview.lastData=EPoverview.workData;
        
        newName=get(EPoverview.handles.overview.hDataName,'String');
        
        EPoverview.workData.dataName=newName;
        
        [err]=ep_checkEPfile(EPoverview.workData);
        
        ep_editData('start');
        
    case 'renameExp'
        
        EPoverview.lastData=EPoverview.workData;
        
        newName=get(EPoverview.handles.overview.hExpName,'String');
        
        EPoverview.workData.ename=newName;
        
        [err]=ep_checkEPfile(EPoverview.workData);
        
        ep_editData('start');
        
    case 'montage'
        
        EPoverview.lastData=EPoverview.workData;
        
        newMontage=get(EPoverview.handles.overview.hMontage,'String');
        
        EPoverview.workData.montage=newMontage;
        
        [err]=ep_checkEPfile(EPoverview.workData);
        
        ep_editData('start');
        
    case 'Fs'
        
        EPoverview.lastData=EPoverview.workData;
        
        newFs=str2num(get(EPoverview.handles.overview.hFs,'String'));
        
        EPoverview.workData.Fs=newFs;
        
        if strcmp(EPoverview.workData.dataType,'continuous')
            numEpochs=floor(size(EPoverview.workData.data,2)/EPoverview.workData.Fs); %excess time points are tacked onto final epoch
            if numEpochs == 0
                numEpochs =1;
            end;
            EPoverview.workData.analysis.blinkTrial=zeros(numSubs,numEpochs);
            EPoverview.workData.analysis.saccadeTrial=zeros(numSubs,numEpochs);
            EPoverview.workData.analysis.saccadeOnset=zeros(numSubs,numEpochs);
            EPoverview.workData.analysis.moveTrial=zeros(numSubs,numEpochs);
            EPoverview.workData.analysis.badTrials=zeros(numSubs,numEpochs);
            EPoverview.workData.analysis.badChans=zeros(numSubs,numEpochs,numChan);
            disp('Resetting artifact correction fields to zero since length of one-second epochs has been changed.');
        end;
        
        [err]=ep_checkEPfile(EPoverview.workData);
        
        ep_editData('start');
        
    case 'baseline'
        
        EPoverview.lastData=EPoverview.workData;
        
        if strcmp(EPoverview.mode,'overview')
            newBaseline=(str2num(get(EPoverview.handles.overview.hBaseline,'String')))/(1000/EPoverview.workData.Fs);
        elseif strcmp(EPoverview.mode,'samples')
            newBaseline=(str2num(get(EPoverview.handles.samples.hBaseline,'String')))/(1000/EPoverview.workData.Fs);
        else
            msg{1}='Baseline change not supported from this pane.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        EPoverview.workData.baseline=newBaseline;
        
        EPoverview.workData.timeNames=(((0:length(EPoverview.workData.timeNames)-1)-newBaseline)*(1000/EPoverview.workData.Fs));
        
        [err]=ep_checkEPfile(EPoverview.workData);
        
        ep_editData('start');
        
    case 'ced'
        
        EPoverview.lastData=EPoverview.workData;
        
        newCed=get(EPoverview.handles.overview.hCed,'String');
        
        EPoverview.workData.ced=newCed;
        
        [err]=ep_checkEPfile(EPoverview.workData);
        
        ep_editData('start');
        
        
    case 'editSubs'
        
        EPoverview.lastData=EPoverview.workData;
        
        if EPmain.noTable
            tableData=EPtable.edit.subjects.tableData;
            weights=EPtable.edit.subjects.tableData{:,3};
            for i=1:size(EPtable.edit.subjects.tableData,1)
                if isnumeric(EPtable.edit.subjects.tableData{i,2})
                    tableData{i,2}=cast(tableData{i,2},'logical'); %workaround for Matlab bug
                end;
            end;
        else
            tableData=get(EPoverview.handles.subjects.hTable,'Data');
        end;
        
        %subjects reordered?
        if any(cell2mat(tableData(:,1))-[1:size(tableData,1)]')
            [tableData,index] = sortrows(tableData,1);
            
            EPoverview.workData.data=EPoverview.workData.data(:,:,:,index,:);
            if ~isempty(EPoverview.workData.facData)
                EPoverview.workData.facData=EPoverview.workData.facData(:,:,:,index,:);
            end;
            if isfield(EPoverview.workData,'noise')
                EPoverview.workData.noise=EPoverview.workData.noise(:,:,:,index,:);
            end;
            if isfield(EPoverview.workData,'std')
                EPoverview.workData.std=EPoverview.workData.std(:,:,:,index,:);
            end;
            EPoverview.workData.subNames=EPoverview.workData.subNames(index);
            EPoverview.workData.subTypes=EPoverview.workData.subTypes(index);
            EPoverview.workData.subjectSpecs=EPoverview.workData.subjectSpecs(index,:);
            EPoverview.workData.events=EPoverview.workData.events(index,:);
            
            EPoverview.workData.analysis.blinkTrial= EPoverview.workData.analysis.blinkTrial(index,:);
            EPoverview.workData.analysis.saccadeTrial= EPoverview.workData.analysis.saccadeTrial(index,:);
            EPoverview.workData.analysis.saccadeOnset= EPoverview.workData.analysis.saccadeOnset(index,:);
            EPoverview.workData.analysis.moveTrial= EPoverview.workData.analysis.moveTrial(index,:);
            EPoverview.workData.analysis.badTrials= EPoverview.workData.analysis.badTrials(index,:);
            EPoverview.workData.analysis.badChans= EPoverview.workData.analysis.badChans(index,:,:);
            
            reordered=1;
            
        else
            EPoverview.subjects.select=cell2mat(tableData(:,2));
            EPoverview.subjects.weights=cell2mat(tableData(:,3));
            EPoverview.workData.subNames =tableData(:,4);
            EPoverview.workData.subTypes =tableData(:,5);
            EPoverview.workData.subjectSpecs =tableData(:,6:5+numSpecs);
            reordered=0;
            set(EPoverview.handles.subjects.hWeightsSum,'String',sprintf('%f',sum(EPoverview.subjects.weights)));
        end;
        
        [err]=ep_checkEPfile(EPoverview.workData);
        
        if reordered
            EPtable=[];
            ep_editData('start');
        end;
        
        
    case 'deleteSubs'
        if sum(EPoverview.subjects.select) ==0
            msg{1}='No subjects specified for deletion.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        if sum(EPoverview.subjects.select) ==length(EPoverview.workData.subNames)
            msg{1}='There would be nothing left.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        EPoverview.lastData=EPoverview.workData;
        
        keepSubjects=not(EPoverview.subjects.select);
        
        EPoverview.workData.avgNum=EPoverview.workData.avgNum(keepSubjects,:);
        EPoverview.workData.subNum=EPoverview.workData.subNum(keepSubjects,:);
        EPoverview.workData.data=EPoverview.workData.data(:,:,:,keepSubjects,:);
        if ~isempty(EPoverview.workData.facData)
            EPoverview.workData.facData=EPoverview.workData.facData(:,:,:,keepSubjects,:);
        end;
        if isfield(EPoverview.workData,'noise')
            EPoverview.workData.noise=EPoverview.workData.noise(:,:,:,keepSubjects,:);
        end;
        if isfield(EPoverview.workData,'std')
            EPoverview.workData.std=EPoverview.workData.std(:,:,:,keepSubjects,:);
        end;
        EPoverview.workData.subNames=EPoverview.workData.subNames(keepSubjects);
        EPoverview.workData.subTypes=EPoverview.workData.subTypes(keepSubjects);
        EPoverview.workData.subjectSpecs=EPoverview.workData.subjectSpecs(keepSubjects,:);
        EPoverview.workData.events=EPoverview.workData.events(keepSubjects,:);
        
        EPoverview.workData.analysis.blinkTrial= EPoverview.workData.analysis.blinkTrial(keepSubjects,:);
        EPoverview.workData.analysis.saccadeTrial= EPoverview.workData.analysis.saccadeTrial(keepSubjects,:);
        EPoverview.workData.analysis.saccadeOnset= EPoverview.workData.analysis.saccadeOnset(keepSubjects,:);
        EPoverview.workData.analysis.moveTrial= EPoverview.workData.analysis.moveTrial(keepSubjects,:);
        EPoverview.workData.analysis.badTrials= EPoverview.workData.analysis.badTrials(keepSubjects,:);
        EPoverview.workData.analysis.badChans= EPoverview.workData.analysis.badChans(keepSubjects,:,:);
        
        EPoverview.subjects.select=repmat(false,length(EPoverview.workData.subNames),1);
        EPtable=[];
        
        [err]=ep_checkEPfile(EPoverview.workData);
        
        ep_editData('start');
        
    case 'addSubs'
        
        if EPmain.noTable
            tableData=EPtable.edit.subjects.tableData;
        else
            tableData=get(EPoverview.handles.subjects.hTable,'Data');
        end;
        weights=cell2mat(tableData(:,3));
        
        if ~any(weights)
            msg{1}='No weights specified for forming new subject average.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        EPoverview.lastData=EPoverview.workData;
        
        totalWeight=sum(abs(weights));
        newSubData=zeros(size(EPoverview.workData.data,1),size(EPoverview.workData.data,2),length(EPoverview.workData.cellNames),1,max(numSGLfacs,1));
        for i=find(weights)'
            theWeight=weights(i);
            newSubData=newSubData+(theWeight/totalWeight)*EPoverview.workData.data(:,:,:,i,:);
        end;
        EPoverview.workData.data(:,:,:,end+1,:)=newSubData;
        if ~isempty(EPoverview.workData.facData)
            newSubData=zeros(size(EPoverview.workData.facData,1),size(EPoverview.workData.facData,2),length(EPoverview.workData.cellNames),1,max(numCMBfacs,1));
            for i=find(weights)'
                theWeight=weights(i);
                newSubData=newSubData+(theWeight/totalWeight)*EPoverview.workData.facData(:,:,:,i,:);
            end;
            EPoverview.workData.facData(:,:,:,end+1,:)=newSubData;
        end
        if isfield(EPoverview.workData,'noise')
            newSubData=zeros(length(EPoverview.workData.chanNames),length(EPoverview.workData.timeNames),length(EPoverview.workData.cellNames),1);
            for i=find(weights)'
                theWeight=weights(i);
                newSubData=newSubData+(theWeight/totalWeight)*EPoverview.workData.noise(:,:,:,i,:);
            end;
            EPoverview.workData.noise(:,:,:,end+1,:)=newSubData;
        end;
        if isfield(EPoverview.workData,'std')
            newSubData=zeros(length(EPoverview.workData.chanNames),length(EPoverview.workData.timeNames),length(EPoverview.workData.cellNames),1);
            for i=find(weights)'
                theWeight=weights(i);
                newSubData=newSubData+(theWeight/totalWeight)*EPoverview.workData.std(:,:,:,i,:);
            end;
            EPoverview.workData.std(:,:,:,end+1,:)=newSubData;
        end;
        
        for i=1:length(EPoverview.workData.cellNames)
            if ~isempty(find(EPoverview.workData.avgNum(find(weights),i) == -1))
                EPoverview.workData.avgNum(length(EPoverview.workData.subNames)+1,i) = -1; %bad number
            elseif ~isempty(find(EPoverview.workData.avgNum(find(weights),i) == 0))
                EPoverview.workData.avgNum(length(EPoverview.workData.subNames)+1,i) = 0; %unknown number
            else
                EPoverview.workData.avgNum(length(EPoverview.workData.subNames)+1,i)=sum(EPoverview.workData.avgNum(find(weights),i));
            end
            
            if ~isempty(find(EPoverview.workData.subNum(find(weights),i) == -1))
                EPoverview.workData.subNum(length(EPoverview.workData.subNames)+1,i) = -1; %bad number
            elseif ~isempty(find(EPoverview.workData.subNum(find(weights),i) == 0))
                EPoverview.workData.subNum(length(EPoverview.workData.subNames)+1,i) = 0; %unknown number
            else
                EPoverview.workData.subNum(length(EPoverview.workData.subNames)+1,i)=sum(EPoverview.workData.subNum(find(weights),i));
            end
            
            %since this function is not allowed for single subject files, can assume are averaged data
            EPoverview.workData.analysis.blinkTrial(length(EPoverview.workData.subNames)+1,i)= sum(EPoverview.workData.analysis.blinkTrial(find(weights),i));
            EPoverview.workData.analysis.saccadeTrial(length(EPoverview.workData.subNames)+1,i)= sum(EPoverview.workData.analysis.saccadeTrial(find(weights),i));
            EPoverview.workData.analysis.saccadeOnset(length(EPoverview.workData.subNames)+1,i)= sum(EPoverview.workData.analysis.saccadeOnset(find(weights),i));
            EPoverview.workData.analysis.moveTrial(length(EPoverview.workData.subNames)+1,i)= sum(EPoverview.workData.analysis.moveTrial(find(weights),i));
            EPoverview.workData.analysis.badTrials(length(EPoverview.workData.subNames)+1,i)= sum(EPoverview.workData.analysis.badTrials(find(weights),i));
            EPoverview.workData.analysis.badChans(length(EPoverview.workData.subNames)+1,i,:)= sum(EPoverview.workData.analysis.badChans(find(weights),i,:));
        end;
        
        %make sure new cell doesn't duplicate existing subject names
        newName=[];
        for i=1:length(weights)
            if weights(i) ~= 0
                if weights(i) > 0
                    theSign='+';
                else
                    theSign='-';
                end;
                newName=[newName theSign EPoverview.workData.subNames{i}];
            end;
        end;
        
        comb=1;
        while any(strcmp([newName num2str(comb)], EPoverview.workData.subNames))
            comb=comb+1;
        end;
        if comb > 1
            newName=[newName '_' num2str(comb)];
        end;
        
        EPoverview.workData.subNames{end+1}=newName;
        
        if length(find(weights)) > 1
            EPoverview.workData.subTypes{end+1}='GAV';
        else
            EPoverview.workData.subTypes{end+1}=EPoverview.workData.subTypes(weights ~= 0);
        end;
        EPoverview.workData.subjectSpecs(end+1,:)=cell(1,size(EPoverview.workData.subjectSpecs,2));
        EPoverview.workData.events(end+1,:)=cell(1,size(EPoverview.workData.events,2));
        
        EPoverview.subjects.weights=repmat(0,length(EPoverview.workData.subNames),1);
        EPoverview.subjects.select=repmat(false,length(EPoverview.workData.subNames),1);
        
        [err]=ep_checkEPfile(EPoverview.workData);
        
        EPtable=[];
        ep_editData('start');
        
    case 'clearSubWeights'
        
        EPoverview.subjects.weights=repmat(0,length(EPoverview.workData.subNames),1);
        EPoverview.subjects.select=repmat(false,length(EPoverview.workData.subNames),1);
        
        ep_editData('start');
        
    case 'allSubs'
        
        EPoverview.subjects.weights=repmat(1,length(EPoverview.workData.subNames),1);
        EPoverview.subjects.select=repmat(true,length(EPoverview.workData.subNames),1);
        
        ep_editData('start');
        
    case 'appendSubs'
        
        disp('Making assumption that file format is same as that of the original file.');
        importFormat=EPoverview.workData.fileFormat;
        fileType=EPoverview.workData.dataType;
        if any(strcmp(fileType,{'continuous','single_trial'}))
            msg{1}='It is not possible to append additional subjects to a ''continuous'' or ''single trial'' type of file.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        inArg{1}='format';
        inArg{2}=importFormat;
        inArg{3}='type';
        inArg{4}=fileType;
        
        [newData]=ep_readData(inArg);
        
        if isempty(newData)
            return
        end;
        
        if EPmain.noTable
            tableData=EPtable.edit.subjects.tableData;
        else
            tableData=get(EPoverview.handles.subjects.hTable,'Data');
        end;
        
        if ~strcmp(fileType,newData.dataType)
            msg{1}='The file types do not match.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        if length(EPoverview.workData.cellNames) ~= length(newData.cellNames)
            msg{1}='The numbers of cells do not match.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        if length(EPoverview.workData.chanNames) ~= length(newData.chanNames)
            msg{1}='The numbers of channels do not match.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        if length(EPoverview.workData.timeNames) ~= length(newData.timeNames)
            msg{1}='The numbers of time points do not match.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        if length(EPoverview.workData.facNames) ~= length(newData.facNames)
            msg{1}='The numbers of factors do not match.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        if size(EPoverview.workData.subjectSpecs,2) ~= size(newData.subjectSpecs,2)
            msg{1}='The numbers of subject specs do not match.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        newNames=EPoverview.workData.subNames;
        for i=1:length(newData.subNames)
            if any(strcmp(newData.subNames{i},newNames))
                subNameSuffix='sub001';
                sameName=1;
                suffix=0;
                while sameName
                    sameName=0;
                    for j=1:length(newNames)
                        if strcmp(newNames{j},subNameSuffix)
                            sameName=1;
                        end;
                    end;
                    if sameName
                        suffix=suffix+1;
                        subNameSuffix=['sub' sprintf('%03d',suffix)];
                    end;
                end;
                newData.subNames{i,1}=subNameSuffix;
            end;
            newNames{end+1,1}=newData.subNames{i};
        end;
        
        combinedSubs = [EPoverview.workData.subNames; newData.subNames];
        if length(unique(combinedSubs)) ~= length(combinedSubs)
            msg{1}='Subject names cannot be duplicated.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        EPoverview.lastData=EPoverview.workData;
        
        EPoverview.workData.data(:,:,:,end+1:end+length(newData.subNames),:)=newData.data;
        if ~isempty(EPoverview.workData.facData)
            if ~isempty(newData.facData)
                EPoverview.workData.facData(:,:,:,end+1:end+length(newData.subNames),:)=newData.facData;
            else
                EPoverview.workData.facData(:,:,:,end+1:end+length(newData.subNames),:)= zeros(size(newData.facData,1),size(newData.facData,2),length(newData.cellNames),length(newData.subNames),numCMBfacs);
            end;
        end;
        
        if isfield(EPoverview.workData,'noise')
            if isfield(newData,'noise')
                EPoverview.workData.noise(:,:,:,end+1:end+length(newData.subNames),:)=newData.noise;
            else
                EPoverview.workData.noise(:,:,:,end+1:end+length(newData.subNames),:)= zeros(length(newData.chanNames),length(newData.timeNames),length(newData.cellNames),length(newData.subNames),max(length(newData.facNames),1));
            end;
        end;
        if isfield(EPoverview.workData,'std')
            if isfield(newData,'std')
                EPoverview.workData.std(:,:,:,end+1:end+length(newData.subNames),:)=newData.std;
            else
                EPoverview.workData.std(:,:,:,end+1:end+length(newData.subNames),:)= zeros(length(newData.chanNames),length(newData.timeNames),length(newData.cellNames),length(newData.subNames),max(length(newData.facNames),1));
            end;
        end;
        
        EPoverview.workData.avgNum(end+1:end+length(newData.subNames),:)=newData.avgNum;
        EPoverview.workData.subNum(end+1:end+length(newData.subNames),:)=newData.subNum;
        EPoverview.workData.subNames(end+1:end+length(newData.subNames))=newData.subNames;
        EPoverview.workData.subTypes(end+1:end+length(newData.subNames))=newData.subTypes;
        if ~isempty(EPoverview.workData.subjectSpecs)
            EPoverview.workData.subjectSpecs(end+1:end+length(newData.subNames),:)=newData.subjectSpecs;
        end;
        EPoverview.workData.events(end+1:end+length(newData.subNames),:)=newData.events;
        
        
        EPoverview.workData.analysis.blinkTrial(end+1:end+length(newData.subNames),:)= newData.analysis.blinkTrial;
        EPoverview.workData.analysis.saccadeTrial(end+1:end+length(newData.subNames),:)= newData.analysis.saccadeTrial;
        EPoverview.workData.analysis.saccadeOnset(end+1:end+length(newData.subNames),:)= newData.analysis.saccadeOnset;
        EPoverview.workData.analysis.moveTrial(end+1:end+length(newData.subNames),:)= newData.analysis.moveTrial;
        EPoverview.workData.analysis.badTrials(end+1:end+length(newData.subNames),:)= newData.analysis.badTrials;
        EPoverview.workData.analysis.badChans(end+1:end+length(newData.subNames),:,:)= newData.analysis.badChans;
        
        EPoverview.subjects.select=[EPoverview.subjects.select; repmat(false,length(newData.subNames),1)];
        EPoverview.subjects.weights=[EPoverview.subjects.weights; repmat(0,length(newData.subNames),1)];
        
        [err]=ep_checkEPfile(EPoverview.workData);
        
        EPtable=[];
        ep_editData('start');
        
    case 'exportSubs'
        
        if EPmain.noTable
            tableData=EPtable.edit.subjects.tableData;
        else
            tableData=get(EPoverview.handles.subjects.hTable,'Data');
        end;
        tableData=tableData(:,4:end);
        
        [FileName,PathName,FilterIndex] = uiputfile('*.txt','Export Table');
        
        if isnumeric(FileName)
            if FileName == 0
                msg{1}='No file name specified.';
                [msg]=ep_errorMsg(msg);
                return
            end;
        end;
        
        [pathstr, name, ext] = fileparts(FileName);
        if ~strcmp(ext,'.txt')
            FileName=[FileName '.txt'];
        end;
        
        fid=fopen([PathName FileName],'w');
        
        for i=1:size(tableData,1)
            for j=1:size(tableData,2)
                theCell=tableData{i,j};
                if isnumeric(theCell)
                    fprintf(fid,'%d', theCell);
                else
                    fprintf(fid,'%s', theCell);
                end;
                if j < size(tableData,2)
                    fprintf(fid,'\t');
                end;
            end;
            if i < size(tableData,1)
                fprintf(fid,'\r');
            end;
        end;
        
        ep_editData('start');
        
    case 'addSubSpec'
        
        specName = inputdlg('Name of new spec?');
        pause(1); %pause to avoid crash on Windows due to bug in Matlab 2009
        if ~isempty(specName)
            EPoverview.lastData=EPoverview.workData;
            EPoverview.workData.subjectSpecNames(end+1)=specName;
            
            for i=1:numSubs
                EPoverview.workData.subjectSpecs{i,length(EPoverview.workData.subjectSpecNames)}='';
            end;
        end;
        
        EPtable=[];
        ep_editData('start');
        
    case 'minusSubSpec'
        
        [Selection,ok] = listdlg('PromptString','Choose specs to delete','ListString',EPoverview.workData.subjectSpecNames);
        if ok
            EPoverview.lastData=EPoverview.workData;
            deleteList=setdiff([1:length(EPoverview.workData.subjectSpecNames)],Selection);
            EPoverview.workData.subjectSpecNames=EPoverview.workData.subjectSpecNames(deleteList);
            EPoverview.workData.subjectSpecs=EPoverview.workData.subjectSpecs(:,deleteList);
        end;
        
        EPtable=[];
        ep_editData('start');
        
    case 'editCells'
        
        [uniqueCells waveOrder m]=unique(EPoverview.workData.cellNames,'first'); %waveOrder is the true order of the unique cells (wave numbers of the first occurences)
        [a b cellOrder]=unique(waveOrder,'first'); %cellOrder is the true order of the unique cells (numbered by cells)
        [n xout] = hist(m,length(uniqueCells));
        
        EPoverview.lastData=EPoverview.workData;
        
        if EPmain.noTable
            tableData=EPtable.edit.cells.tableData;
            weights=EPtable.edit.cells.tableData{:,3};
            for i=1:size(EPtable.edit.cells.tableData,1)
                if isnumeric(EPtable.edit.cells.tableData{i,2})
                    tableData{i,2}=cast(tableData{i,2},'logical'); %workaround for Matlab bug
                end;
            end;
        else
            tableData=get(EPoverview.handles.cells.hTable,'Data');
        end;
        
        reordered=0;
        %cells reordered?
        if any(cell2mat(tableData(:,1))-[1:size(tableData,1)]')
            reordered=1;
            [tableData,index] = sortrows(tableData,1);
            
            events2=cell(size(EPoverview.workData.events));
            data2=zeros(size(EPoverview.workData.data));
            if ~isempty(EPoverview.workData.facData)
                facData2=zeros(size(EPoverview.workData.facData));
            end;
            if isfield(EPoverview.workData,'noise')
                noise2=zeros(size(EPoverview.workData.noise));
            end;
            if isfield(EPoverview.workData,'std')
                std2=zeros(size(EPoverview.workData.std));
            end;
            cellNames2=cell(size(EPoverview.workData.cellNames));
            cellTypes2=cell(size(EPoverview.workData.cellTypes));
            avgNum2=zeros(size(EPoverview.workData.avgNum));
            subNum2=zeros(size(EPoverview.workData.subNum));
            trialNames2=cell(size(EPoverview.workData.trialNames));
            trialSpecs2=cell(size(EPoverview.workData.trialSpecs));
            blinkTrial2=zeros(size(EPoverview.workData.analysis.blinkTrial));
            saccadeTrial2=zeros(size(EPoverview.workData.analysis.saccadeTrial));
            saccadeOnset2=zeros(size(EPoverview.workData.analysis.saccadeOnset));
            moveTrial2=zeros(size(EPoverview.workData.analysis.moveTrial));
            badTrials2=zeros(size(EPoverview.workData.analysis.badTrials));
            badChans2=zeros(size(EPoverview.workData.analysis.badChans));
            
            counter=0;
            for i=1:numCells %going through new order of reordered cells
                oldCell=index(i);
                trialIndex=find(m == find(cellOrder==oldCell));
                numTrials=length(trialIndex); %number of trials in the cell
                
                events2(:,counter+1:counter+numTrials)=EPoverview.workData.events(:,trialIndex);
                data2(:,:,counter+1:counter+numTrials,:,:)=EPoverview.workData.data(:,:,trialIndex,:,:);
                if ~isempty(EPoverview.workData.facData)
                    facData2(:,:,counter+1:counter+numTrials,:)=EPoverview.workData.facData(:,:,trialIndex,:,:);
                end;
                if isfield(EPoverview.workData,'noise')
                    noise2(:,:,counter+1:counter+numTrials,:)=EPoverview.workData.noise(:,:,trialIndex,:,:);
                end;
                if isfield(EPoverview.workData,'std')
                    std2(:,:,counter+1:counter+numTrials,:)=EPoverview.workData.std(:,:,trialIndex,:,:);
                end;
                cellNames2(counter+1:counter+numTrials)=EPoverview.workData.cellNames(trialIndex);
                cellTypes2(counter+1:counter+numTrials)=EPoverview.workData.cellTypes(trialIndex);
                avgNum2(:,counter+1:counter+numTrials)=EPoverview.workData.avgNum(:,trialIndex);
                subNum2(:,counter+1:counter+numTrials)=EPoverview.workData.subNum(:,trialIndex);
                if strcmp(EPoverview.workData.dataType,'single_trial')
                    trialNames2(counter+1:counter+numTrials)=EPoverview.workData.trialNames(trialIndex);
                    trialSpecs2(counter+1:counter+numTrials,:)=EPoverview.workData.trialSpecs(trialIndex,:);
                end;
                blinkTrial2(:,counter+1:counter+numTrials)= EPoverview.workData.analysis.blinkTrial(:,trialIndex);
                saccadeTrial2(:,counter+1:counter+numTrials)= EPoverview.workData.analysis.saccadeTrial(:,trialIndex);
                saccadeOnset2(:,counter+1:counter+numTrials)= EPoverview.workData.analysis.saccadeOnset(:,trialIndex);
                moveTrial2(:,counter+1:counter+numTrials)= EPoverview.workData.analysis.moveTrial(:,trialIndex);
                badTrials2(:,counter+1:counter+numTrials)= EPoverview.workData.analysis.badTrials(:,trialIndex);
                badChans2(:,counter+1:counter+numTrials,:)= EPoverview.workData.analysis.badChans(:,trialIndex,:);
                
                counter=counter+numTrials;
            end;
            EPoverview.workData.events=events2;
            EPoverview.workData.data=data2;
            if ~isempty(EPoverview.workData.facData)
                EPoverview.workData.facData=facData2;
            end;
            if isfield(EPoverview.workData,'noise')
                EPoverview.workData.noise=noise2;
            end;
            if isfield(EPoverview.workData,'std')
                EPoverview.workData.std=std2;
            end;
            
            EPoverview.workData.avgNum=avgNum2;
            EPoverview.workData.subNum=subNum2;
            EPoverview.workData.trialNames=trialNames2;
            EPoverview.workData.trialSpecs=trialSpecs2;
            EPoverview.workData.cellNames=cellNames2;
            EPoverview.workData.cellTypes=cellTypes2;
            EPoverview.workData.analysis.blinkTrial=blinkTrial2;
            EPoverview.workData.analysis.saccadeTrial=saccadeTrial2;
            EPoverview.workData.analysis.saccadeOnset=saccadeOnset2;
            EPoverview.workData.analysis.moveTrial=moveTrial2;
            EPoverview.workData.analysis.badTrials=badTrials2;
            EPoverview.workData.analysis.badChans=badChans2;
        else
            
            EPoverview.cells.select=cell2mat(tableData(:,2));
            EPoverview.cells.weights=cell2mat(tableData(:,3));
            
            if ~strcmp(EPoverview.workData.dataType,'single_trial')
                EPoverview.workData.cellTypes=tableData(:,5);
                set(EPoverview.handles.cells.hWeights,'String',sprintf('%f',sum(EPoverview.cells.weights)));
            end;
            
            %make sure new cell doesn't duplicate existing cell names unless single_trial data
            if (length(unique(EPoverview.workData.cellNames)) == length(EPoverview.workData.cellNames))
                for i=1:numCells %going through new order of reordered cells
                    EPoverview.workData.cellNames(m==find(cellOrder==i))=tableData(i,4);
                end;
                %cell name has been changed to be the same as another so merge the trials into one cell
            elseif strcmp(EPoverview.workData.dataType,'single_trial')
                for i=1:numCells %going through new order of reordered cells
                    EPoverview.workData.cellNames(m==find(cellOrder==i))=tableData(i,4);
                end;
                %rename the combined trial names
                trialCount=ones(numCells,1);
                uniqueCells=unique(EPoverview.workData.cellNames,'first');
                for wave=1:length(EPoverview.workData.cellNames)
                    theCell=strcmp(EPoverview.workData.cellNames(wave),uniqueCells);
                    EPoverview.workData.trialNames(wave)=trialCount(theCell);
                    trialCount(theCell)=trialCount(theCell)+1;
                end;
                %group the trials by cell
                [uniqueCellNames m cellIDs]=unique(EPoverview.workData.cellNames); %get number of trials in each cell
                cellNums=hist(cellIDs,unique(cellIDs));
                trialCounter=zeros(numCells,1);
                cellNames2=cell(size(EPoverview.workData.cellNames));
                cellTypes2=cell(size(EPoverview.workData.cellTypes));
                trialNames2=zeros(size(EPoverview.workData.trialNames));
                trialSpecs2=cell(size(EPoverview.workData.trialSpecs));
                events2=cell(size(EPoverview.workData.events));
                avgNum2=zeros(size(EPoverview.workData.avgNum));
                subNum2=zeros(size(EPoverview.workData.subNum));
                data2=zeros(size(EPoverview.workData.data));
                if ~isempty(EPoverview.workData.facData)
                    facData2=zeros(size(EPoverview.workData.facData));
                end;
                if isfield(EPoverview.workData,'noise')
                    noise2=zeros(size(EPoverview.workData.noise));
                end;
                if isfield(EPoverview.workData,'std')
                    std2=zeros(size(EPoverview.workData.std));
                end;
                
                blinkTrial2=zeros(size(EPoverview.workData.analysis.blinkTrial));
                saccadeTrial2=zeros(size(EPoverview.workData.analysis.saccadeTrial));
                saccadeOnset2=zeros(size(EPoverview.workData.analysis.saccadeOnset));
                moveTrial2=zeros(size(EPoverview.workData.analysis.moveTrial));
                badTrials2=zeros(size(EPoverview.workData.analysis.badTrials));
                badChans2=zeros(size(EPoverview.workData.analysis.badChans));
                
                for theTrial = 1:length(cellIDs)
                    newCell=cellIDs(theTrial);
                    trialCounter(newCell)=trialCounter(newCell)+1;
                    data2(:,:,sum(cellNums(1:newCell-1))+trialCounter(newCell))=EPoverview.workData.data(:,:,theTrial,:,:);
                    if ~isempty(EPoverview.workData.facData)
                        facData2(:,:,sum(cellNums(1:newCell-1))+trialCounter(newCell))=EPoverview.workData.facData(:,:,theTrial,:,:);
                    end;
                    if isfield(EPoverview.workData,'noise')
                        noise2(:,:,sum(cellNums(1:newCell-1))+trialCounter(newCell))=EPoverview.workData.noise(:,:,theTrial,:,:);
                    end;
                    if isfield(EPoverview.workData,'std')
                        std2(:,:,sum(cellNums(1:newCell-1))+trialCounter(newCell))=EPoverview.workData.std(:,:,theTrial,:,:);
                    end;
                    
                    cellNames2(sum(cellNums(1:newCell-1))+trialCounter(newCell))=EPoverview.workData.cellNames(theTrial);
                    cellTypes2(sum(cellNums(1:newCell-1))+trialCounter(newCell))=EPoverview.workData.cellTypes(theTrial);
                    trialNames2(sum(cellNums(1:newCell-1))+trialCounter(newCell))=EPoverview.workData.trialNames(theTrial);
                    trialSpecs2(sum(cellNums(1:newCell-1))+trialCounter(newCell),:)=EPoverview.workData.trialSpecs(theTrial,:);
                    events2(1,sum(cellNums(1:newCell-1))+trialCounter(newCell))=EPoverview.workData.events(1,theTrial);
                    avgNum2(1,sum(cellNums(1:newCell-1))+trialCounter(newCell))=EPoverview.workData.avgNum(1,theTrial);
                    subNum2(1,sum(cellNums(1:newCell-1))+trialCounter(newCell))=EPoverview.workData.subNum(1,theTrial);
                    blinkTrial2(1,sum(cellNums(1:newCell-1))+trialCounter(newCell))= EPoverview.workData.analysis.blinkTrial(1,theTrial);
                    saccadeTrial2(1,sum(cellNums(1:newCell-1))+trialCounter(newCell))= EPoverview.workData.analysis.saccadeTrial(1,theTrial);
                    saccadeOnset2(1,sum(cellNums(1:newCell-1))+trialCounter(newCell))= EPoverview.workData.analysis.saccadeOnset(1,theTrial);
                    moveTrial2(1,sum(cellNums(1:newCell-1))+trialCounter(newCell))= EPoverview.workData.analysis.moveTrial(1,theTrial);
                    badTrials2(1,sum(cellNums(1:newCell-1))+trialCounter(newCell))= EPoverview.workData.analysis.badTrials(1,theTrial);
                    badChans2(1,sum(cellNums(1:newCell-1),1)+trialCounter(newCell),:)= EPoverview.workData.analysis.badChans(1,theTrial,:);
                end;
                
                EPoverview.workData.cellNames=cellNames2;
                EPoverview.workData.cellTypes=cellTypes2;
                EPoverview.workData.trialNames=trialNames2;
                EPoverview.workData.trialSpecs=trialSpecs2;
                EPoverview.workData.events=events2;
                EPoverview.workData.avgNum=avgNum2;
                EPoverview.workData.subNum=subNum2;
                EPoverview.workData.data=data2;
                if ~isempty(EPoverview.workData.facData)
                    EPoverview.workData.data=facData2;
                end;
                if isfield(EPoverview.workData,'noise')
                    EPoverview.workData.noise=noise2;
                end;
                if isfield(EPoverview.workData,'std')
                    EPoverview.workData.std=std2;
                end;
                
                EPoverview.workData.analysis.blinkTrial=blinkTrial2;
                EPoverview.workData.analysis.saccadeTrial=saccadeTrial2;
                EPoverview.workData.analysis.saccadeOnset=saccadeOnset2;
                EPoverview.workData.analysis.moveTrial=moveTrial2;
                EPoverview.workData.analysis.badTrials=badTrials2;
                EPoverview.workData.analysis.badChans=badChans2;
                
            else
                disp('Cell names cannot be duplicated except in single trial data.');
            end;
        end;
        
        [err]=ep_checkEPfile(EPoverview.workData);
        
        if reordered
            EPtable=[];
            ep_editData('start');
        end;
        
    case 'deleteCells'
        if sum(EPoverview.cells.select) ==0
            msg{1}='No cells specified for deletion.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        if sum(EPoverview.cells.select) ==length(unique(EPoverview.workData.cellNames))
            msg{1}='There would be nothing left.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        EPoverview.lastData=EPoverview.workData;
        
        keepCells=not(EPoverview.cells.select);
        
        %         [uniqueCells waveOrder m]=unique(EPoverview.workData.cellNames,'first'); %waveOrder is the true order of the unique cells (wave numbers of the first occurences)
        %         [a b cellOrder]=unique(waveOrder,'first'); %cellOrder is the true order of the unique cells (numbered by cells)
        %         keepCells=ismember(m,cellOrder(keepCells));
        %
        EPoverview.workData.avgNum=EPoverview.workData.avgNum(:,keepCells);
        EPoverview.workData.subNum=EPoverview.workData.subNum(:,keepCells);
        EPoverview.workData.data=EPoverview.workData.data(:,:,keepCells,:,:);
        if ~isempty(EPoverview.workData.facData)
            EPoverview.workData.facData=EPoverview.workData.facData(:,:,keepCells,:,:);
        end;
        if isfield(EPoverview.workData,'noise')
            EPoverview.workData.noise=EPoverview.workData.noise(:,:,keepCells,:,:);
        end;
        if isfield(EPoverview.workData,'std')
            EPoverview.workData.std=EPoverview.workData.std(:,:,keepCells,:,:);
        end;
        EPoverview.workData.cellNames=EPoverview.workData.cellNames(keepCells);
        EPoverview.workData.cellTypes=EPoverview.workData.cellTypes(keepCells);
        
        EPoverview.workData.events=EPoverview.workData.events(:,keepCells);
        if strcmp(EPoverview.workData.dataType,'single_trial')
            EPoverview.workData.trialNames=EPoverview.workData.trialNames(keepCells);
            EPoverview.workData.trialSpecs=EPoverview.workData.trialSpecs(keepCells,:);
        end;
        
        EPoverview.workData.analysis.blinkTrial= EPoverview.workData.analysis.blinkTrial(:,keepCells);
        EPoverview.workData.analysis.saccadeTrial= EPoverview.workData.analysis.saccadeTrial(:,keepCells);
        EPoverview.workData.analysis.saccadeOnset= EPoverview.workData.analysis.saccadeOnset(:,keepCells);
        EPoverview.workData.analysis.moveTrial= EPoverview.workData.analysis.moveTrial(:,keepCells);
        EPoverview.workData.analysis.badTrials= EPoverview.workData.analysis.badTrials(:,keepCells);
        EPoverview.workData.analysis.badChans= EPoverview.workData.analysis.badChans(:,keepCells,:);
        
        EPoverview.cells.weights=repmat(0,length(EPoverview.workData.cellNames),1);
        EPoverview.cells.select=repmat(false,length(unique(EPoverview.workData.cellNames)),1);
        
        [err]=ep_checkEPfile(EPoverview.workData);
        
        EPtable=[];
        ep_editData('start');
        
    case 'addCells'
        
        if EPmain.noTable
            tableData=EPtable.edit.cells.tableData;
        else
            tableData=get(EPoverview.handles.cells.hTable,'Data');
        end;
        weights=cell2mat(tableData(:,3));
        
        if ~any(weights)
            msg{1}='No weights specified for forming new cell average.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        EPoverview.lastData=EPoverview.workData;
        
        totalWeight=sum(abs(weights));
        newCellData=zeros(size(EPoverview.workData.data,1),size(EPoverview.workData.data,2),1,length(EPoverview.workData.subNames),max(numSGLfacs,1));
        for i=find(weights)'
            theWeight=weights(i);
            newCellData=newCellData+(theWeight/totalWeight)*EPoverview.workData.data(:,:,i,:,:);
        end;
        EPoverview.workData.data(:,:,end+1,:,:)=newCellData;
        if ~isempty(EPoverview.workData.facData)
            newCellData=zeros(size(EPoverview.workData.facData,1),size(EPoverview.workData.facData,2),1,length(EPoverview.workData.subNames),max(numCMBfacs,1));
            for i=find(weights)'
                theWeight=weights(i);
                newCellData=newCellData+(theWeight/totalWeight)*EPoverview.workData.facData(:,:,i,:,:);
            end;
            EPoverview.workData.facData(:,:,end+1,:,:)=newCellData;
        end;
        if isfield(EPoverview.workData,'noise')
            newCellData=zeros(length(EPoverview.workData.chanNames),length(EPoverview.workData.timeNames),1,length(EPoverview.workData.subNames));
            for i=find(weights)'
                theWeight=weights(i);
                newCellData=newCellData+(theWeight/totalWeight)*EPoverview.workData.noise(:,:,i,:,:);
            end;
            EPoverview.workData.noise(:,:,end+1,:,:)=newCellData;
        end;
        if isfield(EPoverview.workData,'std')
            newCellData=zeros(length(EPoverview.workData.chanNames),length(EPoverview.workData.timeNames),1,length(EPoverview.workData.subNames));
            for i=find(weights)'
                theWeight=weights(i);
                newCellData=newCellData+(theWeight/totalWeight)*EPoverview.workData.std(:,:,i,:,:);
            end;
            EPoverview.workData.std(:,:,end+1,:,:)=newCellData;
        end;
        
        for i=1:length(EPoverview.workData.subNames)
            if ~isempty(find(EPoverview.workData.avgNum(i,find(weights)) == -1))
                EPoverview.workData.avgNum(i,length(EPoverview.workData.cellNames)+1) = -1; %bad number
            elseif ~isempty(find(EPoverview.workData.avgNum(i,find(weights)) == 0))
                EPoverview.workData.avgNum(i,length(EPoverview.workData.cellNames)+1) = 0; %unknown number
            else
                EPoverview.workData.avgNum(i,length(EPoverview.workData.cellNames)+1)=sum(EPoverview.workData.avgNum(i,find(weights)));
            end
            
            if ~isempty(find(EPoverview.workData.subNum(i,find(weights)) == -1))
                EPoverview.workData.subNum(i,length(EPoverview.workData.cellNames)+1) = -1; %bad number
            elseif ~isempty(find(EPoverview.workData.subNum(i,find(weights)) == 0))
                EPoverview.workData.subNum(i,length(EPoverview.workData.cellNames)+1) = 0; %unknown number
            else
                EPoverview.workData.subNum(i,length(EPoverview.workData.cellNames)+1)=sum(EPoverview.workData.subNum(i,find(weights)));
            end
            
            EPoverview.workData.analysis.blinkTrial(i,length(EPoverview.workData.cellNames)+1)= sum(EPoverview.workData.analysis.blinkTrial(i,find(weights)));
            EPoverview.workData.analysis.saccadeTrial(i,length(EPoverview.workData.cellNames)+1)= sum(EPoverview.workData.analysis.saccadeTrial(i,find(weights)));
            EPoverview.workData.analysis.saccadeOnset(i,length(EPoverview.workData.cellNames)+1)= sum(EPoverview.workData.analysis.saccadeOnset(i,find(weights)));
            EPoverview.workData.analysis.moveTrial(i,length(EPoverview.workData.cellNames)+1)= sum(EPoverview.workData.analysis.moveTrial(i,find(weights)));
            EPoverview.workData.analysis.badTrials(i,length(EPoverview.workData.cellNames)+1)= sum(EPoverview.workData.analysis.badTrials(i,find(weights)));
            EPoverview.workData.analysis.badChans(i,length(EPoverview.workData.cellNames)+1,:)= sum(EPoverview.workData.analysis.badChans(i,find(weights),:));
        end;
        
        %make sure new cell doesn't duplicate existing cell names
        newName=[];
        for i=1:length(weights)
            if weights(i) ~= 0
                if weights(i) > 0
                    theSign='+';
                else
                    theSign='-';
                end;
                newName=[newName theSign EPoverview.workData.cellNames{i}];
            end;
        end;
        
        comb=1;
        while any(strcmp([newName num2str(comb)], EPoverview.workData.cellNames))
            comb=comb+1;
        end;
        if comb > 1
            newName=[newName '_' num2str(comb)];
        end;
        
        EPoverview.workData.cellNames{end+1}=newName;
        
        if length(find(weights ~= 0)) > 1
            EPoverview.workData.cellTypes{end+1}='CMB';
        else
            EPoverview.workData.cellTypes(end+1)=EPoverview.workData.cellTypes(weights ~= 0);
        end;
        
        if ~isempty(EPoverview.workData.trialNames)
            EPoverview.workData.trialNames(end+1)=1;
        end;
        
        EPoverview.workData.events(:,end+1)=cell(size(EPoverview.workData.events,1),1);
        
        EPoverview.cells.weights=repmat(0,length(EPoverview.workData.cellNames),1);
        EPoverview.cells.select=repmat(false,length(EPoverview.workData.cellNames),1);
        
        [err]=ep_checkEPfile(EPoverview.workData);
        
        EPtable=[];
        ep_editData('start');
        
    case 'clearCellWeights'
        
        EPoverview.cells.weights=repmat(0,length(EPoverview.workData.cellNames),1);
        EPoverview.cells.select=repmat(false,length(EPoverview.workData.cellNames),1);
        
        ep_editData('start');
        
    case 'allCells'
        
        EPoverview.cells.weights=repmat(1,length(EPoverview.workData.cellNames),1);
        EPoverview.cells.select=repmat(true,length(EPoverview.workData.cellNames),1);
        
        ep_editData('start');
        
    case 'appendCells'
        
        disp('Making assumption that file format is same as that of the original file.');
        importFormat=EPoverview.workData.fileFormat;
        fileType=EPoverview.workData.dataType;
        if strcmp(fileType,'continuous')
            msg{1}='It is not possible to append additional cells to this type of file.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        inArg{1}='format';
        inArg{2}=importFormat;
        inArg{3}='type';
        inArg{4}=fileType;
        
        [newData]=ep_readData(inArg);
        
        if isempty(newData)
            return
        end;
        
        if EPmain.noTable
            tableData=EPtable.edit.cells.tableData;
        else
            tableData=get(EPoverview.handles.cells.hTable,'Data');
        end;
        
        if ~strcmp(fileType,newData.dataType)
            msg{1}='The file types do not match.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        if length(EPoverview.workData.subNames) ~= length(newData.subNames)
            msg{1}='The numbers of subjects do not match.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        if length(EPoverview.workData.chanNames) ~= length(newData.chanNames)
            msg{1}='The numbers of channels do not match.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        if length(EPoverview.workData.timeNames) ~= length(newData.timeNames)
            msg{1}='The numbers of time points do not match.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        if length(EPoverview.workData.facNames) ~= length(newData.facNames)
            msg{1}='The numbers of factors do not match.';
            [msg]=ep_errorMsg(msg);
            return
        end;

        if length(EPoverview.workData.trialSpecNames) ~= length(newData.trialSpecNames)
            msg{1}='The numbers of trial specs do not match.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        if ~strcmp(EPoverview.workData.dataType,'single_trial')
            newCells=EPoverview.workData.cellNames;
            for i=1:length(newData.cellNames)
                if any(strcmp(newData.cellNames{i},newCells))
                    cellNameSuffix='cell001';
                    sameCell=1;
                    suffix=0;
                    while sameCell
                        sameCell=0;
                        for j=1:length(newCells)
                            if strcmp(newCells{j},cellNameSuffix)
                                sameCell=1;
                            end;
                        end;
                        if sameCell
                            suffix=suffix+1;
                            cellNameSuffix=['cell' sprintf('%03d',suffix)];
                        end;
                    end;
                    newData.cellNames{i,1}=cellNameSuffix;
                end;
                newCells{end+1,1}=newData.cellNames{i};
            end;
            
            combinedCells = [EPoverview.workData.cellNames; newData.cellNames];
            if (length(unique(combinedCells)) ~= length(combinedCells))
                msg{1}='Cell names cannot be duplicated except in single trial data.';
                [msg]=ep_errorMsg(msg);
                return
            end;
        end;
        
        EPoverview.lastData=EPoverview.workData;
        
        EPoverview.workData.data(:,:,end+1:end+length(newData.cellNames),:,:)=newData.data;
        
        if ~isempty(EPoverview.workData.facData)
            if ~isempty(newData.facData)
                EPoverview.workData.facData(:,:,end+1:end+length(newData.cellNames),:,:)=newData.facData;
            else
                EPoverview.workData.facData(:,:,end+1:end+length(newData.cellNames),:,:)= zeros(size(newData.facData,1),size(newData.facData,2),length(newData.cellNames),length(newData.subNames),max(numCMBfacs,1));
            end;
        end;
        
        if isfield(EPoverview.workData,'noise')
            if isfield(newData,'noise')
                EPoverview.workData.noise(:,:,end+1:end+length(newData.cellNames),:,:)=newData.noise;
            else
                EPoverview.workData.noise(:,:,end+1:end+length(newData.cellNames),:,:)= zeros(length(newData.chanNames),length(newData.timeNames),length(newData.cellNames),length(newData.subNames),max(length(newData.facNames),1));
            end;
        end;
        if isfield(EPoverview.workData,'std')
            if isfield(newData,'std')
                EPoverview.workData.std(:,:,end+1:end+length(newData.cellNames),:,:)=newData.std;
            else
                EPoverview.workData.std(:,:,end+1:end+length(newData.cellNames),:,:)= zeros(length(newData.chanNames),length(newData.timeNames),length(newData.cellNames),length(newData.subNames),max(length(newData.facNames),1));
            end;
        end;
        
        EPoverview.workData.avgNum(:,end+1:end+length(newData.cellNames))=newData.avgNum;
        EPoverview.workData.subNum(:,end+1:end+length(newData.cellNames))=newData.subNum;
        EPoverview.workData.cellNames(end+1:end+length(newData.cellNames))=newData.cellNames;
        EPoverview.workData.cellTypes(end+1:end+length(newData.cellNames))=newData.cellTypes;
        EPoverview.workData.events(:,end+1:end+length(newData.cellNames))=newData.events;
        
        EPoverview.workData.analysis.blinkTrial(:,end+1:end+length(newData.cellNames))= newData.analysis.blinkTrial;
        EPoverview.workData.analysis.saccadeTrial(:,end+1:end+length(newData.cellNames))= newData.analysis.saccadeTrial;
        EPoverview.workData.analysis.saccadeOnset(:,end+1:end+length(newData.cellNames))= newData.analysis.saccadeOnset;
        EPoverview.workData.analysis.moveTrial(:,end+1:end+length(newData.cellNames))= newData.analysis.moveTrial;
        EPoverview.workData.analysis.badTrials(:,end+1:end+length(newData.cellNames))= newData.analysis.badTrials;
        EPoverview.workData.analysis.badChans(:,end+1:end+length(newData.cellNames),:)= newData.analysis.badChans;
        if strcmp(EPoverview.workData.dataType,'single_trial')
            EPoverview.workData.trialSpecs(end+1:end+length(newData.cellNames),:)= newData.trialSpecs;
        end;
        
        if (length(unique(EPoverview.workData.cellNames)) ~= length(EPoverview.workData.cellNames))
            %rename the combined trial names
            trialCount=ones(length(unique(EPoverview.workData.cellNames)),1);
            uniqueCells=unique(EPoverview.workData.cellNames,'first');
            for wave=1:length(EPoverview.workData.cellNames)
                theCell=strcmp(EPoverview.workData.cellNames(wave),uniqueCells);
                EPoverview.workData.trialNames(wave)=trialCount(theCell);
                trialCount(theCell)=trialCount(theCell)+1;
            end;
        end;
        
        EPoverview.cells.select=[EPoverview.cells.select; repmat(false,length(unique(newData.cellNames)),1)];
        EPoverview.cells.weights=[EPoverview.cells.weights; repmat(0,length(unique(newData.cellNames)),1)];
        
        [err]=ep_checkEPfile(EPoverview.workData);
        
        EPtable=[];
        ep_editData('start');
        
    case 'exportCells'
        
        if EPmain.noTable
            tableData=EPtable.edit.cells.tableData;
        else
            tableData=get(EPoverview.handles.cells.hTable,'Data');
        end;
        tableData=tableData(:,4:end);
        
        [FileName,PathName,FilterIndex] = uiputfile('*.txt','Export Table');
        
        if isnumeric(FileName)
            if FileName == 0
                msg{1}='No file name specified.';
                [msg]=ep_errorMsg(msg);
                return
            end;
        end;
        
        [pathstr, name, ext] = fileparts(FileName);
        if ~strcmp(ext,'.txt')
            FileName=[FileName '.txt'];
        end;
        
        fid=fopen([PathName FileName],'w');
        
        for i=1:size(tableData,1)
            for j=1:size(tableData,2)
                theCell=tableData{i,j};
                if isnumeric(theCell)
                    fprintf(fid,'%d', theCell);
                else
                    fprintf(fid,'%s', theCell);
                end;
                if j < size(tableData,2)
                    fprintf(fid,'\t');
                end;
            end;
            if i < size(tableData,1)
                fprintf(fid,'\r');
            end;
        end;
        
        ep_editData('start');
        
    case 'trialSpecs'
        
        EPoverview.lastData=EPoverview.workData;
        
        if EPmain.noTable
            tableData=EPtable.edit.trials.tableData;
        else
            tableData =get(EPoverview.handles.trials.hTable,'Data');
        end;
        
        EPoverview.workData.cellNames=tableData(:,1);
        EPoverview.workData.trialNames=tableData(:,2);
        EPoverview.workData.trialSpecs=tableData(:,3:end);
        
        [err]=ep_checkEPfile(EPoverview.workData);
        
    case 'exportTrials'
        
        if EPmain.noTable
            tableData=EPtable.edit.trials.tableData;
        else
            tableData =get(EPoverview.handles.trials.hTable,'Data');
        end;
        
        [FileName,PathName,FilterIndex] = uiputfile('*.txt','Export Table');
        
        if isnumeric(FileName)
            if FileName == 0
                msg{1}='No file name specified.';
                [msg]=ep_errorMsg(msg);
                return
            end;
        end;
        
        [pathstr, name, ext] = fileparts(FileName);
        if ~strcmp(ext,'.txt')
            FileName=[FileName '.txt'];
        end;
        
        fid=fopen([PathName FileName],'w');
        
        for i=1:size(tableData,1)
            for j=1:size(tableData,2)
                theCell=tableData{i,j};
                if isnumeric(theCell)
                    fprintf(fid,'%d', theCell);
                else
                    fprintf(fid,'%s', theCell);
                end;
                if j < size(tableData,2)
                    fprintf(fid,'\t');
                end;
            end;
            if i < size(tableData,1)
                fprintf(fid,'\r');
            end;
        end;
        
        ep_editData('start');
        
    case 'editChannels'
        
        EPoverview.lastData=EPoverview.workData;
        
        chanNum=length(EPoverview.workData.chanNames);
        
        if EPmain.noTable
            tableData=EPtable.edit.channels.tableData;
            weights=EPtable.edit.channels.tableData{:,3};
            for i=1:size(EPtable.edit.channels.tableData,1)
                if isnumeric(EPtable.edit.channels.tableData{i,2})
                    tableData{i,2}=cast(tableData{i,2},'logical'); %workaround for Matlab bug
                end;
            end;
        else
            tableData =get(EPoverview.handles.channels.hTable,'Data');
        end;
        
        
        %channels reordered?  Can't edit implicit sites, which must also stay at the end.
        reordered=0;
        if any(cell2mat(tableData(1:chanNum,1))-[1:chanNum]')
            reordered=1;
            [tableData(1:chanNum,:),index] = sortrows(tableData(1:chanNum,:),1);
            if ~isempty(EPoverview.workData.facVecS)
                EPoverview.workData.facVecS=EPoverview.workData.facVecS(index,:);
            else
                EPoverview.workData.data=EPoverview.workData.data(index,:,:,:,:);
            end;
            if isfield(EPoverview.workData,'noise')
                EPoverview.workData.noise=EPoverview.workData.noise(index,:,:,:);
            end;
            if isfield(EPoverview.workData,'std')
                EPoverview.workData.std=EPoverview.workData.std(index,:,:,:);
            end;
            if ~isempty(EPoverview.workData.facData)
                EPoverview.workData.facData=EPoverview.workData.facData(index,:,:,:,:);
            end
            
            EPoverview.workData.chanNames=EPoverview.workData.chanNames(index);
            EPoverview.workData.chanTypes=EPoverview.workData.chanTypes(index);
            EPoverview.workData.eloc=EPoverview.workData.eloc(index);
            EPoverview.channels.select(1:numChans)=EPoverview.channels.select(index);
            EPoverview.channels.weights(1:numChans)=EPoverview.channels.weights(index);
            EPoverview.workData.analysis.badChans=EPoverview.workData.analysis.badChans(:,:,index);
            
        else
            EPoverview.channels.select(1:chanNum)=cell2mat(tableData(1:chanNum,2));
            EPoverview.channels.weights(1:chanNum)=cell2mat(tableData(1:chanNum,3));
            set(EPoverview.handles.channels.hWeights,'String',sprintf('%f',sum(EPoverview.channels.weights)));
            EPoverview.workData.chanNames=tableData(1:chanNum,4);
            EPoverview.workData.chanTypes=tableData(1:chanNum,5);
        end;
        
        [err]=ep_checkEPfile(EPoverview.workData);
        
        if reordered
            EPtable=[];
            ep_editData('start');
        end;
        
    case 'deleteChans'
        
        if sum(EPoverview.channels.select) ==0
            msg{1}='No channels specified for deletion.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        if sum(EPoverview.channels.select(1:numChans)) == numChans
            msg{1}='There would be no data left.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        EPoverview.lastData=EPoverview.workData;
        
        keepChans=not(EPoverview.channels.select(1:numChans));
        keepImplicit=not(EPoverview.channels.select(numChans+1:end));
        
        if ~isempty(EPoverview.workData.facVecS)
            EPoverview.workData.facVecS=EPoverview.workData.facVecS(keepChans,:);
        else
            EPoverview.workData.data=EPoverview.workData.data(keepChans,:,:,:,:);
        end;
        
        if isfield(EPoverview.workData,'noise')
            EPoverview.workData.noise=EPoverview.workData.noise(keepChans,:,:,:);
        end;
        if isfield(EPoverview.workData,'std')
            EPoverview.workData.std=EPoverview.workData.std(keepChans,:,:,:);
        end;
        if ~isempty(EPoverview.workData.facData)
            EPoverview.workData.facData=EPoverview.workData.facData(keepChans,:,:,:,:);
        end;
        
        EPoverview.workData.chanNames=EPoverview.workData.chanNames(keepChans);
        EPoverview.workData.chanTypes=EPoverview.workData.chanTypes(keepChans);
        EPoverview.workData.eloc=EPoverview.workData.eloc(keepChans);
        EPoverview.workData.implicit=EPoverview.workData.implicit(keepImplicit);
        
        EPoverview.workData.analysis.badChans=EPoverview.workData.analysis.badChans(:,:,keepChans);
        
        EPoverview.channels.select=repmat(false,length(keepChans)+length(keepImplicit),1);
        
        [err]=ep_checkEPfile(EPoverview.workData);
        
        EPtable=[];
        ep_editData('start');
        
    case 'addChans'
        
        if EPmain.noTable
            tableData=EPtable.edit.channels.tableData;
        else
            tableData=get(EPoverview.handles.channels.hTable,'Data');
        end;
        
        weights=cell2mat(tableData(:,3));
        
        if ~any(weights)
            msg{1}='No weights specified for forming new channel average.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        EPoverview.lastData=EPoverview.workData;
        
        totalWeight=sum(abs(weights));
        newChanData=zeros(1,numPoints,numCells,numSubs,max(numSGLfacs,1));
        newFacData=zeros(1,numPoints,numCells,numSubs,max(numCMBfacs,1));
        newFacVecSData=zeros(1,numFacs);
        newBadChanData=zeros(numSubs,numCells);
        for i=find(weights)'
            theWeight=weights(i);
            if ~isempty(EPoverview.workData.facVecS)
                newFacVecSData=newFacVecSData+(theWeight/totalWeight)*EPoverview.workData.facVecS(i,:);
            else
                newChanData=newChanData+(theWeight/totalWeight)*EPoverview.workData.data(i,:,:,:,:);
            end;
            if ~isempty(EPoverview.workData.facData)
                newFacData=newFacData+(theWeight/totalWeight)*EPoverview.workData.newFacData(i,:,:,:,:);
            end;
            newBadChanData=newBadChanData+(theWeight/totalWeight)*EPoverview.workData.analysis.badChans(:,:,i);
        end;
        
        if ~isempty(EPoverview.workData.facVecS)
            EPoverview.workData.facVecS(end+1,:)=newFacVecSData;
        else
            EPoverview.workData.data(end+1,:,:,:,:)=newChanData;
        end;
        
        if ~isempty(EPoverview.workData.facData)
            EPoverview.workData.newFacData(end+1,:,:,:,:)=newFacData;
        end;
        
        EPoverview.workData.analysis.badChans(:,:,end+1)=newBadChanData;
        
        if isfield(EPoverview.workData,'noise')
            newChanData=zeros(1,numPoints,numCells,numSubs);
            for i=find(weights)'
                theWeight=weights(i);
                newChanData=newChanData+(theWeight/totalWeight)*EPoverview.workData.noise(i,:,:,:);
            end;
            EPoverview.workData.noise(end+1,:,:,:,:)=newChanData;
        end;
        if isfield(EPoverview.workData,'std')
            newChanData=zeros(1,numPoints,numCells,numSubs);
            for i=find(weights)'
                theWeight=weights(i);
                newChanData=newChanData+(theWeight/totalWeight)*EPoverview.workData.std(i,:,:,:);
            end;
            EPoverview.workData.std(end+1,:,:,:,:)=newChanData;
        end;
        
        %make sure new channel doesn't duplicate existing channel names
        comb=1;
        while any(strcmp(['combined' num2str(comb)], EPoverview.workData.chanNames))
            comb=comb+1;
        end;
        EPoverview.workData.chanNames{end+1}=['combined' num2str(comb)];
        EPoverview.workData.chanTypes{end+1}='REG';
        EPoverview.workData.eloc(end+1)=cell2struct(cell(1,length(fieldnames(EPoverview.workData.eloc))),fieldnames(EPoverview.workData.eloc),2);
        
        EPoverview.channels.weights=repmat(0,numChans+numImplicit+1,1);
        EPoverview.channels.select=repmat(false,numChans+numImplicit+1,1);
        
        [err]=ep_checkEPfile(EPoverview.workData);
        
        EPtable=[];
        ep_editData('start');
        
    case 'clearChanWeights'
        
        EPoverview.channels.weights=repmat(0,numChans+numImplicit,1);
        EPoverview.channels.select=repmat(false,numChans+numImplicit,1);
        
        ep_editData('start');
        
    case 'allChans'
        
        EPoverview.channels.weights=repmat(1,numChans+numImplicit,1);
        EPoverview.channels.select=repmat(true,numChans+numImplicit,1);
        
        ep_editData('start');
        
    case 'appendChans'
        
        disp('Making assumption that file format is same as that of the original file.');
        importFormat=EPoverview.workData.fileFormat;
        fileType=EPoverview.workData.dataType;
        
        inArg{1}='format';
        inArg{2}=importFormat;
        inArg{3}='type';
        inArg{4}=fileType;
        
        [newData]=ep_readData(inArg);
        
        if isempty(newData)
            return
        end;
        
        if EPmain.noTable
            tableData=EPtable.edit.channels.tableData;
        else
            tableData=get(EPoverview.handles.channels.hTable,'Data');
        end;
        
        if ~strcmp(fileType,newData.dataType)
            msg{1}='The file types do not match.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        if numCells ~= length(newData.cellNames)
            msg{1}='The numbers of cells do not match.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        if numSubs ~= length(newData.subNames)
            msg{1}='The numbers of subjects do not match.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        if numPoints ~= length(newData.timeNames)
            msg{1}='The numbers of time points do not match.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        if length(EPoverview.workData.facNames) ~= length(newData.facNames)
            msg{1}='The numbers of factors do not match.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        if size(EPoverview.workData.facVecS,2) ~= size(newData.facVecS,2)
            msg{1}='The two datasets are not equally factor compressed.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        EPoverview.lastData=EPoverview.workData;
        
        if ~isempty(EPoverview.workData.facVecS)
            EPoverview.workData.facVecS(end+1:length(newData.chanNames),:)=newData.facVecS;
        else
            EPoverview.workData.data(end+1:length(newData.chanNames),:,:,:,:)=newData.data;
        end;
        
        if ~isempty(EPoverview.workData.facData)
            if ~isempty(newData.facData)
                EPoverview.workData.facData(end+1:end+length(newData.chanNames),:,:,:,:)=newData.facData;
            else
                EPoverview.workData.facData(end+1:end+length(newData.chanNames),:,:,:,:)= zeros(size(newData.facData,1),size(newData.facData,2),length(newData.cellNames),length(newData.subNames),max(length(newData.facNames),1));
            end;
        end;
        
        if isfield(EPoverview.workData,'noise')
            if isfield(newData,'noise')
                EPoverview.workData.noise(end+1:end+length(newData.chanNames),:,:,:)=newData.noise;
            else
                EPoverview.workData.noise(end+1:end+length(newData.chanNames),:,:,:)= zeros(length(newData.chanNames),length(newData.timeNames),length(newData.cellNames),length(newData.subNames),max(length(newData.facNames),1));
            end;
        end;
        if isfield(EPoverview.workData,'std')
            if isfield(newData,'std')
                EPoverview.workData.std(end+1:end+length(newData.chanNames),:,:,:)=newData.std;
            else
                EPoverview.workData.std(end+1:end+length(newData.chanNames),:,:,:)= zeros(length(newData.chanNames),length(newData.timeNames),length(newData.cellNames),length(newData.subNames),max(length(newData.facNames),1));
            end;
        end;
        
        EPoverview.workData.analysis.badChans(:,:,end+1:end+length(newData.chanNames))=newData.analysis.badChans;
        
        EPoverview.workData.chanNames(end+1:end+length(newData.chanNames))=newData.chanNames;
        EPoverview.workData.chanTypes(end+1:end+length(newData.chanNames))=newData.chanTypes;
        EPoverview.workData.eloc(end+1:end+length(newData.chanNames))=newData.eloc;
        
        EPoverview.channels.select=repmat(false,length(EPoverview.workData.chanNames)+length(EPoverview.workData.implicit),1);
        EPoverview.channels.weights=repmat(0,length(EPoverview.workData.chanNames)+length(EPoverview.workData.implicit),1);
        
        [err]=ep_checkEPfile(EPoverview.workData);
        
        EPtable=[];
        ep_editData('start');
        
    case 'exportChans'
        
        if EPmain.noTable
            tableData=EPtable.edit.cells.tableData;
        else
            tableData=get(EPoverview.handles.chans.hTable,'Data');
        end;
        tableData=tableData(:,4:end);
        
        [FileName,PathName,FilterIndex] = uiputfile('*.txt','Export Table');
        
        if isnumeric(FileName)
            if FileName == 0
                msg{1}='No file name specified.';
                [msg]=ep_errorMsg(msg);
                return
            end;
        end;
        
        [pathstr, name, ext] = fileparts(FileName);
        if ~strcmp(ext,'.txt')
            FileName=[FileName '.txt'];
        end;
        
        fid=fopen([PathName FileName],'w');
        
        for i=1:size(tableData,1)
            for j=1:size(tableData,2)
                theCell=tableData{i,j};
                if isnumeric(theCell)
                    fprintf(fid,'%d', theCell);
                else
                    fprintf(fid,'%s', theCell);
                end;
                if j < size(tableData,2)
                    fprintf(fid,'\t');
                end;
            end;
            if i < size(tableData,1)
                fprintf(fid,'\r');
            end;
        end;
        
        ep_editData('start');
        
    case 'changeStartSamp'
        
        newStartSamp=str2double(get(EPoverview.handles.samples.startSamp,'string'));
        if isnan(newStartSamp)
            set(EPoverview.handles.samples.startSamp,'string',sprintf('%d', 1))
            msg{1}='The new start sample must be a number.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        if newStartSamp < 1
            set(EPoverview.handles.samples.startSamp,'string',sprintf('%d', 1))
            msg{1}='The new start sample cannot be earlier than the existing start sample time.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        if newStartSamp == 1
            msg{1}='The start sample was not changed.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        if newStartSamp > length(EPoverview.workData.timeNames)
            msg{1}='The new start sample cannot be larger than the number of total samples.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        EPoverview.lastData=EPoverview.workData;
        
        trimSamples=newStartSamp-1;
        
        EPoverview.workData.baseline=EPoverview.workData.baseline-trimSamples;
        
        EPoverview.workData.timeNames=EPoverview.workData.timeNames(trimSamples+1:end);
        
        if ~isempty(EPoverview.workData.facVecT)
            EPoverview.workData.facVecT=EPoverview.workData.facVecT(trimSamples+1:end,:);
        else
            EPoverview.workData.data=EPoverview.workData.data(:,trimSamples+1:end,:,:,:);
        end;
        
        if isfield(EPoverview.workData,'noise')
            EPoverview.workData.noise=EPoverview.workData.noise(:,trimSamples+1:end,:,:,:);
        end;
        
        if isfield(EPoverview.workData,'std')
            EPoverview.workData.std=EPoverview.workData.std(:,trimSamples+1:end,:,:,:);
        end;
        
        if ~isempty(EPoverview.workData.facData)
            EPoverview.workData.facData=EPoverview.workData.facData(:,trimSamples+1:end,:,:,:);
        end;
        
        for subject = 1:length(EPoverview.workData.subNames)
            for wave = 1:length(EPoverview.workData.cellNames)
                for event = 1:length(EPoverview.workData.events(subject,wave))
                    EPoverview.workData.events{subject,wave}(event).offset=EPoverview.workData.events{subject,wave}(event).offset-trimSamples;
                end;
            end;
        end;
        
        [err]=ep_checkEPfile(EPoverview.workData);
        
        ep_editData('start');
        
    case 'changeEndSamp'
        
        newEndSamp=str2double(get(EPoverview.handles.samples.endSamp,'string'));
        if isnan(newEndSamp)
            set(EPoverview.handles.samples.endSamp,'string',sprintf('%d', (length(EPoverview.workData.timeNames))))
            msg{1}='The new end sample must be a number.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        if newEndSamp > length(EPoverview.workData.timeNames)
            set(EPoverview.handles.samples.endSamp,'string',sprintf('%d', (length(EPoverview.workData.timeNames))))
            msg{1}='The new end sample cannot be later than the existing end sample time.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        if newEndSamp == length(EPoverview.workData.timeNames)
            msg{1}='The end sample was not changed.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        if newEndSamp < 1
            msg{1}='The new end sample cannot be smaller than one.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        EPoverview.lastData=EPoverview.workData;
        
        trimSamples=length(EPoverview.workData.timeNames)-newEndSamp;
        
        EPoverview.workData.timeNames=EPoverview.workData.timeNames(1:end-trimSamples);
        
        if ~isempty(EPoverview.workData.facVecT)
            EPoverview.workData.facVecT=EPoverview.workData.facVecT(1:end-trimSamples,:);
        else
            EPoverview.workData.data=EPoverview.workData.data(:,1:end-trimSamples,:,:,:);
        end;
        
        if isfield(EPoverview.workData,'noise')
            EPoverview.workData.noise=EPoverview.workData.noise(:,1:end-trimSamples,:,:,:);
        end;
        
        if isfield(EPoverview.workData,'std')
            EPoverview.workData.std=EPoverview.workData.std(:,1:end-trimSamples,:,:,:);
        end;
        
        if ~isempty(EPoverview.workData.facData)
            EPoverview.workData.facData=EPoverview.workData.facData(:,1:end-trimSamples,:,:,:);
        end;
        
        [err]=ep_checkEPfile(EPoverview.workData);
        
        ep_editData('start');
        
    case 'halfRate'
        
        if mod(length(EPoverview.workData.timeNames),2) ~= 0
            msg{1}='The total number of samples must be an even number.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        EPoverview.lastData=EPoverview.workData;
        
        EPoverview.workData.baseline=EPoverview.workData.baseline/2;
        EPoverview.workData.Fs=EPoverview.workData.Fs/2;
        
        EPoverview.workData.timeNames=EPoverview.workData.timeNames(1:2:end);
        
        if ~isempty(EPoverview.workData.facVecT)
            newFacVecT=zeros(numPoints/2,numFacs);
            for i=1:length(EPoverview.workData.timeNames)
                newFacVecT(i,:)=(EPoverview.workData.facVecT((i-1)*2+1,:)+EPoverview.workData.facVecT(i*2,:))./2;
            end;
        else
            newData=zeros(numChans,numPoints/2,numWaves,numSubs);
            for i=1:length(EPoverview.workData.timeNames)
                newData(:,i,:,:)=(EPoverview.workData.data(:,(i-1)*2+1,:,:,:)+EPoverview.workData.data(:,i*2,:,:,:))./2;
            end;
        end;
        
        EPoverview.workData.data=newData;
        
        if isfield(EPoverview.workData,'noise')
            newData=zeros(numChans,numPoints/2,numWaves,numSubs);
            for i=1:length(EPoverview.workData.timeNames)
                newData(:,i,:,:)=(EPoverview.workData.noise(:,(i-1)*2+1,:,:,:)+EPoverview.workData.noise(:,i*2,:,:,:))./2;
            end;
            
            EPoverview.workData.noise=newData;
        end;
        
        if isfield(EPoverview.workData,'std')
            newData=zeros(numChans,numPoints/2,numWaves,numSubs);
            for i=1:length(EPoverview.workData.timeNames)
                newData(:,i,:,:)=(EPoverview.workData.std(:,(i-1)*2+1,:,:,:)+EPoverview.workData.std(:,i*2,:,:,:))./2;
            end;
            
            EPoverview.workData.std=newData;
        end;
        
        if ~isempty(EPoverview.workData.facData)
            newData=zeros(numChans,numPoints/2,numWaves,numSubs);
            for i=1:length(EPoverview.workData.timeNames)
                newData(:,i,:,:)=(EPoverview.workData.facData(:,(i-1)*2+1,:,:,:)+EPoverview.workData.facData(:,i*2,:,:,:))./2;
            end;
            
            EPoverview.workData.facData=newData;
        end;
        
        for subject = 1:numSubs
            for wave = 1:numWaves
                for event = 1:length(EPoverview.workData.events{subject,wave})
                    EPoverview.workData.events{subject,wave}(event).offset=(EPoverview.workData.events{subject,wave}(event).offset)/2;
                end;
            end;
        end;
        
        [err]=ep_checkEPfile(EPoverview.workData);
        
        ep_editData('start');
        
    case 'editEvents'
        
        EPoverview.lastData=EPoverview.workData;
        
        tableData=get(EPoverview.handles.events.hTable,'Data');
        [uniqueCells waveOrder m]=unique(EPoverview.workData.cellNames,'first'); %waveOrder is the true order of the unique cells (wave numbers of the first occurences)
        [a b cellOrder]=unique(waveOrder,'first'); %cellOrder is the true order of the unique cells (numbered by cells)
        [n xout] = hist(m,length(uniqueCells));
        
        eventCounter=0;
        for subject=1:numSubs
            for wave=1:numWaves
                for event=1:length(EPoverview.workData.events{subject,wave})
                    eventCounter=eventCounter+1;
                    EPoverview.workData.events{subject,wave}(event).type=tableData{eventCounter,6};
                    EPoverview.workData.events{subject,wave}(event).sample=tableData{eventCounter,7};
                    EPoverview.workData.events{subject,wave}(event).value=tableData{eventCounter,8};
                    EPoverview.workData.events{subject,wave}(event).offset=-(tableData{eventCounter,9}/(1000/EPoverview.workData.Fs)+EPoverview.workData.baseline);
                    EPoverview.workData.events{subject,wave}(event).duration=tableData{eventCounter,10};
                end;
            end;
        end;
        
        EPoverview.events.select=cell2mat(tableData(:,1));
        
        [err]=ep_checkEPfile(EPoverview.workData);
        
    case 'deleteEvents'
        if sum(EPoverview.events.select) ==0
            msg{1}='No events specified for deletion.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        EPoverview.lastData=EPoverview.workData;
        
        keepEvents=not(EPoverview.events.select);
        
        eventCounter=0;
        for subject=1:numSubs
            for wave=1:numWaves
                eventList=[];
                for event=1:length(EPoverview.workData.events{subject,wave})
                    eventCounter=eventCounter+1;
                    if keepEvents(eventCounter)
                        eventList=[eventList event];
                    end;
                end;
                EPoverview.workData.events{subject,wave}=EPoverview.workData.events{subject,wave}(eventList);
            end;
        end;
        
        EPoverview.events.select=repmat(false,numEvents,1);
        
        [err]=ep_checkEPfile(EPoverview.workData);
        
        ep_editData('start');
        
    case 'clearEventWeights'
        
        EPoverview.events.weights=repmat(0,numEvents,1);
        EPoverview.events.select=repmat(false,numEvents,1);
        
        ep_editData('start');
        
    case 'allEvents'
        
        EPoverview.events.weights=repmat(1,numEvents,1);
        EPoverview.events.select=repmat(true,numEvents,1);
        
        ep_editData('start');
        
    case 'clearEventWeights'
        
        EPoverview.channels.weights=repmat(0,numEvents,1);
        EPoverview.channels.select=repmat(false,numEvents,1);
        
        ep_editData('start');
        
    case 'exportEvents'
        
        tableData=get(EPoverview.handles.events.hTable,'Data');
        tableData=tableData(:,2:end);
        
        [FileName,PathName,FilterIndex] = uiputfile('*.txt','Export Table');
        
        if isnumeric(FileName)
            if FileName == 0
                msg{1}='No file name specified.';
                [msg]=ep_errorMsg(msg);
                return
            end;
        end;
        
        [pathstr, name, ext] = fileparts(FileName);
        if ~strcmp(ext,'.txt')
            FileName=[FileName '.txt'];
        end;
        
        fid=fopen([PathName FileName],'w');
        
        for i=1:size(tableData,1)
            for j=1:size(tableData,2)
                theCell=tableData{i,j};
                if isnumeric(theCell)
                    fprintf(fid,'%d', theCell);
                else
                    fprintf(fid,'%s', theCell);
                end;
                if j < size(tableData,2)
                    fprintf(fid,'\t');
                end;
            end;
            if i < size(tableData,1)
                fprintf(fid,'\r');
            end;
        end;
        
        ep_editData('start');
        
    case 'editFacs'
        
        EPoverview.lastData=EPoverview.workData;
        
        if EPmain.noTable
            tableData=EPtable.edit.factors.tableData;
            weights=EPtable.edit.factors.tableData{:,3};
            for i=1:size(EPtable.edit.factors.tableData,1)
                if isnumeric(EPtable.edit.factors.tableData{i,2})
                    tableData{i,2}=cast(tableData{i,2},'logical'); %workaround for Matlab bug
                end;
            end;
        else
            tableData=get(EPoverview.handles.factors.hTable,'Data');
        end;
        
        %factors reordered?
        if any(cell2mat(tableData(:,1))-[1:size(tableData,1)]')
            reordered=1;
            [tableData,index] = sortrows(tableData,1);
            EPoverview.workData.facNames=EPoverview.workData.facNames(index);
            EPoverview.workData.facTypes=EPoverview.workData.facTypes(index);
            
            
            singleFacs=find(ismember(EPoverview.workData.facTypes,'SGL')); %combined factors always last and so not reordered.
            combFacs=find(ismember(EPoverview.workData.facTypes,'CMB'))-length(singleFacs);
            EPoverview.workData.facVar=EPoverview.workData.facVar(index(singleFacs));
            EPoverview.workData.facVarQ=EPoverview.workData.facVarQ(index(singleFacs));
            EPoverview.workData.data=EPoverview.workData.data(:,:,:,:,index(singleFacs));
            if ~isempty(EPoverview.workData.facData)
                EPoverview.workData.facData=EPoverview.workData.facData(:,:,:,:,index(combFacs));
            end;
            if ~isempty(EPoverview.workData.facVecT)
                EPoverview.workData.facVecT=EPoverview.workData.facVecT(:,index(singleFacs));
            end;
            if ~isempty(EPoverview.workData.facVecS)
                EPoverview.workData.facVecS=EPoverview.workData.facVecS(:,index(singleFacs));
            end;
            
            
        else %table data edited but not reordered
            reordered=0;
            EPoverview.factors.select=cell2mat(tableData(:,2));
            EPoverview.factors.weights=cell2mat(tableData(:,3));
            EPoverview.workData.facNames =tableData(:,4);
            EPoverview.workData.facTypes =tableData(:,5);
            set(EPoverview.handles.factors.hWeightSum,'String',sprintf('%f',sum(EPoverview.factors.weights)));
        end;
        
        [err]=ep_checkEPfile(EPoverview.workData);
        
        if reordered
            EPtable=[];
            ep_editData('start');
        end;
        
    case 'deleteFacs'
        if sum(EPoverview.factors.select) ==0
            msg{1}='No factors specified for deletion.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        if sum(EPoverview.factors.select) ==numFacs
            msg{1}='There would be nothing left.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        EPoverview.lastData=EPoverview.workData;
        
        keepFactors=not(EPoverview.factors.select);
        
        singleFacs=find(ismember(EPoverview.workData.facTypes,'SGL')) ;
        singleFacsNum=length(singleFacs);
        keepSingleFacs=zeros(singleFacsNum,1);
        for i=1:singleFacsNum
            if keepFactors(singleFacs(i))
                keepSingleFacs(i)=1;
            end;
        end;
        
        keepSingleFacs=intersect(singleFacs,find(keepFactors));
        
        if ~isempty(EPoverview.workData.facVecT)
            EPoverview.workData.facVecT=EPoverview.workData.facVecT(:,keepSingleFacs);
        end;
        if ~isempty(EPoverview.workData.facVecS)
            EPoverview.workData.facVecS=EPoverview.workData.facVecS(:,keepSingleFacs);
        end;
        
        EPoverview.workData.data=EPoverview.workData.data(:,:,:,:,keepSingleFacs);
        EPoverview.workData.facNames=EPoverview.workData.facNames(keepFactors);
        EPoverview.workData.facTypes=EPoverview.workData.facTypes(keepFactors);
        if ~isempty(EPoverview.workData.facData)
            EPoverview.workData.facData=EPoverview.workData.facData(:,:,:,:,keepFactors(length(singleFacs)+1:end));
        end;
        
        EPoverview.factors.select=repmat(false,length(EPoverview.workData.facNames),1);
        
        [err]=ep_checkEPfile(EPoverview.workData);
        
        EPtable=[];
        ep_editData('start');
        
    case 'addFacs'
        
        if EPmain.noTable
            tableData=EPtable.edit.factors.tableData;
        else
            tableData=get(EPoverview.handles.factors.hTable,'Data');
        end;
        weights=cell2mat(tableData(:,3));
        
        if ~any(weights)
            msg{1}='No weights specified for forming new factor average.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        if length(find(weights)) ==1
            msg{1}='Need to specify at least two factors to combine.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        EPoverview.lastData=EPoverview.workData;
        
        totalWeight=sum(abs(weights));
        newFacData=zeros(numChans,numPoints,numWaves,numSubs,1);
        for i=find(weights)'
            theWeight=weights(i);
            newFacData=newFacData+(theWeight/totalWeight)*ep_expandFacs(EPoverview.workData,[],[],[],[],i);
        end;
        
        EPoverview.workData.facData(:,:,:,:,end+1)=newFacData;
        
        EPoverview.workData.facNames{end+1}='combined';
        if length(find(weights~=0)) > 1
            EPoverview.workData.facTypes{end+1}='CMB';
        end;
        
        EPoverview.factors.weights=repmat(0,length(EPoverview.workData.facNames),1);
        EPoverview.factors.select=repmat(false,length(EPoverview.workData.facNames),1);
        
        [err]=ep_checkEPfile(EPoverview.workData);
        
        EPtable=[];
        ep_editData('start');
        
    case 'clearFacWeights'
        
        EPoverview.factors.weights=repmat(0,numFacs,1);
        EPoverview.factors.select=repmat(false,numFacs,1);
        
        ep_editData('start');
        
    case 'allFacs'
        
        EPoverview.factors.weights=repmat(1,numFacs,1);
        EPoverview.factors.select=repmat(true,numFacs,1);
        
        ep_editData('start');
        
    case 'appendFacs'
        
        disp('Making assumption that file format is same as that of the original file.');
        importFormat=EPoverview.workData.fileFormat;
        fileType=EPoverview.workData.dataType;
        
        inArg{1}='format';
        inArg{2}=importFormat;
        inArg{3}='type';
        inArg{4}=fileType;
        
        [newData]=ep_readData(inArg);
        
        if isempty(newData)
            return
        end;
        
        if EPmain.noTable
            tableData=EPtable.edit.factors.tableData;
        else
            tableData=get(EPoverview.handles.factors.hTable,'Data');
        end;
        
        if ~strcmp(fileType,newData.dataType)
            msg{1}='The file types do not match.';
            [msg]=ep_errorMsg(msg);
            return
        end;
         
        if length(EPoverview.workData.cellNames) ~= length(newData.cellNames)
            msg{1}='The numbers of cells do not match.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        if length(EPoverview.workData.chanNames) ~= length(newData.chanNames)
            msg{1}='The numbers of channels do not match.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        if length(EPoverview.workData.timeNames) ~= length(newData.timeNames)
            msg{1}='The numbers of time points do not match.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        if length(EPoverview.workData.subames) ~= length(newData.subNames)
            msg{1}='The numbers of subjects do not match.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        if size(EPoverview.workData.facVecS,2) ~= size(newData.facVecS,2)
            msg{1}='The two datasets are not equally factor compressed.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        if size(EPoverview.workData.facVecT,2) ~= size(newData.facVecT,2)
            msg{1}='The two datasets are not equally factor compressed.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        EPoverview.lastData=EPoverview.workData;
        
        if ~isempty(EPoverview.workData.facVecS)
            EPoverview.workData.facVecS(:,end+1:length(newData.facNames))=newData.facVecS;
        end;
        
        if ~isempty(EPoverview.workData.facVecT)
            EPoverview.workData.facVecT(:,end+1:length(newData.facNames))=newData.facVecT;
        end;
        
        EPoverview.workData.data(:,:,:,:,end+1:end+size(newData.data,5))=newData.data;
        EPoverview.workData.facData(:,:,:,:,end+1:end+size(newData.facData,5))=newData.facData;
        EPoverview.workData.facNames(end+1:end+length(newData.facNames))=newData.facNames;
        EPoverview.workData.facTypes(end+1:end+length(newData.facNames))=newData.facTypes;
        
        EPoverview.factors.select=[EPoverview.factors.select repmat(false,length(newData.facNames),1)];
        EPoverview.factors.weights=[EPoverview.factors.weights repmat(0,length(newData.facNames),1)];
        
        [err]=ep_checkEPfile(EPoverview.workData);
        
        EPtable=[];
        ep_editData('start');
        
    case 'exportFacs'
        
        if EPmain.noTable
            tableData=EPtable.edit.factors.tableData;
        else
            tableData=get(EPoverview.handles.factors.hTable,'Data');
        end;
        tableData=tableData(:,4:end);
        
        [FileName,PathName,FilterIndex] = uiputfile('*.txt','Export Table');
        
        if isnumeric(FileName)
            if FileName == 0
                msg{1}='No file name specified.';
                [msg]=ep_errorMsg(msg);
                return
            end;
        end;
        
        [pathstr, name, ext] = fileparts(FileName);
        if ~strcmp(ext,'.txt')
            FileName=[FileName '.txt'];
        end;
        
        fid=fopen([PathName FileName],'w');
        
        for i=1:size(tableData,1)
            for j=1:size(tableData,2)
                theCell=tableData{i,j};
                if isnumeric(theCell)
                    fprintf(fid,'%d', theCell);
                else
                    fprintf(fid,'%s', theCell);
                end;
                if j < size(tableData,2)
                    fprintf(fid,'\t');
                end;
            end;
            if i < size(tableData,1)
                fprintf(fid,'\r');
            end;
        end;
        
        ep_editData('start');
        
    case 'exportQC'
        
        if EPmain.noTable
            tableData=EPtable.edit.QC.tableData;
        else
            tableData=get(EPoverview.handles.QC.hTable,'Data');
        end;
        
        [FileName,PathName,FilterIndex] = uiputfile('*.txt','Export Table');
        
        if isnumeric(FileName)
            if FileName == 0
                msg{1}='No file name specified.';
                [msg]=ep_errorMsg(msg);
                return
            end;
        end;
        
        [pathstr, name, ext] = fileparts(FileName);
        if ~strcmp(ext,'.txt')
            FileName=[FileName '.txt'];
        end;
        
        fid=fopen([PathName FileName],'w');
        
        for i=1:size(tableData,1)
            for j=1:size(tableData,2)
                theCell=tableData{i,j};
                if isnumeric(theCell)
                    fprintf(fid,'%d', theCell);
                else
                    fprintf(fid,'%s', theCell);
                end;
                if j < size(tableData,2)
                    fprintf(fid,'\t');
                end;
            end;
            if i < size(tableData,1)
                fprintf(fid,'\r');
            end;
        end;
        
        ep_editData('start');
        
    case 'exportPCA'
        
        if EPmain.noTable
            tableData=EPtable.edit.PCA.tableData;
        else
            tableData=get(EPoverview.handles.PCA.hTable,'Data');
        end;
        
        [FileName,PathName,FilterIndex] = uiputfile('*.txt','Export Table');
        
        if isnumeric(FileName)
            if FileName == 0
                msg{1}='No file name specified.';
                [msg]=ep_errorMsg(msg);
                return
            end;
        end;
        
        [pathstr, name, ext] = fileparts(FileName);
        if ~strcmp(ext,'.txt')
            FileName=[FileName '.txt'];
        end;
        
        fid=fopen([PathName FileName],'w');
        
        for i=1:size(tableData,1)
            for j=1:size(tableData,2)
                theCell=tableData{i,j};
                if isnumeric(theCell)
                    fprintf(fid,'%d', theCell);
                else
                    fprintf(fid,'%s', theCell);
                end;
                if j < size(tableData,2)
                    fprintf(fid,'\t');
                end;
            end;
            if i < size(tableData,1)
                fprintf(fid,'\r');
            end;
        end;
        
        ep_editData('start');
        
    case 'done'
        try
            EPver=ver('EP_Toolkit');
        catch
            EPver='unavailable'; %workaround for bug in earlier version of Matlab
        end;
        EPoverview.workData.EPver=EPver;
        EPoverview.workData.ver=ver;
        EPoverview.workData.date=date;
        EPoverview.workData.history(end+1)={'ep_editData'};
        
        [err]=ep_checkEPfile(EPoverview.workData);
        
        EPdata=EPoverview.workData;
        
        if ~exist([EPdataset.EPwork filesep 'EPwork'],'dir')
            msg{1}='The work directory cannot be found.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        delete([EPdataset.EPwork filesep 'EPwork' filesep EPdataset.dataset(EPoverview.dataset).dataName '.mat']);
        EPdataset.dataset=EPdataset.dataset(setdiff([1:length(EPdataset.dataset)],EPoverview.dataset));
        EPdataset=ep_saveEPdataset(EPdataset,EPdata,EPoverview.dataset,'no');
        
        close('EditData');
        EPoverview=[];
        EPoverview='done';
        ep('start');
        
    case 'undo'
        
        if isempty(EPoverview.lastData)
            msg{1}='Nothing to undo.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        temp=EPoverview.workData;
        EPoverview.workData=EPoverview.lastData;
        EPoverview.lastData=temp;
        
        numPoints=length(EPoverview.workData.timeNames);
        numChans=length(EPoverview.workData.chanNames);
        numCells=length(unique(EPoverview.workData.cellNames));
        numWaves=length(EPoverview.workData.cellNames);
        numSubs=length(EPoverview.workData.subNames);
        numSpecs=length(EPoverview.workData.subjectSpecNames);
        numImplicit=length(EPoverview.workData.implicit);
        numFacs=length(EPoverview.workData.facNames);
        
        numEvents=0;
        for subject=1:numSubs
            for wave=1:numWaves
                numEvents=numEvents+length(EPoverview.workData.events{subject,wave});
            end;
        end;
        
        EPoverview.subjects.select=repmat(false,numSubs,1);
        EPoverview.cells.select=repmat(false,numCells,1);
        EPoverview.channels.select=repmat(false,numChans+numImplicit,1);
        EPoverview.subjects.weights=repmat(0,numSubs,1);
        EPoverview.cells.weights=repmat(0,numCells,1);
        EPoverview.channels.weights=repmat(0,numChans+numImplicit,1);
        EPoverview.events.select=repmat(false,numEvents,1);
        EPoverview.events.weights=repmat(0,numEvents,1);
        EPoverview.factors.select=repmat(false,numFacs,1);
        EPoverview.factors.weights=repmat(0,numFacs,1);
        
        EPtable=[];
        
        ep_editData('start');
        
    case 'cancel'
        close('EditData');
        EPoverview=[];
        
    otherwise
        disp(['Don''t recognize the task ' varargin{1}]);
        
end

if ~strcmp(varargin{1},{'cancel','done'})
    refresh;
    drawnow;
end;


