function ep_postprocessData(inputFiles,inputFormat,fileType,outputFormat,referenceMethod,refChan1,refChan2,baseline)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% ep_postprocessData(sessionFiles,inputFormat,fileType,outputFormat,referenceMethod,refChan1,refChan2,baseline)
% Rereferences and baseline corrects data.
% Will refuse to operate on a factor file as it would be invalid.  The PCA results are affected by whether the
% data are baseline corrected or how it is referenced so such operations should be applied prior to the PCA.
%
%	Reads in specified data files, rereferences them and baseline corrects them.
%   Saves the resulting output data.
%
%Inputs
%   inputFiles: cell array of file names to correct.  Assumes active directory if path not specified.  (default: will ask for files)
%   inputFormat: input file format.
%   fileType: input file type.
%   outputFormat: output file format.
%   referenceMethod: Type of referencing: 'none','average','traditional'.
%   refChan1: First channel used for traditional referencing.  Zero for not used.
%   refChan2: Second channel used for traditional referencing.  Zero for not used.
%   baseline: Samples used for baseline correction.  Empty if no baseline correction
%
%Outputs
%	Writes postprocessed file.
%
% History:
%
% by Joseph Dien (7/20/09)
% jdien07@mac.com
%
% bugfix 1/27/10 JD
% Update baseline field when using Postprocess function to baseline correct data.
%
% modified 2/16/10 JD
% analysis fields no longer optional.
% Eliminated chantype field for implicit channels.
%
% bugfix 2/27/10 JD
% Eliminated updating of baseline field.  The part of the segment used for baseline correction may not necessarily
% correspond to the prestimulus period.
%
% bugfix 5/26/10 JD
% Fixed crash when rereferencing and there are no implicit channels.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%     Copyright (C) 1999-2011  Joseph Dien
%
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
%
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
%
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

if isempty(inputFiles)
    [inputFiles, pathname] = uigetfile('*.*','Open:','MultiSelect','on');
    activeDirectory=pathname;
    if ~iscell(inputFiles)
        temp=inputFiles;
        inputFiles=[];
        inputFiles{1}=temp;
    end;
    if inputFiles{1}==0
        msg{1}='No filenames selected. You have to click on a name';
        [msg]=ep_errorMsg(msg);
        return
    end
    if ~iscell(inputFiles)
        temp=inputFiles;
        inputFiles=[];
        inputFiles{1}=temp;
    end;
    for theFile=1:size(inputFiles,2)
        inputFiles{theFile}=[activeDirectory inputFiles{theFile}];
    end;
else
    files=dir;
    for theFile=1:size(inputFiles,2)
        [activeDirectory, name, ext] = fileparts(inputFiles{theFile});
        if isempty(activeDirectory)
            activeDirectory=pwd;
            inputFiles{theFile}=[activeDirectory filesep inputFiles{theFile}];
        end;
        if ~exist(inputFiles{theFile},'file')
            msg{1}=['Error: The file ' inputFiles{theFile} ' is not in the directory.'];
            [msg]=ep_errorMsg(msg);
            return
        end;
    end;
end;

inputFiles=inputFiles';
sessionNum = size(inputFiles,1);

readArg{1}='format';
readArg{2}=inputFormat;
readArg{3}='type';
readArg{4}=fileType;

for file=1:length(inputFiles)
    Name=deblank(inputFiles{file});
    thisReadArg=readArg;
    thisReadArg{end+1}='file';
    [pathstr, fileName, ext]=fileparts(Name);
    thisReadArg{end+1}=Name;
    inputData=ep_readData(thisReadArg);
    
    if length(inputData.facNames) > 1
        msg{1}=['Error: The file ' fileName ' is a factor file.'];
        [msg]=ep_errorMsg(msg);
        return
    end;
    
    numChans=length(inputData.chanNames);
    numPoints=length(inputData.timeNames);
    numCells=length(unique(inputData.cellNames));
    numWaves=length(inputData.cellNames);
    numSubs=length(inputData.subNames);
    numFacs=size(inputData.data,5);
    numCMBfacs=size(inputData.facData,5);
    suffix=[];
    
    if ~strcmp(referenceMethod,'none')
        %make implicit reference channel explicit
        if ~isempty(inputData.implicit)
            theRef=find(strcmp({inputData.implicit.type},'REF'));
        else
            theRef=[];
        end;
        if length(theRef) == 1 %can't make implicit reference explicit if there is more than one such ref site
            inputData.data(end+1,:,:,:,:)=zeros(1,numPoints,numWaves,numSubs,numFacs);
            if ~isempty(inputData.facData)
                inputData.facData(end+1,:,:,:,:)=zeros(1,numPoints,numWaves,numSubs,numCMBfacs);
            end;
            if isfield(inputData,'noise')
                inputData.noise(end+1,:,:,:,:)=zeros(1,numPoints,numWaves,numSubs,numCMBfacs);
            end;
            if isfield(inputData,'std')
                inputData.std(end+1,:,:,:,:)=zeros(1,numPoints,numWaves,numSubs,numCMBfacs);
            end;
            inputData.chanNames{end+1}=inputData.implicit(theRef).labels;
            inputData.chanTypes{end+1}='EEG';
            theEloc=inputData.implicit(theRef);
            inputData.eloc(end+1)=theEloc;
            inputData.implicit=inputData.implicit([1:theRef-1 theRef+1:end]);
                    inputData.analysis.badChans(:,:,end+1)=zeros(numSubs,numWaves);

            
            numChans=numChans+1;
        end;
        
        suffix=[suffix 'r'];
        
        %rereference data
        switch referenceMethod
            case 'Average'
                referenceData=mean(inputData.data,1);
                for theChan=1:numChans
                    inputData.data(theChan,:,:,:,:)=inputData.data(theChan,:,:,:,:)-referenceData;
                end;
                if ~isempty(inputData.facData)
                    referenceData=mean(inputData.facData,1);
                    for theChan=1:numChans
                        inputData.facData(theChan,:,:,:,:)=inputData.facData(theChan,:,:,:,:)-referenceData;
                    end;
                end;
                if isfield(inputData,'noise')
                    referenceData=mean(inputData.noise,1);
                    for theChan=1:numChans
                        inputData.noise(theChan,:,:,:,:)=inputData.noise(theChan,:,:,:,:)-referenceData;
                    end;
                end;
                if isfield(inputData,'std')
                    referenceData=mean(inputData.std,1);
                    for theChan=1:numChans
                        inputData.std(theChan,:,:,:,:)=inputData.std(theChan,:,:,:,:)-referenceData;
                    end;
                end;
            case 'Traditional'
                refChans=[];
                if (refChan1 > 0) && (refChan1 <= numChans)
                    refChans=[refChans refChan1];
                    inputData.chanTypes{refChan1}='REF';
                end;
                if (refChan2 > 0) && (refChan2 <= numChans)
                    refChans=[refChans refChan2];
                    inputData.chanTypes{refChan2}='REF';
                end;
                if isempty(refChans)
                    msg{1}=['Error: Reference channels ' num2str(refChan1) ' and ' num2str(refChan2) ' invalid.'];
                    [msg]=ep_errorMsg(msg);
                    return
                end;
                referenceData=mean(inputData.data(refChans,:,:,:,:),1);
                for theChan=1:numChans
                    inputData.data(theChan,:,:,:,:)=inputData.data(theChan,:,:,:,:)-referenceData;
                end;
                if ~isempty(inputData.facData)
                    referenceData=mean(inputData.facData(refChans,:,:,:,:),1);
                    for theChan=1:numChans
                        inputData.facData(theChan,:,:,:,:)=inputData.facData(theChan,:,:,:,:)-referenceData;
                    end;
                end;
                if isfield(inputData,'noise')
                    referenceData=mean(inputData.noise(refChans,:,:,:,:),1);
                    for theChan=1:numChans
                        inputData.noise(theChan,:,:,:,:)=inputData.noise(theChan,:,:,:,:)-referenceData;
                    end;
                end;
                if isfield(inputData,'std')
                    referenceData=mean(inputData.std(refChans,:,:,:,:),1);
                    for theChan=1:numChans
                        inputData.std(theChan,:,:,:,:)=inputData.std(theChan,:,:,:,:)-referenceData;
                    end;
                end;
            otherwise
                msg{1}=['Error: Reference method ' referenceMethod ' not recognized.'];
                [msg]=ep_errorMsg(msg);
                return
        end;
    end;
    
    %baseline correct data
    if ~isempty(baseline)
        if isnumeric(baseline)
            if (max(baseline) <= numPoints) && min(baseline) > 0
                for theChan=1:numChans
                    baselineData=mean(inputData.data(theChan,baseline,:,:,:),2);
                    for thePoint=1:numPoints
                        inputData.data(theChan,thePoint,:,:,:)=inputData.data(theChan,thePoint,:,:,:)-baselineData;
                    end;
                end;
                if ~isempty(inputData.facData)
                    for theChan=1:numChans
                        baselineData=mean(inputData.facData(theChan,baseline,:,:,:),2);
                        for thePoint=1:numPoints
                            inputData.facData(theChan,thePoint,:,:,:)=inputData.facData(theChan,thePoint,:,:,:)-baselineData;
                        end;
                    end;
                end
                if isfield(inputData,'noise')
                    for theChan=1:numChans
                        baselineData=mean(inputData.noise(theChan,baseline,:,:,:),2);
                        for thePoint=1:numPoints
                            inputData.noise(theChan,thePoint,:,:,:)=inputData.noise(theChan,thePoint,:,:,:)-baselineData;
                        end;
                    end;
                end
                suffix=[suffix 'b'];
            else
                msg{1}=['Error: Baseline points ' num2str(baseline) ' invalid.'];
                [msg]=ep_errorMsg(msg);
                return
            end;
        else
            msg{1}=['Error: Baseline points ' baseline ' invalid.'];
            [msg]=ep_errorMsg(msg);
            return
        end;
    end;
    
    if ~isempty(suffix)
        [pathstr, name, ext] = fileparts(inputFiles{file});
        
        outFileName = [pathstr filesep name '_' suffix ext];
        
        [err]=ep_writeData(inputData,outFileName,outputFormat);
    end;
end;
