function ep_showTopos(startTime,endTime)
% ep_showTopos - ep_showTopos(startTime,endTime) -
% Displays the scalp topographies of the data and performs source analyses.
%
%Input:
%  EPtopos         : Structured array with the data and accompanying information.  See readData.
%
%  Time is counted for an epoch as, say, -200 to 800 ms.  In this case the samples have some length to them depending on
% the digitization rate, such as 4ms for a 250Hz sampling rate.  The points prior to the baseline are counted from the
% left side of the baseline sample and the points following the baseline are counted from the right side of the baseline
% sample.  Thus, when the actual samples are calculated, one first adds the baseline to the numbers, yielding 0 to
% 1000ms.  One then must add the length of the baseline to the first number to obtain 4 to 1000ms.  Then one would
% divide the numbers by the sample size, resulting in 1 to 250 samples, which are the correct samples to use.
% For continuous data or other such data where there is no baseline, one must by this convention start with 0 ms,
% as in 0-1000ms and 1000-2000ms.

%History
%  by Joseph Dien (4/20/10)
%  jdien07@mac.com

% bugfix 5/9/10 JD
% Fixed crash when using Topo button of View EEG pane and not all four colors are being used.
% Fixed can only change channel and latency settings for number of rows equal to number of pages of factors.
% Fixed sometimes crashes when first color is set to none.
%
% modified 5/15/10 JD
% Added white marker to topos for electrode corresponding to the waveform figures.
% Small black dots indicate electrode locations in topographical plots.
% May click on electrode dots to move the waveform plot channel.
% May right-click on topographical plot to obtain expanded 2D plot.
% May right-click on topographical plot to obtain expanded 3D plot.
% May right-click on topographical plot to obtain basic dipole analysis.
% Eliminated sorting of channel names.
%
% bugfix 6/6/10 JD
% Fixed crash when trying to display 3D plot or dipole source using .ced file generated from a .elp file.
%
% modified 6/15/10 JD
% Added dipole analysis of jack-knifed PCA results to the Topos function of the View Pane.
% Marks when a file isn't saved yet.
%
% bugfix 7/23/10 JD
% Fixed crash when doing dipole or 3D function and ced is either empty or "none".
% Fixed 3D, dipole, and jackknife results incorrect for topos not on the first page (when there are multiple pages of
% topos).
%
% bugfix 12/26/10 JD
% Fixed bottom of Topos window being cut off on laptop screens.
%
% bugfix 1/9/11 JD
% Fixed channels not being unstandardized correctly in jack-knife, resulting in some inaccuracy in dipole results.
% Fixed crash when trying to display data where some channels are missing electrode coordinates.
%
% bugfix 2/12/11 JD
% Fixed contents of topos window getting shifted upwards off window when OS X Dock is at bottom of screen.
%
% bugfix 2/14/11 JD
% Fixed crash when displaying data with only one timepoint.
%
% bugfix 3/24/11 JD
% Fixed crash when displaying data with only two timepoints.

%     Copyright (C) 1999-2011  Joseph Dien
%
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
%
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
%
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

global EPdataset EPmain EPtopos

scrsz = get(0,'ScreenSize');
windowHeight=scrsz(4)-100;
textColors={'blue','red',[0 .5 0],'black'};

if EPtopos.page==0 %set up topoplot for the first time
    
    EPtopos.RGBcolors=[0 0 1;1 0 0;0 1 0;0 0 0]; %colors for the waveforms
    
    EPtopos.handles.topos.topoWindow = figure('Name', 'TopoWindow', 'NumberTitle', 'off', 'Position',[201 101 800 windowHeight]);
    EPtopos.plotWidth=100;
    EPtopos.plotHeight=80;
    EPtopos.topoSize=80;
    EPtopos.margin=.1;
    
    EPtopos.perPage=min(floor(windowHeight/100)-1,10); %how many rows of figures per page
    EPtopos.page=1; %current page
    EPtopos.chans=[];
    EPtopos.points=[];
    
    %make sure the plotted datasets are compatible
    numChans=0;
    EPtopos.numFacs=0;
    for color=1:4
        if EPmain.view.dataset(color) <= length(EPdataset.dataset) %if not set to "none"
            if numChans == 0
                windowName=[EPdataset.dataset(EPmain.view.dataset(color)).dataName '-' EPdataset.dataset(EPmain.view.dataset(color)).subNames{EPmain.view.subject(color)}];
                set(EPtopos.handles.topos.topoWindow,'Name',windowName);
                EPtopos.firstTime=min([EPdataset.dataset(EPmain.view.dataset(color)).timeNames]);
                EPtopos.lastTime=max([EPdataset.dataset(EPmain.view.dataset(color)).timeNames]);
                EPtopos.sampleSize=1000/EPdataset.dataset(EPmain.view.dataset(color)).Fs;
                implicit=EPdataset.dataset(EPmain.view.dataset(color)).implicit; %use implicits of the first dataset
                EPtopos.chanNames=EPdataset.dataset(EPmain.view.dataset(color)).chanNames; %use channel names of the first dataset
                EPtopos.eloc=EPdataset.dataset(EPmain.view.dataset(color)).eloc;
                EPtopos.ced=EPdataset.dataset(EPmain.view.dataset(color)).ced;
                EPtopos.dataName=EPdataset.dataset(EPmain.view.dataset(color)).dataName;
                if ~isempty(implicit)
                    implicit=implicit(find(~strcmp('FID',{implicit.type})));
                    EPtopos.chanNames=[EPtopos.chanNames {implicit.labels}]; %use channel names of the first dataset
                    EPtopos.eloc=[EPtopos.eloc implicit];
                end;
                numChans=length(EPtopos.chanNames);
                EPtopos.numFacs=length(EPdataset.dataset(EPmain.view.dataset(color)).facNames);
                if isfield(EPtopos.eloc,'labels')
                    eLabels={EPtopos.eloc.labels};
                else
                    eLabels=cell(numChans,1);
                end;
                for chan=1:length(eLabels)
                    if isempty(eLabels{chan})
                        eLabels{chan}=EPtopos.chanNames{chan};
                    end;
                end;
                %                 [B,chanIX{color}] = sort(eLabels);
                %                 if ~isempty(EPtopos.eloc)
                %                     sortedEloc=EPtopos.eloc(chanIX{color});
                %                 end;
                %EPtopos.chanNames=EPtopos.chanNames(chanIX{color});
                EPtopos.theFirstColor=color;
            else
                newImplicit=EPdataset.dataset(EPmain.view.dataset(color)).implicit;
                if ~isempty(newImplicit)
                    newImplicit=newImplicit(find(~strcmp('FID',{newImplicit.type})));
                end;
                if numChans ~= length(EPdataset.dataset(EPmain.view.dataset(color)).chanNames)+length(newImplicit)
                    msg{1}='Error: The number of channels are not compatible.';
                    [msg]=ep_errorMsg(msg);
                    done
                    return
                end;
                EPtopos.firstTime =max(min([EPdataset.dataset(EPmain.view.dataset(color)).timeNames]),EPtopos.firstTime);
                EPtopos.lastTime = min(max([EPdataset.dataset(EPmain.view.dataset(color)).timeNames]),EPtopos.lastTime);
                if EPtopos.sampleSize ~= 1000/EPdataset.dataset(EPmain.view.dataset(color)).Fs
                    msg{1}='Error: The sampling rates are not compatible.';
                    [msg]=ep_errorMsg(msg);
                    done
                    return
                end;
                EPtopos.numFacs=max(EPtopos.numFacs,length(EPdataset.dataset(EPmain.view.dataset(color)).facNames));
                newEloc=[EPdataset.dataset(EPmain.view.dataset(color)).eloc newImplicit];
                if ~isempty(EPtopos.eloc) && ~isempty(newEloc)
                    eLabels={newEloc.labels};
                    for chan=1:length(eLabels)
                        if isempty(eLabels{chan})
                            eLabels{chan}=EPdataset.dataset(EPmain.view.dataset(color)).chanNames{chan};
                        end;
                    end;
                    %[B,chanIX{color}] = sort(eLabels);
                    %newEloc=newEloc(chanIX{color});
                    if any([EPtopos.eloc.theta]-[newEloc.theta]) || any([EPtopos.eloc.radius]-[newEloc.radius])
                        msg{1}='Error: The electrode coordinates are not compatible.';
                        [msg]=ep_errorMsg(msg);
                        done
                        return
                    end;
                elseif xor(isempty(EPtopos.eloc),isempty(newEloc)) %if one but not the other has no electrode coordinates
                    msg{1}='Error: The electrode coordinates are not compatible.';
                    [msg]=ep_errorMsg(msg);
                    done
                    return
                else
                    %if both have no electrode coordinates
                    %chanIX{color}=chanIX{1};
                end;
            end;
        end;
    end;
    
    EPtopos.numFacs=max(1,EPtopos.numFacs); %numFacs should be at least one
    EPtopos.numPages=ceil(EPtopos.numFacs/EPtopos.perPage);
    
    EPtopos.firstTime=max(EPtopos.firstTime,startTime); %start time counting from left side of each sample
    EPtopos.lastTime=min(EPtopos.lastTime,endTime); %end time counting from left side of each sample
    
    %compute epoch samples for each color
    EPtopos.numPoints=0;
    for color=1:4
        if EPmain.view.dataset(color) <= length(EPdataset.dataset)
            startSamp(color)=(EPtopos.firstTime/EPtopos.sampleSize)+EPdataset.dataset(EPmain.view.dataset(color)).baseline+1;
            lastSamp(color)=(EPtopos.lastTime/EPtopos.sampleSize)+EPdataset.dataset(EPmain.view.dataset(color)).baseline+1;
            EPtopos.numPoints=lastSamp(color)-startSamp(color)+1;
        end;
    end
    
    %organize the data for plotting
    EPtopos.plotColors=[];
    EPtopos.thePlotColors=[];
    EPtopos.totalData=zeros(numChans,EPtopos.numPoints,EPtopos.numFacs,4); %default is zero voltage if, for example, two factor datasets with different numbers of factors
    theDataset=0;
    for color=1:4
        if EPmain.view.dataset(color) <= length(EPdataset.dataset)
            if theDataset ~= EPmain.view.dataset(color) %load in the dataset if different from the one already loaded in.
                EPdata=ep_loadEPdataset(EPdataset,EPmain.view.dataset(color));
                theDataset=EPmain.view.dataset(color);
            end;
            if isempty(EPdata.trialNames) %averaged data
                theCell=EPmain.view.cell(color);
            else  %if single_trial data
                theCell=intersect(find(strcmp(EPmain.view.theCells{EPmain.view.dataset(color)}(EPmain.view.cell(color)),EPdata.cellNames)),...
                    find(EPmain.view.trial(color)==EPdata.trialNames));
            end;
            facList=[1:length(EPdataset.dataset(EPmain.view.dataset(color)).facNames)];
            if isempty(facList)
                facList=[1:EPtopos.numFacs]; %if not a factor file, then just add the one set of data to all the rows.
                theData=ep_expandFacs(EPdata,[],startSamp(color):lastSamp(color),theCell,EPmain.view.subject(color),[]);
                for fac=facList
                    EPtopos.totalData(1:length(EPdata.chanNames),1:size(theData,2),fac,color)=theData;
                end;
            else
                EPtopos.totalData(1:length(EPdata.chanNames),:,facList,color)=ep_expandFacs(EPdata,[],startSamp(color):lastSamp(color),theCell,EPmain.view.subject(color),[]);
                if isfield(EPdata,'pca')
                    if isfield(EPdata.pca,'jack')
                        if isfield(EPdata.pca.jack,'FacPat')
                            for theJK=1:size(EPdata.pca.jack.FacPat,3)
                                for theFac=1:size(EPdata.pca.jack.FacPat,2)
                                    EPtopos.jack(color).FacPat(:,theFac,theJK)=EPdata.pca.jack.FacPat(:,theFac,theJK).*EPdata.pca.jack.varSD(:,theJK);
                                    EPtopos.jack(color).PCAmode=EPdata.pca.PCAmode;
                                end;
                            end;
                        elseif isfield(EPdata.pca.jack,'FacPatST')
                            for theJK=1:size(EPdata.pca.jack.FacPatST,3)
                                for theFac=1:size(EPdata.pca.jack.FacPatST,2)
                                    EPtopos.jack(color).FacPat(:,theFac,theJK)=EPdata.pca.jack.FacPatST(:,theFac,theJK).*EPdata.pca.jack.varSDST(floor((theFac-1)/EPdata.pca.numFacs2)+1,:,theJK)';
                                    EPtopos.jack(color).PCAmode=EPdata.pca.PCAmode2;
                                end;
                            end;
                        end;
                    end;
                end;
                
            end;
            
            
            %EPtopos.totalData(:,:,:,color)=EPtopos.totalData(chanIX{color},:,:,color); %rearrange order of channels to be consistent with other datasets
            EPtopos.plotColors=[EPtopos.plotColors color];
            EPtopos.thePlotColors=[EPtopos.thePlotColors; EPtopos.RGBcolors(color,:)];
            
            if isempty(EPtopos.chans)
                %find max channels and latencies, based on the first non-empty dataset
                EPtopos.chans=ones(EPtopos.numFacs,1); %default chan is one
                EPtopos.points=ones(EPtopos.numFacs,1); %default point is one
                for theFactor=1:length(facList)
                    [C EPtopos.points(theFactor)]=max(max(reshape(shiftdim(abs(EPtopos.totalData(:,:,theFactor,color)),1),EPtopos.numPoints,[])'));
                    [C EPtopos.chans(theFactor)]=max(max(reshape(abs(EPtopos.totalData(:,:,theFactor,color)),numChans,[])'));
                end;
            end;
        end;
    end;
    
    EPtopos.plotMVmin=min(min(min(min(EPtopos.totalData))));
    EPtopos.plotMVmax=max(max(max(max(EPtopos.totalData))));
end;

if EPtopos.page < EPtopos.numPages
    numRows=EPtopos.perPage;
else
    numRows=mod(EPtopos.numFacs-1,EPtopos.perPage)+1;
end;

clf(EPtopos.handles.topos.topoWindow)

EPtopos.handles.done = uicontrol('Style', 'pushbutton', 'String', 'Done',...
    'Position', [20 windowHeight-110 50 30], 'Callback', @done);

EPtopos.handles.plotMVmin = uicontrol('Style','edit',...
    'String',num2str(EPtopos.plotMVmin),'HorizontalAlignment','left',...
    'Position',[100 windowHeight-110 50 20],'Callback',@changeMV);

EPtopos.handles.plotMVmax = uicontrol('Style','edit',...
    'String',num2str(EPtopos.plotMVmax),'HorizontalAlignment','left',...
    'Position',[160 windowHeight-110 50 20],'Callback',@changeMV);

for color=1:4
    if EPmain.view.dataset(color) <= length(EPdataset.dataset)
        uicontrol('Style','text','ForegroundColor',textColors{color},...
            'String',EPmain.view.theCells{EPmain.view.dataset(color)}{EPmain.view.cell(color)},'HorizontalAlignment','left',...
            'Position',[100+EPtopos.plotWidth+50+(color-1)*(EPtopos.topoSize+50) windowHeight-110 80 20]);
    else
        uicontrol('Style','text','ForegroundColor',textColors{color},...
            'String','none','HorizontalAlignment','left',...
            'Position',[100+EPtopos.plotWidth+50+(color-1)*(EPtopos.topoSize+50) windowHeight-110 80 20]);
    end;
end;

if EPtopos.numPages > 1
    %page buttons
    if EPtopos.numFacs <= 100
        for i=1:EPtopos.numPages
            EPtopos.handles.pageButton(i) = uicontrol('Style', 'pushbutton', 'String', num2str(i*EPtopos.perPage),...
                'Position', [(20 + (i-1)*40) windowHeight-70 30 20], 'Callback', ['global EPtopos;','EPtopos.page=' num2str(i) ';','ep_showTopos(EPtopos.firstTime,EPtopos.lastTime)']);
        end;
        set(EPtopos.handles.pageButton(EPtopos.page),'ForegroundColor','blue');
    else
        EPtopos.handles.pageMenu = uicontrol('Style', 'popupmenu', 'String', num2str([ceil((1:EPtopos.numPages)*EPtopos.perPage)]'),'Value',EPtopos.page,...
            'Position', [20 windowHeight-70 100 20],...
            'Callback', ['global EPtopos;','temp=get(EPtopos.handles.pageMenu,''Value'');','if temp ~=0,EPtopos.page=temp;end;','ep_showTopos(EPtopos.firstTime,EPtopos.lastTime)']);
    end;
end;

%start drawing the figures
set(EPtopos.handles.topos.topoWindow ,'DefaultAxesXTickLabel',[])
set(EPtopos.handles.topos.topoWindow ,'DefaultAxesYTickLabel',[])
set(EPtopos.handles.topos.topoWindow ,'DefaultAxesXTick',[])
set(EPtopos.handles.topos.topoWindow ,'DefaultAxesYTick',[])
set(EPtopos.handles.topos.topoWindow,'DefaultAxesColorOrder',EPtopos.thePlotColors)

numChans=length(EPtopos.eloc);
%start code from EEGlab's topoplot

locChans=true(numChans,1);
Rd=[];
Th=[];
for chan=1:numChans
    theRd=EPtopos.eloc(chan).radius;
    if isempty(theRd)
        locChans(chan)=false;
    end;
    theTheta=EPtopos.eloc(chan).theta;
    if isempty(theTheta)
        locChans(chan)=false;
    end;
    if locChans(chan)
        Rd=[Rd theRd];
        Th=[Th theTheta];
    end;
end;
EPtopos.locChans=find(locChans);

Th = pi/180*Th; 
[x,y] = pol2cart(Th,Rd);
plotrad = min(1.0,max(Rd)*1.02);            % default: just outside the outermost electrode location
plotrad = max(plotrad,0.5);                 % default: plot out to the 0.5 head boundary
squeezefac = .5/plotrad;
x    = x*squeezefac;    
y    = y*squeezefac;  
%end code from EEGlab's topoplot

for row=1:numRows
    theFac=(EPtopos.page-1)*EPtopos.perPage+row;
    if ~isempty(EPdataset.dataset(EPmain.view.dataset(EPtopos.theFirstColor)).facNames)
        uicontrol('Style','text',...
            'String',EPdataset.dataset(EPmain.view.dataset(EPtopos.theFirstColor)).facNames{theFac},'HorizontalAlignment','left',...
            'Position',[20 windowHeight-(140+(row-1)*100) 60 20]);
    end
    EPtopos.handles.latency(row)=uicontrol('Style','edit',...
        'String',num2str(EPtopos.firstTime+EPtopos.points(theFac)*EPtopos.sampleSize),'HorizontalAlignment','left',...
        'Position',[20 windowHeight-(160+(row-1)*100) 50 20],...
        'Callback',@changeLatency);
    EPtopos.handles.channel(row)=uicontrol('Style','popupmenu',...
        'String',EPtopos.chanNames',...
        'Value',EPtopos.chans(theFac),...
        'Position',[15 windowHeight-(180+(row-1)*100) 80 20],...
        'Callback',@changeChannel);
    
    %draw waveforms
    if EPtopos.numPoints > 1
        EPtopos.handles.topos.hWave(row) = axes('Units','pixel','position',[100 windowHeight-(200+(row-1)*100) EPtopos.plotWidth EPtopos.plotHeight]);
        EPtopos.handles.topos.hLines{row} = plot([EPtopos.firstTime:EPtopos.sampleSize:EPtopos.lastTime],squeeze(EPtopos.totalData(EPtopos.chans(theFac),:,theFac,EPtopos.plotColors)));
        axis([EPtopos.firstTime EPtopos.lastTime EPtopos.plotMVmin EPtopos.plotMVmax]); %left side of first sample to left side of last sample
        if EPtopos.direction ==2
            set(EPtopos.handles.topos.hWave(row),'YDir','reverse')
        end;
        line([EPtopos.firstTime+EPtopos.sampleSize EPtopos.lastTime],[0 0],'Color','black','LineWidth',1) % zero line
        line([0 0],[0 EPtopos.plotMVmax],'Color','black','LineWidth',1) %stimulus onset
        %text(EPtopos.firstTime+(EPtopos.lastTime-EPtopos.firstTime)/20, EPtopos.plotMVmin+(EPtopos.plotMVmax-EPtopos.plotMVmin)/10, EPtopos.chanNames(EPtopos.chans(theFac)));
        line(repmat(EPtopos.firstTime+EPtopos.points(theFac)*EPtopos.sampleSize,length([EPtopos.plotMVmin:EPtopos.plotMVmax])),[EPtopos.plotMVmin:EPtopos.plotMVmax],'Color',[.5 .5 .5],'LineWidth',1);
    end;
    
    %draw topos
    for col=1:4
        if EPmain.view.dataset(col) <= length(EPdataset.dataset)
            EPtopos.handles.topos.topo(row,col) = axes('Units','pixel','position',[100+EPtopos.plotWidth+50+(col-1)*(EPtopos.topoSize+50) windowHeight-(200+(row-1)*100) EPtopos.topoSize EPtopos.topoSize]);
            el_topoplot(EPtopos.totalData(:,EPtopos.points(theFac),theFac,col), EPtopos.eloc,'maplimits',[EPtopos.plotMVmin EPtopos.plotMVmax]);
            line([y(EPtopos.chans(theFac)) y(EPtopos.chans(theFac))],[x(EPtopos.chans(theFac)) x(EPtopos.chans(theFac))],'Marker','o','MarkerFaceColor','white'); %add marker for the electrode of the waveform plot
            for i=1:length(EPtopos.locChans)
                EPtopos.handles.topos.line(row,col,i)=line([y(i) y(i)],[x(i) x(i)],'color','black','ButtonDownFcn',['global EPtopos;','sel_typ = get(gcbf,''SelectionType'');','if strcmp(sel_typ,''normal'')','EPtopos.chans(' num2str(theFac) ')=' num2str(EPtopos.locChans(i)) ';','ep_showTopos(EPtopos.firstTime,EPtopos.lastTime);','end;']);
            end;
            
            %         EPtopos.handles.topos.d2Button(row,col) = uicontrol('Style', 'pushbutton', 'String', '2D',...
            %             'Position', [100+EPtopos.plotWidth+50+(col-1)*(EPtopos.topoSize+50) windowHeight-(200+(row-1)*100)-20 30 20],...
            %             'Callback', ['global EPtopos;','figure;','el_topoplot(EPtopos.totalData(:,EPtopos.points(' num2str(theFac) '),' num2str(theFac) ',' num2str(col) '), EPtopos.eloc,''maplimits'',[EPtopos.plotMVmin EPtopos.plotMVmax]);']);
            
            hcmenu = uicontextmenu;
            % Define callbacks for context menu items that change linestyle
            hcb1 = ['global EPtopos;','figure;','el_topoplot(EPtopos.totalData(:,EPtopos.points(' num2str(theFac) '),' num2str(theFac) ',' num2str(col) '), EPtopos.eloc,''maplimits'',[EPtopos.plotMVmin EPtopos.plotMVmax]);'];
            hcb3 = ['set(gco, ''LineStyle'', ''-'')'];
            % Define the context menu items and install their callbacks
            item1 = uimenu(hcmenu, 'Label', '2D', 'Callback', hcb1);
            item2 = uimenu(hcmenu, 'Label', '3D', 'Callback', @D3head);
            item3 = uimenu(hcmenu, 'Label', 'Dipole',  'Callback', @dipoles);
            item4 = uimenu(hcmenu, 'Label', 'Jack-Knife',  'Callback', @jackknife);
            % Attach the context
            set(EPtopos.handles.topos.topo(row,col),'UIContextMenu',hcmenu)
            theChildren=get(EPtopos.handles.topos.topo(row,col),'Children');
            for i=1:length(theChildren)
                set(theChildren(i),'UIContextMenu',hcmenu)
            end;
            for i=1:length(EPtopos.locChans)
                set(EPtopos.handles.topos.line(row,col,i),'UIContextMenu',hcmenu)
            end;
        end;
    end;
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function done(src,eventdata)
global EPtopos EPmain EPdataset

%save results of jack-knife PCA dipole analyses
if isfield(EPtopos,'jack')
    for col=1:4
        if EPmain.view.dataset(col) <= length(EPdataset.dataset) %if not set to "none"
            if ~isempty(EPtopos.jack(col))
                if isfield(EPtopos.jack(col),'sources')
                    if ~isempty(EPtopos.jack(col).sources)
                        EPdata=ep_loadEPdataset(EPdataset,EPmain.view.dataset(col));
                        EPdata.pca.jack.sources=EPtopos.jack(col).sources;
                        
                        try
                            EPver=ver('EP_Toolkit');
                        catch
                            EPver='unavailable'; %workaround for bug in earlier version of Matlab
                        end;
                        EPdata.EPver=EPver;
                        EPdata.ver=ver;
                        EPdata.date=date;
                        EPdata.history(end+1)={'PCA jackknife source analysis'};
                        
                        [err]=ep_checkEPfile(EPdata);
                        
                        if ~exist([EPdataset.EPwork filesep 'EPwork'],'dir')
                            warndlg('The work directory cannot be found.')
                            return
                        end;
                        
                        delete([EPdataset.EPwork filesep 'EPwork' filesep EPdataset.dataset(EPmain.view.dataset(col)).dataName '.mat']);
                        EPdataset.dataset=EPdataset.dataset(setdiff([1:length(EPdataset.dataset)],EPmain.view.dataset(col)));
                        EPdataset=ep_saveEPdataset(EPdataset,EPdata,EPmain.view.dataset(col),'no');
                    end;
                end;
            end;
        end;
    end;
end;

set(EPmain.handles.view.waves,'enable','on');
set(EPmain.handles.view.topos,'enable','on');
set(EPmain.handles.view.done,'enable','on');
set(EPmain.handles.view.marker1,'enable','on');
set(EPmain.handles.view.marker2,'enable','on');
set(EPmain.handles.view.leftEpoch,'enable','on');
set(EPmain.handles.view.rightEpoch,'enable','on');
set(EPmain.handles.view.topVolt,'enable','on');
set(EPmain.handles.view.bottomVolt,'enable','on');
for color=1:4
    set(EPmain.handles.view.dataset(color),'enable','on');
    if EPmain.view.dataset(color) <= length(EPdataset.dataset)
        set(EPmain.handles.view.cell(color),'enable','on');
        set(EPmain.handles.view.subject(color),'enable','on');
        set(EPmain.handles.view.trial(color),'enable','on');
        set(EPmain.handles.view.factor(color),'enable','on');
    end;
end;

close(EPtopos.handles.topos.topoWindow);
ep('start');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function changeMV(src,eventdata)
global EPtopos

plotMVmin=str2num(get(EPtopos.handles.plotMVmin,'string'));
plotMVmax=str2num(get(EPtopos.handles.plotMVmax,'string'));

if plotMVmin < plotMVmax
    EPtopos.plotMVmin=plotMVmin;
    EPtopos.plotMVmax=plotMVmax;
end;

ep_showTopos(EPtopos.firstTime,EPtopos.lastTime);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function changeLatency(src,eventdata)
global EPtopos

if EPtopos.page < EPtopos.numPages
    numRows=EPtopos.perPage;
else
    numRows=mod(EPtopos.numFacs-1,EPtopos.perPage)+1;
end;

for row=1:numRows
    theFac=(EPtopos.page-1)*EPtopos.perPage+row;
    theLatency=str2num(get(EPtopos.handles.latency(row),'string'));
    if theLatency ~= EPtopos.firstTime+EPtopos.points(theFac)*EPtopos.sampleSize
        if (theLatency >= EPtopos.firstTime) && (theLatency <= EPtopos.lastTime)
            newSample=round((theLatency-EPtopos.firstTime)/EPtopos.sampleSize);
            if (newSample > 0) && (newSample <= EPtopos.numPoints)
                EPtopos.points(theFac)=round((theLatency-EPtopos.firstTime)/EPtopos.sampleSize);
            end;
        end;
    end;
end;
            
ep_showTopos(EPtopos.firstTime,EPtopos.lastTime);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function changeChannel(src,eventdata)
global EPtopos

if EPtopos.page < EPtopos.numPages
    numRows=EPtopos.perPage;
else
    numRows=mod(EPtopos.numFacs-1,EPtopos.perPage)+1;
end;

for row=1:numRows
    theFac=(EPtopos.page-1)*EPtopos.perPage+row;
    theChannel=get(EPtopos.handles.channel(row),'value');
    EPtopos.chans(theFac)=theChannel;
end;
            
ep_showTopos(EPtopos.firstTime,EPtopos.lastTime);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function D3head(src,eventdata)
global EPtopos

[row,col,lin] = ind2sub(size(EPtopos.handles.topos.line),find(EPtopos.handles.topos.line==gco));

if isempty(row)
    [row,col] = ind2sub(size(EPtopos.handles.topos.topo),find(EPtopos.handles.topos.topo==gco));
end;

if isempty(row)
    temp=get(gco);
    [row,col] = ind2sub(size(EPtopos.handles.topos.topo),find(EPtopos.handles.topos.topo==temp.Parent));
end;

row=row+EPtopos.perPage*(EPtopos.page-1);

%check to see if spline file needs to be generated
if ~isempty(EPtopos.ced) && ~strcmp('none',EPtopos.ced)
    [pathstr, name, ext] = fileparts(EPtopos.ced);
    CEDloc=which(EPtopos.ced);
else
    name=EPtopos.dataName;
    CEDloc=[pwd filesep 'temp'];
end;
if isempty(which([name '.spl']))
    [pathstr2, name2, ext2] = fileparts(CEDloc);
    if max(abs([EPtopos.eloc.X])) <= 1
        MNItransform=[ 0 -15 0 0.08 0 -1.571 102 93 100 ]; %assume eloc coordinates are from a .elp file
    else
        MNItransform=[ 0 -15 4 0.05 0 -1.571 10.2 12 12.2 ]; %assume eloc coordinates are from a .sfp file
    end;
    headplot('setup', EPtopos.eloc, [pathstr2 filesep name '.spl'],'transform',MNItransform); %save spline file in same place as the ced file.
end;

figure
[hdaxis cbaraxis] = headplot(EPtopos.totalData(:,EPtopos.points(row),row,col),which([name '.spl']),'cbar',0,'maplimits',[EPtopos.plotMVmin EPtopos.plotMVmax]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function sources=computeDipoles(topo,eloc,sampleSize)

data=[];
cfg=[];

%adapted from EEGlab's eeglab2fieldtrip under GPL
data.elec.pnt   = zeros(length( eloc ), 3);
for ind = 1:length( eloc )
    data.elec.label{ind} = eloc(ind).labels;
    if ~isempty(eloc(ind).X)
        data.elec.pnt(ind,1) = eloc(ind).X;
        data.elec.pnt(ind,2) = eloc(ind).Y;
        data.elec.pnt(ind,3) = eloc(ind).Z;
    else
        data.elec.pnt(ind,:) = [0 0 0];
    end;
end;

if max(abs([eloc.X])) <= 1
    MNItransform=[ 0 -15 0 0.08 0 -1.571 102 93 100 ]; %assume eloc coordinates are from a .elp file
else
    MNItransform=[ 0 -15 4 0.05 0 -1.571 10.2 12 12.2 ]; %assume eloc coordinates are from a .sfp file
end;
transfmat = traditionaldipfit(MNItransform);
data.elec.pnt = transfmat * [ data.elec.pnt ones(size(data.elec.pnt,1),1) ]';
data.elec.pnt = data.elec.pnt(1:3,:)';
%end adapted section

data.fsample = 1000/(sampleSize);
data.avg  = topo;
data.trial{1}  = topo;
data.var  = ones(length(topo),1);
data.time{1} = 0;
data.label = { eloc.labels };

cfg.numdipoles  = 2;
cfg.symmetry    = 'x';
cfg.channel     = 'all';
cfg.gridsearch  = 'yes';
cfg.nonlinear   = 'yes';
cfg.hdmfile     = which('standard_BESA.mat');
cfg.grid.xgrid  = 'auto';
cfg.grid.ygrid  = 'auto';
cfg.grid.zgrid  = 'auto';
cfg.grid.resolution = 50;

[source] = ft_dipolefitting(cfg, data);

sources.momxyz(1,1:3)=source.dip.mom(1:3);
sources.momxyz(2,1:3)=source.dip.mom(4:6);

if (abs(source.dip.pos(1,1)) < .1) && (abs(source.dip.pos(2,1)) < .1)
    disp('Restarting dipole analysis with one dipole - two dipole solution blew up.');
    cfg.numdipoles  = 1;
    cfg.symmetry    = [];
    [source] = ft_dipolefitting(cfg, data);
    sources.momxyz=[];
    sources.momxyz(1,1:3)=source.dip.mom(1:3);
end;

sources.posxyz=source.dip.pos;
sources.rv=source.dip.rv;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function dipoles(src,eventdata)
global EPtopos

[row,col,lin] = ind2sub(size(EPtopos.handles.topos.line),find(EPtopos.handles.topos.line==gco));

if isempty(row)
    [row,col] = ind2sub(size(EPtopos.handles.topos.topo),find(EPtopos.handles.topos.topo==gco));
end;

if isempty(row)
    temp=get(gco);
    [row,col] = ind2sub(size(EPtopos.handles.topos.topo),find(EPtopos.handles.topos.topo==temp.Parent));
end;

row=row+EPtopos.perPage*(EPtopos.page-1);

topo=squeeze(EPtopos.totalData(:,EPtopos.points(row),row,col));
eloc=EPtopos.eloc;
sources=computeDipoles(topo,eloc,EPtopos.sampleSize);

dipplot(sources,'mri',which('avg152t1.mat'),'color', { 'g' 'b' },'coordformat','MNI','verbose','off');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function jackknife(src,eventdata)
global EPtopos

[row,col,lin] = ind2sub(size(EPtopos.handles.topos.line),find(EPtopos.handles.topos.line==gco));

if isempty(row)
    [row,col] = ind2sub(size(EPtopos.handles.topos.topo),find(EPtopos.handles.topos.topo==gco));
end;

if isempty(row)
    temp=get(gco);
    [row,col] = ind2sub(size(EPtopos.handles.topos.topo),find(EPtopos.handles.topos.topo==temp.Parent));
end;

row=row+EPtopos.perPage*(EPtopos.page-1);

if ~isfield(EPtopos,'jack')
    msg{1}='Error: Jack-knife dipole analysis only works with PCA dataset.';
    [msg]=ep_errorMsg(msg);
    return
end;

if ~strcmp(EPtopos.jack(col).PCAmode,'spat')
    msg{1}='Error: Jack-knife dipole analysis only works with spatial or temporo-spatial PCA dataset.';
    [msg]=ep_errorMsg(msg);
    return
end;

eloc=EPtopos.eloc;
topo=squeeze(EPtopos.totalData(:,EPtopos.points(row),row,col));
sources=computeDipoles(topo,eloc,EPtopos.sampleSize);
dipColors{1}='r';
for theJK=1:size(EPtopos.jack(col).FacPat,3)
    topo=EPtopos.jack(col).FacPat(:,(EPtopos.page-1)*EPtopos.perPage+row,theJK);
    sources(end+1)=computeDipoles(topo,eloc,EPtopos.sampleSize);
    dipColors{end+1}='b';
end;

EPtopos.jack(col).sources=sources;

dipplot(sources,'mri',which('avg152t1.mat'),'color', dipColors,'coordformat','MNI','verbose','off','spheres','on','dipolelength',0,'summary','3d');

%jack-knife t-test for hemispheric main effect
%using statistical test presented by: Miller, J., Patterson, T., & Ulrich, R. (1998). Jackknife-based method for measuring LRP onset latency differences. Psychophysiology, 35(1), 99-115.

numJK=length(sources);
DipAmp=zeros(numJK,2);

numJKpairs=0;
for JK=1:numJK
    momxyz=EPtopos.jack(col).sources(JK).momxyz;
    if size(momxyz,1) == 2
        numJKpairs=numJKpairs+1;
        DipAmp(numJKpairs,1)=sqrt(momxyz(1,1)^2+momxyz(1,2)^2+momxyz(1,3)^2);
        DipAmp(numJKpairs,2)=sqrt(momxyz(2,1)^2+momxyz(2,2)^2+momxyz(2,3)^2);
    end;
end;

JD=DipAmp(1:numJKpairs,1)-DipAmp(1:numJKpairs,2);
JM=mean(JD);

SDjk=sqrt(((numJKpairs-1)/numJKpairs)*sum((JD-JM).^2));

if numJKpairs > 1
    disp(['The t-value for the amplitude of the two hemispheric dipoles is ' num2str(sum(JD)/SDjk) ' with ' num2str(numJKpairs-1) ' degrees of freedom.']);
    if sum(JD) > 0 
        disp('Right hemisphere larger than left.');
    elseif sum(JD) < 0 
        disp('Left hemisphere larger than right.');
    else
        disp('No difference between hemispheres.');
    end;
    disp('Note that I have not worked out yet how to generalize the robust statistic to the jack-knife inferential test.');
else
    disp('Too few solutions with hemispheric dipoles to calculate hemispheric t-test.');
end;




