function ep_showWaves(plotMVmin,plotMVmax,startTime,endTime,marker1,marker2)
% ep_showWaves - ep_showWaves(plotMVmin,plotMVmax,startTime,endTime,marker1,marker2) -
% Displays the waveforms of the data.
%
%Input:
%  EPwaves         : Structured array with the data and accompanying information.  See readData.
%
%  Time is counted for an epoch as, say, -200 to 800 ms.  In this case the samples have some length to them depending on
% the digitization rate, such as 4ms for a 250Hz sampling rate.  The points prior to the baseline are counted from the
% left side of the baseline sample and the points following the baseline are counted from the right side of the baseline
% sample.  Thus, when the actual samples are calculated, one first adds the baseline to the numbers, yielding 0 to
% 1000ms.  One then must add the length of the baseline to the first number to obtain 4 to 1000ms.  Then one would
% divide the numbers by the sample size, resulting in 1 to 250 samples, which are the correct samples to use.
% For continuous data or other such data where there is no baseline, one must by this convention start with 0 ms,
% as in 0-1000ms and 1000-2000ms.

%History
%  by Joseph Dien (5/15/09)
%  jdien07@mac.com
%
%  bugfix 8/27/09 JD
%  Location of windows appearing partly off screen on some systems.  Fixed.
%
%  modified 1/16/10 JD
%  Rounds off ms labels in the scale graph.
%
%  modified 1/26/10 JD
%  Sped up waveform drawing by relying more on EPwork cache.
%  Added ability to plot data from continuous data files by showing only one second at a time.
%  Epoch ms now displayed as from beginning of first sample to end of last sample.
%
%  modified 2/16/10 JD
%  Can now handle displaying waves from multiple datasets where some of the channels are implicit in one but not the
%  other dataset.
%  Eliminated chantype field for implicit channels.
%
%  bugfix 2/27/10 JD
%  Fixed crash when viewing data with regional channels.
%
%  bugfix 3/16/10 JD
%  Fixed crash when viewing data with no implicit channels.
%
%  bugfix 3/23/10 JD
%  Fixed crash when viewing data with no electrode coordinate information.
%
%  modified 3/28/10 JD
%  Reduced memory use of figures by odd workaround of dividing the data by an arbitrary scaling factor.
%
%  bugfix 3/30/10 JD
%  Fixed baseline bar not appearing in expanded waveform figures.
%  Fixed stimulus onset bar not appearing in waveforms figures.
%  Fixed baseline bar and stimulus onset bar not appearing in expanded waveform figures when click lands on a waveform.
%  Fixed real reason figures using much more memory than needed.
%
%  modified 4/23/10 JD
%  Scale of waveforms no longer constrained to be at least +/- 1 microvolt.


%     Copyright (C) 1999-2011  Joseph Dien
%
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
%
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
%
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

global EPdataset EPmain EPwaves

%EPwaves.plotMVmin=min([plotMVmin; -1]);
%EPwaves.plotMVmax=max([plotMVmax; 1]);
EPwaves.plotMVmin=plotMVmin;
EPwaves.plotMVmax=plotMVmax;

EPwaves.marker1=marker1;
EPwaves.marker2=marker2;

scrsz = get(0,'ScreenSize');
windowHeight=scrsz(4);

EPwaves.RGBcolors=[0 0 1;1 0 0;0 1 0;0 0 0]; %colors for the waveforms

EPwaves.handles.waves.hWaveWindow = figure('Name', 'WaveWindow', 'NumberTitle', 'off', 'Position',[201 1 scrsz(3)-200 scrsz(4)]);
%, 'MenuBar', 'none'
EPwaves.plotWidth=.05;
EPwaves.plotHeight=.04;
EPwaves.margin=.1;

%make sure the plotted datasets are compatible
numChans=0;
for color=1:4
    if EPmain.view.dataset(color) <= length(EPdataset.dataset) %if not set to "none"
        if numChans == 0
            windowName=[EPdataset.dataset(EPmain.view.dataset(color)).dataName '-' EPdataset.dataset(EPmain.view.dataset(color)).subNames{EPmain.view.subject(color)}];
            if ~isempty(EPdataset.dataset(EPmain.view.dataset(color)).facNames)
                windowName=[windowName '-' EPdataset.dataset(EPmain.view.dataset(color)).facNames{EPmain.view.factor(color)}];
            end;
            set(EPwaves.handles.waves.hWaveWindow,'Name',windowName);
            EPwaves.firstTime=min([EPdataset.dataset(EPmain.view.dataset(color)).timeNames]);
            EPwaves.lastTime=max([EPdataset.dataset(EPmain.view.dataset(color)).timeNames]);
            EPwaves.sampleSize=1000/EPdataset.dataset(EPmain.view.dataset(color)).Fs;
            implicit=EPdataset.dataset(EPmain.view.dataset(color)).implicit; %use implicits of the first dataset
            EPwaves.chanNames=EPdataset.dataset(EPmain.view.dataset(color)).chanNames; %use channel names of the first dataset
            eloc=EPdataset.dataset(EPmain.view.dataset(color)).eloc;
            if ~isempty(implicit)
                implicit=implicit(find(~strcmp('FID',{implicit.type})));
                EPwaves.chanNames=[EPwaves.chanNames {implicit.labels}]; %use channel names of the first dataset
                eloc=[eloc implicit];
            end;
            numChans=length(EPwaves.chanNames);
            if isfield(eloc,'labels')
                eLabels={eloc.labels};
            else
                eLabels=cell(numChans,1);
            end;
            for chan=1:length(eLabels)
                if isempty(eLabels{chan})
                    eLabels{chan}=EPwaves.chanNames{chan};
                end;
            end;
            [B,chanIX{color}] = sort(eLabels);
            if ~isempty(eloc)
                eloc=eloc(chanIX{color});
            end;
            EPwaves.chanNames=EPwaves.chanNames(chanIX{color});
            theFirstColor=color;
        else
            newImplicit=EPdataset.dataset(EPmain.view.dataset(color)).implicit;
            if ~isempty(newImplicit)
                newImplicit=newImplicit(find(~strcmp('FID',{newImplicit.type})));
            end;
            if numChans ~= length(EPdataset.dataset(EPmain.view.dataset(color)).chanNames)+length(newImplicit)
                msg{1}='Error: The number of channels are not compatible.';
                [msg]=ep_errorMsg(msg);
                close(EPwaves.handles.waves.hWaveWindow);
                return
            end;
            EPwaves.firstTime =max(min([EPdataset.dataset(EPmain.view.dataset(color)).timeNames]),EPwaves.firstTime);
            EPwaves.lastTime = min(max([EPdataset.dataset(EPmain.view.dataset(color)).timeNames]),EPwaves.lastTime);
            if EPwaves.sampleSize ~= 1000/EPdataset.dataset(EPmain.view.dataset(color)).Fs
                msg{1}='Error: The sampling rates are not compatible.';
                [msg]=ep_errorMsg(msg);
                close(EPwaves.handles.waves.hWaveWindow);
                return
            end;
            newEloc=[EPdataset.dataset(EPmain.view.dataset(color)).eloc newImplicit];
            if ~isempty(eloc) && ~isempty(newEloc)
                eLabels={newEloc.labels};
                for chan=1:length(eLabels)
                    if isempty(eLabels{chan})
                        eLabels{chan}=EPdataset.dataset(EPmain.view.dataset(color)).chanNames{chan};
                    end;
                end;
                [B,chanIX{color}] = sort(eLabels);
                newEloc=newEloc(chanIX{color});
                if any([eloc.theta]-[newEloc.theta]) || any([eloc.radius]-[newEloc.radius])
                    msg{1}='Error: The electrode coordinates are not compatible.';
                    [msg]=ep_errorMsg(msg);
                    close(EPwaves.handles.waves.hWaveWindow);
                    return
                end;
            elseif xor(isempty(eloc),isempty(newEloc)) %if one but not the other has no electrode coordinates
                msg{1}='Error: The electrode coordinates are not compatible.';
                [msg]=ep_errorMsg(msg);
                close(EPwaves.handles.waves.hWaveWindow);
                return
            else
                %if both have no electrode coordinates
                chanIX{color}=chanIX{1};
            end;
        end;
    end;
end;

EPwaves.firstTime=max(EPwaves.firstTime,startTime);
EPwaves.lastTime=min(EPwaves.lastTime,endTime);

if isempty(EPwaves.marker1)
    EPwaves.marker1=0;
end;

if isempty(EPwaves.marker2)
    EPwaves.marker2=0;
end;

if (EPwaves.marker1 > EPwaves.lastTime) || (EPwaves.marker1 < EPwaves.firstTime)
    EPwaves.marker1=0;
end;

if (EPwaves.marker2 > EPwaves.lastTime) || (EPwaves.marker2 < EPwaves.firstTime)
    EPwaves.marker2=0;
end;

%compute epoch samples for each color
EPwaves.numPoints=0;
for color=1:4
    if EPmain.view.dataset(color) <= length(EPdataset.dataset)
        startSamp(color)=(EPwaves.firstTime/EPwaves.sampleSize)+EPdataset.dataset(EPmain.view.dataset(color)).baseline+1;
        lastSamp(color)=(EPwaves.lastTime/EPwaves.sampleSize)+EPdataset.dataset(EPmain.view.dataset(color)).baseline;
        EPwaves.numPoints=lastSamp(color)-startSamp(color)+1;
    end;
end

%organize the data for plotting
EPwaves.plotColors=[];
EPwaves.thePlotColors=[];
EPwaves.totalData=zeros(numChans,EPwaves.numPoints,4);
theDataset=0;
for color=1:4
    if EPmain.view.dataset(color) <= length(EPdataset.dataset)
        if theDataset ~= EPmain.view.dataset(color) %load in the dataset if different from the one already loaded in.
            EPdata=ep_loadEPdataset(EPdataset,EPmain.view.dataset(color));
            theDataset=EPmain.view.dataset(color);
        end;
        if isempty(EPdata.trialNames) %averaged data
            theCell=EPmain.view.cell(color);
        else  %if single_trial data
            theCell=intersect(find(strcmp(EPmain.view.theCells{EPmain.view.dataset(color)}(EPmain.view.cell(color)),EPdata.cellNames)),...
                find(EPmain.view.trial(color)==EPdata.trialNames));
        end;
        EPwaves.totalData(1:length(EPdata.chanNames),:,color)=ep_expandFacs(EPdata,[],startSamp(color):lastSamp(color),theCell,EPmain.view.subject(color),EPmain.view.factor(color));
        EPwaves.totalData(:,:,color)=EPwaves.totalData(chanIX{color},:,color); %rearrange order of channels to be consistent with other datasets
        EPwaves.plotColors=[EPwaves.plotColors color];
        EPwaves.thePlotColors=[EPwaves.thePlotColors; EPwaves.RGBcolors(color,:)];
    end;
end;

%compute screen locations of channels
EPwaves.X=zeros(numChans,1);
EPwaves.Y=zeros(numChans,1);
hasLoc=zeros(numChans,1);

if ~isempty(eloc) %if there are electrode coordinates
    nonLoc=0;
    for chan=1:numChans
        theta=((eloc(chan).theta+90)/180)*pi;
        radius=eloc(chan).radius;
        if isempty(theta)
            nonLoc=nonLoc+1;
            EPwaves.X(chan)=.02;
            EPwaves.Y(chan)=.02+((EPwaves.plotHeight+.01)*nonLoc);
        else
            [EPwaves.X(chan),EPwaves.Y(chan)] = pol2cart(theta,radius);
            hasLoc(chan)=1;
        end;
    end;
    
    for chan=1:length(implicit)
        theta=((implicit(chan).theta+90)/180)*pi;
        radius=implicit(chan).radius;
        if isempty(theta)
            EPwaves.X(end+1)=0;
            EPwaves.Y(end+1)=0;
            hasLoc(end+1)=0;
        else
            [EPwaves.X(end+1),EPwaves.Y(end+1)] = pol2cart(theta,radius);
            hasLoc(end+1)=1;
        end;
    end;
else %if there are no electrode coordinates
    graphSize=ceil(sqrt(numChans));
    for chan=1:numChans
        EPwaves.X(chan)=1-(mod(chan-1,graphSize)+1)*(EPwaves.plotWidth+.01);
        EPwaves.Y(chan)=1-(floor((chan-1)/graphSize)+1)*(EPwaves.plotHeight+.01);
        hasLoc(chan)=1;
    end;
    for chan=1:length(implicit)
        if strcmp(implicit(chan).type,'REF')
            EPwaves.X(end+1)=1-(mod(length(EPwaves.X),graphSize)+1)*(EPwaves.plotWidth+.01);
            EPwaves.Y(end+1)=1-(floor(length(EPwaves.Y)/graphSize)+1)*(EPwaves.plotHeight+.01);
            hasLoc(end+1)=1;
        end;
    end;
end;

locList=find(hasLoc);
EPwaves.X(locList)=(1-EPwaves.X(locList))/2;
EPwaves.Y(locList)=(EPwaves.Y(locList)+1)/2;

EPwaves.X(locList)=((EPwaves.X(locList)-min(EPwaves.X(locList)))/(max(EPwaves.X(locList)-min(EPwaves.X(locList)))*(1+2*EPwaves.margin)))+EPwaves.margin;
EPwaves.Y(locList)=((EPwaves.Y(locList)-min(EPwaves.Y(locList)))/(max(EPwaves.Y(locList)-min(EPwaves.Y(locList)))*(1+2*EPwaves.margin)))+EPwaves.margin;

set(EPwaves.handles.waves.hWaveWindow ,'DefaultAxesXTickLabel',[])
set(EPwaves.handles.waves.hWaveWindow ,'DefaultAxesYTickLabel',[])
set(EPwaves.handles.waves.hWaveWindow ,'DefaultAxesXTick',[])
set(EPwaves.handles.waves.hWaveWindow ,'DefaultAxesYTick',[])
set(EPwaves.handles.waves.hWaveWindow,'DefaultAxesColorOrder',EPwaves.thePlotColors)

EPwaves.handles.waves.hWave=zeros(numChans,1);

for chan=1:numChans
    EPwaves.handles.waves.hWave(chan) = axes('position',[EPwaves.X(chan) EPwaves.Y(chan) EPwaves.plotWidth EPwaves.plotHeight]);
    EPwaves.handles.waves.hLines{chan} = plot([EPwaves.firstTime+EPwaves.sampleSize:EPwaves.sampleSize:EPwaves.lastTime],squeeze(EPwaves.totalData(chan,:,EPwaves.plotColors)));
    axis([EPwaves.firstTime+EPwaves.sampleSize EPwaves.lastTime EPwaves.plotMVmin EPwaves.plotMVmax]);
    if EPwaves.direction ==2
        set(EPwaves.handles.waves.hWave(chan),'YDir','reverse')
    end;
    line([EPwaves.firstTime+EPwaves.sampleSize EPwaves.lastTime],[0 0],'Color','black','LineWidth',1) % zero line
    line([0 0],[0 EPwaves.plotMVmax],'Color','black','LineWidth',1) %stimulus onset
    text(EPwaves.firstTime+(EPwaves.lastTime-EPwaves.firstTime)/20, EPwaves.plotMVmin+(EPwaves.plotMVmax-EPwaves.plotMVmin)/10, EPwaves.chanNames(chan));
    if EPwaves.marker1
        line(repmat(EPwaves.marker1,length([EPwaves.plotMVmin:EPwaves.plotMVmax])),[EPwaves.plotMVmin:EPwaves.plotMVmax],'Color',[.5 .5 .5],'LineWidth',1);
    end;
    if EPwaves.marker2
        line(repmat(EPwaves.marker2,length([EPwaves.plotMVmin:EPwaves.plotMVmax])),[EPwaves.plotMVmin:EPwaves.plotMVmax],'Color',[.5 .5 .5],'LineWidth',1);
    end;
    set(EPwaves.handles.waves.hWave(chan),'ButtonDownFcn',@expandChan);
    for i=1:length(EPwaves.handles.waves.hLines{chan})
        set(EPwaves.handles.waves.hLines{chan}(i),'ButtonDownFcn',@expandChan);
    end;
end;

%scale graph
EPwaves.handles.waves.graph = axes('position',[.02 .02 EPwaves.plotWidth EPwaves.plotHeight]);
axis([EPwaves.firstTime+EPwaves.sampleSize EPwaves.lastTime EPwaves.plotMVmin EPwaves.plotMVmax]);
if EPwaves.direction ==2
    set(EPwaves.handles.waves.graph,'YDir','reverse')
end;
line([EPwaves.firstTime+EPwaves.sampleSize EPwaves.lastTime],[0 0],'Color','black','LineWidth',1) % zero line
line([0 0],[0 EPwaves.plotMVmax],'Color','black','LineWidth',1) %stimulus onset
text(EPwaves.firstTime+(EPwaves.lastTime-EPwaves.firstTime)/20, EPwaves.plotMVmin-(EPwaves.plotMVmax-EPwaves.plotMVmin)/8, sprintf('%+d',round(EPwaves.firstTime)));
text(EPwaves.lastTime-(EPwaves.lastTime-EPwaves.firstTime)/5, EPwaves.plotMVmin-(EPwaves.plotMVmax-EPwaves.plotMVmin)/8, sprintf('%+d',round(EPwaves.lastTime)));
text(EPwaves.firstTime-(EPwaves.lastTime-EPwaves.firstTime)/3, EPwaves.plotMVmax-(EPwaves.plotMVmax-EPwaves.plotMVmin)/8, sprintf('%+2.2f',EPwaves.plotMVmax));
text(EPwaves.firstTime-(EPwaves.lastTime-EPwaves.firstTime)/3, EPwaves.plotMVmin+(EPwaves.plotMVmax-EPwaves.plotMVmin)/8, sprintf('%+2.2f',EPwaves.plotMVmin));
if EPwaves.marker1
    line(repmat(EPwaves.marker1,length([EPwaves.plotMVmin:EPwaves.plotMVmax])),[EPwaves.plotMVmin:EPwaves.plotMVmax],'Color',[.5 .5 .5],'LineWidth',1);
end;
if EPwaves.marker2
    line(repmat(EPwaves.marker2,length([EPwaves.plotMVmin:EPwaves.plotMVmax])),[EPwaves.plotMVmin:EPwaves.plotMVmax],'Color',[.5 .5 .5],'LineWidth',1);
end;
set(gca,'ButtonDownFcn','selectmoveresize')

set(EPwaves.handles.waves.hWaveWindow ,'Color',[.5 .5 .5])

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function expandChan(src,eventdata)
global EPwaves

scrsz = get(0,'ScreenSize');
chan=find([EPwaves.handles.waves.hWave] == src);

if isempty(chan)
    for theChan=1:length(EPwaves.handles.waves.hLines)
        for theLine=1:length(EPwaves.handles.waves.hLines{theChan})
            if src == EPwaves.handles.waves.hLines{theChan}(theLine)
                chan=theChan;
            end;
        end;
    end;
end;

if isempty(chan)
    disp('Could not find the channel for some reason.')
    return
end;

if isfield(EPwaves.handles.waves,'hExpandedFigure')
    if ~isempty(EPwaves.handles.waves.hExpandedFigure)
        if ishandle(EPwaves.handles.waves.hExpandedFigure)
            close(EPwaves.handles.waves.hExpandedFigure)
        end;
    end;
end;

EPwaves.handles.waves.hExpandedFigure = figure('Name', [EPwaves.chanNames{chan}], 'NumberTitle', 'off', 'Position',[scrsz(3)/2 scrsz(4)/2 600 400]);

set(EPwaves.handles.waves.hExpandedFigure,'DefaultAxesColorOrder',EPwaves.thePlotColors)
EPwaves.handles.waves.hExpandedAxes = axes('position',[.05 .05 .9 .9]);
plot([EPwaves.firstTime+EPwaves.sampleSize:EPwaves.sampleSize:EPwaves.lastTime],squeeze(EPwaves.totalData(chan,:,EPwaves.plotColors)));
axis([EPwaves.firstTime EPwaves.lastTime EPwaves.plotMVmin EPwaves.plotMVmax]);
if EPwaves.direction ==2
    set(EPwaves.handles.waves.hExpandedAxes,'YDir','reverse')
end;
line([EPwaves.firstTime+EPwaves.sampleSize EPwaves.lastTime],[0 0],'Color','black','LineWidth',1) % zero line
line([0 0],[0 EPwaves.plotMVmax],'Color','black','LineWidth',1) %stimulus onset

text(EPwaves.firstTime+(EPwaves.lastTime-EPwaves.firstTime)/20, EPwaves.plotMVmin+(EPwaves.plotMVmax-EPwaves.plotMVmin)/10, EPwaves.chanNames(chan));
if EPwaves.marker1
    try
        eval('line(repmat(EPwaves.marker1,length([EPwaves.plotMVmin:EPwaves.plotMVmax])),[EPwaves.plotMVmin:EPwaves.plotMVmax],''Color'',[.5 .5 .5],''LineWidth'',1)');
    catch
    end
end;
if EPwaves.marker2
    try
        eval('line(repmat(EPwaves.marker2,length([EPwaves.plotMVmin:EPwaves.plotMVmax])),[EPwaves.plotMVmin:EPwaves.plotMVmax],''Color'',[.5 .5 .5],''LineWidth'',1);');
    catch
    end;
end;




