function [outData]=ep_mergeEPfiles(files,mergeName)
% ep_mergeEPfiles - [outData]=ep_mergeEPfiles(files,mergeName) -
% Merge list of EP data files so that they are combined together.
% They must be either subject average or combined subject average files or single subject files from the same subject.
% Factor files are not supported.
% They must have the same set of channels, timepoints, and subject specs.
% If for a single subject file, must have the same trial spec names.
% The result is a combined set of cells and a combined number of subjects.
% Any subjects without the full set of cells will have zeroed data for the missing cells.
% The information about the experiment and so forth are taken from the first file in the list.
%
%Inputs
% Optional
%   files: A 2D cell array with file names to merge (including path names) in the first column.
%          the remaining columns are input arguments for the readData function.
%   mergeName: the name of the resulting merged file.
%
%Outputs:
%  EPdata         : Structured array with the data and accompanying information.  See readData.
%
%History
%  by Joseph Dien (3/4/09)
%  jdien07@mac.com
%
% modified 5/30/09 JD
% Added cellType, subType, chanType, facName, facType, and analysis fields.  Reads in files directly rather than .mat files.
%
% modified 6/26/09 JD
% added mergeName input and dataName field.
%
%  bugfix 10/28/09 JD
%  Crash due to not encoding cell and subject labels correctly.
%  Not filling in cellType and trialSpec fields correctly for single-trial data.
%
%  bugfix 1/18/10 JD
%  Fixed crash in EP when merge operation aborted due to problem with files.
%
%  bugfix 1/23/10 JD
%  Fixed contents of analysis fields being not arranged correctly.
%
% modified 2/11/10 JD
% analysis fields no longer optional.
%
%  bugfix 4/30/10 JD
%  Fixed crash when reading single_trial data using single file mode.
%
%  bugfix 5/23/10 JD
%  Fixed crash when merging average files.
%  Added 'exact' keyword to strmatch commands to ensure that the cell and subject names are matched exactly rather than
%  based on the beginning of the names.
%
% modified 5/26/10 JD
% Added eloc to parameters passed to files to handle cases where ced is non-empty but eloc is empty so doesn't have to
% keep accessing .ced file on every pass.
% Fixed not successfully merging average files together.
%
%  bugfix 6/2/10 JD
% Only adds trial names for single trial data files.
% 
%  modified 10/17/10 JD
%  Added support for saccadeTrials and saccadeOnset fields.

outData=[];

if nargin < 1
    files=[];
end

if isempty(files)
    [files, pathname] = uigetfile('*.*','Open:','MultiSelect','on');
    activeDirectory=pathname;
    if ~iscell(files)
        temp=files;
        files=[];
        files{1}=temp;
    end;
    if files{1}==0
        msg{1}='No filenames selected. You have to click on a name';
        [msg]=ep_errorMsg(msg);
        return
    end
    
    if ~iscell(files)
        temp=files;
        files=[];
        files{1}=temp;
    end;
else
    for theFile=1:size(files,1)
        if ~exist(char(files{theFile,1}),'file')
            msg{1}=['Error: The file ' files{theFile,2} ' is not in the directory.'];
            [msg]=ep_errorMsg(msg);
            return
        end;
    end;
end;

numFiles = size(files,1);

if numFiles==1
    msg{1}='Only one file selected.';
    [msg]=ep_errorMsg(msg);
    return
end


totalSubjectNames=cell(0);
totalSubjectSpecs=cell(0);
totalCellNames=cell(0);
uniqueCellNames=cell(0);
ced=[];
eloc=[];

%compile data from each file
for theFile=1:numFiles
    thisFile=files{theFile,1};
    temp=[];
    temp{1}='file';
    temp(2:size(files,2)+1)=files(theFile,:);
    if ~isempty(ced) && ~any(strcmp('ced',files(theFile,:)))
        temp{end+1}='ced';
        temp{end+1}=ced;
    end;
    if ~isempty(eloc)
        temp{end+1}='eloc';
        temp{end+1}=eloc;
    end;
    
    if theFile >1
        temp{end+1}='silent';
        temp{end+1}='on';
    end;
    
    data=ep_readData(temp);
    
    if isempty(data)
        outData=[];
        msg{1}='File not read successfully.  Aborting effort to merge data files.';
        [msg]=ep_errorMsg(msg);
        return
    end;
    
    if strcmp(data.dataType,'single_trial')
        if theFile==1
            theSingleSubject=data.subNames{1};
        else
            if ~strcmp(data.subNames{1},theSingleSubject)
                outData=[];
                msg{1}=['Error: When merging single trial data, all must be from the same subject (' theSingleSubject ').'];
                [msg]=ep_errorMsg(msg);
                return
            end;
        end;
    elseif ~any(strcmp(data.dataType,{'average'}))
        outData=[];
        msg{1}=['Error: The file ' thisFile ' is the ' data.dataType ' data type.'];
        [msg]=ep_errorMsg(msg);
        return
    end;
    
    %information that is assumed to be identical between all the files and is culled from the first such file
    if theFile ==1
        outData.dataType=data.dataType;
        outData.fileFormat = data.fileFormat;
        outData.montage = data.montage;
        outData.chanNames = data.chanNames;
        outData.timeNames = data.timeNames;
        outData.chanTypes= data.chanTypes;
        outData.cellTypes = data.cellTypes;
        outData.subjectSpecNames = data.subjectSpecNames;
        outData.facNames= data.facNames;
        outData.facTypes = data.facTypes;
        try
            EPver=ver('EP_Toolkit');
        catch
            EPver='unavailable'; %workaround for bug in earlier version of Matlab
        end;
        outData.EPver = EPver;
        outData.ver = ver;
        outData.date = date;
        outData.Fs = data.Fs;
        outData.baseline = data.baseline;
        outData.ename = data.ename;
        outData.dataName = mergeName;
        outData.trialSpecNames = data.trialSpecNames;
        outData.trialNames = [];
        outData.trialSpecs = [];
        outData.fileName = data.fileName;
        outData.ced = data.ced;
        outData.eloc = data.eloc;
        outData.implicit = data.implicit;
        outData.facVecS = data.facVecS;
        outData.facVecT = data.facVecT;
        ced = data.ced;
        eloc = data.eloc;
        if isempty(ced)
            ced='none';
        end;
        numChans = size(data.data,1);
        numPoints = size(data.data,2);
        numEvents = length(data.events);
        
        newBlinkTrial=[];
        newMoveTrial=[];
        newBadTrials=[];
        newBadChans=[];
    else
        
        if ~isempty(setdiff(outData.chanNames,data.chanNames))
            outData=[];
            msg{1}=['Error: The file ' thisFile ' has different channels.'];
            [msg]=ep_errorMsg(msg);
            return
        end;
        if ~isempty(setdiff(outData.timeNames,data.timeNames))
            outData=[];
            msg{1}=['Error: The file ' thisFile ' has different time points.'];
            [msg]=ep_errorMsg(msg);
            return
        end;
        if ~isempty(setdiff(outData.subjectSpecNames,data.subjectSpecNames))
            outData=[];
            msg{1}=['Error: The file ' thisFile ' has different subject specs.'];
            [msg]=ep_errorMsg(msg);
            return
        end;
        if ~isempty(setdiff(outData.trialSpecNames,data.trialSpecNames)) && strcmp(data.dataType,'single_trial')
            outData=[];
            msg{1}=['Error: The file ' thisFile ' has different channels.'];
            [msg]=ep_errorMsg(msg);
            return
        end;        
    end;
    
    if strcmp(data.dataType,'single_trial')
        newTrialNames{theFile}= data.trialNames;
        newTrialSpecs{theFile}= data.trialSpecs;
    end;
    
    newCellNames{theFile}= data.cellNames;
    newData{theFile}= data.data;
    newSubNames{theFile}= data.subNames;
    newSubTypes{theFile}= data.subTypes;
    newCellTypes{theFile}= data.cellTypes;
    newSubjectSpecs{theFile}= data.subjectSpecs;
    newEvents{theFile}= data.events;
    newAvgNum{theFile}= data.avgNum;
    newSubNum{theFile}= data.subNum;
    
    newBlinkTrial{theFile}= data.analysis.blinkTrial;
    newSaccadeTrial{theFile}= data.analysis.saccadeTrial;
    newSaccadeOnset{theFile}= data.analysis.saccadeOnset;
    newMoveTrial{theFile}= data.analysis.moveTrial;
    newBadTrials{theFile}= data.analysis.badTrials;
    newBadChans{theFile}= data.analysis.badChans;
    
    for i=1:length(data.cellNames)
        totalCellNames(end+1)=data.cellNames(i);
        if ~any(strcmp(data.cellNames{i},uniqueCellNames))
            uniqueCellNames{end+1}=char(data.cellNames{i});
        end;
    end;
    
    for i=1:length(data.subNames)
        if ~any(strcmp(data.subNames{i},totalSubjectNames))
            totalSubjectNames(end+1)=data.subNames(i);
            if ~isempty(char(data.subjectSpecs{i,:}))
                totalSubjectSpecs(end+1,:)=char(data.subjectSpecs{i,:});
            end;
        end;
    end;
end;

numCells=length(uniqueCellNames);
numSubs=length(totalSubjectNames);

if strcmp(outData.dataType,'average')
    numWaves=numCells;
elseif strcmp(data.dataType,'single_trial')
    numWaves=length(totalCellNames);
end;

outData.data=zeros(numChans,numPoints,numWaves,numSubs);
outData.facData=[];
outData.avgNum=zeros(numSubs,numCells);
outData.subNum=zeros(numSubs,numCells);
outData.events=cell(numSubs,numCells);
outData.analysis.blinkTrial=zeros(numSubs,numWaves);
outData.analysis.saccadeTrial=zeros(numSubs,numWaves);
outData.analysis.saccadeOnset=zeros(numSubs,numWaves);
outData.analysis.moveTrial=zeros(numSubs,numWaves);
outData.analysis.badTrials=zeros(numSubs,numWaves);
outData.analysis.badChans=zeros(numSubs,numWaves,numChans);

outData.subNames=totalSubjectNames;
outData.subjectSpecs=totalSubjectSpecs;

%merge the data from the files together
trialCount=zeros(numCells,1);
outCount=0;
for theFile=1:numFiles
    for theSub=1:length(newSubNames{theFile})
        newSub=strmatch(newSubNames{theFile}{theSub},totalSubjectNames,'exact');
        outData.subTypes(newSub)=newSubTypes{theFile}(theSub);
        for theCell=1:length(newCellNames{theFile})
            newCell=strmatch(newCellNames{theFile}{theCell},uniqueCellNames,'exact');
            newWave=newCell;
            if strcmp(data.dataType,'single_trial')
                newWave=outCount+theCell;
                for theSpec=1:size(newTrialSpecs{theFile},2)
                    outData.trialSpecs{newWave,theSpec}=newTrialSpecs{theFile}{theCell,theSpec};
                end;
            end;
            trialCount(newCell)=trialCount(newCell)+1;
            if strcmp(data.dataType,'single_trial')
                outData.trialNames(newWave)=trialCount(newCell);
            end;
            outData.analysis.blinkTrial(newSub,newWave)=newBlinkTrial{theFile}(theSub,theCell);
            outData.analysis.saccadeTrial(newSub,newWave)=newSaccadeTrial{theFile}(theSub,theCell);
            outData.analysis.saccadeOnset(newSub,newWave)=newSaccadeOnset{theFile}(theSub,theCell);
            outData.analysis.moveTrial(newSub,newWave)=newMoveTrial{theFile}(theSub,theCell);
            outData.analysis.badTrials(newSub,newWave)=newBadTrials{theFile}(theSub,theCell);
            outData.analysis.badChans(newSub,newWave,:)=newBadChans{theFile}(theSub,theCell,:);
            outData.cellNames(newWave)=uniqueCellNames(newCell);
            outData.cellTypes(newWave)=newCellTypes{theFile}(theCell);
            
            if any(any(outData.data(:,:,newWave,newSub)))
                disp(' ');
                disp('**************************************************************');
                disp(['Warning: The file ' thisFile ' duplicated the data for subject ' totalSubjectNames{newSub} ' for cell ' totalCellNames{newCell} '.']);
                disp('**************************************************************');
                disp(' ');
            end;
            outData.data(:,:,newWave,newSub)=newData{theFile}(:,:,theCell,theSub);
            outData.avgNum(newSub,newWave)=newAvgNum{theFile}(theSub,theCell);
            outData.subNum(newSub,newWave)=newSubNum{theFile}(theSub,theCell);
            outData.events(newSub,newWave)=newEvents{theFile}(theSub,theCell);
        end;
        outCount=outCount+length(newCellNames{theFile});
    end;
end;

try
    EPver=ver('EP_Toolkit');
catch
    EPver='unavailable'; %workaround for bug in earlier version of Matlab
end;
outData.EPver=EPver;
outData.ver=ver;
outData.date=date;
outData.history={'mergeEPfiles',files};
outData.facData=[]; %factor files not supported so assumed no such data.
if isempty(outData.trialSpecs) && strcmp(data.dataType,'single_trial')
    outData.trialSpecs=cell(size(outData.data,3),0);
end;

[err]=ep_checkEPfile(outData);



