function [EPdata]=ep_readData(varargin);
%  [EPdata]=ep_readData(varargin);
%       imports data from a variety of file formats relying on FieldTrip I/O code and converts into EP file format.
%
%Inputs:
%  OPTIONAL KEYWORDS (accepts either a single cell string or a series of variables)
%  'file'       : Name of the file to be imported.  It needs to have an
%                 appropriate suffix in order for it to be recognized properly.
%                 See the tutorial file.
%                 If none is specified it will assume that it is an EGI EGIA average file.
%
%  'format'     : File format of the file.
%  'samples'    : Samples included in analysis.
%  'cells'      : Cells included in analysis (trials for single_trial data).  Should be specified in order
%                 desired for output.  See example.
%  'subjects'   : Subjects included in analysis.  Should be specified in
%                 order desired for output.
%  'channels'   : Channels included in analysis.
%  'montage'    : The electrode montage if an EGIS file.  If not specified and the SLAY resource is not in the file, it
%                 will be requested.  The list of montages can be found in askForMontage.m.
%  'name'       :  The name of the experiment.
%  'labels'      : Renames the condition names subject names (e.g., 'labels',{{'cell1','cell2'},{'sub1','sub2'}}).
%                   If only one cell label is specified, it will be applied to all the data.
%  'type'       : The type of the data.  Should be followed by the data type.
%                 'continuous' raw data from a single subject, not segmented into epochs.
%                 'single_trial' segmented single subject not averaged.
%                 'average' file with trials averaged so there is only one waveform per condition.
%                 'grand_average' file with multiple subjects averaged together.
%                 'factors' output of PCA results, where "subjects" are factors.
%  'NumberSubjects'   : The number of subjects or factors.
%  'prestim'   : The number of msec prior to the stimulus onset (prior to using the samples keyword if used).
%  'ced'      : The name of the .ced file for electrode coordinates.  Use 'none' if none.  Can be invalid ced file.
%  'eloc'      : The eloc information (only if good info).
%  'silent'   : If 'on' rather than 'off', do not provide output to command line for standard messages. (default off)
%  'reference'  : Specify the reference channel if present explicitly in the dataset so that it is not dubbed a bad channel.
%  'textPrefs'  : Specifies the parameters for text files.  Structured variable:
%                 .firstRow : the first row of the data.
%                 .firstCol : the first column of the data.
%                 .lastCol  : the last column of the data.  Zero equals last column.
%                 .orientation : the orientation of the data.  1=channels are columns, 2=channels are rows.
%
%  Reads in FieldTrip I/O outputs: data, hdr, eventHdr.  Assumes 'average' data is organized
%  with the subjects grouped together by cell (e.g., C1S1 C1S2 C1S3 C2S1 C2S2 C2S3).
%  Recommended but not necessarily required is a .ced file (EEGlab format) for electrode coordinates.
%
%Outputs:
%  EPdata         : Structured array with the data and accompanying information.
%    .data      : 5D data matrix [channels, time points, cells/trials, subjects, factors].
%                 Trials are grouped by cell in single_trial format.
%    .noise     : 4D matrix [channels, time points, cells/trials, subjects] mirroring .data.
%                 This is the +/- reference average (Schimmel, 1967) which provides an estimate of the noise level
%                 in averaged data by flipping every other trial.  Optional.
%    .std       : 4D matrix [channels, time points, cells/trials, subjects] mirroring .data.
%                 This is the standard deviation from averaging trials.  Optional.
%    .facData   : 5D data matrix [channels, time points, cells/trials, subjects, factors] for combined factors (only when facVecS & facVecT are used).
%                 They are represented separately from .data because they can't be compacted using the facVec mechanism.
%    .montage   : String with the montage information, if available.
%    .fileFormat: The file format.
%    .dataType  : The type of the data: 'continuous', 'single_trial', or 'average' (default: average)
%    .chanNames : The channel names.
%    .timeNames : The msec of the sample onset with respect to the stimulus onset time.
%    .subNames  : The subject names
%    .cellNames : The cell names (once for each trial for single_trial files).
%    .trialNames: The trial number ID per cell (starting from 1). (single_trial data only)
%    .facNames  : The factor names (only for factor files)
%    .chanTypes : The type of the channel: EEG, MEG, ANS (autonomic), REF (explicit current reference), REG (regional average)
%    .subTypes  : The type of the subject: RAW (single trial), AVG (subject average), GAV (grand average)
%    .cellTypes : The type of the cell: SGL (one cell), CMB (combination of cells)
%    .facTypes  : The type of the factor: SGL (one factor), CMB (combination of factors)
%    .EPver     : The EP Toolkit version information.
%    .ver       : The Matlab version information.
%    .date      : The date the file was created.
%    .Fs        : The sampling frequency in Hz.
%    .baseline  : The number of samples prior to the trigger event (positive number).
%                 Thus, the ms in the timeNames refers to the offset of each sample.  A baseline of 50 samples means
%                 that the epoch started 200 ms (50 samples) prior to the stimulus onset.  A baseline of 1 sample means
%                 that the epoch started 4 ms (1 sample) prior to the stimulus onset.
%    .ename     : The name of the experiment.
%    .dataName  : A descriptive name for the dataset, used to differentiate the active datasets during analysis.
%    .trialSpecs       : Cell array (trial,spec) of specific information for trials (single_trial data only)
%    .trialSpecNames   : Cell array of the name of each trial spec type.
%    .subjectSpecs     : Cell array (subject,spec) of specific information for subjects.  Can be empty if no specs.
%    .subjectSpecNames : Cell array of the name of each subject spec type.
%    .events    : Cell array of event structured variables (subject,cell/trial)
%      .type      = string (events of "trial" type dropped as they are redundant with cell name information.)
%      .sample    = expressed in samples, the first sample of a recording is 1
%      .value     = number or string
%      .offset    = expressed in samples (0=first sample of epoch, negative means event after first sample of epoch)
%      .duration  = expressed in samples
%      .timestamp = expressed in timestamp units, which vary over systems (optional)
%    .avgNum    : Number of waveforms going into averages (subject,cell)
%                 0 means unknown and -1 means bad.
%    .subNum    : Number of subjects going into averages (subject,cell)
%                 0 means unknown and -1 means bad.
%    .fileName  : Name of original file.
%    .history   : Command used to create current file.
%    .ced       : The name of the .ced file for electrode coordinates.
%    .eloc      : The electrode location information, one for each channel (see readlocs header)
%                 eloc is the same length as the channels, with REG channels having a blank entry.
%    .implicit  : The electrode information for implicit references and fiducial locations (see readlocs header)
%    .FacPat	: Optional.  Factor pattern matrix from doPCA (or doPCAst if two-step PCA performed).  Editable.  (rows=variables, cols=factors)
%    .facVar    : Optional.  %age of variance accounted for by each rotated factor from doPCA or doPCAst.  Editable.
%    .facVarQ   : Optional.  %age of variance uniquely accounted for by each rotated factor from doPCA or doPCAst.  Editable.
%    .facVecT   : For temporal PCA factor files, the factor waveform.  Used to compress the data. (rows=points,cols=factors)
%    .facVecS   : For spatial PCA factor files, the factor scalp topography.  Used to compress the data.(rows=chans,cols=factors)
%    .analysis (for continuous files, divided into one second epochs and "trials" refer to these epochs.
%       .blinkTrial : Array of blink-corrected trials (subject,cell/trial)
%       .saccadeTrial : Array of saccade-corrected trials (subject,cell/trial)
%       .saccadeOnset : Array of onset in samples of detected saccades (subject,cell/trial)
%       .moveTrial : Array of movement-corrected trials (subject,cell/trial)
%       .badTrials : Array of bad trials (subject,cell/trial)
%       .badChans : Array of corrected bad channels (subject,cell/trial,channel). -1 in a session file means still bad.
%                   Negative numbers in an average file means number of still bad channels that went into the average
%                   (or rather, were left out of the average).
%    .pca (all optional)
%        fields from ep_doPCA and ep_doPCAst steps.  See them for documentation.
%        Not affected by editing.
%
% With respect to the FieldTrip data structure:
%  According to Vladimir Litvak,
%  "In case of events with duration that define trials event.sample is the first sample of a trial and event.offset is
%  the offset of the trigger with respect to the trial. An offset of 0 means that the first sample of the trial
%  corresponds to the trigger. A  positive offset indicates that the first sample is later than the trigger,
%  a negative offset indicates that the trial begins before the trigger."


%
%  Example:
%  [EPdata]=readData;
%  [EPdata]=readData('file','NT9coll.ave_af.egis','cells',[1:3]);
%  [EPdata]=readData('format','ns_avg');

%History:
%  by Joseph Dien (2/7/08)
%  jdien07@mac.com
%
%  bugfix 3/31/08 JD
%  Handles formats other than EGIS properly by adding information about number of cells, subjects, and ordering.
%
%  modified 4/1/08 JD
%  Added support for EGI's segmented simple binary format.
%
%  modified 4/30/08 JD
%  accepts EGIS session files.
%
%  modified 11/4/08 JD
%  outputs structured variable including montage and file format and name
%  information.  Also, data is now a 4D data matrix.  Also, now autodetects
%  the electrode montage in EGIS files.
%
%  modified 11/10/08 JD
%  inputs changed to keyword list approach.
%
%  modified and bugfix (1/31/09) JD
%  Added initEP.  Added version and date fields to output structure.  Fixed reversal of order for subjects and cells
%  specified with format keyword.
%
%  modified (2/3/09) JD
%  Added Channels keyword.  Samples changed to set of numbers rather than just first and last.  Format no longer
%  includes fields for numbers of cells and subjects.
%
%  modified and bugfix (3/25/09) JD
%  Added information on date, version, sampling rate, baseline, experiment name, data type, and specs.
%  Subject names for EGIS average files now correctly detected.
%  Generalized to handle session files too with addition of trial specs, trial names, and events.
%  Also handles factor output files.  Tested out with Neuroscan files.
%  Added .ced field for electrode coordinate information.
%
%  modified (4/17/09) JD
%  Groups epochs in single trial files by cell.
%  eloc and chanType and implicit fields added to the files.
%
%  modified (4/27/09) JD
%  Added subNum field.  Merged 'subject_average', 'combined_average' and 'grand_average' data types into 'average'.
%  Deblanks cell names for EGIS files.
%
%  modified and bugfix 7/22/09 JD
%  Eliminated the latency field from the event structure.  Dropping initial samples results in event offset being
%  adjusted.   Dropped events of "trial" type as they are redundant with cell name information.  Added cellTypes field.
%  Fixed some crashes when handling simple binary files.  Added factors as 5th dimension.  Added support for text files.
%  Added support for EP files.  Added analysis subfields blinkTrial, moveTrial, badTrials, and badChans.
%  Added support for factor file compression via facVecT and facVecS fields.  Added dataName and facData fields.
%  EGIS subject specs converted to strings.
%
%  bugfix 8/4/09 JD
%  Added workaround for defective EGIS average headers generated by NetStation (the LastDone field which indicates number of subjects is incorrect).
%  Also, fix for NetStation not outputing subject numbers in the EGIS average header.
%
%  bugfix 8/29/09 JD
%  For "other" format files, not distinguishing cell names that start with the same characters (e.g., "1" and "11" not
%  distinguished).  Thanks to Grega Repovs.
%  For single_trial data, was incorrectly reporting on the command line the wrong number of cells being read in.
%
%  modified 8/30/09 JD
%  Added settings for text files.
%
%  bugfix 9/5/09 JD
%  Crash when reading factor file other than EGIS format.
%  Header defective (led to subsequent crash) when reading simple binary average file.
%  Crash when setting reference channel type.
%  Unable to find ced file when already specified in the file.
%  Text files with non-number columns sometimes not being read properly.
%
%  bugfix 9/16/09 JD
%  Text file import was skipping the first timepoint.
%  Not reading correct number of text columns.
%
%  bugfix 10/12/09 JD
%  Crash when reading "other" format files in single file mode where event values are empty.
%
%  bugfix 10/18/09 JD
%  Now aborts when NaN or inf values are in the data.
%
%  modified 10/31/09 JD
%  Reads in eloc information if present in eeglab_set file.
%
%  bugfix 11/9/09 JD
%  Number of subjects for continuous simple binary files incorrectly set.
%  Crash when reading continuous simple binary files.
%
%  modified 11/15/09 JD
%  When reading simple binary files, if the cell names cannot be deduced, put all of the segments into the same single
%  cell rather than just aborting.
%
% bugfix and modified 11/20/09 JD
% Replaced "union" commands with "unique" commands because certain situations caused the "union" command to crash in
% Matlab 2007.
% When importing a text file, ignore tabs at the end of the line.
% Drops CELL and TRSP events for all simple binary files since NetStation loses the associated information when exporting simple binary files.
%
%  modified 1/15/10 JD
%  When reading in an EGIS format file, if none of the EGI montages are chosen, then it will allow for a .ced file to be chosen.
%
%  modified 1/26/10 JD
%  When reading in file formats that label the channels, they will be reordered to match the order of the ced file.  Also, if any channels
%  are missing from the data file then they will be added as bad channels.  Also, can now accommodate channels that are in the data file
%  but not in the ced file.  The type field in the .ced files is now used since the
%  EEGlab bug was apparently fixed and it is now functional.  The type field must now be present in the .ced file and assumptions
%  will no longer be made about which ones are REF or FID types.
%
%  bugfix 1/30/10
%  Fixed incorrect subject ID being extracted from EGIS session headers.
%  Fixed crash due to events with empty value fields (now uses type field instead if value field is empty).
%  Fixed crash due to empty type fields in eloc information by assuming they are EEG channels.
%
%  bugfix 2/4/10
%  Fixed crash when loading factor file with CED information.
%  Fixed loss of some factor information when loading EP format factors file.
%
%  modified & bugfix 2/27/10 JD
%  When importing data, epochs that are entirely flat across all channels are marked as bad.
%  Fixed crash when two REF channels (as in M1-M2 mean mastoid channels).
%  Analysis fields no longer optional.
%  Fixed crash when loading in an EP file with ced information.
%  Fixed crash when loading in a .set file that does not have ced information included in its header.
%  Initializes subject spec names as cell rather than empty array.
%  When importing EP file format data, correctly checks for data type even for "factors" and "grand_average".
%  Eliminated chantype field for implicit channels.
%  Now reads the nsweeps field of Neuroscan AVG files to fill in the avgNum and subNum fields.
%  Implicit reference channels will no longer be marked as bad.
%
%  bugfix 3/5/10 JD
%  One dimensional cell array fields are now standardized to be column vectors.  Some file formats were causing crashes
%  in the Edit function because some of these fields were coming out as row vectors.
%
%  modified 3/15/10 JD
%  if .type field is missing from eloc information, then add it (assume channels are "EEG").
%
%  bugfix 3/27/10 JD
%  Fixed crash when loading in .txt file format data.
%  Fixed crash when loading in data where event information is empty.
%  Fixed text files treating all channels as being implicit when used with ced files with channel names different than
%  the default channel names.
%  For file formats with fixed channel orders, use channel names from ced file if available.
%  Tried to reinstate support for .edf file format.
%
%  bugfix 4/18/10 JD
%  Fixed wrong number of channel names when reading fixed order file formats (like EGIS) and the reference channel is implicit.
%
%  bugfix 5/1/10 JD
%  Fixed crash when reading data using single file mode with .set files which contain the .eloc information.
%  Fixed crash when reading data file that was originally in .set format and then was saved in EP file format.
%
%  modified 5/12/10 JD
%  For files with ced set to "eeglab", change to either name in chaninfo.filename field if present, else "none".
%
%  bugfix 5/22/10 JD
%  Fixed crash when importing .set file or EP file with unavailable or invalid ced file named in ced field.
%  Fixed channel selection not operating on channel coordinates eloc field.
%
%  bugfix 6/17/10 JD
%  Fixed not keeping FID channels in implicit channel info for fixed channel order file formats (e.g., EGIS, text).
%  Fixed electrode information not matching the data for second file onwards when reading files in single cell file
%  mode.
%  Can now be passed eloc information so don't have to access ced file on every pass for single cell file mode.
%  Fixed cell labels and sub labels not being applied to EP file formats.
%  Fixed crash for file formats with fixed-order channels when ced file has wrong number of channels.
%  Fixed not ignoring extra tab at end of line of text files, resulting in "not-a-number" errors.
%  Fixed crash when loading EP or .set file with name of original ced file in addition to eloc information.
%  Fixed losing the electrode coordinate information (eloc and and ced) when loading in a .study file.
%
%  bugfix 8/2/10 JD
%  Fixed crash if using "trial" events to determine name of cells and the .value field is empty.
%
%  bugfix 8/25/10 JD
%  Fixed not obtaining channel names from eloc info when provided by function call, as when merging files during Single File mode.
%  Fixed errors when importing .study files where the group or the session fields were left blank.
%
%  bugfix 10/3/10 JD
%  Fixed events in the final sample of a segment being assigned to the succeeding segment and causing a crash if the
%  segment was already the last one.
%  Fixed crash when the montage keyword was followed by a blank, as when preprocessing a batch of non-EGIS files
%  containing more than one data file.
%
%  modified 10/12/10 JD
%  For continuous files, analysis fields refer to one second epochs.
% 
%  modified 10/16/10 JD
%  Added support for HEOG saccade correction.
% 
%  bugfix 11/3/10 JD
%  Now accepts upper case file suffixes (e.g., .EGIS).
% 
%  bugfix 12/6/10 JD
%  When reading in EGIS session files with implicit ref, adds it in explicitly, thus avoiding crash in eyeblink
%  correction routine.
% 
%  modified 2/15/11 JD
%  Added support for CSV text files in addition to tab-delimited text files.
%
% modified 2/26/11 JD
% Added support for EGI Matlab files

%
%     Copyright (C) 1999-2011  Joseph Dien
%
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
%
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
%
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.
%

ep_initEP;

samples=[];
cells=[];
subjects=[];
channels=[];
EPdata=[];
montage=[];
fileFormat=[];
dataType=[];
chanNames=[];
timeNames=[];
cellNames=[];
trialNames=[];
subNames=[];
facNames=[];
ename=[];
dataName=[];
trialSpecs=[];
trialSpecNames=[];
subjectSpecs={};
subjectSpecNames={};
eventHdr=[];
avgNum=[];
subNum=[];
baseline=[];
sampleRate=[];
events=[];
ced=[];
eloc=[];
implicit=eloc;
chanTypes=[];
cellTypes=[];
subTypes=[];
facTypes=[];
facVecT=[];
facVecS=[];

fileName=[];
fileFormat=[];
numSubs=[];
numFacs=[];
numCells=[];
numWaves=[];
cellLabels=[];
subLabels=[];
silent='off';
refChan=[];
blinkTrial=[];
saccadeTrial=[];
saccadeOnset=[];
moveTrial=[];
badTrials=[];
badChans=[];
noise=[];
stanDev=[];
facData=[];
textPrefs.firstRow=1;
textPrefs.firstCol=1;
textPrefs.lastCol=0;
textPrefs.orientation=1;

if ~isempty(varargin)
    if isa(varargin{1},'cell') && nargin==1 %if keywords were input as a single cell string
        inputSet=varargin{1};
    else
        inputSet=varargin;
    end;
else
    inputSet=[];
end;

argNum=length(inputSet);
if mod(argNum,2) ~= 0
    msg{1}='The keywords need to all be in pairs, with a keyword followed by the keyword information.';
    [msg]=ep_errorMsg(msg);
    return
end;

argCount=1;
while argCount <= argNum
    switch inputSet{argCount}
        case 'file'
            argCount=argCount+1;
            if argCount > argNum
                msg{1}='The ''file'' keyword must be followed by a file name.';
                [msg]=ep_errorMsg(msg);
                return
            end;
            if ~ischar(inputSet{argCount})
                msg{1}='The ''file'' keyword must be followed by a file name.';
                [msg]=ep_errorMsg(msg);
                return
            end;
            fileName=inputSet{argCount};
            argCount=argCount+1;
        case 'format'
            argCount=argCount+1;
            if argCount > argNum
                msg{1}='The ''format'' keyword must be followed by a format name.';
                [msg]=ep_errorMsg(msg);
                return
            end;
            if ~ischar(inputSet{argCount})
                msg{1}='The ''format'' keyword must be followed by a format name.';
                [msg]=ep_errorMsg(msg);
                return
            end;
            fileFormat=inputSet{argCount};
            argCount=argCount+1;
        case 'samples'
            argCount=argCount+1;
            if argCount > argNum
                msg{1}='The ''samples'' keyword must be followed by a list of samples.';
                [msg]=ep_errorMsg(msg);
                return
            end;
            if ~isnumeric(inputSet{argCount})
                msg{1}='The ''samples'' keyword must be followed by a set of numbers (e.g., [1:250]).';
                [msg]=ep_errorMsg(msg);
                return
            end;
            samples=inputSet{argCount};
            argCount=argCount+1;
        case 'cells'
            argCount=argCount+1;
            if argCount > argNum
                msg{1}='The ''cells'' keyword must be followed by a list of cells.';
                [msg]=ep_errorMsg(msg);
                return
            end;
            if ~isnumeric(inputSet{argCount})
                msg{1}='The ''cells'' keyword must be followed by a set of numbers (e.g., [1:3]).';
                [msg]=ep_errorMsg(msg);
                return
            end;
            cells=inputSet{argCount};
            argCount=argCount+1;
        case 'subjects'
            argCount=argCount+1;
            if argCount > argNum
                msg{1}='The ''subjects'' keyword must be followed by a list of subjects.';
                [msg]=ep_errorMsg(msg);
                return
            end;
            if ~isnumeric(inputSet{argCount})
                msg{1}='The ''subjects'' keyword must be followed by a set of numbers (e.g., [1:3]).';
                [msg]=ep_errorMsg(msg);
                return
            end;
            subjects=inputSet{argCount};
            argCount=argCount+1;
        case 'channels'
            argCount=argCount+1;
            if argCount > argNum
                msg{1}='The ''channels'' keyword must be followed by a list of channels.';
                [msg]=ep_errorMsg(msg);
                return
            end;
            if ~isnumeric(inputSet{argCount})
                msg{1}='The ''channels'' keyword must be followed by a set of numbers (e.g., [1:100 105:129]).';
                [msg]=ep_errorMsg(msg);
                return
            end;
            channels=inputSet{argCount};
            argCount=argCount+1;
        case 'montage'
            argCount=argCount+1;
            if argCount > argNum
                msg{1}='The ''montage'' keyword must be followed by a montage name.';
                [msg]=ep_errorMsg(msg);
                return
            end;
            if ~ischar(inputSet{argCount}) && ~isempty(inputSet{argCount})
                msg{1}='The ''montage'' keyword must be followed by a montage name.';
                [msg]=ep_errorMsg(msg);
                return
            end;
            montage=inputSet{argCount};
            argCount=argCount+1;
        case 'name'
            argCount=argCount+1;
            if argCount > argNum
                msg{1}='The ''name'' keyword must be followed by an experiment name.';
                [msg]=ep_errorMsg(msg);
                return
            end;
            if ~ischar(inputSet{argCount})
                msg{1}='The ''name'' keyword must be followed by an experiment name.';
                [msg]=ep_errorMsg(msg);
                return
            end;
            ename=inputSet{argCount};
            argCount=argCount+1;
        case 'labels'
            argCount=argCount+1;
            if argCount > argNum
                msg{1}='The ''labels'' keyword must be followed by a set of condition labels and a set of subject labels, both within a set.';
                [msg]=ep_errorMsg(msg);
                return
            end;
            if ~iscell(inputSet{argCount})
                msg{1}='The ''labels'' keyword must be followed by a set of condition labels and a set of subject labels, both within a set.';
                [msg]=ep_errorMsg(msg);
                return
            end;
            if length(inputSet{argCount}) ~=2
                msg{1}='The ''labels'' keyword must be followed by a set of condition labels and a set of subject labels, both within a set.';
                [msg]=ep_errorMsg(msg);
                return
            end;
            cellLabels=inputSet{argCount}{1};
            subLabels=inputSet{argCount}{2};
            if ~iscell(cellLabels)
                cellLabels=cellstr(cellLabels);
            end;
            if ~iscell(subLabels)
                subLabels=cellstr(subLabels);
            end;
            argCount=argCount+1;
        case 'type'
            argCount=argCount+1;
            if argCount > argNum
                msg{1}='The ''type'' keyword must be followed by a data type.';
                [msg]=ep_errorMsg(msg);
                return
            end;
            if ~ischar(inputSet{argCount})
                msg{1}='The ''type'' keyword must be followed by a data type.';
                [msg]=ep_errorMsg(msg);
                return
            end;
            if ~any(strcmp(inputSet{argCount},{'continuous','single_trial','average','grand_average', 'factors'}))
                msg{1}='The data type must be one of the following: continuous, single_trial, average, factors.';
                [msg]=ep_errorMsg(msg);
                return
            end;
            dataType=inputSet{argCount};
            argCount=argCount+1;
        case 'NumberSubjects'
            argCount=argCount+1;
            if argCount > argNum
                msg{1}='The ''NumberSubjects'' keyword must be followed by number of subjects or factors.';
                [msg]=ep_errorMsg(msg);
                return
            end;
            if ~isnumeric(inputSet{argCount})
                msg{1}='The ''NumberSubjects'' keyword must be followed by number of subjects or factors.';
                [msg]=ep_errorMsg(msg);
                return
            end;
            numSubs=inputSet{argCount};
            argCount=argCount+1;
        case 'prestim'
            argCount=argCount+1;
            if argCount > argNum
                msg{1}='The ''prestim'' keyword must be followed by the number of msec prior to the event onset.';
                [msg]=ep_errorMsg(msg);
                return
            end;
            if ~isnumeric(inputSet{argCount})
                msg{1}='The ''prestim'' keyword must be followed by the number of msec prior to the event onset.';
                [msg]=ep_errorMsg(msg);
                return
            end;
            baseline=inputSet{argCount};
            argCount=argCount+1;
        case 'ced'
            argCount=argCount+1;
            if argCount > argNum
                msg{1}='The ''ced'' keyword must be followed by the name of the .ced file.';
                [msg]=ep_errorMsg(msg);
                return
            end;
            if ~ischar(inputSet{argCount})
                msg{1}='The ''ced'' keyword must be followed by the name of the .ced file.';
                [msg]=ep_errorMsg(msg);
                return
            end;
            ced=inputSet{argCount};
            argCount=argCount+1;
        case 'eloc'
            argCount=argCount+1;
            if argCount > argNum
                msg{1}='The ''eloc'' keyword must be followed by the electrode information.';
                [msg]=ep_errorMsg(msg);
                return
            end;
            if ~isempty(inputSet{argCount}) && ~isstruct(inputSet{argCount})
                msg{1}='The ''eloc'' keyword must be followed by the electrode information.';
                [msg]=ep_errorMsg(msg);
                return
            end;
            eloc=inputSet{argCount};
            argCount=argCount+1;
        case 'silent'
            argCount=argCount+1;
            if argCount > argNum
                msg{1}='The ''silent'' keyword must be followed by on or off.';
                [msg]=ep_errorMsg(msg);
                return
            end;
            if ~ischar(inputSet{argCount})
                msg{1}='The ''silent'' keyword must be followed by on or off.';
                [msg]=ep_errorMsg(msg);
                return
            end;
            if ~any(strcmp(inputSet{argCount},{'on','off'}))
                msg{1}='The ''silent'' keyword must be followed by on or off.';
                [msg]=ep_errorMsg(msg);
                return
            end;
            silent=inputSet{argCount};
            argCount=argCount+1;
        case 'reference'
            argCount=argCount+1;
            if argCount > argNum
                msg{1}='The ''reference'' keyword must be followed by the number of the reference channel(s).';
                [msg]=ep_errorMsg(msg);
                return
            end;
            if ~isnumeric(inputSet{argCount})
                msg{1}='The ''reference'' keyword must be followed by the number of the reference channel(s).';
                [msg]=ep_errorMsg(msg);
                return
            end;
            refChan=inputSet{argCount};
            argCount=argCount+1;
        case 'textPrefs'
            argCount=argCount+1;
            if argCount > argNum
                msg{1}='The ''textPrefs'' keyword must be followed by a structured variable with the preferences.';
                [msg]=ep_errorMsg(msg);
                return
            end;
            if ~isstruct(inputSet{argCount})
                msg{1}='The ''textPrefs'' keyword must be followed by a structured variable with the preferences.';
                [msg]=ep_errorMsg(msg);
                return
            end;
            if ~isfield(inputSet{argCount},'firstRow') || ~isfield(inputSet{argCount},'firstCol') || ~isfield(inputSet{argCount},'lastCol') || ~isfield(inputSet{argCount},'orientation')
                msg{1}='The structured variable must have all four fields included.';
                [msg]=ep_errorMsg(msg);
                return
            end;
            textPrefs=inputSet{argCount};
            argCount=argCount+1;
        otherwise
            keyword=inputSet{argCount};
            if isnumeric(inputSet{argCount})
                keyword=num2str(inputSet{argCount});
            end;
            msg{1}=[keyword ' is not a keyword.'];
            [msg]=ep_errorMsg(msg);
            return
    end;
end;

if isempty(fileName)
    [fileName, pathname] = uigetfile('*.*','Open:');
    if ~ischar(fileName)
        fileName=[];
    end;
    
    if (isempty(fileName))
        msg{1}='No filename selected. You have to click on a name';
        [msg]=ep_errorMsg(msg);
        return
    end
    fileName = [pathname fileName];
else
    if ~exist(fileName,'file')
        msg{1}=['The file ' fileName ' does not exist.'];
        [msg]=ep_errorMsg(msg);
        return
    end;
end;

%determine file type
[pathstr, name, fileSuffix] = fileparts(fileName);
[pathstr2, name2, fileSuffix2] = fileparts(name);

if any(strcmp(fileFormat,{'egi_egia','egi_egis'})) && ~any(strcmpi(fileSuffix,{'.egis','.ave','.gav','.raw','.ses'}))
    msg{1}='EGIS file names must end in .egis, .ave, .gav, .raw, or .ses to be recognized.';
    if any(strcmpi(fileSuffix2,{'.egis','.ave','.gav','.raw','.ses'}))
        msg{2}='It appears that you may have a hidden suffix at the end of your file name.';
    end;
    [msg]=ep_errorMsg(msg);
    return
end;

if strcmp(fileFormat,'egi_sbin') && ~any(strcmpi(fileSuffix,{'.sbin','.raw'}))
    msg{1}='SBIN file names must end in .raw or .sbin to be recognized.';
    if any(strcmp(fileSuffix2,{'.sbin','.raw'}))
        msg{2}='It appears that you may have a hidden suffix at the end of your file name.';
    end;
    [msg]=ep_errorMsg(msg);
    return
end;

if strcmp(fileFormat,'ep_mat') && ~strcmpi(fileSuffix,'.ept')
    msg{1}='EP file names must end in .ept to be recognized.';
    if strcmp(fileSuffix2,'.ept')
        msg{2}='It appears that you may have a hidden suffix at the end of your file name.';
    end;
    [msg]=ep_errorMsg(msg);
    return
end;

if isempty(fileFormat)
    switch fileSuffix
        case '.sbin'
            fileFormat='egi_sbin';
        case '.egis'
            fileFormat='egi_egia';
        case '.mat'
            fileFormat='ep_mat';
        otherwise
            if strcmp(silent,'off')
                disp('Assuming file format is EGIS format.');
            end;
            fileFormat='egi_egia';
    end;
end;

if strcmp(fileFormat,'egi_egis') && ~strcmp(dataType,'single_trial')
    dataType='single_trial';
    if strcmp(silent,'off')
        disp('EGIS session files are always single trial format.');
    end;
end;

if strcmp(fileFormat,'egi_egia') && any(strcmp(dataType,{'single_trial','continuous'}))
    dataType='average';
    if strcmp(silent,'off')
        disp('EGIS average files are never single trial or continuous data types.  Defaulting to assume the file is an average file.');
    end;
end;

if strcmp(fileFormat,'ep_mat')
    try
        load('-mat', fileName);
    catch
        msg{1}=['The attempt to load in the file ' fileName 'resulted in the error:' lasterr];
        [msg]=ep_errorMsg(msg);
        return
    end;
    fiveDdata=EPdata.data;
    timeNames=EPdata.timeNames;
    subNames=EPdata.subNames;
    numSubs=length(subNames);
    cellNames=EPdata.cellNames;
    numWaves=length(cellNames);
    numCells=length(unique(cellNames));
    if ~isempty(dataType)
        switch dataType
            case {'continuous','single_trial'}
                if ~strcmp(dataType,EPdata.dataType)
                    disp(['File indicates its data type is actually ' EPdata.dataType '.']);
                    dataType=EPdata.dataType;
                end;
            case {'average','grand_average'}
                if any(strcmp(EPdata.dataType,{'continuous','single_trial'}))
                    disp(['File indicates its data type is actually ' EPdata.dataType '.']);
                    dataType=EPdata.dataType;
                end;
            case {'factors'}
                if any(strcmp(EPdata.dataType,{'continuous','single_trial'}))
                    dataType=EPdata.dataType;
                end;
        end;
    else
        dataType=EPdata.dataType;
    end;
    events=EPdata.events;
    trialNames=EPdata.trialNames;
    trialSpecs=EPdata.trialSpecs;
    avgNum=EPdata.avgNum;
    subNum=EPdata.subNum;
    subjectSpecs=EPdata.subjectSpecs;
    chanNames=EPdata.chanNames;
    facNames=EPdata.facNames;
    facTypes=EPdata.facTypes;
    chanTypes=EPdata.chanTypes;
    cellTypes=EPdata.cellTypes;
    subTypes=EPdata.subTypes;
    sampleRate=EPdata.Fs;
    trialSpecNames=EPdata.trialSpecNames;
    subjectSpecNames=EPdata.subjectSpecNames;
    if ~iscell(subjectSpecNames)
        subjectSpecNames={};
    end;
    facVecS=EPdata.facVecS;
    facVecT=EPdata.facVecT;
    facData=EPdata.facData;
    if isempty(baseline)
        baseline=EPdata.baseline;
    end;
    if any(strcmp(ced,{'eeglab','none'})) || isempty(ced)
        ced=EPdata.ced;
        eloc=EPdata.eloc;
    end;
    if isempty(montage)
        montage=EPdata.montage;
    end;
    if isfield(EPdata,'analysis')
        if isfield(EPdata.analysis,'blinkTrial')
            blinkTrial= EPdata.analysis.blinkTrial;
            if strcmp(dataType,'continuous')
                if (size(fiveDdata,2) >= sampleRate*2) && size(blinkTrial,2) ==1
                    blinkTrial = []; %backward compatibility conversion
                end;
            end;
        end;
        if isfield(EPdata.analysis,'saccadeTrial')
            saccadeTrial= EPdata.analysis.saccadeTrial;
            if strcmp(dataType,'continuous')
                if (size(fiveDdata,2) >= sampleRate*2) && size(saccadeTrial,2) ==1
                    saccadeTrial = []; %backward compatibility conversion
                end;
            end;
        end;
        if isfield(EPdata.analysis,'saccadeOnset')
            saccadeOnset= EPdata.analysis.saccadeOnset;
            if strcmp(dataType,'continuous')
                if (size(fiveDdata,2) >= sampleRate*2) && size(saccadeOnset,2) ==1
                    saccadeOnset = []; %backward compatibility conversion
                end;
            end;
        end;
        if isfield(EPdata.analysis,'moveTrial')
            moveTrial= EPdata.analysis.moveTrial;
            if strcmp(dataType,'continuous')
                if (size(fiveDdata,2) >= sampleRate*2) && size(moveTrial,2) ==1
                    moveTrial = []; %backward compatibility conversion
                end;
            end;
        end;
        if isfield(EPdata.analysis,'badTrials')
            badTrials= EPdata.analysis.badTrials;
            if strcmp(dataType,'continuous')
                if (size(fiveDdata,2) >= sampleRate*2) && size(badTrials,2) ==1
                    badTrials = []; %backward compatibility conversion
                end;
            end;
        end;
        if isfield(EPdata.analysis,'badChans')
            badChans= EPdata.analysis.badChans;
            if strcmp(dataType,'continuous')
                if (size(fiveDdata,2) >= sampleRate*2) && size(badChans,2) ==1
                    badChans = []; %backward compatibility conversion
                end;
            end;
        end;
    end;
    
    if isfield(EPdata,'noise')
        noise= EPdata.noise;
    end;
    if isfield(EPdata,'std')
        stanDev= EPdata.std;
    end;
    if isfield(EPdata,'implicit')
        implicit=EPdata.implicit;
        if isfield(implicit,'chantype')
            implicit=rmfield(implicit,'chantype'); %backward compatibility for older files with a chantype field.
        end;
    end;
elseif (strcmp(fileFormat,'eeglab_set') && strcmp(fileSuffix,'.study'))
    try
        eval(['load(''-mat'',  ''' fileName ''');']);
    catch
        msg{1}=['The attempt to load in the file ' fileName 'resulted in the error:' lasterr];
        [msg]=ep_errorMsg(msg);
        return
    end;
    
    fileList=cell(length(STUDY.datasetinfo),1);
    
    if (length(fileList) > 1) && strcmp(dataType,'continuous')
        msg{1}='Multiple files cannot be combined into a single continuous data file.  You will need to load them in separately.';
        [msg]=ep_errorMsg(msg);
        return
    end;
    
    if (length(unique({STUDY.datasetinfo.subject})) > 1) && strcmp(dataType,'single_trial')
        msg{1}='Multiple subjects cannot be combined into a single single-trial data file.  You will need to load them in separately.';
        [msg]=ep_errorMsg(msg);
        return
    end;
    
    for setFile=1:length(STUDY.datasetinfo)
        theFile=[STUDY.datasetinfo(setFile).filepath filesep STUDY.datasetinfo(setFile).filename];
        if ~exist(theFile,'file')
            theFile=STUDY.datasetinfo(setFile).filename;
            if ~exist(theFile,'file')
                msg{1}=['The file ' theFile ' is missing.'];
                [msg]=ep_errorMsg(msg);
                return
            end;
        end;
        fileList{setFile}=theFile;
    end;
    mergeName=STUDY.name;
    if isempty(mergeName)
        mergeName='mergedData';
    end;
    
    mergeArg=[];
    for file=1:length(fileList)
        mergeArg{file,1}=fileList{file};
        mergeArg{file,2}='format';
        mergeArg{file,3}='eeglab_set';
        mergeArg{file,4}='type';
        mergeArg{file,5}=dataType;
        mergeArg{file,6}='labels';
        
        theSession=STUDY.datasetinfo(file).session;
        if isempty(theSession)
            theSession='Cond';
        end;
        theCondition=STUDY.datasetinfo(file).condition;
        if isempty(theCondition)
            theSession='1';
        end;
        theCellName=[theSession '-' theCondition];
        
        theGroup=STUDY.datasetinfo(file).group;
        if isempty(theGroup)
            theGroup='Sub';
        end;
        theSubject=STUDY.datasetinfo(file).subject;
        if isempty(theSubject)
            theSubject='1';
        end;
        theSubjectName=[theGroup '-' theSubject];
        
        mergeArg{file,7}={theCellName theSubjectName};
    end;
    
    [EPdata]=ep_mergeEPfiles(mergeArg,mergeName);
    if isempty(EPdata)
        return;
    end;
    hdr=[];
    fiveDdata=EPdata.data;
    ced=EPdata.ced;
    eloc=EPdata.eloc;
    timeNames=EPdata.timeNames;
    subNames=EPdata.subNames;
    numSubs=length(subNames);
    cellNames=EPdata.cellNames;
    numWaves=length(cellNames);
    numCells=length(unique(cellNames));
    events=EPdata.events;
    trialNames=EPdata.trialNames;
    trialSpecs=EPdata.trialSpecs;
    avgNum=EPdata.avgNum;
    subNum=EPdata.subNum;
    subjectSpecs=EPdata.subjectSpecs;
    chanNames=EPdata.chanNames;
    facNames=EPdata.facNames;
    facTypes=EPdata.facTypes;
    chanTypes=EPdata.chanTypes;
    cellTypes=EPdata.cellTypes;
    subTypes=EPdata.subTypes;
    sampleRate=EPdata.Fs;
    trialSpecNames=EPdata.trialSpecNames;
    subjectSpecNames=EPdata.subjectSpecNames;
    if ~iscell(subjectSpecNames)
        subjectSpecNames={};
    end;
    facVecS=EPdata.facVecS;
    facVecT=EPdata.facVecT;
    facData=EPdata.facData;
    if isempty(baseline)
        baseline=EPdata.baseline;
    end;
else
    
    %determine data type
    if strcmp(fileFormat,'egi_egis') && isempty(dataType)
        dataType='single_trial';
    end;
    
    if strcmp(fileFormat,'egi_egia') && isempty(dataType)
        dataType='average';
    end;
    
    if strcmp(fileFormat,'ns_avg') && isempty(dataType)
        dataType='average';
    end;
    
    if strcmp(fileFormat,'ns_eeg') && isempty(dataType)
        dataType='single_trial';
    end;
    
    if strcmp(fileFormat,'ns_cnt') && isempty(dataType)
        dataType='continuous';
    end;
    
    if isempty(dataType)
        dataType='single_trial';
        if strcmp(silent,'off')
            disp('Defaulting to assuming the file is single trial segmented data from one subject.');
        end;
    end;
    
    %read in data
    if strcmp(fileFormat,'text')
        fid=fopen(fileName);
        for i=1:textPrefs.firstRow
            temp=fgetl(fid);
        end;
        delim='\t';
        if ~isempty(strfind(temp,',')) && isempty(strfind(temp,'\t'))
            delim=','; %if there are commas and no tabs, assume it is a comma-delimited file.
        end;
        
        numcols=length(regexp(temp,delim))+1; %determine number of columns based on tab markers
        if regexp(temp,[delim '$'])
            numcols=numcols-1; %if there is an extra tab at the end of the line, drop it.
        end;
        
        frewind(fid);
        for i=1:textPrefs.firstRow-1
            temp=fgetl(fid);
        end;
        
        lastCol=textPrefs.lastCol;
        if lastCol ==0
            lastCol=numcols;
        end;
        
        theData=textscan(fid, [repmat('%*s',1,textPrefs.firstCol-1) repmat('%f',1,lastCol-textPrefs.firstCol+1) repmat('%*s',1,numcols-lastCol)],'Delimiter',delim);
        fclose(fid);
        theData=cell2mat(theData);
        
        if textPrefs.orientation ==1
            theData=theData';
        end;
        
        hdr.nChans=size(theData,1);
        hdr.nSamples=size(theData,2);
        hdr.nSamplesPre=0;
        hdr.nTrials=0;
        hdr.Fs=[];
        for i = 1:hdr.nChans
            hdr.label{i}  = ['e' num2str(i)];
        end;
%     elseif strcmp(fileFormat,'edf')
%         hdr = read_biosig_header(fileName);
%         chanindx = 1:hdr.nChans;
%         begsample = 1;
%         endsample = hdr.nSamples*hdr.nTrials;
%         [theData] = read_biosig_data(fileName, hdr, begsample, endsample, chanindx);
%         eventHdr=[];
%         disp('Support for EDF file format was broken due to dropping of Biosig from FieldTrip (which is used to provide file format support). An attempt has been made to reinstate it but the one sample .edf file that I have yielded defective output.  In order to fix this, I would need someone with access to .edf files to work with me to help track down the problem.');
    elseif strcmp(fileFormat,'ns_mat')
        [theNSFData,theEvents]=ep_readNSF(fileName);
        if isempty(theNSFData)
            msg{1}=['The attempt to load in the file ' fileName 'resulted in no data'];
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        theData=zeros(size(theNSFData{1,1},1),size(theNSFData{1,1},2),size(theNSFData,2));
        switch dataType
            case {'continuous','single_trial'}
                cellNames=theNSFData(2,:);
                for i=1:length(cellNames)
                    theIndex=strfind(cellNames{i},'_Segment');
                    cellNames{i}=cellNames{i}(1:theIndex(end)-1);
                    theData(:,:,i)=theNSFData{1,i};
                end;
                %cellNames=unique(theCellNames);
                %trialCount=zeros(length(cellNames));
                %fiveDdata=zeros(size(theData{1,1},1),size(theData{1,1},2),size(theData,2),1,1);
%                 for i=1:size(theData,2)
%                     theCell=strcmp(theCellNames{i},cellNames);
%                     fiveDdata(:,:,theCell,1,1)=theData{1,i};
%                     trialCount(theCell)=trialCount(theCell)+1;
%                     trialNames(i)=trialCount(theCell);
%                 end;
            case {'average','grand_average','factors'}
                subNames=theNSFData(2,:);
                cellNames=theNSFData(2,:);
                for i=1:length(cellNames)
                    theIndex=strfind(cellNames{i},'_Average');
                    cellNames{i}=cellNames{i}(1:theIndex(end)-1);
                    subNames{i}=subNames{i}(theIndex(end)+8:end);
                    theData(:,:,i)=theNSFData{1,i};
                end;
%                 cellNames=unique(theCellNames);
%                 subNames=unique(theSubNames);
%                 fiveDdata=zeros(size(theData{1,1},1),size(theData{1,1},2),size(theData,2),1,1);
%                 for i=1:size(theData,2)
%                     theCell=strcmp(theCellNames{i},cellNames);
%                     theSub=strcmp(theSubNames{i},subNames);
%                     fiveDdata(:,:,theCell,theSub,1)=theData{1,i};
%                 end;
        end;
        hdr.nChans=size(theData,1);
        hdr.nSamples=size(theData,2);
        hdr.nSamplesPre=0;
        hdr.nTrials=0;
        if ~isempty(sampleRate)
            hdr.Fs=sampleRate;
        else
            hdr.Fs=250;
            disp('Assuming default sample rate of 250Hz.');
        end;
        
        for i = 1:hdr.nChans
            hdr.label{i}  = ['e' num2str(i)];
        end;
        
        eventHdr=[];
        if (theEvents{2,end}/(1000/hdr.Fs)) < (size(theData,2)*size(theData,3))
            for i = 1:size(theEvents,2)
                eventHdr(i).value=theEvents{1,i};
                eventHdr(i).type=[];
                eventHdr(i).sample=theEvents{2,i}/(1000/hdr.Fs); %convert latency to samples from milliseconds.
            end;
        else
            disp('Some events fall outside the range of data points.  One possible explanation is that when NetStation is set to drop bad segments, it only drops');
            disp('them from the voltage data, not the event data.  If this was done, then it is no longer possible');
            disp('to determine which event data go with which voltage data.  In any case, the event data will therefore be ignored.');
        end;
        
    else
        try
            [hdr]=ft_read_header(fileName,'dataFormat',fileFormat, 'headerformat',fileFormat);
            
            [theData]=ft_read_data(fileName,'dataformat',fileFormat, 'headerformat',fileFormat);
            
            [eventHdr]=ft_read_event(fileName,'dataformat',fileFormat, 'headerformat',fileFormat,'eventformat',fileFormat);
        catch
            msg{1}='No data were read.  The error message was:';
            msg{2}=lasterr;
            [msg]=ep_errorMsg(msg);
            return
        end;
    end;
    
    if any(any(any(isnan(theData))))
        msg{1}='Error: The data contained non-numbers.';
        [msg]=ep_errorMsg(msg);
        return
    end;
    
    if any(any(any(isinf(theData))))
        msg{1}='Error: The data contained infinite numbers.';
        [msg]=ep_errorMsg(msg);
        return
    end;
    
    if isfield(eventHdr,'sample')
        if any([eventHdr.sample] < 0)
            msg{1}='Error: The events data contained negative samples.';
            [msg]=ep_errorMsg(msg);
            return
        end;
    else
        eventHdr.sample=[];
    end;
    
    if ~isfield(eventHdr,'value')
        eventHdr.value=[];
    end;

    if ~isfield(eventHdr,'type')
        eventHdr.type=[];
    end;

    for event=1:length(eventHdr)
        if ~isempty(eventHdr(event).value)
            eventValues{event}=eventHdr(event).value;
        else
            eventValues{event}=eventHdr(event).type;
        end;
    end;
    
    if isempty(theData) && strcmp(fileFormat,'egi_egia') %if it is actually  egi_egis, may return empty data
        fileFormat='egi_egis';
        dataType='single_trial';
        [theData]=read_data(fileName,'dataformat',fileFormat, 'headerformat',fileFormat);
    end;
    
    if isempty(theData)
        msg{1}='No data were read.  Perhaps file format was misspecified?';
        [msg]=ep_errorMsg(msg);
        return
    end
    
    if (strcmp(fileFormat,'egi_egia') && hdr.orig.fhdr(2) == 3)
        disp('The file indicates that this is actually an EGIS session file.');
        fileFormat='egi_egis';
        dataType = 'single_trial';
        numSubs=1;
        [theData]=read_data(fileName,'dataformat',fileFormat, 'headerformat',fileFormat);
    end;
    
    if (strcmp(fileFormat,'egi_egis') && hdr.orig.fhdr(2) == -1)
        disp('The file indicates that this is actually an EGIS average file.');
        fileFormat='egi_egia';
        if strcmp(dataType,'single_trial')
            dataType = 'average';
        end;
    end;
    
    %Determine number of subjects and factors
    if strcmp(fileFormat,'egi_egia') && isempty(numSubs)
        numSubs = hdr.orig.chdr(1,2); %since NetStation does not properly set the fhdr(11) field, use the number of subjects from the chdr instead
    end;
    
    if strcmp(fileFormat,'egi_sbin') && isempty(numSubs)
        if ~any(strcmp(dataType,{'single_trial','continuous'}))
            numSubs = size(theData,3)/hdr.orig.header_array(14);
        else
            numSubs=1;
        end;
    end;
    
    if strcmp(dataType,'factors') && ~isempty(numSubs)
        numFacs=numSubs; %non-EP files will normally have only subjects OR factors.
        numSubs=1;
        if strcmp(fileFormat,'egi_egia')
            if numFacs == hdr.orig.chdr(1,2)-1;
                numFacs=numFacs+1; %it is assumed that factor files always have an additional summed factors waveform
            end;
        end;
    end;
    
    if isempty(numSubs)
        numSubs=1;
    end;
    
    if strcmp(fileFormat,'egi_egia')
        if ~strcmp(dataType,'factors')
            if (numSubs ~= hdr.orig.chdr(1,2)) && (hdr.orig.chdr(1,2)*hdr.orig.fhdr(18)==size(theData,3))
                numSubs = hdr.orig.chdr(1,2);
                disp(['According to the file, the number of subjects is actually: ' num2str(hdr.orig.chdr(1,2))]);
            end;
        else
            
            if (numFacs ~= hdr.orig.chdr(1,2)) && (hdr.orig.chdr(1,2)*hdr.orig.fhdr(18)==size(theData,3))
                numFacs = hdr.orig.chdr(1,2);
                disp(['According to the file, the number of factors is actually: ' num2str(hdr.orig.chdr(1,2))]);
            end;
        end;
    end;
    
    if strcmp(dataType,'single_trial') && size(theData,3) == 1
        disp('A file that has only a single ''trial'' is actually a ''continuous'' file.');
        dataType='continuous';
    end;
    
    if strcmp(dataType,'continuous')
        if numSubs > 1
            disp(['For continuous data, the number of subjects must be one, rather than ' num2str(numSubs) '.']);
        end;
        numSubs=1;
        avgNum=ones(1,1);
        subNum=ones(1,1);
    end;
    
    if strcmp(dataType,'single_trial')
        if numSubs > 1
            disp(['For single trial data, the number of subjects must be one, rather than ' num2str(numSubs) '.']);
        end;
        numSubs=1;
        avgNum=ones(1,size(theData,3));
        subNum=ones(1,size(theData,3));
    end;
    
    %Determine other general information.
    if ~isempty(hdr.Fs)
        sampleRate=hdr.Fs;
    else
        sampleRate=250;
        disp('Assuming default sample rate of 250Hz.');
    end;
    
    chanNames=hdr.label;
    
    if length(chanNames) ~= length(unique(chanNames))
        msg{1}='Error: Channel names must be unique.';
        [msg]=ep_errorMsg(msg);
        return
    end
    
    if isempty(baseline)
        baseline=hdr.nSamplesPre;
        if isempty(baseline)
            baseline=0;
        end;
    else
        if mod(baseline,(1000/sampleRate)) >0
            baseline=baseline-mod(baseline,(1000/sampleRate));
            disp(['The specified baseline was not evenly divided by the sample lengths of ' num2str(1000/sampleRate) ' msec.']);
            disp(['The baseline has been changed to ' num2str(baseline) ' msec.']);
        end;
        baseline=baseline/(1000/sampleRate); %change to samples
    end;
    
    numSamples=size(theData,2);
    timeNames=(((0:numSamples-1)-baseline)*(1000/sampleRate));
    
    %EGIS session file
    if strcmp(fileFormat,'egi_egis')
        trialCounter=0;
        for theCell = 1:hdr.orig.fhdr(18)
            numSpecs=(hdr.orig.chdr(theCell,5)/2);
            for trial = 1:hdr.orig.chdr(theCell,2)
                trialCounter=trialCounter+1;
                cellNames{trialCounter} = deblank(hdr.orig.cnames{theCell});
                trialNames(trialCounter)=trial;
                for spec=1:numSpecs
                    trialSpecs{trialCounter,spec} = hdr.orig.chdr(theCell,5+(trial-1)*numSpecs+spec);
                end;
            end;
        end;
        numWaves=trialCounter;
        numCells=hdr.orig.fhdr(18);
        
        subNames{1}=['sub' sprintf('%03.0f',hdr.orig.fhdr(11))];
        
        for i=1:size(trialSpecs,2)
            trialSpecNames{i}=['spec' sprintf('%03.0f',i)];
        end;
        trialSpecNames{1}='edit';
        
        subjectSpecs=cell(1,13);
        subjectSpecNames{1}='RunDateMo';
        subjectSpecNames{2}='RunDateDay';
        subjectSpecNames{3}='RunDateYr';
        subjectSpecNames{4}='RunTimeHr';
        subjectSpecNames{5}='RunTimeMin';
        subjectSpecNames{6}='RunTimeSec';
        subjectSpecNames{7}='SubjID';
        subjectSpecNames{8}='Handed';
        subjectSpecNames{9}='Sex';
        subjectSpecNames{10}='Age';
        subjectSpecNames{11}='ExperID';
        subjectSpecNames{12}='Comment';
        subjectSpecNames{13}='Text';
        
        for i=1:11
            subjectSpecs{i}=num2str(hdr.orig.fhdr(i+4));
        end;
        subjectSpecs{12}=char(hdr.orig.fcom);
        subjectSpecs{13}=char(hdr.orig.ftext);
        
        %EGIS average file
    elseif strcmp(fileFormat,'egi_egia')
        numCells = hdr.orig.fhdr(18);
        numWaves=numCells; %EGIS average files are never session files
        
        cellNames=deblank(hdr.orig.cnames);
        if strcmp(dataType,'factors')
            for i=1:numFacs
                facNames{i,1}=['fac' sprintf('%03.0f',i)];
                facTypes{i,1}='SGL';
            end;
            facNames{numFacs,1}='summed factors'; %assume the last one is a summed factor since that is what the EP Toolkit generates by default
            facTypes{numFacs,1}='CMB';
            subNames{1}='summed subjects';
        else
            if ~any(hdr.orig.chdr(1,13+(1:numSubs-1)*hdr.orig.chdr(1,5)/2))
                for i=1:numSubs
                    subNames{i}=['sub' sprintf('%03.0f',i)]; %correct for NetStation bug of not outputing subject numbers
                end;
            else
                for i=1:numSubs
                    subNames{i}=['sub' sprintf('%03.0f',hdr.orig.chdr(1,13+(i-1)*hdr.orig.chdr(1,5)/2))];
                end;
            end;
        end;
        
        subjectSpecs=cell(numSubs,13);
        numSpecs=(hdr.orig.chdr(1,5)/2); %assume subject information is the same for each cell
        for theSub = 1:numSubs
            for spec=1:11
                subjectSpecs{theSub,spec} = num2str(hdr.orig.chdr(1,6+(theSub-1)*numSpecs+spec));
            end;
            subjectSpecs{theSub,12}=char(hdr.orig.fcom);
            subjectSpecs{theSub,13}=char(hdr.orig.ftext);
        end;
        
        subjectSpecNames{1}='RunDateMo';
        subjectSpecNames{2}='RunDateDay';
        subjectSpecNames{3}='RunDateYr';
        subjectSpecNames{4}='RunTimeHr';
        subjectSpecNames{5}='RunTimeMin';
        subjectSpecNames{6}='RunTimeSec';
        subjectSpecNames{7}='SubjID';
        subjectSpecNames{8}='Handed';
        subjectSpecNames{9}='Sex';
        subjectSpecNames{10}='Age';
        subjectSpecNames{11}='ExperID';
        subjectSpecNames{12}='Comment';
        subjectSpecNames{13}='Text';
        
        for theCell=1:numCells
            for theSub=1:numSubs
                numSpecs=(hdr.orig.chdr(theCell,5)/2);
                if strcmp(dataType,'factors') || numSubs == 1
                    subNum(theSub,theCell)=hdr.orig.chdr(theCell,5+(theSub-1)*numSpecs+1);
                    avgNum(theSub,theCell)=0;
                else
                    avgNum(theSub,theCell)=hdr.orig.chdr(theCell,5+(theSub-1)*numSpecs+1);
                    subNum(theSub,theCell)=1;
                end
            end;
        end;
        
        %EGI Simple Binary
    elseif strcmp(fileFormat,'egi_sbin')
        numCells = hdr.orig.header_array(14);
        if strcmp(dataType,'factors')
            for i=1:numFacs
                facNames{i}=['fac' sprintf('%03.0f',i)];
                facTypes{i}='SGL';
            end;
            facNames{numFacs}='summed factors';
            facTypes{numFacs}='CMB';
            subNames{1}='summed subjects';
            numWaves=size(theData,3)/numFacs;
        else
            for i=1:numSubs
                subNames{i}=['sub' sprintf('%03.0f',i)];
            end;
            numWaves=size(theData,3)/numSubs;
        end;
        
        if strcmp(dataType,'average')
            subjectSpecs=cell(numSubs,7);
            for spec=1:7
                subjectSpecs{numSubs,spec} = hdr.orig.header_array(spec+1);
            end;
            subjectSpecNames{1}='RunDateYr';
            subjectSpecNames{2}='RunDateMo';
            subjectSpecNames{3}='RunDateDay';
            subjectSpecNames{4}='RunTimeHr';
            subjectSpecNames{5}='RunTimeMin';
            subjectSpecNames{6}='RunTimeSec';
            subjectSpecNames{7}='RunTimeMsec';
        end;
        
        droppedEvent=0;
        eventIndex=ones(length(eventValues),1);
        for i=1:length(eventValues)
            if isempty(eventValues{i})
                eventIndex(i)=0;
            elseif any(strcmp(eventValues{i},{'CELL','TRSP'})) %the information associated with these two events are lost when NS exports the file.
                eventIndex(i)=0;
                droppedEvent=1;
            end;
        end;
        
        eventValues=eventValues(find(eventIndex));
        eventHdr=eventHdr(find(eventIndex));
        
        if droppedEvent
            disp('Dropping CELL and TRSP events since NetStation loses the associated information when it exports a simple binary file.');
        end;
                
        if strcmp(dataType,'continuous')
            disp('Note that NetStation, at least as of version 4.1.2, does not import the event information associated with');
            disp('continuous simple binary files, so if you have problems with this, this is not the Toolkit''s fault.');
            numCells=1; %continuous data only has one cell
            cellNames{1}='cell01';
        elseif strcmp(dataType,'single_trial')
            if ~isempty(cellLabels)
                numCells=length(unique(cellLabels));
                if length(cellLabels)==1
                    cellNames=repmat(cellLabels,numWaves,1);
                else
                    cellNames=cellLabels;
                end;
            elseif length(eventHdr)==numWaves
                numCells=length(unique(eventValues));
                cellNames=eventValues;
            elseif sum(strcmp('trial',{eventHdr.type})) ==numWaves
                cellNames={eventHdr(strcmp('trial',{eventHdr.type})).value};
                numCells=length(unique(cellNames));
            else
                theTrial=ceil([eventHdr.sample]/numSamples);
                theOffset=mod([eventHdr.sample],numSamples);
                if (length(find(theOffset==1))==numWaves) && length(theTrial(unique(find(theOffset==1))))==numWaves
                    numCells=length(unique(eventValues(find(theOffset==1)))); %an event is at the offset every time
                    cellNames=eventValues(find(theOffset==1));
                else
                    numCells=1; %give up trying to separate the cells and put them all into the same one.
                    for i=1:numWaves
                        cellNames{i}='cell01';
                    end;
                    disp('Can''t figure out the cell names so putting all the segments into the same cell.');
                end;
            end;
            
            uniqueCellNames=unique(cellNames);
            trialCounter=zeros(length(uniqueCellNames),1);
            for theCell=1:length(cellNames)
                newCell=strcmp(cellNames(theCell),uniqueCellNames);
                trialCounter(newCell)=trialCounter(newCell)+1;
                trialNames(theCell)=trialCounter(newCell);
            end;
            trialSpecs=cell(numWaves,0);
        else
            if isempty(cellLabels)
                for theCell=1:numCells
                    cellNames{theCell}=char(hdr.orig.CateNames{theCell});
                end;
            else
                cellNames=cellLabels;
            end;
            numCells=length(unique(cellNames));
        end;
        
        %any other sort of file other than EGIS or Simple Binary
    else
        if (hdr.nSamplesPre== 0 && hdr.nTrials==1) && ~strcmp(dataType,'continuous');
            dataType = 'continuous';
            numSubs=1;
            disp('This is actually a continuous raw data file.');
        end;
        
        if length(eventHdr)==1
            if strcmp(eventHdr.type,'average') && ~any(strcmp(dataType,{'average','grand_average'}))
                dataType='average';
                disp('The file indicates that it is actually an average file.');
            end;
        end;
        
        if strcmp(dataType,'factors')
            for i=1:numFacs
                facNames{i}=['fac' sprintf('%03.0f',i)];
                facTypes{i}='SGL';
            end;
            facNames{numFacs}='summed factors';
            facTypes{numFacs}='CMB';
            subNames{1}='summed subjects';
            numWaves=size(theData,3)/numFacs;
        else
            for i=1:numSubs
                subNames{i}=['sub' sprintf('%03.0f',i)];
            end;
            numWaves=size(theData,3)/numSubs;
        end;
        
        if strcmp(fileFormat,'ns_avg')
            if strcmp(dataType,'grand_average')
                subNum=hdr.orig.nsweeps;
            else
                avgNum=hdr.orig.nsweeps;
            end;
        end;
        
        if ~strcmp(dataType,'single_trial')
            numCells = size(theData,3); %number of cells is not ambiguous if not a single_trial file.
            if isempty(cellNames)
                if ~isempty(eventHdr)
                    if sum(strcmp('trial',{eventHdr.type})) ==numCells
                        cellNames={eventHdr(strcmp('trial',{eventHdr.type})).value};
                        for i=1:length(cellNames)
                            if isempty(cellNames{i})
                                cellNames{i}=['cell' sprintf('%03.0f',i)];
                            end;
                        end;
                    elseif sum(strcmp('trigger',{eventHdr.type})) ==numCells
                        cellNames={eventHdr(strcmp('trigger',{eventHdr.type})).value};
                    else
                        theTrial=ceil([eventHdr.sample]/numSamples);
                        theOffset=mod([eventHdr.sample],numSamples);
                        if (length(find(theOffset==1))==numWaves) && length(theTrial(unique(find(theOffset==1))))==numWaves && ~isempty(cell2mat(eventValues))
                            numCells=length(unique(eventValues(find(theOffset==1)))); %an event is at the offset every time
                            cellNames=eventValues(find(theOffset==1));
                        else
                            for i=1:numCells
                                cellNames{i}=['cell' sprintf('%03.0f',i)];
                            end;
                        end;
                    end;
                else
                    for i=1:numCells
                        cellNames{i}=['cell' sprintf('%03.0f',i)];
                    end;
                end;
            end;
            
            if length(unique(cellNames)) ~= length(cellNames)
                dataType='single_trial';
                disp('The file is actually a single_trial data file.');
            elseif numSubs==1
                if all(strncmp(cellNames,'Sub',3)) %if all the cellNames start with 'Sub' then may be a combined subject average file
                    temp=char(cellNames);
                    subNums=str2num(temp(:,4:6));
                    tempCellNames=cellstr(deblank(temp(:,7:end)));
                    if all(subNums) %all of them must also have three digit numbers after the Sub
                        if all(hist(subNums,unique(subNums))==length(unique(tempCellNames))) %if the putative cells and subject names are fully crossed
                            cellNames=unique(tempCellNames);
                            tempSubNames=cellstr(num2str(subNums));
                            subNames=unique(tempSubNames);
                            numSubs=length(subNames);
                            numCells=length(cellNames);
                            numWaves=numCells;
                            theData2=zeros(length(chanNames),numSamples,numCells,numSubs);
                            for i=1:numCells
                                theData2(:,:,strcmp(tempCellNames{i},cellNames),strcmp(tempSubNames{i},subNames))=theData(:,:,i);
                            end;
                            for i=1:numSubs
                                subNames{i}=['sub' strtrim(subNames{i})];
                            end;
                        end;
                    end;
                end;
            end;
        end;
        
        if strcmp(dataType,'single_trial')
            if isempty(cellNames)
                if ~isempty(cellLabels)
                    if length(cellLabels)==1
                        cellNames=repmat(cellLabels,numWaves,1);
                    else
                        cellNames=cellLabels;
                    end;
                elseif strcmp(fileFormat,'ns_eeg')
                    for i=1:numWaves
                        cellNames{i}='cell001';
                    end;
                elseif length(eventHdr)==1 && strcmp(eventHdr.type,'average')
                    for i=1:numWaves
                        cellNames{i}='cell001';
                    end;
                elseif length(eventHdr)==numWaves
                    cellNames=eventValues;
                else
                    if ~isempty(eventHdr)
                        theTrial=ceil([eventHdr.sample]/numSamples);
                        theOffset=mod([eventHdr.sample],numSamples);
                        if (length(find(theOffset==1))==numWaves) && length(theTrial(unique(find(theOffset==1))))==numWaves && ~isempty(cell2mat(eventValues))
                            cellNames=eventValues(find(theOffset==1));
                        end;
                    end;
                end;
            end;
            
            if isempty(cellNames) %if was unable to determine cell names assume just one cell
                for i=1:numWaves
                    cellNames{i}='cell01';
                end;
            end;
            
            uniqueCellNames=unique(cellNames);
            numCells=length(uniqueCellNames);
            trialCounter=zeros(length(uniqueCellNames),1);
            for theCell=1:length(cellNames)
                newCell=strmatch(cellNames(theCell),uniqueCellNames,'exact');
                trialCounter(newCell)=trialCounter(newCell)+1;
                trialNames(theCell)=trialCounter(newCell);
            end;
            trialSpecs=cell(numWaves,0);
        end
        
    end;
    
    if isempty(subNum)
        if strcmp(dataType,'factors')
            subNum=zeros(numSubs,numCells); %number of subjects is unknown
        else
            subNum=ones(numSubs,numCells);
        end;
    end;
    
    if isempty(avgNum)
        if any(strcmp(dataType,{'continuous','single_trial'}))
            avgNum=ones(numSubs,numCells);
        else
            avgNum=zeros(numSubs,numCells); %number in average is unknown
        end;
    end;
    
    if (mod(size(theData,3),numSubs) ~= 0) && strcmp(dataType,'average')
        msg{1}=['Number of subjects (' num2str(numSubs) ') does not divide evenly into the number of epochs (' num2str(size(theData,3)) ').'];
        [msg]=ep_errorMsg(msg);
        return
    end
    
    if strcmp(dataType,'factors')
        if (mod(size(theData,3),numFacs) ~= 0)
            msg{1}=['Number of factors (' num2str(numFacs) '), including summed factor, does not divide evenly into the number of epochs (' num2str(size(theData,3)) ').'];
            [msg]=ep_errorMsg(msg);
            return
        end
    end;
    
    if isempty(ename)
        if any(strcmp(fileFormat,{'egi_egia','egi_egis','factors'}))
            ename = hdr.orig.ename;
        else
            ename = [];
        end;
    end;
    
    if size(ename,1) > size(ename,2)
        ename=ename';
        if size(ename,1) >1
            ename=[];
        end;
    end;
    
    if ~isempty(ename)
        ename=strtrim(ename);
    end;
    
    if sum(subjects)==0
        subjects=[];
    end
    
    if ~isempty(cells) && (max(cells) > numCells)
        msg{1}=['Largest cell to be kept (' num2str(max(cells)) ') is larger than number of cells (' num2str(numCells) ') in dataset.'];
        [msg]=ep_errorMsg(msg);
        return
    end;
    
    if ~strcmp(dataType,'factors')
        if (numWaves*numSubs ~= size(theData,3))
            msg{1}=['Number of epochs (' num2str(numWaves) ') times subjects (' num2str(numSubs) ') does not equal number of observations (' num2str(size(theData,3)) ').'];
            [msg]=ep_errorMsg(msg);
            return
        end;
    elseif (numWaves*numFacs ~= size(theData,3))
        msg{1}=['Number of epochs (' num2str(numWaves) ') times factors (' num2str(numFacs) ') does not equal number of observations (' num2str(size(theData,3)) ').'];
        [msg]=ep_errorMsg(msg);
        return
    end;
    
    if length(trialNames)==1
        trialStr='trial';
    else
        trialStr='trials';
    end
    if numCells==1
        cellStr='cell';
    else
        cellStr='cells';
    end
    if strcmp(dataType,'factors')
        if numFacs==1
            subStr='factor';
        else
            subStr='factors';
        end
    else
        if numSubs==1
            subStr='subject';
        else
            subStr='subjects';
        end
    end;
    
    if strcmp(dataType,'single_trial')
        disp(['Read in ' num2str(length(unique(cellNames))) ' ' cellStr ' with a total of ' num2str(length(trialNames)) ' ' trialStr '.']);
    elseif  strcmp(dataType,'factors')
        disp(['Read in ' num2str(numCells) ' ' cellStr ' and ' num2str(numFacs) ' ' subStr '.']);
        disp('Making the assumption that the last factor is the combination of the other factors.');
    else
        disp(['Read in ' num2str(numCells) ' ' cellStr ' and ' num2str(numSubs) ' ' subStr '.']);
    end;
    
    if strcmp(dataType,{'continuous'}) && ~isempty(cells)
        disp('Cannot select subset of cells in a continuous raw data file.');
        cells=[];
    end;
    
    if ~isempty(subjects) && (max(subjects) > numSubs)
        msg{1}=['Largest subject to be kept (' num2str(max(subjects)) ') is larger than number of subjects (' num2str(numSubs) ') indicated for dataset.'];
        [msg]=ep_errorMsg(msg);
        return
    end;
    
    if max(samples) > size(theData,2)
        msg{1}='Largest sample is larger than number of timepoints in data.';
        [msg]=ep_errorMsg(msg);
        return
    end;
    
    if max(channels) > size(theData,1)
        msg{1}='Largest channel is larger than number of channels in data.';
        [msg]=ep_errorMsg(msg);
        return
    end;
    
    if min(samples) < 0
        error('No negative samples.');
    end;
    
    if min(channels) < 0
        error('No negative channels.');
    end;
    
    if min(subjects) < 0
        error('No negative subjects.');
    end;
    
    %Reorganize waveforms and events
    
    if ~isempty(eventHdr)
        if ~strcmp(dataType,'factors')
            events=cell(numSubs,length(cellNames));
            for theEvent=1:length(eventHdr)
                if ~isempty(eventHdr(theEvent).sample)
                    epoch=floor((eventHdr(theEvent).sample-1)/numSamples)+1;
                    %assume that subjects are grouped together by cell.
                    cellTrial=floor((epoch-1)/numSubs)+1;
                    subject=mod(epoch,numSubs)+1;
                    oneEvent=eventHdr(theEvent);
                    events{subject,cellTrial}=[events{subject,cellTrial} oneEvent];
                end;
            end;
        else
            events=cell(numSubs,length(cellNames));
            for theEvent=1:length(eventHdr)/numFacs
                if ~isempty(eventHdr(theEvent).sample)
                    epoch=floor((eventHdr(theEvent).sample-1)/numSamples)+1;
                    %assume that the events for all the factors are duplicates of the first as all from same set of subject(s).
                    cellTrial=epoch;
                    subject=1;
                    oneEvent=eventHdr(theEvent);
                    events{subject,cellTrial}=[events{subject,cellTrial} oneEvent];
                end;
            end;
        end;
    end;
    
    %reorganize data into 5D array, assuming that subjects are grouped together by cell.
    if ~strcmp(dataType,'factors')
        fiveDdata=zeros(size(theData,1),size(theData,2),numCells,numSubs,1);
        for sub=1:numSubs
            fiveDdata(:,:,:,sub,1)=theData(:,:,sub:numSubs:numSubs*numCells);
        end;
    else
        fiveDdata=zeros(size(theData,1),size(theData,2),numCells,numSubs,numFacs);
        for fac=1:numFacs
            fiveDdata(:,:,:,:,fac)=theData(:,:,fac:numFacs:numFacs*numCells);
        end;
        
    end;
    
    %rearrange trials so that they are grouped by cell
    if strcmp(dataType,'single_trial')
        [uniqueCellNames m cellIDs]=unique(cellNames); %get number of trials in each cell
        cellNums=hist(cellIDs,unique(cellIDs));
        trialCounter=zeros(numCells,1);
        cellNames2=cell(size(cellNames));
        trialNames2=zeros(size(trialNames));
        trialSpecs2=cell(size(trialSpecs));
        events2=cell(size(events));
        avgNum2=zeros(size(avgNum));
        subNum2=zeros(size(subNum));
        
        for theTrial = 1:length(cellIDs)
            newCell=cellIDs(theTrial);
            trialCounter(newCell)=trialCounter(newCell)+1;
            fiveDdata(:,:,sum(cellNums(1:newCell-1))+trialCounter(newCell),1,1)=theData(:,:,theTrial);
            cellNames2(sum(cellNums(1:newCell-1))+trialCounter(newCell))=cellNames(theTrial);
            trialNames2(sum(cellNums(1:newCell-1))+trialCounter(newCell))=trialNames(theTrial);
            trialSpecs2(sum(cellNums(1:newCell-1))+trialCounter(newCell),:)=trialSpecs(theTrial,:);
            events2(1,sum(cellNums(1:newCell-1))+trialCounter(newCell))=events(1,theTrial);
            avgNum2(1,sum(cellNums(1:newCell-1))+trialCounter(newCell))=avgNum(1,theTrial);
            subNum2(1,sum(cellNums(1:newCell-1))+trialCounter(newCell))=subNum(1,theTrial);
        end;
        
        cellNames=cellNames2;
        trialNames=trialNames2;
        trialSpecs=trialSpecs2;
        events=events2;
        avgNum=avgNum2;
        subNum=subNum2;
    end;
end;

numChan=length(chanNames);

% apply cell and sub names if separately specified
if ~isempty(cellLabels)
    if length(cellLabels)==1
        cellNames=repmat(cellLabels,numWaves,1);
    elseif numWaves == length(cellLabels)
        cellNames=cellLabels;
    else
        disp(['The number of epochs (' num2str(numWaves) ') did not match the number of condition labels (' num2str(length(cellLabels)) ').']);
    end;
end;

if ~isempty(subLabels)
    if length(subLabels) ==1
        subNames{1}=subLabels{1};
    elseif numSubs == length(subLabels)
        subNames=subLabels;
    else
        disp(['The number of subjects (' num2str(numSubs) ') did not match the number of subject labels (' num2str(length(subLabels)) ').']);
    end;
end;

numEpochs=numWaves;
if strcmp(dataType,'continuous')
    numEpochs=floor(size(fiveDdata,2)/sampleRate); %excess time points are tacked onto final epoch
    if numEpochs == 0
        numEpochs =1;
    end;
end;

%add analysis fields
if isempty(blinkTrial)
    blinkTrial=zeros(numSubs,numEpochs);
end;
if isempty(saccadeTrial)
    saccadeTrial=zeros(numSubs,numEpochs);
end;
if isempty(saccadeOnset)
    saccadeOnset=zeros(numSubs,numEpochs);
end;
if isempty(moveTrial)
    moveTrial=zeros(numSubs,numEpochs);
end;
if isempty(badTrials)
    badTrials=zeros(numSubs,numEpochs);
end;
if isempty(badChans)
    badChans=zeros(numSubs,numEpochs,numChan);
end;

%add Type information

if isempty(chanTypes)
    chanTypes=cellstr(repmat('EEG',numChan,1));
end;

for i=1:length(refChan)
    if refChan(i) > numChan
        disp(['The reference channel specified ' num2str(refChan) ' is not explicitly in the dataset.']);
    else
        chanTypes{refChan(i)}='REF';
    end;
end;

if isempty(cellTypes)
    cellTypes=cellstr(repmat('SGL',numWaves,1));
end;

if isempty(subTypes)
    switch dataType
        case {'single_trial', 'continuous'}
            subTypes=cellstr(repmat('RAW',numSubs,1));
        case 'average'
            subTypes=cellstr(repmat('AVG',numSubs,1));
        case 'grand_average'
            subTypes=cellstr(repmat('GAV',numSubs,1));
        case 'factors'
            subTypes=cellstr(repmat('GAV',numSubs,1));
    end;
end;

try
    if any(strcmp(fileFormat,{'egi_egia','egi_egis'})) && ismac && isempty(montage)
        [err, montage]=ep_identifySLAY(fileName);
        if ~isempty(montage)
            if err == 1
                disp('Unable to identify montage information in EGIS file because:');
                disp(montage);
                montage=[];
            end;
        end;
    end;
catch
end;

%Ensure that one-dimensional string arrays are column vectors
chanNames=chanNames(:);
timeNames=timeNames(:);
subNames=subNames(:);
cellNames=cellNames(:);
trialNames=trialNames(:);
facNames=facNames(:);
chanTypes=chanTypes(:);
subTypes=subTypes(:);
cellTypes=cellTypes(:);
facTypes=facTypes(:);

%channel coordinate information
if isempty(montage) && any(strcmp(fileFormat,{'egi_egia','egi_egis'}))
    montage=ep_askForMontage;
end;

if strcmp(fileFormat,'eeglab_set')
    if isfield(hdr,'orig')
        if isfield(hdr.orig,'chanlocs')
            if isfield(hdr.orig,'chaninfo')
                if isfield(hdr.orig.chaninfo,'filename')
                    [pathstr, name, ext] = fileparts(hdr.orig.chaninfo.filename);
                    ced=[name ext];
                else
                    ced='none';
                end;
            else
                ced='none';
            end;
            eloc=hdr.orig.chanlocs;
            if isempty(eloc)
                ced=[];
            end;
        end;
    end;
end;

%test out the ced file and set to null if it's no good.
if ~isempty(ced) && ~strcmp('none',ced) && isempty(eloc)
    try
        whichCed=which(ced);
        eloc = readlocs([whichCed],'filetype','chanedit');
        implicit=eloc(1);
        implicit(1)=[];
        eloc=[];
    catch
        disp(['The ced file ' ced ' did not work for some reason.  The error message was:']);
        disp(lasterr)
        ced = [];
        eloc = [];
        implicit=[];
    end;
end;

if isempty(ced)
    if any(strcmp(fileFormat,{'egi_egia','egi_egis'}))
        [ced]=ep_whichEGIced(montage);
    end;
    if isempty(ced)
        thisFile = mfilename('fullpath');
        [pathstr, name, ext] = fileparts(thisFile);
        theSeps=findstr(pathstr,filesep);
        pathstr=[pathstr(1:theSeps(end)-1) filesep 'electrodes'];
        [ced, pathstr] = uigetfile('*.ced','Electrode Coordinate file:',pathstr);
        if ~isnumeric(ced)
            if isempty(which(ced))
                disp('In order to be used, the ced file must be either in the active directory or in the search path.');
            end;
        end;
    end;
    if isnumeric(ced)
        ced='none';
    end;
end;

if ~isempty(ced) && ~strcmp('none',ced)
    if isempty(eloc)
        try
            whichCed=which(ced);
            eloc = readlocs([whichCed],'filetype','chanedit');
            implicit=eloc(1);
            implicit(1)=[];
        catch
            disp(['The ced file ' ced ' did not work for some reason.  The error message was:']);
            disp(lasterr)
            ced = [];
            eloc = [];
            implicit=[];
        end;
    end;
    if ~isempty(eloc)
        if length(eloc) ~= length(unique({eloc.labels}))
            disp(['The ced file ' ced ' had channel names that were not unique.']);
            ced = [];
            eloc = [];
        end
    end;
    
    if ~isempty(ced)
        if ~isfield(eloc,'type')
            eloc(1).type=[];
            implicit(1).type=[];
            disp('Type field missing from ced file.  Will assume all channels are EEG channels.');
        end;
        for i=1:length(eloc)
            if isempty(eloc(i).type)
                eloc(i).type='EEG'; %if type fields are empty, just assume they are EEG channels.
            end;
        end;
        
        numEEG=length(find(strcmp('EEG',{eloc.type})));
        numREF=length(find(strcmp('REF',{eloc.type})));
        numFID=length(find(strcmp('FID',{eloc.type})));
        
        %Formats that use a fixed channel order and have no channel labels in the header
        if any(strcmp(fileFormat,{'egi_egia','egi_egis','egi_sbin','text','ns_mat'}))
            if numEEG == length(chanNames)  %reference is not marked or is implicit
                eloc2=eloc(find(strcmp('EEG',{eloc.type})));
                temp=eloc(find(strcmp('REF',{eloc.type})));
                if ~isempty(temp)
                    for theRefChan=1:length(temp)
                        fiveDdata(end+1,:,:,:,:)=zeros(size(fiveDdata,2),size(fiveDdata,3),size(fiveDdata,4),size(fiveDdata,5));
                        if ~isempty(facVecS)
                            facVecS(end+1,:)=zeros(size(facVecS,2));
                        end;
                        
                        if ~isempty(facData)
                            facData(end+1,:,:,:,:)=zeros(size(facData,2),size(facData,3),size(facData,4),size(facData,5));
                        end;
                        
                        badChans(:,:,end+1)=zeros(size(badChans,1),size(badChans,2));
                        
                        if ~isempty(noise)
                            noise(end+1,:,:,:,:)=zeros(size(noise,2),size(noise,3),size(noise,4),size(noise,5));
                        end;
                        if ~isempty(stanDev)
                            stanDev(end+1,:,:,:,:)=zeros(size(stanDev,2),size(stanDev,3),size(stanDev,4),size(stanDev,5));
                        end;
                        
                        chanTypes{end+1}='REF';
                        eloc2(end+1)=temp(theRefChan);
                    end;
                end;
                chanNames={eloc2.labels};
            elseif numEEG+numREF == length(chanNames)  %explicit references
                eloc2=eloc(find(strcmp('EEG',{eloc.type})));
                eloc2(end+1:end+numREF)=eloc(find(strcmp('REF',{eloc.type})));
                chanNames={eloc(find(strcmp('EEG',{eloc.type}))).labels};
                chanNames(end+1:end+numREF)={eloc(find(strcmp('REF',{eloc.type}))).labels};
            else
                disp(['The ced file ' ced ' had the wrong number of channels.']);
                ced = [];
                eloc = [];
                eloc2=[];
            end;
            if (numFID > 0) && ~isempty(ced)
                temp=eloc(find(strcmp('FID',{eloc.type})));
                [temp.type] = deal('FID');
                implicit(end+1:end+numFID) = temp;
            end;
            
            %Formats that include channel labels in the header and where channels have flexible ordering or may even be absent
            
        else
            chanCount=0;
            nonCED=[];
            for i=1:length(chanNames)
                if find(strcmpi(chanNames(i),{eloc.labels}))
                    chanCount=chanCount+1;
                else
                    nonCED(end+1)=i;
                end;
            end;
            
            if chanCount > 0
                if numREF
                    impRefs=setdiff({eloc(find(strcmp('REF',{eloc.type}))).labels},chanNames); %implicit refs
                    impRefNums=zeros(length(impRefs),1);
                    for i=1:length(impRefs)
                        impRefNums(i)=find(strcmpi(impRefs{i},{eloc.labels}));
                    end;
                    numImpRefs=length(impRefs);
                else
                    impRefs=[];
                    impRefNums=[];
                    numImpRefs=0;
                end;
                
                extraCED=0; %number of channels only in the CED file
                chanIndex=setdiff(sort([find(strcmp('EEG',{eloc.type})) find(strcmp('REF',{eloc.type}))]),impRefNums);
                for newElocChan=1:length(chanIndex)
                    oldElocChan=chanIndex(newElocChan);
                    oldFileChan=find(strcmpi(eloc(oldElocChan).labels,chanNames));
                    if isempty(oldFileChan)
                        extraCED=extraCED+1;
                    end;
                end;
                numChan=length(chanNames)+extraCED;
                numPoints=length(timeNames);
                numCellTrials=length(cellNames);
                numFacs=size(fiveDdata,5);
                if isempty(badChans)
                    badChans=zeros(numSubs,numCellTrials,numChan);
                end;
                if ~isempty(facVecS)
                    facVecS2=zeros(numChan,numFacs);
                else
                    fiveDdata2=zeros(numChan,numPoints,numCellTrials,numSubs);
                end;
                if ~isempty(noise)
                    noise2=zeros(numChan,numPoints,numCellTrials,numSubs);
                end;
                if ~isempty(stanDev)
                    stanDev2=zeros(numChan,numPoints,numCellTrials,numSubs);
                end;
                chanNames2=cell(1,numChan);
                chanTypes2=cell(1,numChan);
                if ~isempty(badChans)
                    badChans2=ones(numSubs,numCellTrials,numChan); %left out channels will be marked bad
                    if ~strcmp(dataType,{'single_trial','continuous'})
                        badChans2=badChans2*(-1); %if an average file then change to -1 as the marker for a bad channel
                    end;
                end;
                for newElocChan=1:length(chanIndex)
                    oldElocChan=chanIndex(newElocChan);
                    oldFileChan=find(strcmpi(eloc(oldElocChan).labels,chanNames));
                    if ~isempty(oldFileChan) %channel is in both CED and data
                        eloc2(newElocChan)=eloc(oldElocChan); %generate eloc array with only EEG and REF channels in it
                        if ~isempty(facVecS)
                            facVecS2(newElocChan,:)=facVecS(oldFileChan,:);
                        else
                            fiveDdata2(newElocChan,:,:,:)=fiveDdata(oldFileChan,:,:,:);
                        end;
                        if ~isempty(noise)
                            noise2(newElocChan,:,:,:)=noise(oldFileChan,:,:,:);
                        end;
                        if ~isempty(stanDev)
                            stanDev2(newElocChan,:,:,:)=stanDev(oldFileChan,:,:,:);
                        end;
                        chanNames2{newElocChan}=eloc(oldElocChan).labels;
                        chanTypes2{newElocChan}=eloc(oldElocChan).type;
                        if ~isempty(badChans)
                            badChans2(:,:,newElocChan)=badChans(:,:,oldFileChan);
                        end;
                    else %channel is only in the CED
                        eloc2(newElocChan)=eloc(oldElocChan); %generate eloc array with only EEG and REF channels in it
                        chanNames2{newElocChan}=eloc(oldElocChan).labels;
                        chanTypes2{newElocChan}=eloc(oldElocChan).type;
                        if strcmp(chanTypes2{newElocChan},'REF')
                            badChans2(:,:,newElocChan)=0; %implicit reference channels are not bad
                        end;
                    end;
                end;
                for i=1:length(nonCED) %channel is only in the data
                    theNonCED=nonCED(i);
                    eloc2(length(chanIndex)+i).theta=[]; %empty theta is marker for missing electrode coordinates
                    if ~isempty(facVecS)
                        facVecS2(length(chanIndex)+i,:)=facVecS(theNonCED,:);
                    else
                        fiveDdata2(length(chanIndex)+i,:,:,:)=fiveDdata(theNonCED,:,:,:);
                    end;
                    if ~isempty(noise)
                        noise2(length(chanIndex)+i,:,:,:)=noise(theNonCED,:,:,:);
                    end;
                    if ~isempty(stanDev)
                        stanDev2(length(chanIndex)+i,:,:,:)=stanDev(theNonCED,:,:,:);
                    end;
                    chanNames2(length(chanIndex)+i)=chanNames(theNonCED);
                    chanTypes2(length(chanIndex)+i)=chanTypes(theNonCED);
                    if ~isempty(badChans)
                        badChans2(:,:,length(chanIndex)+i)=badChans(:,:,theNonCED);
                    end;
                end;
                if ~isempty(facVecS)
                    facVecS=facVecS2;
                else
                    fiveDdata=fiveDdata2;
                end;
                if ~isempty(noise)
                    noise=noise2;
                end;
                if ~isempty(stanDev)
                    stanDev=stanDev2;
                end;
                chanNames=chanNames2;
                chanTypes=chanTypes2;
                if ~isempty(badChans)
                    badChans=badChans2;
                end;
                if numFID > 0
                    temp=eloc(find(strcmp('FID',{eloc.type})));
                    [temp.type] = deal('FID');
                    implicit(end+1:end+numFID) = temp;
                end;
                if numImpRefs > 0
                    temp=eloc(impRefNums);
                    [temp.type] = deal('REF');
                    implicit(end+1:end+numImpRefs)=temp;
                end;
                
                if ~isempty(nonCED)
                    disp(['There were ' num2str(length(nonCED)) ' channels not represented in the CED file.']);
                end;
                if extraCED > 0
                    disp(['There were ' num2str(extraCED) ' ced file channels not represented in the data file.']);
                end;
            else
                disp(['The ced file ' ced ' channel names did not match those in the file.']);
                ced = [];
                eloc2 = [];
            end;
        end;
        eloc=eloc2;
    end;
end;

%Select subsets of data

if isempty(samples) %select samples
    samples=[1:length(timeNames)];
else
    if ~isempty(facVecT)
        facVecT=facVecT(samples,:);
    else
        fiveDdata=fiveDdata(:,samples,:,:,:);
    end;
    if ~isempty(noise)
        noise=noise(:,samples,:,:,:);
    end;
    if ~isempty(stanDev)
        stanDev=stanDev(:,samples,:,:,:);
    end;
    timeNames=timeNames(samples);
    if min(samples) > 1
        baseline = baseline - min(samples) +1;
        for subject = 1:length(subNames)
            for wave = 1:length(cellNames)
                for event = 1:length(events(subject,wave))
                    events{subject,wave}(event).offset=events{subject,wave}(event).offset-min(samples)+1;
                end;
            end;
        end;
    end;
end;

if isempty(channels) %select channels
    channels=[1:length(chanNames)];
else
    if ~isempty(facVecS)
        facVecS=facVecS(channels,:);
    else
        fiveDdata=fiveDdata(channels,:,:,:);
    end;
    if ~isempty(noise)
        noise=noise(channels,:,:,:);
    end;
    if ~isempty(stanDev)
        stanDev=stanDev(channels,:,:,:);
    end;
    chanNames=chanNames(channels);
    chanTypes=chanTypes(channels);
    if ~isempty(badChans)
        badChans=badChans(:,:,channels);
    end;
    numChan=length(chanNames);
    if ~isempty(eloc)
        eloc=eloc(channels);
    end;
end


if ~isempty(cells) %select cells
    if strcmp(dataType,'single_trial')
        uniqueCellNames=unique(cellNames);
        trials=[];
        for i=cells
            trials=unique([trials,find(strcmp(uniqueCellNames{i},cellNames))]);
        end;
        fiveDdata=fiveDdata(:,:,trials,:,:);
        if ~isempty(noise)
            noise=noise(:,:,trials,:,:);
        end;
        if ~isempty(stanDev)
            stanDev=stanDev(:,:,trials,:,:);
        end;
        
        numCells=length(trials);
        numWaves=length(trials);
        cellNames=cellNames(trials);
        cellTypes=cellTypes(trials);
        trialNames=trialNames(trials);
        trialSpecs=trialSpecs(trials);
        events=events(trials);
        badChans=badChans(trials,:);
        moveTrial=moveTrial(trials);
        blinkTrial=blinkTrial(trials);
        saccadeTrial=saccadeTrial(trials);
        saccadeOnset=saccadeOnset(trials);
        badTrials=badTrials(trials);
    else
        avgNum=avgNum(:,cells);
        subNum=subNum(:,cells);
        fiveDdata=fiveDdata(:,:,cells,:,:);
        if ~isempty(noise)
            noise=noise(:,:,cells,:,:);
        end;
        if ~isempty(stanDev)
            stanDev=stanDev(:,:,cells,:,:);
        end;
        
        numCells=length(cells);
        numWaves=length(cells);
        cellNames=cellNames(cells);
        cellTypes=cellTypes(cells);
        events=events(:,cells);
        badChans=badChans(:,cells,:);
        moveTrial=moveTrial(:,cells);
        blinkTrial=blinkTrial(:,cells);
        saccadeTrial=saccadeTrial(:,cells);
        saccadeOnset=saccadeOnset(:,cells);
        badTrials=badTrials(:,cells);
    end;
end;

if ~isempty(subjects) %select subjects
    fiveDdata=fiveDdata(:,:,:,subjects,:);
    if ~isempty(noise)
        noise=noise(:,:,:,subjects,:);
    end;
    if ~isempty(stanDev)
        stanDev=stanDev(:,:,:,subjects,:);
    end;
    
    numSubs = length(subjects);
    subNames=subNames(subjects);
    subTypes=subTypes(subjects);
    subjectSpecs=subjectSpecs(subjects,:);
    avgNum=avgNum(subjects,:);
    subNum=subNum(subjects,:);
    events=events(subjects,:);
    badChans=badChans(subjects,:,:);
    moveTrial=moveTrial(subjects,:);
    blinkTrial=blinkTrial(subjects,:);
    saccadeTrial=blinkTrial(subjects,:);
    saccadeOnset=blinkTrial(subjects,:);
    badTrials=badTrials(subjects,:);
end;

%Detect bad epochs and mark them
for theSub=1:numSubs
    for theWave=1:numWaves
        if ~any(any(any(fiveDdata(:,:,theWave,theSub,:),1),2),5) %do not include epochs that are zeroed out
            if any(strcmp(dataType,'single_trial'))
                badTrials(theSub,theWave)=1;
            else
                avgNum(theSub,theWave)=-1;
                subNum(theSub,theWave)=-1;
            end;
        end;
    end;
end;


%drop events of "trial" type as they are redundant with cell name information
for subject=1:numSubs
    for wave=1:length(cellNames)
        eventList=[];
        for event=1:length(events{subject,wave})
            if ~strcmp(events{subject,wave}(event).type,'trial')
                eventList=[eventList event];
            end;
        end;
        events{subject,wave}=events{subject,wave}(eventList);
    end;
end;

%Display summary of results

if strcmp(silent,'off')
    disp(['The name of the experiment is: ' ename]);
    if ~isempty(baseline) && ~isempty(sampleRate)
        disp(['The pre-stimulus period of the data is: ' num2str(baseline * (1000/sampleRate)) ' msec.']);
        if min(samples)>1
            disp('after dropping samples as instructed.');
        end;
        if any(strcmp(fileFormat,{'egi_egis','egi_sbin','text'}))
            disp(['Note that the data file format (' fileFormat ') is unable to specify the baseline period so it may be in error, in which case you will need to manually fix it using the Edit function.']);
        end;
    end;
end;

if ~isempty(cells) || ~isempty(subjects)
    if length(trialNames)==1
        trialStr='trial';
    else
        trialStr='trials';
    end
    if length(cellNames)==1
        cellStr='cell';
    else
        cellStr='cells';
    end
    if strcmp(dataType,'factors')
        if numFacs==1
            subStr='summed factor';
        else
            subStr='factors (including the summed factor)';
        end
    else
        if numSubs==1
            subStr='subject';
        else
            subStr='subjects';
        end
    end;
    if strcmp(silent,'off')
        if strcmp(dataType,'single_trial')
            disp(['After selecting, there are ' num2str(numCells) ' ' cellStr ' with a total of ' num2str(length(trialNames)) ' ' trialStr '.']);
        elseif strcmp(dataType,'factors')
            disp(['After selecting, there are ' num2str(numCells) ' ' cellStr ' and ' num2str(numFacs) ' ' subStr '.']);
        else
            disp(['After selecting, there are ' num2str(numCells) ' ' cellStr ' and ' num2str(numSubs) ' ' subStr '.']);
        end;
    end;
end;

%Final construction of output file
if any(strcmp(dataType,{'grand_average','factors'}))
    dataType = 'average';
end;

EPdata.data=fiveDdata;
if ~isempty(noise)
    EPdata.noise=noise;
end;
if ~isempty(stanDev)
    EPdata.std=stanDev;
end;

if isempty(dataName)
    [pathstr, name, ext] = fileparts(fileName);
    dataName=name;
end;

EPdata.montage=montage;
EPdata.fileFormat=fileFormat;
EPdata.chanNames=chanNames;
EPdata.timeNames=timeNames;
EPdata.subNames=subNames;
EPdata.cellNames=cellNames;
EPdata.trialNames=trialNames;
EPdata.facNames=facNames;
EPdata.chanTypes=chanTypes;
EPdata.subTypes=subTypes;
EPdata.cellTypes=cellTypes;
EPdata.facTypes=facTypes;
try
    EPver=ver('EP_Toolkit');
catch
    EPver='unavailable'; %workaround for bug in earlier version of Matlab
end;
EPdata.EPver=EPver;
EPdata.ver=ver;
EPdata.date=date;
EPdata.Fs=sampleRate;
EPdata.baseline=baseline;
EPdata.dataName=dataName;
EPdata.ename=ename;
EPdata.dataType=dataType;
EPdata.trialSpecs=trialSpecs;
EPdata.trialSpecNames=trialSpecNames;
EPdata.subjectSpecs=subjectSpecs;
EPdata.subjectSpecNames=subjectSpecNames;
EPdata.events=events;
EPdata.avgNum=avgNum;
EPdata.subNum=subNum;
EPdata.fileName=fileName;
EPdata.history={'readData',varargin};
EPdata.ced=ced;
EPdata.eloc=eloc;
EPdata.implicit=implicit;
EPdata.facVecT=facVecT;
EPdata.facVecS=facVecS;
EPdata.facData=facData;

EPdata.analysis.blinkTrial=blinkTrial;
EPdata.analysis.saccadeTrial=saccadeTrial;
EPdata.analysis.saccadeOnset=saccadeOnset;
EPdata.analysis.moveTrial=moveTrial;
EPdata.analysis.badTrials=badTrials;
EPdata.analysis.badChans=badChans;

[err]=ep_checkEPfile(EPdata);
