function [data, peakLatency, peakSamp, peakChan] = ep_PCAoutput(FactorResults, cnames, cellcoll, exclChan, params, paramNames);
% ep_PCAoutput - [data, peakLatency, peakSamp, peakChan] = ep_PCAoutput(FactorResults, cnames, cellcoll, exclChan, params, paramNames); - output PCA results to an EGIS file
%Inputs
%  FactorResults: Structured variable with results and information about analysis.
%  .FacPat	: Factor pattern matrix - produces standardized variables from scores, scaled by communality  (rows=variables, cols=factors)
%  .FacStr	: Factor structure matrix - correlation between factors and variables (rows=variables, cols=factors)
%  .FacScr	: Factor scores, variance standardized, non-mean corrected  (rows=variables, cols=factors)
%  .FacCof   : Factor scoring coefficients (rows=variables, cols=factors)
%  .FacCor	: Factor correlations
%  .scree    : The full set of unrotated eigenvalues, prior to truncation.
%  .facVar   : %age of variance accounted for by each rotated factor, ignoring other factors.
%			  Calculated based on variance of relationship matrix (if COV or SCP, each variable weighted).
%  .facVarQ  : %age of variance uniquely accounted for by each rotated factor.
%			  Calculated based on variance of relationship matrix (if COV or SCP, each variable weighted)
%  .facVarTot   : Total variance accounted for.
%  .varSD    : Standard deviations of the variables.
%  .PCAmode     : PCAmode used
%  .ROTATION    : Rotation used
%  .MAT_TYPE    : Relationship matrix used
%  .KAISER      : Loading option used
%  .timepoints  : Number of time points in original data
%  .numchan     : Number of channels in original data
%  .numSubs     : Number of subjects.
%  .numCells    : Number of cells
%  .numFacs     : Number of factors
%  .montage     : The electrode montage.
%  .chanNames   : The channel names.
%  .timeNames   : The timepoint names.
%  .subNames    : The subject names.
%  .cellNames   : The cell names.
%  .chanTypes : The type of the channel: EEG, MEG, ANS (autonomic), REF (explicit reference), REG (regional average)
%  .subTypes  : The type of the subject: RAW (single trial), AVG (subject average), GAV (grand average)
%  .cellTypes : The type of the cell: SGL (one cell), CMB (combination of cells)
%  .subjectSpecs     : Cell array (subject,spec) of specific information for subjects
%  .subjectSpecNames : Cell array of the name of each subject spec type.
%  .Fs          : The sampling frequency in Hz.
%  .baseline    : The number of samples in the baseline.
%  .ename       : The name of the experiment.
%  .dataType    : The type of the data: 'analysis'
%
%  -----Optional------
%  cnames	 : Cell array of cell names.
%  cellcoll	 : Cell array of which input cells are going into each output cell (output cell, input cells).
%				Example:
%					Cells = {[6 8];...
%					[1 2 3 4]};
%
%  exclChan	 : Channels not to be reported as peak channel (typically eye channels and midline channels)
%
%  params    : parameters to correlate against factor results to produce parametric PCA solutions (weights, parameter).
%              The rows should correspond to input cells and subjects (e.g., [C1S1; C1S2; C2S1; C2S2]).
%              Rows with a value of -999 will be dropped.
%  paramNames: Cell array of parameter names.
%
%Outputs
%  peakLatency:	Peak latencies (absolute microvolts) in msec, minus the baseline period.  (cells, factors, subjects)
%  peakSamp:	Peak latencies (absolute microvolts) in samples.  (cells, factors, subjects)
%  peakChan:	Peak channel at peak latency (absolute microvolts), not including channels excluded by exclChan.  (cells, factors, subjects)
%  data  : Structured array with the data and accompanying information.  See readData.
%
%  Outputs EP data variable with the factor results.  The factor scores and the factor pattern are multiplied to produce
%  the portion of the grand average accounted for by the factoring.
%  For a two-step PCA, there are the number of the first-stage factors multiplied by the second stage factors.
%  The second stage factors vary most quickly (e.g., for temporospatial PCA, T1S1 T1S2 T1S3...T2S1...etc.).
%  The cells input array allows the output
%  cells to be any desired combination of input cells.  In addition, a grand average of all the input cells is
%  generated and added to the end of the file.  The observations are the factors.  In addition to the input
%  factors, an additional observation is added to the end that is the total amount of the grand average accounted
%  for by the factor analysis.  Optionally, may specify parameters that will be used for parametric PCA results,
%  which will be output as additional cells.  Parametric cells contain the microvolt change corresponding to a one standard
%  deviation change in the parameter.
%  When using parametric PCA, also saves a .mat file with "Scores" appended to the name containing the
%  scores matrix with factors*chans as cols and obs as rows to facilitate analysis.
%  For when outputting separate subject files, parametric correlations calculated separately for each subject.

%History
%  by Joseph Dien (10/1/00)
%  jdien07@mac.com
%
%  modified (1/9/01) JD
%  Added parametric PCA option.
%
%  bugfix(1/10/01) JD
%  Transpose spatial data blocks before writing out to disk.  Also, changed data input to varSD.
%
%  modified (2/27/01) JD
%  Added peakLatency output.
%
%  modified (5/29/01) JD
%  Added peakChan and peakSamp output.
%
%  corrected (8/22/01) JD
%  Gave added to factor reconstructions to assist identification.
%
%  modified (2/2/02) JD
%  Eliminated assumption that cell numbers of dataset are in ascending order.
%
%  modified (6/7/02) JD
%  Also, modified to eliminate warning message when no exclusion channels.
%
%  modified (6/26/02) JD
%  added option to output matlab matrices rather than EGIS file.
%
%  corrected (6/30/02) JD
%  Fixed subject specs to include subject ID number so NetStation will label observations properly and NAvg so BESA will have N= info.
%
%  bugfix (7/9/02) JD
%  Fixed bug in parametric PCA output.  Results were not being calculated properly.
%
%  upgraded (8/22/02) JD
%  Changed to work under Matlab 6.5
%
%  modified (11/3/02) JD
%  Missing number for parameter vectors is now 999.
%
%  bugfix (3/24/03) JD
%  Eliminated x10 scaling in parametric PCA output.
%
%  bugfix & modified (4/25/03) JD
%  Parametric output after first parametric cell was computed incorrectly.  Also,
%  will change bins to 500 for EGI format output since losing too much resolution
%  when bins equal 1 (as with NS output) since EGIS files are saved in integer format.
%  Also now saves scores matrix when using parametric option.
%
%  modified (5/11/03) JD
%  Added individual subject file output option.
%
% bugfix (5/1/04) JD
% Fixed naming of scores .mat file
%
% bugfix (1/6/05) JD
% Fixed fclose statement, individual file output option, and Mat output option.  Thanks to Robert Frank!
%
% modified (6/16/05) JD
% Added error check for cell numbers.
%
% modified (11/15/05) JD
% In parametric PCA procedure, drop cells corresponding to missing data
% parameters when generating the mean voltage map.
%
% bugfix (12/9/07) JD
% Fixed scaling error when input data is not 500 bins per microvolt
%
% modified (12/19/07) JD
% Changed input cell names and parameter names arrays to cell arrays.
% Changed cellcoll to cell array, thus eliminating need to pad it out with zeroes.
% Deleted old filetype function (only worked on OS 9 Macs).
% Eliminated indexdata.  Eliminated the +gave option.
%
% modified (1/27/08) JD
% EGIS files always written as big-endian since NetStation makes this assumption.
%
% modified (2/21/08) JD
% Copyright notice appears only once per session.  Input factor results are now packaged in a
% structured variable.  Accommodates two-step PCA results by itself.  Subjects mode fixed.
%
% bugfix (6/2/08) JD
% Fixed sign of parametric output.  For example, positive correlation
% should result in positive voltage for positive channels but negative for
% negative channels.  Instead it was producing positive voltages at both
% types of channels.  Sign is now determined solely by correlation.
%
% modified (9/9/08) JD
% Automatically sets File Type and Creator Type if on an OS X Mac.
%
% modified (11/5/08) JD
% Automatically sets montage type if on an OS X Mac.
%
% modified (2/24/09) JD
% Gets baseline, experiment name, and sampling rate from structured variable.  Deleted cal field.  Assumes bins equals 1.
%
%  modified (11/3/09) JD
%  Missing number for parameter vectors changed from 999 to -999 (less likely to have problems with freq parameter).
%
% modified (3/11/09) JD
% Changed name to PCAoutput.m and changed from saving EGIS file to outputing EP data structure.
% Made the input fields, other than the data structure, optional.
%
% bugfix (4/25/09) JD
% Subs option had been unintentionally eliminated by last modification.  Restored the option.
%
% modified 5/10/09 JD
% Added cellType and subType and facNames and facTypes fields.  Eliminated subs option in favor of a single EP file
% with both subject average and grand average information in a 5D data field.
%
% bugfix 7/17/09 JD
% cellTypes correctly set to be a vector.
%
% modified 6/26/09 JD
% Moved facName and facType to doPCA and doPCAst.  Addition of .pca field moved to this function.
% Data represented by factor loadings no longer expanded into full .data field in order to avoid memory limitations.
% Instead, factor loadings are only expanded as needed by ep_expFac function, using facVecS and facVecT.
% Added dataName and facData fields.
%
% bugfix 8/3/09 JD
% crash when doing PCA of single trial data fixed.
%
% bugfix 8/26/09 JD
% Made sumFacs more memory efficient to reduce number of out of memory errors.
%
%  modified 2/15/10 JD
%  Analysis fields no longer optional.
%
% bugfix 3/14/10 JD
% ensure that summary factor, subject, and cell names are not already taken.  If so, add a unique suffix to the name.
%
% bugfix 6/16/10 JD
% avoid adding grand average to subjects dimension if the data type is continuous.
% 
%  modified 10/17/10 JD
%  Added support for saccadeTrials and saccadeOnset fields.

%     Copyright (C) 1999-2011  Joseph Dien
%
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
%
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
%
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

peakLatency=[];
peakSamp=[];
peakChan=[];
data=[];

ep_initEP;

if ~isfield(FactorResults,'varSD')
    error('No varSD field.');
end
varSD=FactorResults.varSD;
if ~isfield(FactorResults,'numchan')
    error('No numchan field.');
end
numChans=FactorResults.numchan;
if ~isfield(FactorResults,'timepoints')
    error('No timepoints field.');
end
numPoints=FactorResults.timepoints;
if ~isfield(FactorResults,'numSubs')
    error('No numSubs field.');
end
numSubs=FactorResults.numSubs;
if ~strcmp(FactorResults.dataType,'continuous')
    numTotSubs=numSubs+1; %includes grand average
else
    numTotSubs=numSubs; %no grand average for continuous data
end;

if ~isfield(FactorResults,'numCells')
    error('No numCells field.');
end
numCells=FactorResults.numCells;

if ~isfield(FactorResults,'montage')
    montage=askForMontage;
else
    montage=FactorResults.montage;
end
if ~isfield(FactorResults,'Fs')
    error('No Fs field.');
end
SampleRate=FactorResults.Fs;
if ~isfield(FactorResults,'baseline')
    error('No baseline field.');
end
base=FactorResults.baseline*(1000/SampleRate);

if ~isfield(FactorResults,'ename')
    error('No ename field.');
end

if nargin < 5
    params = [];
    paramNames = [];
end

if  nargin < 4
    exclChan=[];
end;

if  nargin < 2
    cnames = FactorResults.cellNames;
    cellcoll = num2cell([1:FactorResults.numCells]);
end

if length(cnames) ~= length(cellcoll)
    error('Error - The number of Cell Names does not match the number of output cell specifications');
end;

if length(paramNames) ~= size(params,2)
    error('Error - The number of Parameter Cell Names does not match the number of parameter output cell specifications');
end;

if isfield(FactorResults,'PCAmode2')  %if this is the result of a two-step PCA
    if ~isfield(FactorResults,'varSDST')
        error('No varSDST field.');
    end
    varSDST=FactorResults.varSDST;
    PCAmode=FactorResults.PCAmode2;
    numfacs2 = FactorResults.numFacs2;	%number of factors retained in second PCA step
    numfacs1 = FactorResults.numFacs;	%number of factors retained in first PCA step

    disp(['Generating PCA file with ' num2str(numfacs1) ' factors from the first PCA step and ' num2str(numfacs2) ' factors in the second.']);
    
    tempFacPatST=zeros(size(FactorResults.FacPatST,1),numfacs1*numfacs2);
    tempFacPat=zeros(size(FactorResults.FacPat,1),numfacs1*numfacs2);
    for i = 1:numfacs1
        for k = 1:numfacs2
            tempFacPatST(:,(i-1)*numfacs2+k)=diag(FactorResults.varSDST(i,:))*(FactorResults.FacPatST(:,(i-1)*numfacs2+k));
            tempFacPat(:,(i-1)*numfacs2+k)=diag(varSD)*FactorResults.FacPat(:,i);
        end;
    end;
    if strcmp(PCAmode,'temp')
        facVecT=tempFacPatST;
        facVecS=tempFacPat;
    elseif strcmp(PCAmode,'spat')
        facVecS=tempFacPatST;
        facVecT=tempFacPat;
    else
        error('PCAmode in factor results structure must be set to either temp or spat');
    end;
 
    FacScr=FactorResults.FacScrST;
    facNames=FactorResults.facNames2;
    facTypes=FactorResults.facTypes2;
    data.pca=FactorResults;
    facVar=FactorResults.facVarST;
    facVarQ=FactorResults.facVarQST;
    %FacPat=FactorResults.FacPatST;
    
    ObsNum=1; %the number of channels or timepoints not accounted for by factor loadings and thus accounted for by factor scores.
    dataChanNum=1; %the number of channels represented by the factor scores
    dataTimeNum=1; %the number of time points represented by the factor scores
        
elseif isfield(FactorResults,'PCAmode')
    PCAmode=FactorResults.PCAmode;
    if ~isfield(FactorResults,'FacScr')
        error('No FacScr field.');
    end
    FacScr=FactorResults.FacScr;
    if ~isfield(FactorResults,'FacPat')
        error('No FacPat field.');
    end
    
    if strcmp(PCAmode,'temp')
        facVecT=diag(varSD)*FactorResults.FacPat;
        facVecS=[];
        dataChanNum=numChans;
        dataTimeNum=1;
    elseif strcmp(PCAmode,'spat')
        facVecT=[];
        facVecS=diag(varSD)*FactorResults.FacPat;
        dataChanNum=1;
        dataTimeNum=numPoints;
    end;
    
    numfacs1 = FactorResults.numFacs;	%number of factors retained in first PCA step
    disp(['Generating PCA file with ' num2str(numfacs1) ' factors.']);
    
    facNames=FactorResults.facNames;
    facTypes=FactorResults.facTypes;
    data.pca=FactorResults;
    facVar=FactorResults.facVar;
    facVarQ=FactorResults.facVarQ;
    %FacPat=FactorResults.FacPat;
    
    if strcmp(PCAmode,'temp')
        ObsNum = numChans;
    elseif strcmp(PCAmode,'spat')
        ObsNum = numPoints;
    else
        error('PCAmode must be set to either temp or spat');
    end;
    
else
    msg{1}='Not a PCA analysis file.';
    [msg]=ep_errorMsg(msg);
    return
end;

%ensure that summary factor name is not already taken
PCAname='all';
sameName=1;
suffix=0;
PCAnameSuffix=PCAname;
while sameName
    sameName=0;
    if any(strcmp(PCAnameSuffix,facNames))
        sameName=1;
    end;
    if sameName
        suffix=suffix+1;
        PCAnameSuffix=[PCAname '-' num2str(suffix)];
    end;
end;
facNames{end+1} = PCAnameSuffix;
facTypes{end+1} = 'CMB';

%ensure that summary cell name is not already taken
PCAname='all';
sameName=1;
suffix=0;
PCAnameSuffix=PCAname;
while sameName
    sameName=0;
    if any(strcmp(PCAnameSuffix,cnames))
        sameName=1;
    end;
    if sameName
        suffix=suffix+1;
        PCAnameSuffix=[PCAname '-' num2str(suffix)];
    end;
end;
cnames{end+1} = PCAnameSuffix;

if mod(size(FacScr,1),(ObsNum * numSubs)) ~= 0
    error('Error - The number of factor scores is not evenly divided by the number of subjects and the channels/time points.');
end;

NumCell = size(FacScr,1)/(ObsNum * numSubs);	%Find number of cells

for i = 1:length(cellcoll)
    temp=cellcoll{i};
    if (max(temp) > NumCell)
        error(['Error - You have specified a cell number (' num2str(max(temp)) ') in cellcoll that is larger than the number of cells present (' num2str(NumCell) ').']);
    end;
    if (min(temp) < 1)
        error('Error - You have specified a cell number in cellcoll that is smaller than zero.');
    end;
end;

if ~isempty(params)
    cnames = [cnames; paramNames];
    if size(params,1) ~= (size(FacScr,1)/ObsNum)
        error('Error - number of weights for each parameter needs to be equal to the number of subjects times cells');
    end;
end;

%Sum together factor scores across subjects in each cell
NUM_FAC = size(FacScr,2);
avgCells = zeros(ObsNum*NumCell,NUM_FAC);
CellCount = zeros(NumCell,1);

for cellLoop = 1:NumCell
    for sub = 1:numSubs
        avgCells((cellLoop-1)*ObsNum+1:cellLoop*ObsNum,:) = avgCells((cellLoop-1)*ObsNum+1:cellLoop*ObsNum,:) + FacScr((cellLoop-1)*numSubs*ObsNum+(sub-1)*ObsNum+1:(cellLoop-1)*numSubs*ObsNum+sub*ObsNum,:);
        CellCount(cellLoop) = CellCount(cellLoop)+1;
    end;
end;

numOutputCells = length(cellcoll)+1;

%Divide by number of summed factor scores to produce mean factor scores for each cell

warning off MATLAB:divideByZero
for k = 1:NumCell
    for i = 1:ObsNum
        avgCells((k-1)*ObsNum+i,:) = avgCells((k-1)*ObsNum+i,:) / CellCount(k);
    end;
end;
warning on MATLAB:divideByZero

cellTypes=cell(length(cellcoll),1);
for i = 1:length(cellcoll)
    if length(cellcoll(i)) ==1
        cellTypes{i}='SGL';
    else
        cellTypes{i}='CMB';
    end;
end;

peakSamp=zeros(numOutputCells,NUM_FAC,numSubs);
peakChan=zeros(numOutputCells,NUM_FAC,numSubs);

fprintf('%60s\n',' ' );

fiveDdata=zeros(dataChanNum,dataTimeNum,numOutputCells,numTotSubs,NUM_FAC);
sumFacs = zeros(numChans, numPoints,numOutputCells,numTotSubs);
%start loop for each subject
for sub = 1:numSubs
        avgCells = zeros(ObsNum*NumCell,NUM_FAC);
        obsPerSub = size(FacScr,1)/numSubs;
        for theCell=1:NumCell
            avgCells((theCell-1)*ObsNum+1:theCell*ObsNum,:)=FacScr((theCell-1)*numSubs*ObsNum+(sub-1)*ObsNum+1:(theCell-1)*numSubs*ObsNum+sub*ObsNum,:);
        end;
    
    %Compute output cells by collapsing together cells according to cell specifications
    OutCells = zeros((length(cellcoll)+1)*ObsNum,NUM_FAC);	%construct matrix of mean factor scores (output cells*chans/points,factors)
    
    for i = 1:length(cellcoll)
        Count = 0;
        for j = 1:length(cellcoll{i})
            if cellcoll{i}(j) ~= 0
                for obs = 1:ObsNum
                    OutCells((i-1)*ObsNum+obs,:) = OutCells((i-1)*ObsNum+obs,:) + avgCells((cellcoll{i}(j)-1)*ObsNum+obs,:);
                end;
                Count = Count +1;
            end;
        end;
        for obs = 1:ObsNum
            OutCells((i-1)*ObsNum+obs,:) = OutCells((i-1)*ObsNum+obs,:) / Count;
        end;
    end;
    
    Count = 0;
    for cellLoop = 1:NumCell
        for obs = 1:ObsNum
            OutCells((numOutputCells-1)*ObsNum+obs,:) = OutCells((numOutputCells-1)*ObsNum+obs,:) + avgCells((cellLoop-1)*ObsNum+obs,:);	%compute mean of all cells for 'total' cell
        end;
        Count = Count +1;
    end;
    
    for k = 1:ObsNum
        OutCells((numOutputCells-1)*ObsNum+k,:) = OutCells((numOutputCells-1)*ObsNum+k,:) / Count;
    end;
    
    %Construct parametric PCA output if so instructed
    if ~isempty(params)
        
        %Compute correlations with factor scores
        scoresMatrix = zeros((size(FacScr,1)/ObsNum),(ObsNum*NUM_FAC));
        for i = 1:(size(FacScr,1)/ObsNum)
            CellScores = FacScr((i-1)*ObsNum+1:i*ObsNum,:);	%cell's scores (chans, factors)
            scoresMatrix(i,:) = reshape(CellScores, 1, numel(CellScores));	%building matrix with factors*chans as cols and obs as rows
        end;
        
        eval(['save ' fname 'S' num2str(sub) 'scores.mat scoresMatrix']);
        
        for i = 1:size(params,2)
            index = find(params(:,i)~=-999);	%observations with non-missing params and thus retained for correlation
            correl = corrcoef([scoresMatrix(index,:) params(index,i)]);	%column vector of correlations
            correl = correl(1:size(correl,2)-1,size(correl,2));	%extract just the correlations between the parameter and the chans (chans*factors)
            corrMatrix = reshape(correl,ObsNum,NUM_FAC);	%matrix of correlations (chans,factors)
            
            Count = 0;
            parametricMean=zeros(ObsNum,NUM_FAC);
            for cellLoop = 1:NumCell
                if params(cellLoop,i)~=-999	%generate mean ERP for parametric computation, not including missing data
                    for obs = 1:ObsNum
                        parametricMean(obs,:) = parametricMean(obs,:) + avgCells((cellLoop-1)*ObsNum+obs,:);	%compute mean of all cells
                    end;
                    Count = Count +1;
                end;
            end;
            
            parametricMean=parametricMean/Count;
            
            OutCells = [OutCells; abs(parametricMean).*corrMatrix];	%add parametric PCA output cell
        end;
        
        numOutputCells=numOutputCells+size(params,2);
    end;
    
    for outcell = 1:numOutputCells
        fprintf('%s%-60s',repmat(sprintf('\b'),1,60),sprintf('%s%4d%s%4d','Working on cell# ', outcell,' and subject# ', sub))
        for factor = 1:NUM_FAC
            datablock=[];
            for Obs = 1:ObsNum	%chans for temporal or points for spatial
                datablock(Obs,:) = OutCells((outcell-1)*ObsNum+Obs, factor);
            end;
            if strcmp(PCAmode,'spat')
                datablock = datablock';	%make so that columns are time points and rows are channels
            end;
            [x peakSamp(outcell, factor, sub)] = max(sum(abs(datablock)));	%find (absolute) peak time.  x is a dummy variable.
            [x chans] = sort(abs(datablock(:,peakSamp(outcell, factor))));	%find (absolute) peak chan.  x is a dummy variable.
            topchan = length(chans);
            if ~isempty(exclChan)
                while any(exclChan == chans(topchan))
                    topchan = topchan -1;
                    if topchan == 0
                        error('All channels excluded from peakchan calculation.');
                    end;
                end;
            end;
            peakChan(outcell, factor, sub)=chans(topchan);
            fiveDdata(1:dataChanNum,1:dataTimeNum,outcell,sub,factor)=datablock;
            
            %in order to combine different factors, cannot add together just the loadings or just the scores since the
            %differences in scaling can be lost.
            if ~isempty(facVecS)
                datablock=facVecS(:,factor)*datablock;
            end;
            if ~isempty(facVecT)
                datablock=datablock*facVecT(:,factor)';
            end;
            sumFacs(:,:,outcell,sub)=sumFacs(:,:,outcell,sub)+datablock;
        end;
    end;
    peakLatency= (peakSamp.*(1000/SampleRate))-base;  %Convert to msec.
end;
fprintf('%60s\n',' ' );

disp('Generating summed factors.');

if ~strcmp(FactorResults.dataType,'continuous')
    fiveDdata(1:dataChanNum,1:dataTimeNum,:,numSubs+1,:)=mean(fiveDdata(1:dataChanNum,1:dataTimeNum,:,1:numSubs,:),4); %add grand average waveforms
    sumFacs(:,:,:,numSubs+1,:)=mean(sumFacs(:,:,:,1:numSubs,:),4); %add grand average waveforms
end;

data.data=fiveDdata;
data.montage=FactorResults.montage;
data.fileFormat=FactorResults.fileFormat;
data.chanNames=FactorResults.chanNames;
data.timeNames=FactorResults.timeNames;
data.subNames=FactorResults.subNames;
data.subTypes=FactorResults.subTypes;

if ~strcmp(FactorResults.dataType,'continuous')
    %ensure that summary subject name is not already taken
    PCAname='grand average';
    sameName=1;
    suffix=0;
    PCAnameSuffix=PCAname;
    while sameName
        sameName=0;
        if any(strcmp(PCAnameSuffix,data.subNames))
            sameName=1;
        end;
        if sameName
            suffix=suffix+1;
            PCAnameSuffix=[PCAname '-' num2str(suffix)];
        end;
    end;
    data.subNames{end+1}=PCAnameSuffix;
    data.subTypes{end+1}='GAV';
end;

data.facNames=facNames;
data.cellNames=cnames;
data.trialNames=FactorResults.trialNames;
data.chanTypes=FactorResults.chanTypes;
data.cellTypes=cellTypes;
data.cellTypes{end+1}='CMB';
if ~isempty(data.trialNames)
    data.trialNames(end+1)=1;
end;
data.facTypes=facTypes;

data.eloc=FactorResults.eloc;
data.ced=FactorResults.ced;
data.implicit=FactorResults.implicit;
try
    EPver=ver('EP_Toolkit');
catch
    EPver='unavailable'; %workaround for bug in earlier version of Matlab
end;
data.EPver=EPver;
data.ver=ver;
data.date=date;
data.Fs=FactorResults.Fs;
data.baseline=FactorResults.baseline;
data.ename=FactorResults.ename;

[pathstr, name, ext] = fileparts(FactorResults.fileName);
data.dataName=name;

data.dataType=FactorResults.dataType;
data.trialSpecs=cell(length(cnames),0);
data.trialSpecNames=[];
data.subjectSpecs=FactorResults.subjectSpecs;
data.subjectSpecs(end+1,:)=cell(1,length(FactorResults.subjectSpecNames));
data.subjectSpecNames=FactorResults.subjectSpecNames;
data.events=cell(numTotSubs,numOutputCells);
data.avgNum=zeros(numTotSubs,numOutputCells);
data.subNum=ones(numTotSubs,numOutputCells)*numSubs;
data.fileName=FactorResults.fileName;
data.history={'PCAoutput',cnames, cellcoll, exclChan, params, paramNames};

data.pca=FactorResults;
data.facVar=facVar;
data.facVarQ=facVarQ;
data.facVecT=facVecT;
data.facVecS=facVecS;
data.facData=sumFacs;

data.analysis.blinkTrial=zeros(numTotSubs,numOutputCells);
data.analysis.saccadeTrial=zeros(numTotSubs,numOutputCells);
data.analysis.saccadeOnset=zeros(numTotSubs,numOutputCells);
data.analysis.moveTrial=zeros(numTotSubs,numOutputCells);
data.analysis.badTrials=zeros(numTotSubs,numOutputCells);
data.analysis.badChans=zeros(numTotSubs,numOutputCells,numChans);

fprintf('%s\r',' ' );
fprintf('%s\r','Done with reconstructing PCA waveforms...' );

[err]=ep_checkEPfile(data);

