function r = readses(fnamein, fnameout, fnum_rec)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% r = readses(fnamein, fnameout, fnum_rec) - Decompresses and converts ERPSS data to matlab format.
%
%Inputs
%	fnamein: Filename and path of source.  Required!
%	fnameout: Filename of output file.  Default = fnamein.mat
%	fnum_rec: Number of records to put in each sub file. Default=5000
%
%Outputs
%	r: status report.  Returns a 0 on error and a 1 when successful.
%
% History:
%
% by Joseph Dien (2/99)
% Tulane University
% jdien@tulane.edu
%
% Modified by William Town 3-12-99
%	Documented, converted for SGI's, and other small additions.
%
% Modified by William Town 3-25-99
%	Added speed enhancements, command line input, and more
%	documentation.
%
% Modified by Joseph Dien 3-27-99
%   Modified endian code and added more documentation.
%
% Modified by Joseph Dien 4-6-99
%   Fix for swapping long variables.
%
% Modified by Joseph Dien 6-8-99
%   Eliminated long variable swap (wasn't needed after all)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Format of ERPSS raw files as developed by Jonathan Hansen:
%
%Structure of raw file is a main header (ref_h) followed by a series of
%records.  Each record is furthermore divided into an initial record
%header (srd_h) followed by the data which is a fixed number of timepoints.
%The channels are multiplexed so that all the channels for a given timepoint
%are recorded before the next timepoint is recorded.  Data is optionally
%compressed as described later.
%
%MAIN HEADER FORMAT (from edheader.h)
%unsigned short	refh_byteswab 	- constant for checking endian of file
%unsigned short refh_eid		- ?
%array of shorts
%	flags
%	# of channels
%	if normalized, |cal|, in v
%	if normalized, in pts/v
%	# times processed
%	dig. prog version
%	log(2) pts in records
%	pts in records
%	"ed in eff."
%	"ed chksum"
%	"log(2) tc"
%	"# AR count slots"
%array of longs
%	unique time (unix)
%	"For orig dig."
%	6 empty slots
%Character & string area with various sorts of information
%
%RECORD HEADER FORMAT(from rdf.h)
%unsigned shorts
%	byte swab indicator (should read -21931)
%	id number
%	flags
%	nchans
%	record size log base 2 (each chan)
%	record size in points (each chan)
%	# A/D errors (duplicated samples)
%	#pauses since beginning
%	error detection flag
%	# events lost
%	# events in event structure array
%	if compressed, number of encoded shorts
%	padded out to 512 bytes
%longs
%	sampling period in microseconds
%	record number (cats with offsets)
%	UNIX time, start of digitization
%	UNIX time, last startup from pause
%	cumulative start item this list
%	checksum to 0L for header and data
%event list (data for each event listed together before that of next event)
%	char - condition code
%	char - offset (0-31 samples from start of record)
%	unsigned short - event #
%
%DATA FORMAT
%
%ERPSS data is normally saved in little-endian format.
%This means that when the short variables (2-bytes) are read,
%one byte at a time, the low bit is read in first.
%The decompress code byteswaps the data before working on it
%if the computer is, say, a Mac (which is big-endian).
%NOTE***The byteswapping is done on the input variable so
%it is changed by the call to decompress, not just the
%output variables!!!!!
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%
% Set the defaults
%
if nargin < 1			% Must have input file, return error
   r = 0;
   disp('Error!  You must give a filename.');
   return
end
if nargin < 3
   fnum_rec = 5000;
end
if nargin < 2
   fnameout = eval(['''' fnamein '.mat' '''']);
end

% Determine computer endian type

switch computer
 	case { 'MAC2','SUN4','SOL2','SGI','SGI64'}, complendian = 0;
	case {'PCWIN64' 'PCWIN','LNX86'}, complendian = 1;
	otherwise error('endian of computer not known')
end;


% Set defaults from main file header
%
ed_maxchans = 256;	%Maximum number of channels
refh_size = 4096;	%Size of header in bytes
refh_ssize = ed_maxchans + 16;	%Number of short header variables
refh_lsize = 8;	%Number of long header variables
refh_csize = refh_size - 2*ed_maxchans - 16*2 - 8*4 - 2*2;	%Bytes left over for header text area

%
% Set defaults from record header
%
srd_ssize = 24;	%Number of short variables in record headers
srd_lsize = 6;  %Number of long variables in record headers
srd_esize = 110;  %Number of slots for events in record headers


%
% Open the file and report any errors
%
sesFID = fopen(fnamein,'r','ieee-le');
if (sesFID == -1)
   error('Error opening input file!');
end;


%
% Find the length of the file
%
fseek(sesFID, 0, 'eof');
fsize = ftell(sesFID);
fseek(sesFID, 0, 'bof');


%
% Find if file is actually big-endian.  If so, close file and start again.
%
filelendian = 1;  %Start off assuming file is little-endian but will automatically correct if not.

[refh_byteswab] = fread(sesFID, 1, 'uint16');
if (refh_byteswab == 21930)  %refh_byteswab is a constant.  Equals 21930 if unintentionally swapped.
   fclose(sesFID);
   sesFID = fopen(fnamein,'r','ieee-be');
   [refh_byteswab] = fread(sesFID, 1, 'uint16');
   filelendian = 0;  %file is big-endian
end;


%
% Read in main header
%
[refh_eid] = fread(sesFID, 1, 'uint16');  %read in "id (EIDLOG) if canonical" (?)
[refh_s] = fread(sesFID, refh_ssize, 'uint16');  %read in short header variables
[refh_l] = fread(sesFID, refh_lsize, 'uint32');  %read in long variable headers
   
[refh_c] = fread(sesFID, refh_csize, 'char');  %read in text portion of header
nchan = refh_s(2);
npoints = refh_s(8);


%
% Initialize record headers and counters
%
srd_cond = [];
srd_offset = [];
srd_event = [];
sesdata = [];
srd_s = [];
srd_l = [];
recordnum = 1;
filenum = 1;


%
% Add records row by row until EOF
%
while (ftell(sesFID) ~= fsize)
   temp1 = [];
   temp2 = [];
   temp3 = [];
   srd_s = [srd_s; fread(sesFID, srd_ssize, 'uint16')'];	%read in shorts of record header
   temp = fread(sesFID, srd_lsize, 'uint32')';	%read in longs of record header
      
   srd_l = [srd_l; temp];
   
   disp(['record #' num2str(srd_l(size(srd_l,1),2))]);

   for i = 1:srd_esize  %read in events
      temp1 = [temp1 fread(sesFID, 1, 'uchar')];
      temp2 = [temp2 fread(sesFID, 1, 'uchar')];
      temp3 = [temp3 fread(sesFID, 1, 'uint16')];
   end;
   srd_offset = [srd_offset; temp1];
   srd_cond = [srd_cond; temp2];
   srd_event = [srd_event; temp3];
   
   srd_nes = srd_s(size(srd_s,1),12);  %size of compressed data
   sesdatain = uint8(fread(sesFID, srd_nes*2, 'uchar'));
   % Decompress the data
   sesdataout = decompressERPSS(sesdatain, nchan, npoints, xor(filelendian, complendian));
   sesdataout = reshape(sesdataout, nchan, npoints)';
   sesdata = [sesdata; sesdataout];
   
   % Create a new file for every fnum_rec records
   if recordnum == fnum_rec
      eval (['save ' fnameout '-' num2str(filenum) ' refh_byteswab ' ...
      'refh_c refh_eid refh_l refh_s sesdata srd_cond srd_event srd_l srd_offset srd_s']);
      srd_cond = [];
      srd_offset = [];
      srd_event = [];
      sesdata = [];
      srd_s = [];
      srd_l = [];
      filenum = filenum + 1;
      recordnum = 0;
   end;
   recordnum = recordnum +1;
end;

%
% Save output from final record
%
eval (['save ' fnameout '-' num2str(filenum) ' refh_byteswab ' ...
	'refh_c refh_eid refh_l refh_s sesdata srd_cond srd_event srd_l srd_offset srd_s']);


%
% Close file
%
fclose(sesFID);


%
% Return success
%
r = 1;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% EEG Data Import End
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
