function r = writeses(fnamein, fnameout, fnum_seg)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% r = writeses(fnamein, fnameout, fnum_seg) - Compresses and converts matlab data to ERPSS format.
%
%Inputs
%	fnamein: Filename and path of source.  Required!
%	fnameout: Filename of output file.  Default = fnameout.mat
%	fnum_seg: Number of segments that file was divided into.  Default = 1.
%
%Outputs
%	r: status report.  Returns a 0 on error and a 1 when successful.
%
% History:
%
% by Joseph Dien (5/99)
% University of California, Davis
% jdien@marzen.ucdavis.edu
%
%Modified from readses
%
%Modified by Joseph Dien 6-8-99
%  Dropped long variable byteswap since redundant with Matlab's I/O byteswapping.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Format of ERPSS raw files as developed by Jonathan Hansen:
%
%Structure of raw file is a main header (ref_h) followed by a series of
%records.  Each record is furthermore divided into an initial record
%header (srd_h) followed by the data which is a fixed number of timepoints.
%The channels are multiplexed so that all the channels for a given timepoint
%are recorded before the next timepoint is recorded.  Data is optionally
%compressed as described later.
%
%MAIN HEADER FORMAT (from edheader.h)
%unsigned short	refh_byteswab 	- constant for checking endian of file
%unsigned short refh_eid		- ?
%array of shorts
%	flags
%	# of channels
%	if normalized, |cal|, in v
%	if normalized, in pts/v
%	# times processed
%	dig. prog version
%	log(2) pts in records
%	pts in records
%	"ed in eff."
%	"ed chksum"
%	"log(2) tc"
%	"# AR count slots"
%array of longs
%	unique time (unix)
%	"For orig dig."
%	6 empty slots
%Character & string area with various sorts of information
%
%RECORD HEADER FORMAT(from rdf.h)
%unsigned shorts
%	byte swab indicator
%	id number
%	flags
%	nchans
%	record size log base 2 (each chan)
%	record size in points (each chan)
%	# A/D errors (duplicated samples)
%	#pauses since beginning
%	error detection flag
%	# events lost
%	# events in event structure array
%	if compressed, number of encoded shorts
%	padded out to 512 bytes
%longs
%	sampling period in microseconds
%	record number (cats with offsets)
%	UNIX time, start of digitization
%	UNIX time, last startup from pause
%	cumulative start item this list
%	checksum to 0L for header and data
%event list (data for each event listed together before that of next event)
%	char - condition code
%	char - offset (0-31 samples from start of record)
%	unsigned short - event #
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%
% Set the defaults
%
if nargin < 1			% Must have input file, return error
   r = 0;
   disp('Error!  You must give a filename.');
   return
end
if nargin < 3
   fnum_seg = 1;
end
if nargin < 2
   fnameout = eval(['''' fnameout '.mat' '''']);
end

% Determine computer endian type

switch computer
 	case {'MAC2','SUN4','SOL2','SGI','SGI64'}, complendian = 0;
	case {'PCWIN','LNX86'}, complendian = 1;
	otherwise error('endian of computer not known')
end;

% Set defaults from main file header
%
ed_maxchans = 256;	%Maximum number of channels
refh_size = 4096;	%Size of header in bytes
refh_ssize = ed_maxchans + 16;	%Number of short header variables
refh_lsize = 8;	%Number of long header variables
refh_csize = refh_size - 2*ed_maxchans - 16*2 - 8*4 - 2*2;	%Bytes left over for header text area

%
% Set defaults from record header
%
srd_ssize = 24;	%Number of short variables in record headers
srd_lsize = 6;  %Number of long variables in record headers
srd_esize = 110;  %Number of slots for events in record headers

%
% Create the output file and report any errors
%
sesFID = fopen(fnameout,'w+','ieee-le');
if (sesFID == -1)
   error('Error opening output file!');
end;


filelendian = 1;  %Files are stored as little-endian by default

%
% Write out segments of file
%
for segment = 1:fnum_seg
	
	eval (['load ' fnamein '-' num2str(segment) 'blink'])	%load in segment
%	eval (['load ' fnamein '-' num2str(segment)])	%load in segment

	%
	% Write main header
	%
	if segment == 1
		nchan = refh_s(2);
		npoints = refh_s(8);
		fwrite(sesFID, refh_byteswab, 'uint16');  %write "id (EIDLOG) if canonical" (?)
		fwrite(sesFID, refh_eid, 'uint16');  %write "id (EIDLOG) if canonical" (?)
		fwrite(sesFID, refh_s, 'uint16');  %write short header variables

		fwrite(sesFID, refh_l, 'uint32');  %write long variable headers
   

		fwrite(sesFID, refh_c, 'char');  %write text portion of header
	end;

	%
	% Write out records of file
	%
	for record = 1:size(srd_s,1)
	 
		disp(['segment #' num2str(segment) ' record #' num2str(record)]);
   
		% Compress the data
		temp1 = reshape(sesdata(1+(record-1)*npoints:npoints+(record-1)*npoints,:)',nchan*npoints,1);	%data of current record
		temp1 = ((abs(temp1) < 32768) .* temp1) + ((abs(temp1) > 32767) * 32767 .* sign(temp1));	%Keep range within 12 bits
		temp1 = ((temp1 < 0).*(bitcmp(abs(temp1),16)+1))+((temp1 >= 0).*(temp1));	%convert negative numbers to two's complement coding
		temp2 = uint8(temp1);				%low byte
		temp3 = uint8(bitshift(temp1,-8,8));	%high byte (including sign bit)
		sesdatain = [];
		for i = 1:size(temp1,1)
			sesdatain = [sesdatain; temp3(i,1); temp2(i,1)];	%build uint8 format data matrix with bigendian order
		end;
		[sesdataout nes] = compressERPSS(sesdatain, nchan, xor(filelendian, complendian));
		srd_s(record,12) = nes;  %size of compressed data
		fwrite(sesFID, srd_s(record,:), 'uint16')';	%write shorts of record header
		fwrite(sesFID, srd_l(record,:), 'uint32')';	%write longs of record header

		for i = 1:srd_esize  %write out events
			fwrite(sesFID, srd_offset(record,i), 'uchar');
			fwrite(sesFID, srd_cond(record,i), 'uchar');
			fwrite(sesFID, srd_event(record,i), 'uint16');
		end;
		fwrite(sesFID, double(sesdataout(1:nes*2))', 'uchar');
	end;
end;

%
% Close file
%
fclose(sesFID);


%
% Return success
%
r = 1;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% EEG Data Export End
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
