% Function to obtain freq response for iirfilt in EEGlab
%
% Usage:
%	[f_axis, gain, h, w, ideal_actual_corr, fs, m] = IIR_freq_rsps(lowcut, highcut, srate, pnts, notch);
%
% Where:
%   lowcut is lower-end cutoff (hz); for low pass filter use 0
%   high cut is upper-end cutoff (hz); for high pass filter use 0
%   srate is sample rate (hz)
%   pnts is length of data vector you intend to filter (use EEG.pnts usually)
%   notch (1 = notch filter, 0 = bandpass filter [default])
%
% Outputs:
%   f_axis: frequency axis for frequency response
%   gain: gain at frequencies of f_axis
%   ideal_actual_corr: correlation between ideal transfer function and
%                      actual transfer function -- if below .95 you best
%                      plot the functions and see if the filter is adequate
%   fs:  frequencies for ideal freq response function
%   m:   ideal transfer function
%
% Version 0.9
%
% Written July, 2011, by John JB Allen jallen@u.arizona.edu
%
% See Also:  pop_optFIR, pop_optFIR_compare, pop_about_optFIR_compare, optFIR.m, optFIR_compare, optFIR_design, FIR_filt, FIR_freq_rsps

function [f_axis, gain, h, w, ideal_actual_corr, fs, m] = IIR_freq_rsps(lowcut, highcut, srate, pnts, notch)

if nargin < 5 
    doc IIR_freq_rsps;
    return
end

% Convert highcut code of 0 for high pass to nyquist
if highcut == 0
    highcut = srate/2;
end


% check for signal processing toolbox
if exist('fir2','file') ~= 2
      error('Optimal FIR Toolbox requies the Signal Processing Toolbox, which cannot be found in your environment!');
end

% set up parameters -- make FFT array dependent on srate
NumInFFTArray = 2*(2^nextpow2(srate)); % twice the next power of 2 that is >= srate

% in case of slow sample rates, get a decent freq response by making longer array
NumInFFTArray = max(NumInFFTArray,1024);  % for slow sample rates, make this longer for better response

% define frequency vector from 0 to 1, where 1 is nyquist
T = NumInFFTArray/srate;  % length of epoch in time
fs = 0:1/T:srate/2;     % frequency vector from 0 to Nyquist

% Define ideal frequency function
m = fs >= lowcut & fs <= highcut;   % where the frequency is between cutpoints
if notch == 1
    m = ~m;    % invert pass and stop freqs for notch filter
end

% Define "Spike" -- impulse response with single spike at midpoint, so freq
% representation will be all freqs equally from 0 to nyquist
% Spike is all zeros, with a single value of 1 at midpoint
spike = zeros(1,pnts);
if mod(pnts,2) == 0
    spike((size(spike,2))/2)=1;
else
    spike((size(spike,2)-1)/2)=1;
end

% now use iirfilt on spike to get to get filtered version using iirfit
% wrap in evalc so you don't get once again the output from iirfilt
evalc('spike_filt= iirfilt( spike, srate, lowcut, highcut, 0, 0, notch)'); % 0 0 is default to use entire epoch and default transition bandwidth 


% Freq Response
[h,w] = freqz(spike_filt);
gain = abs(h);
f_axis = w/pi*(srate/2);

% Correlation between ideal and actual
% Because filt order is not equal to initial array size, need to resample
% to make the correlation work...
ideal_actual_corr = corr(m',resample(abs(h),length(fs),length(h)));
