% Function to compare options of EEGLab's built-in filters (eegfilt and iirfilt)
%   to optimal FIR filter from optFIR plugin
%
% No data are altered in this process
% Requires Signal Processing Toolbox to be installed.
%
% Usage:
%   filtspecs = optFIR_compare(EEG,lowcut,highcut,notch, eegfilt_default_selected, eegfilt_srate_selected, iirfilt_selected, optFIR_selected, original_data_selected, eegfilt_default_data_selected, eegfilt_srate_data_selected, iirfilt_data_selected, optFIR_data_selected,optFIR_filt_length)
%
%
% Where:
%   EEG is eeglab structure
%   lowcut is lower-end cutoff (hz); for low pass filter use 0
%   high cut is upper-end cutoff (hz); for high pass filter use 0
%   notch is logical notch out frequencies, 0 = no (default for bandpass, lowpass, highpass), 1 = yes (notch)
%
% Output:
%   filtspecs is structure with relevant filter parameters for later
%   plotting or examination (e.g. transfer functions)
%
% Notes:
%   Option to compare:
%      1) Ideal Transfer Function to various filter approaches (eegfilt_default_selected, eegfilt_srate_selected, iirfilt_selected, optFIR_selected)
%      2) Data filtered using various filter approches (original_data_selected, eegfilt_default_data_selected, eegfilt_srate_data_selected, iirfilt_data_selected, optFIR_data_selected)
%
% Version 0.9
%
% Written July, 2011, by John JB Allen jallen@u.arizona.edu
%
% See Also:  pop_optFIR, pop_optFIR_compare, pop_about_optFIR_compare, optFIR.m, optFIR_design, FIR_filt, FIR_freq_rsps, IIR_freq_rsps

function filtspecs = optFIR_compare(EEG, lowcut, highcut, notch, eegfilt_default_selected, eegfilt_srate_selected, iirfilt_selected, optFIR_selected, original_data_selected, eegfilt_default_data_selected, eegfilt_srate_data_selected, iirfilt_data_selected, optFIR_data_selected,optFIR_filt_length)

if nargin == 0 
    doc optFIR_compare;
    return
end


%% A few things to get set up

filtspecs = struct;

% Convert highcut code of 0 for high pass to nyquist
if highcut == 0
    highcut = EEG.srate/2;
end

% Speed up by making dataset small if only comparing transfer functions
if original_data_selected+eegfilt_default_data_selected+eegfilt_srate_data_selected+iirfilt_data_selected+optFIR_data_selected == 0
    if length(size(EEG.data)) == 3
        EEG.data = EEG.data(1,:,:);
        EEG.nbchan = 1;
    elseif length(size(EEG.data)) == 2
        EEG.data = EEG.data(1,:);
        EEG.nbchan = 1;
    end
end

% eegfilt and iirfilt specify low pass with 0 as high cut, 
if highcut == round(EEG.srate/2)
    highcut_eegfilt = 0;
else
    highcut_eegfilt = highcut;
end

%% Now calculate filters for only those comparisons needed

% If user chooses to save a previewed dataset, will need this var to give EEG setname an update
if notch
    notchstring = 'notch ';
else
    notchstring = '';
end

if eegfilt_default_selected || eegfilt_default_data_selected
    eegfilt_default = EEG;
    tic;
    [eegfilt_default.data, eegfilt_default.fweights] = eegfilt( EEG.data, EEG.srate, lowcut, highcut_eegfilt , 0, [], notch);
    eegfilt_default.time_to_filter = toc;
    % Make set name in case user chooses to preview data and save this set
    eegfilt_default.setname = [EEG.setname ' post filtering with '  num2str(lowcut) '-' num2str(highcut) 'hz' notchstring ' EEGlab Basic FIR Filter (order = ' num2str(length(eegfilt_default.fweights)-1) ')'];
    % Make command to run this from UI Pushbutton on eegplot, including getting handle to this plot and closing it, otherwise pushing the button again will refilter filtered data rather than original data
    eegfilt_default.cmd = ['h_eegplot = gcf; EEG.data = eegfilt(EEG.data, EEG.srate,' num2str(lowcut) ',' num2str(highcut_eegfilt) ', 0, [], ' num2str(notch) ...
        '); close(h_eegplot); [ALLEEG EEG CURRENTSET] = pop_newset(ALLEEG, EEG,1, ''setname'', ''' eegfilt_default.setname '''); eeglab redraw;']; % for use later if wish to save this filtered version, otherwise does not execute
    % Provide feedback to console
    if exist('cprintf','file') ~=2  % for older versions of Matlab prior to cprintf
        fprintf('\n%s%5.2f%s\n','Default Basic (eegfilt) FIR filtering complete in ',eegfilt_default.time_to_filter,' seconds.');
    else
        cprintf([.5 0 1],'Default Basic (eegfilt) FIR filtering complete in %5.2f seconds.\n', eegfilt_default.time_to_filter);
    end;
end;

if eegfilt_srate_selected  || eegfilt_srate_data_selected
    eegfilt_srate = EEG;
    tic;
    [eegfilt_srate.data, eegfilt_srate.fweights] = eegfilt( EEG.data, EEG.srate, lowcut, highcut_eegfilt , 0, round(EEG.srate*optFIR_filt_length), notch);
    eegfilt_srate.time_to_filter = toc;
    % Make set name in case user chooses to preview data and save this set
    eegfilt_srate.setname = [EEG.setname ' post filtering with '  num2str(lowcut) '-' num2str(highcut) 'hz' notchstring ' EEGlab Basic FIR Filter (order = ' num2str(length(eegfilt_srate.fweights)-1) ')'];
    % Make command to run this from UI Pushbutton on eegplot, including getting handle to this plot and closing it, otherwise pushing the button again will refilter filtered data rather than original data
    eegfilt_srate.cmd = ['h_eegplot = gcf; EEG.data = eegfilt(EEG.data, EEG.srate,' num2str(lowcut) ',' num2str(highcut_eegfilt) ', 0,' num2str(round(EEG.srate*optFIR_filt_length)) ', ' num2str(notch) ...
        '); close(h_eegplot); [ALLEEG EEG CURRENTSET] = pop_newset(ALLEEG, EEG,1, ''setname'', ''' eegfilt_srate.setname '''); eeglab redraw;']; % for use later if wish to save this filtered version, otherwise does not execute
    % Provide feedback to console
    if exist('cprintf','file') ~=2  % for older versions of Matlab prior to cprintf
        fprintf('\n%s%5.2f%s\n','Basic (eegfilt) FIR filtering (with same order as optfilt) complete in ',eegfilt_srate.time_to_filter,' seconds.');
    else
        cprintf([.5 0 1],'Basic (eegfilt) FIR filtering (with same order as optfilt) complete in %5.2f seconds.\n', eegfilt_srate.time_to_filter);
    end;
end

if iirfilt_selected || iirfilt_data_selected
    iirfilt_default = EEG;
    tic;
    [iirfilt_default.data] = iirfilt( EEG.data, EEG.srate, lowcut, highcut_eegfilt , 0, 0, notch);
    iirfilt_default.time_to_filter = toc;
    % Make set name in case user chooses to preview data and save this set
    iirfilt_default.setname = [EEG.setname ' post filtering with '  num2str(lowcut) '-' num2str(highcut_eegfilt) 'hz' notchstring ' EEGlab Short IIR Filter'];
    % Make command to run this from UI Pushbutton on eegplot, including getting handle to this plot and closing it, otherwise pushing the button again will refilter filtered data rather than original data
    iirfilt_default.cmd = ['h_eegplot = gcf; EEG.data = iirfilt(EEG.data, EEG.srate,' num2str(lowcut) ',' num2str(highcut_eegfilt) ', 0, 0, ' num2str(notch) ...
        '); close(h_eegplot); [ALLEEG EEG CURRENTSET] = pop_newset(ALLEEG, EEG,1, ''setname'', ''' iirfilt_default.setname '''); eeglab redraw;']; % for use later if wish to save this filtered version, otherwise does not execute
    % Provide feedback to console
    if exist('cprintf','file') ~=2  % for older versions of Matlab prior to cprintf
        fprintf('\n%s%5.2f%s\n','Short IIR (iirfilt) filtering complete in ',iirfilt_default.time_to_filter,' seconds.');
    else
        cprintf([.5 0 1],'Short IIR (iirfilt) filtering complete in %5.2f seconds.\n', iirfilt_default.time_to_filter);
    end;
end

if optFIR_selected || optFIR_data_selected % screen cprint feedback built into this function 
    optFIR_filt = EEG;
    tic;
    [optFIR_filt.data, optFIR_filt.fweights] = optFIR(EEG.data, EEG.srate, lowcut, highcut, notch, 0, optFIR_filt_length);
    optFIR_filt.time_to_filter = toc;
    % Make set name in case user chooses to preview data and save this set
    optFIR_filt.setname = [EEG.setname ' post filtering with '  num2str(lowcut) '-' num2str(highcut) 'hz' notchstring ' optimal FIR (order = ' num2str(length(optFIR_filt.fweights)-1) ')'];
    % Make command to run this from UI Pushbutton on eegplot, including getting handle to this plot and closing it, otherwise pushing the button again will refilter filtered data rather than original data
    optFIR_filt.cmd = ['h_eegplot = gcf; EEG.data = optFIR(EEG.data, EEG.srate,' num2str(lowcut) ',' num2str(highcut) ',' num2str(notch) ', 0,' num2str(optFIR_filt_length) ...
        '); close(h_eegplot); [ALLEEG EEG CURRENTSET] = pop_newset(ALLEEG, EEG,1, ''setname'', ''' optFIR_filt.setname '''); eeglab redraw;']; % for use later if wish to save this filtered version, otherwise does not execute
end
             
%% Derive Freq Response and ideal-actual correlations and figure legends

% Note that fs and m will be the same for all of these, so we don't need to
% worry which of these if loops defines fs or m

if eegfilt_default_selected || eegfilt_default_data_selected
    [eegfilt_default.f_axis, eegfilt_default.gain, eegfilt_default.h, eegfilt_default.w, eegfilt_default.ideal_actual_corr, fs, m] = FIR_freq_rsps(lowcut, highcut, eegfilt_default.srate, 0, conv(eegfilt_default.fweights,eegfilt_default.fweights(end:-1:1),'same')); % convolve fweights with reversed vector to be consistent with the fact that eegfilt uses filtfilt
    eegfilt_default.legend = ['Basic FIR (eegfilt default)         r_i_d_e_a_l=' sprintf('%3.2f',eegfilt_default.ideal_actual_corr) ', t_f_i_l_t = ' sprintf('%3.2f',eegfilt_default.time_to_filter) ' s'];
end

if eegfilt_srate_selected || eegfilt_srate_data_selected
    [eegfilt_srate.f_axis, eegfilt_srate.gain, eegfilt_srate.h, eegfilt_srate.w, eegfilt_srate.ideal_actual_corr, fs, m] = FIR_freq_rsps(lowcut, highcut, eegfilt_srate.srate, 0, conv(eegfilt_srate.fweights,eegfilt_srate.fweights(end:-1:1),'same')); % convolve fweights with reversed vector to be consistent with the fact that eegfilt uses filtfilt
    eegfilt_srate.legend = ['Basic FIR (eegfilt order ',num2str(eegfilt_srate.srate*optFIR_filt_length),')     r_i_d_e_a_l=' sprintf('%3.2f',eegfilt_srate.ideal_actual_corr) ', t_f_i_l_t = ' sprintf('%3.2f',eegfilt_srate.time_to_filter) ' s'];
end

if iirfilt_selected || iirfilt_data_selected
    [iirfilt_default.f_axis, iirfilt_default.gain, iirfilt_default.h, iirfilt_default.w, iirfilt_default.ideal_actual_corr, fs, m] = IIR_freq_rsps(lowcut, highcut, iirfilt_default.srate, iirfilt_default.pnts, 0);
    iirfilt_default.legend = ['Short IIR (iirfilt default)              r_i_d_e_a_l=' sprintf('%3.2f',iirfilt_default.ideal_actual_corr) ', t_f_i_l_t = ' sprintf('%3.2f',iirfilt_default.time_to_filter) ' s'];
end

if optFIR_selected || optFIR_data_selected
    [optFIR_filt.f_axis, optFIR_filt.gain, optFIR_filt.h, optFIR_filt.w, optFIR_filt.ideal_actual_corr, fs, m] = FIR_freq_rsps(lowcut, highcut, optFIR_filt.srate, 0, optFIR_filt.fweights);
    optFIR_filt.legend = ['Optimal FIR (optFIR order ',num2str(optFIR_filt.srate*optFIR_filt_length),') r_i_d_e_a_l=' sprintf('%3.2f',optFIR_filt.ideal_actual_corr) ', t_f_i_l_t = ' sprintf('%3.2f',optFIR_filt.time_to_filter) ' s'];
end

%% Plot relevant freq responses
figure('numbertitle', 'off', 'name',['Comparison of ' sprintf('%3.1f-%3.1f hz',lowcut,highcut) ' Frequency Response Magnitudes at ' num2str(EEG.srate) ' hz sample rate']);
title(['Comparison of ' sprintf('%3.1f-%3.1f hz',lowcut,highcut) ' Frequency Response Magnitudes at ' num2str(EEG.srate) ' hz sample rate']);
xlabel 'Hz';
ylabel 'Gain';
hold on;

% Plot ideal Transfer Function and generate legend entry
plot(fs,m,'-k');
legend_text{1} = ['Ideal                                                        t_f_i_l_t for ', num2str(EEG.nbchan), ' chan(s)'];

% counter for number of legend entries
n_legend = 1;

% Plot transfer functions for selected filters and generate legend entries
if eegfilt_default_selected 
   plot(eegfilt_default.f_axis, eegfilt_default.gain,'-r');
   n_legend = n_legend+1;
   legend_text{n_legend} = eegfilt_default.legend;
end

if eegfilt_srate_selected 
   plot(eegfilt_srate.f_axis, eegfilt_srate.gain,'-g');
   n_legend = n_legend+1;
   legend_text{n_legend} = eegfilt_srate.legend;
end

if iirfilt_selected
   plot(iirfilt_default.f_axis, iirfilt_default.gain,'-c');
   n_legend = n_legend+1;
   legend_text{n_legend} = iirfilt_default.legend;
end

if optFIR_selected
   plot(optFIR_filt.f_axis, optFIR_filt.gain,'color',[0.5 0 1]); % keep with purple theme
   n_legend = n_legend+1;
   legend_text{n_legend} = optFIR_filt.legend;
end

% Add killer legend and adjust y axis to make room for it!
legend(legend_text);
ylim([0 1.8]);
xlim([min(fs) max(fs)]);

%% Plot overlay of data if requested
if original_data_selected+eegfilt_default_data_selected+eegfilt_srate_data_selected+iirfilt_data_selected+optFIR_data_selected == 2
    n_dataset = 0;  % we'll count up to two datasets
    
    if original_data_selected
        n_dataset = n_dataset +1;
        dataset{n_dataset} = EEG;
        dataset{n_dataset}.cmd = 'eval(msgbox(''Selected dataset is unfiltered data.  No action taken''));';
        legend_text{n_dataset} = 'Original data before filtering';
    end
    
    if eegfilt_default_data_selected
        n_dataset = n_dataset +1;
        dataset{n_dataset} = eegfilt_default;
        dataset{n_dataset}.cmd = 'eval(filtspecs.eegfilt_default.cmd);';
        legend_text{n_dataset} = 'after BASIC FIR (eegfilt default)';
    end
    
    if eegfilt_srate_data_selected
        n_dataset = n_dataset +1;
        dataset{n_dataset} = eegfilt_srate;
        dataset{n_dataset}.cmd = 'eval(filtspecs.eegfilt_srate.cmd);';
        legend_text{n_dataset} = ['after BASIC FIR (eegfilt filter order ',num2str(eegfilt_srate.srate*optFIR_filt_length),')'];
    end
    
    if iirfilt_data_selected
        n_dataset = n_dataset +1;
        dataset{n_dataset} = iirfilt_default;
        dataset{n_dataset}.cmd = 'eval(filtspecs.iirfilt_default.cmd);';
        legend_text{n_dataset} = 'after Short IIR (iirfilt default)';
    end
    
    if optFIR_selected
        n_dataset = n_dataset +1;
        dataset{n_dataset} = optFIR_filt;
        dataset{n_dataset}.cmd = 'eval(filtspecs.optFIR_filt.cmd);';
        legend_text{n_dataset} = ['after Optimal FIR (optFIR order ', num2str(optFIR_filt.srate*optFIR_filt_length), ')'];
    end

    if isfield(dataset{1}, 'event') % check to see if events exist, in order to plot if they do
        events2plot = dataset{1}.event;
    else
        events2plot = struct;  % empty structure
    end

    eegplot( dataset{1}.data, 'srate', dataset{1}.srate, 'winlength', 5, 'events', events2plot, 'eloc_file', dataset{1}.chanlocs, ...
        'title', ['Black = ', legend_text{1},'; Red = ', legend_text{2}], ...
        'limits', [dataset{1}.xmin dataset{1}.xmax]*1000, 'data2', dataset{2}.data);
    % Place menu to allow user to save either version of data as new set
    save_menu=uimenu('label','Save Data', 'tag','save_data','ForegroundColor',[0.5 0 1]); 
    uimenu(save_menu, 'label', ['Save Black (' legend_text{1} ') as new set'],'ForegroundColor', [0 0 0], ...
        'callback', dataset{1}.cmd);
    uimenu(save_menu, 'label', ['Save Red (' legend_text{2} ') as new set'],'ForegroundColor', [1 0 0], ...
        'callback', dataset{2}.cmd);
    title(['Black = ', legend_text{1},'; Red = ', legend_text{2}, '; use Save Data menu to save either set'],...
        'VerticalAlignment', 'bottom', 'HorizontalAlignment', 'right');
    set(get(gca,'Title'),'Color',[0.5 0 1])

% Abandoned this approach for buttons, as when figure is resized, they do not move, due to the way that eegplot redraws
%     h_eegplot = gcf;
%     eegplot_dims = get(h_eegplot,'OuterPosition'); % get eegplot dimensions and location
%     width_buffer = 300;  % text can take up all but this much of width
%     textwidth = eegplot_dims(3)-width_buffer;
%     
%     uicontrol('Style', 'text', 'String', ['Black = ', legend_text{1},'; Red = ', legend_text{2}], ...
%         'Position', [round(width_buffer/2) eegplot_dims(4)-120 textwidth 40] ); % will center it on width
%     uicontrol('Style', 'pushbutton', 'ForegroundColor', [0 0 0], 'String', 'Save Black as new set',...
%         'Position', [650 35 150 20],...
%         'Callback', dataset{1}.cmd);
%     uicontrol('Style', 'pushbutton', 'ForegroundColor', [1 0 0], 'String', 'Save Red as new set',...
%         'Position', [650 10 150 20],...
%         'Callback', dataset{2}.cmd);

end


%% Organize FilterSpecs for output
if eegfilt_default_selected || eegfilt_default_data_selected
    filtspecs = filtspecs2struc(filtspecs, 'eegfilt_default', eegfilt_default, fs, m, lowcut, highcut_eegfilt, notch);
end

if eegfilt_srate_selected  || eegfilt_srate_data_selected
    filtspecs = filtspecs2struc(filtspecs, 'eegfilt_srate', eegfilt_srate, fs, m, lowcut, highcut_eegfilt, notch);
end

if iirfilt_selected || iirfilt_data_selected
    filtspecs = filtspecs2struc(filtspecs, 'iirfilt_default', iirfilt_default, fs, m, lowcut, highcut_eegfilt, notch);
end

if optFIR_selected || optFIR_data_selected 
    filtspecs = filtspecs2struc(filtspecs, 'optFIR_filt', optFIR_filt, fs, m, lowcut, highcut_eegfilt, notch);
end


