% Pop function to implement optimal FIR filter via Cook and Miller 
%   Optimal Digital Filtering approach (with hamming option applied 
%   when truncating filter to final filter order). See: 
%
% Cook, E.W., & Miller, G.A. (1992).  Digital Filtering: Background and tutorial for 
%   psychophysiologists. Psychophysiology, 3, 350-367
%
% Requires Signal Processing Toolbox to be installed.
%
% Usage:
%   EEG = pop_optFIR(EEG,lowcut,highcut)        % Hi/Lo/Band-pass filter
%   EEG = pop_optFIR(EEG,lowcut,highcut,notch)  % If notch = 1, notch
%   filter 
%   EEG = pop_optFIR(EEG,lowcut,highcut,notch,plot_gain)  % If plot_gain = 1, plot freq response
%   EEG = pop_optFIR(EEG,lowcut,highcut,notch,plot_gain,plot_data)  % If  plot_data = 1, superimpose 
%                                                                     filtered data on original data
%   EEG = pop_optFIR(EEG,lowcut,highcut,notch,plot_gain,plot_data,filt_length)  % Specify filter length in seconds (longer filter will 
%                                                                                 have sharper rolloff, but larger edge effects)
%
% Where:
%   EEG is eeglab structure
%   lowcut is lower-end cutoff (hz); for low pass filter use 0
%   high cut is upper-end cutoff (hz); for high pass filter use 0
%   notch is logical notch out frequencies, 0 = no (default for bandpass, lowpass, highpass), 1 = yes (notch)
%   plot_code is optional freq response plot, 0 = no (default), 1 = yes
%   plot_data is optional plot overlaying filtered data on unfiltered data, 0 = no (default), 1 = yes
%   filt_length is filter length in seconds (longer filter will have sharper rolloff, but larger edge effects)
%  
%
% Notes:
%   Default final filter order is one second of data (plus one point)
%   Becuase of this, this filter is best applied to continuous data
%   With Epoch data, and padding, filter will work but will produce edge 
%        effects that should be examined by plotting data
%
% Version 0.9
%
% Written July, 2011, by John JB Allen jallen@u.arizona.edu
%
% See Also:  pop_optFIR_compare, pop_about_optFIR_compare, optFIR.m, optFIR_compare, optFIR_design, FIR_filt, FIR_freq_rsps, IIR_freq_rsps

function [EEG com] = pop_optFIR(EEG,lowcut,highcut,varargin)
% the command output is a hidden output that does not have to
% be described in the header

com = ''; % this initialization ensure that the function will return something
% if the user press the cancel button

% check input arguments, and if not enough provided, popup gui
plotit = 0;
plotdata = 0;
filt_length = 1;
if nargin == 7
    filt_length = varargin{4};
    plotdata = varargin{3};
    plotit = varargin{2};
    notch = varargin{1};
elseif nargin == 6
    plotdata = varargin{3};
    plotit = varargin{2};
    notch = varargin{1};
elseif nargin == 5
    plotit = varargin{2};
    notch = varargin{1};
elseif nargin == 4
    notch = varargin{1};
elseif nargin == 0
    doc pop_optFIR;
    return
end;

if nargin < 3  % popup window parameters
    uilist = { ...
        { 'style' 'text' 'string' 'Lo Cut in Hz (use 0 for high pass filter):'  } ...
        { 'style' 'edit' 'string' '' 'tag' 'lowcut'} ...
        { 'style' 'text' 'string' 'High Cut in Hz (use 0 for low pass filter):' } ...
        { 'style' 'edit' 'string' '' 'tag' 'highcut'  } ...
        { 'style' 'text' 'string' 'Filter Length in seconds (default = 1):' } ...
        { 'style' 'edit' 'string' '1' 'tag' 'filt_length'  } ...
        { 'style' 'checkbox' 'string' 'Notch filter'  'value' 0 'tag' 'notch_selected' 'tag' 'notch' } ...
        { 'style' 'text' 'string' '' } ...
        { 'style' 'checkbox' 'string' 'Plot frequency response'  'value' 0 'tag' 'plot_gain_selected' } ...
        { 'style' 'text' 'string' '' } ...
        { 'style' 'checkbox' 'string' 'Compare unfiltered and filtered data' 'value' 0 'tag' 'plot_data_selected' } ...
        { 'style' 'text' 'string' '' } ...
        };
    
    % geometry of pop box
    line_geo = [1.25 0.25];
    geometry = {line_geo line_geo line_geo line_geo line_geo line_geo};
    
    % result of user input
    [result tmp1 tmp2 res] = inputgui( 'geometry', geometry, 'uilist', uilist, 'title', 'Optimal FIR Filtering (Cook & Miller, 1992) -- Optimal FIR plugin version 0.9 ', ...
        'helpcom', 'pophelp(''pop_optFIR'')');
    
    if ~isempty(result)  % empty if user hits cancel button
        if ~isempty(res.lowcut)
            lowcut = eval(res.lowcut);
        else
            error('Please specify low cut for filter.');
        end
        if ~isempty(res.highcut)
            highcut = eval(res.highcut);
        else
            error('Please specify high cut for filter.');
        end
        notch = res.notch;
        filt_length = eval(res.filt_length);
        plotit = res.plot_gain_selected;
        plotdata = res.plot_data_selected;
    end
    
end

if plotit  % if user wants comparison plot with unfiltered data, make a copy
    unfilt_EEG = EEG;
end

if nargin >=4 || ~isempty(result) % call function if arguments exist
    EEG.data = optFIR(EEG.data,EEG.srate,lowcut,highcut,notch,plotit,filt_length);
    % return the string command
    com = sprintf('EEG = pop_optFIR(EEG, %d, %d, %d, %d, %d, %d);', ...
        lowcut, highcut, notch, plotit, plotdata, filt_length);
end

% Give EEG setname an update
if notch
    notchstring = 'notch ';
else
    notchstring = '';
end
EEG.setname = [EEG.setname ' post filtering with ' sprintf('%s %s %s %s %s', num2str(lowcut), '-', num2str(highcut), 'hz', notchstring, ' optimal FIR (order = ', num2str(round(EEG.srate*filt_length)),')')];

if plotdata
    if isfield(EEG, 'event') % check to see if events exist, in order to plot if they do
        events2plot = unfilt_EEG.event;
    else
        events2plot = struct;  % empty structure
    end
    eegplot( unfilt_EEG.data, 'srate', unfilt_EEG.srate, 'winlength', 5, 'events', events2plot, 'eloc_file', unfilt_EEG.chanlocs, ...
        'title', ['Black = Unfiltered, Red = Filtered with ', sprintf('%s %s %s %s %s', num2str(lowcut), '-', num2str(highcut), 'hz', notchstring, ' optimal FIR ')], ...
        'limits', [unfilt_EEG.xmin unfilt_EEG.xmax]*1000, 'data2', EEG.data);
end

return;
