% Pop function to compare options of EEGLab's built-in filters (eegfilt and iirfilt)
%   to optimal FIR filter from optFIR plugin
%
% No data (or small animals) are altered in this process
% Requires Signal Processing Toolbox to be installed.
%
% Usage:
%   filtspecs = pop_optFIR_compare(EEG,lowcut,highcut,notch, eegfilt_default_selected, eegfilt_srate_selected, iirfilt_selected, optFIR_selected, original_data_selected, eegfilt_default_data_selected, eegfilt_srate_data_selected, iirfilt_data_selected, optFIR_data_selected)
%
%  Note: if too few arguments, pops up window to obtain them
%
% Where:
%   EEG is eeglab structure
%   lowcut is lower-end cutoff (hz); for low pass filter use 0
%   high cut is upper-end cutoff (hz); for high pass filter use 0
%   notch is logical notch out frequencies, 0 = no (default for bandpass, lowpass, highpass), 1 = yes (notch)
%
% Notes:
%   Option to compare:
%      1) Ideal Transfer Function to various filter approaches (eegfilt_default_selected, eegfilt_srate_selected, iirfilt_selected, optFIR_selected)
%      2) Data filtered using various filter approches (original_data_selected, eegfilt_default_data_selected, eegfilt_srate_data_selected, iirfilt_data_selected, optFIR_data_selected)
%
% Version 0.9
%
% Written July, 2011, by John JB Allen jallen@u.arizona.edu
%
% See Also:  pop_optFIR, pop_about_optFIR_compare, optFIR.m, optFIR_compare, optFIR_design, FIR_filt, FIR_freq_rsps, IIR_freq_rsps

function [filtspecs com] = pop_optFIR_compare(EEG, lowcut, highcut, notch, eegfilt_default_selected, eegfilt_srate_selected, iirfilt_selected, optFIR_selected, original_data_selected, eegfilt_default_data_selected, eegfilt_srate_data_selected, iirfilt_data_selected, optFIR_data_selected, optFIR_filt_length)
% the command output is a hidden output that does not have to
% be described in the header

com = ''; filtspecs = struct; % this initialization ensure that the function will return something
% if the user press the cancel button

res = struct;  % initialize since final command checks for this even if we avoid gui that generates it

if nargin < 13  % popup window parameters
    uilist = { ...
        { 'style' 'text' 'string' 'Required:  Enter filter specifications:' 'fontweight' 'bold' } ...
        { 'style' 'text' 'string' '' } ...
        { 'style' 'text' 'string' '        Lo Cut in Hz (use 0 for high pass filter):' } ...
        { 'style' 'edit' 'string' '' 'tag' 'lowcut' } ...
        { 'style' 'text' 'string' sprintf('%s %d %s', '        High Cut in Hz (use 0 for high pass filter):') } ...
        { 'style' 'edit' 'string' '' 'tag' 'highcut'} ...
        { 'style' 'text' 'string' '        Notch filter (0 = no, 1 = yes)' } ...
        { 'style' 'edit' 'string' '0' 'tag' 'notch'} ...
        { 'style' 'text' 'string' '        Filter length (secs) for optFIR and eegfilt of same order (default = 1 sec)' } ...
        { 'style' 'edit' 'string' '1' 'tag' 'optFIR_filt_length'} ...
        { 'style' 'text' 'string' 'Required:  Compare filter transfer functions (check at least two):' 'fontweight' 'bold' } ...
        { 'style' 'text' 'string' '' } ...
        { 'style' 'text' 'string' '' } ...
        { 'style', 'checkbox', 'string' '     Basic FIR (eegfilt) -- default order' 'value' 0 'tag' 'eegfilt_default_selected' } ...
        { 'style' 'text' 'string' '' } ...
        { 'style', 'checkbox', 'string' '     Basic FIR (eegfilt) -- same order as optimal FIR' 'value' 0 'tag' 'eegfilt_srate_selected' } ...
        { 'style' 'text' 'string' '' } ...
        { 'style', 'checkbox', 'string' '     Short IIR (iirfilt) -- default order' 'value' 0 'tag' 'iirfilt_selected' } ...
        { 'style' 'text' 'string' '' } ...
        { 'style', 'checkbox', 'string' '     Optimal FIR (optFIR)' 'value' 1 'tag' 'optFIR_selected' } ...
        { 'style' 'text' 'string' 'Optional:  Compare filtered data (check two below to see data plot, or none for no plot):' 'fontweight' 'bold' } ...
        { 'style' 'text' 'string' '' } ...
        { 'style' 'text' 'string' '' } ...
        { 'style', 'checkbox', 'string' '     Original unfiltered data' 'value' 1 'tag' 'original_data_selected' } ...
        { 'style' 'text' 'string' '' } ...
        { 'style', 'checkbox', 'string' '     Basic FIR (eegfilt) -- default order' 'value' 0 'tag' 'eegfilt_default_data_selected' } ...
        { 'style' 'text' 'string' '' } ...
        { 'style', 'checkbox', 'string' '     Basic FIR (eegfilt) -- same order as optimal FIR' 'value' 0 'tag' 'eegfilt_srate_data_selected' } ...
        { 'style' 'text' 'string' '' } ...
        { 'style', 'checkbox', 'string' '     Short IIR (iirfilt) -- default order' 'value' 0 'tag' 'iirfilt_data_selected' } ...
        { 'style' 'text' 'string' '' } ...
        { 'style', 'checkbox', 'string' '     Optimal FIR (optFIR)' 'value' 1 'tag' 'optFIR_data_selected' } ...
        { 'style' 'text' 'string' '' } ...
        { 'style' 'text' 'string' '' } ...
        { 'style' 'text' 'string' 'Note: if you receive ''Out of memory'' error with 32-bit systems, uncheck all boxes'} ...
        { 'style' 'text' 'string' '' } ...
        { 'style' 'text' 'string' '          below the heading ''Compare filtered data'' and you can compare transfer functions only.'} ...
        { 'style' 'text' 'string' '' } ...
        };
    
    % geometry of pop box
    line_geo = [1 0.2];
    line_check_geo = [0.1 1];
    geometry = {line_geo line_geo line_geo line_geo line_geo line_geo line_check_geo line_check_geo line_check_geo line_check_geo line_geo line_check_geo line_check_geo line_check_geo line_check_geo line_check_geo line_geo line_geo line_geo};
    
    % result of user input
    [result tmp1 tmp2 res] = inputgui( 'geometry', geometry, 'uilist', uilist, 'title', 'Compare optFIR to Basic FIR or Short IIR Filtering -- Optimal FIR plugin version 0.9', ...
        'helpcom', 'pophelp(''pop_optFIR_compare'')');
    
    % check for inconsistencies in plot options, and assign vars to GUI
    % inputs
    if ~isempty(res)  % empty if user hits cancel button
        if ~isempty(res.lowcut)
            lowcut = eval(res.lowcut);
        else
            error('Please specify low cut for filter.');
        end
        if ~isempty(res.highcut)
            highcut = eval(res.highcut);
        else
            error('Please specify high cut for filter.');
        end
        if ~isempty(res.notch)
            notch = eval(res.notch);
        else
            error('Please specify 0 or 1 for notch option.');
        end
        if ~isempty(res.optFIR_filt_length)
            optFIR_filt_length = eval(res.optFIR_filt_length);
        else
            error('Please specify a filter length in seconds.');
        end
        if res.eegfilt_default_selected+res.eegfilt_srate_selected+res.iirfilt_selected+res.optFIR_selected < 1
            error('Please select at least one filter type for transfer function comparison.');
        end
        if res.original_data_selected+res.eegfilt_default_data_selected+res.eegfilt_srate_data_selected+res.iirfilt_data_selected+res.optFIR_data_selected > 2
            error('You may only select a maximum of two filter types for data comparison.');
        end;
        if res.original_data_selected+res.eegfilt_default_data_selected+res.eegfilt_srate_data_selected+res.iirfilt_data_selected+res.optFIR_data_selected == 1
            error('Please select two filter types for data comparison, or no data types to skip data plot.');
        end;
        % Now this next part does look a bit pointless, but ... the reason
        % to assign the values from these fields in the structure to
        % individual variables is to ensure compatibility with using this
        % function from the command line with all parameters supplied.  It
        % seems you can pass a field from a structure into a function as an
        % argument, but you cannot define the function in a way to accept a
        % field of a structure (since the structure will not exist), so the
        % input parameter must be a variable, and that is whey we do this
        % next little excercise in assigning these...
        eegfilt_default_selected=res.eegfilt_default_selected;
        eegfilt_srate_selected=res.eegfilt_srate_selected;
        iirfilt_selected=res.iirfilt_selected;
        optFIR_selected=res.optFIR_selected;
        original_data_selected=res.original_data_selected;
        eegfilt_default_data_selected=res.eegfilt_default_data_selected;
        eegfilt_srate_data_selected=res.eegfilt_srate_data_selected;
        iirfilt_data_selected=res.iirfilt_data_selected;
        optFIR_data_selected=res.optFIR_data_selected;
    end
end

if ~isempty(res) || nargin == 13 % call function if arguments exist (res exists if gui was complted)
                                 % and nargin is 13 if command line fully completed
    filtspecs = optFIR_compare(EEG, lowcut, highcut, notch, eegfilt_default_selected, eegfilt_srate_selected, iirfilt_selected, optFIR_selected, original_data_selected, eegfilt_default_data_selected, eegfilt_srate_data_selected, iirfilt_data_selected, optFIR_data_selected,optFIR_filt_length);
    % return the string command
    com = sprintf('filtspecs = pop_optFIR_compare(EEG, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d);', ...
        lowcut, highcut, notch, eegfilt_default_selected, eegfilt_srate_selected, iirfilt_selected, optFIR_selected, original_data_selected, eegfilt_default_data_selected, eegfilt_srate_data_selected, iirfilt_data_selected, optFIR_data_selected, optFIR_filt_length);
end

return;
