/*
 *	Raw data compression algorithm - takes data produced by raw_decompress.
 *	Modification of Hansen code to act in mex unit of matlab.
 *  Input is npoints by nchan data matrix to be compressed.  This array
 *  must be in uint8 format which means that each word has been split in two.
 *  Second input variable is the number of channels.
 *  Third input variable is a byteswap switch (0 not to swap, 1 to swap endian).
 *  Output is the compressed data array in vector format, also in uint8 format.
 *  The first timepoint of the output array will not be compressed.
 *  The subsequent timepoints will be expressed as the change (delta) from
 *  the previous timepoint and will be compressed using a 12-bit encoding scheme.
 *  The second output variable is the length of the output variable (in bytes).
 *	Joseph Dien, 5/9/99
 */


#include "mex.h"
#include "matrix.h"
#include <math.h>

void mexFunction(
	int nlhs, mxArray *plhs[],
	int nrhs, const mxArray *prhs[])
	{
	short *x;
	char *y, *i;
	unsigned int m, n;
	char t;
        short *                 inptr;
        short *                 outptr;
        int                     nch;
        int                     ntot;	/*total length of input array(npts*nchan*2)*/
        int                     lendian;
	double *			pnch;
	double *			pnnts;
	double *			plendian;	/*byteswap the results?*/
	double *			pnes;	/*how long is the compressed data*/

	register short *	rop;
	register int		nnib;
	register int		delta;
	register short *	rip;
	register short *	rlp;
	register short *	limp;
	int					dims[2];

/* Check for proper number of arguments.  */
if (nrhs !=3)  {
	mexErrMsgTxt("Must have three input arguments.");}
else if (nlhs !=2) {
	mexErrMsgTxt("Two output arguments required ([compressed_data length_of_vector]).");
	}

/*Check input matrices to make sure they have the proper format */
n = mxGetNumberOfDimensions(prhs[0]);
if (n > 2) {
	mexErrMsgTxt("Input matrix must be a 2D matrix");}
n = mxGetN(prhs[0]); 
m = mxGetM(prhs[0]);
if (!mxIsUint8(prhs[0]) || mxIsComplex(prhs[0])) {
         mexErrMsgTxt("Input must be real uint8 format.");
        } 
if ((m%2) != 0) {
	 mexErrMsgTxt("Input array must have an even number of rows.");
	}
	
n = mxGetNumberOfDimensions(prhs[1]);
if (n > 2) {
	mexErrMsgTxt("Number_of_channels must be a 2D matrix");}
n = mxGetN(prhs[1]); 
m = mxGetM(prhs[1]);
if ((m != 1) || (n != 1)) {
		mexErrMsgTxt("Number_of_channels must be a single number");
		}
if (!mxIsDouble(prhs[1]) || mxIsComplex(prhs[1])) {
         mexErrMsgTxt("Number_of_channels must be real double precision format.");
        } 
	
n = mxGetNumberOfDimensions(prhs[2]);
if (n > 2) {
	mexErrMsgTxt("Swap_switch must be a 2D matrix");}
n = mxGetN(prhs[2]); 
m = mxGetM(prhs[2]);
if ((m != 1) || (n != 1)) {
		mexErrMsgTxt("Swap_switch must be a single number");
		}
if (!mxIsDouble(prhs[2]) || mxIsComplex(prhs[2])) {
         mexErrMsgTxt("Swap_switch must be real double precision format.");
        } 

/* Assign pointers to each input.  */
x = (short *) mxGetPr(prhs[0]);
pnch = mxGetPr(prhs[1]);
plendian = mxGetPr(prhs[2]);
ntot = mxGetM(prhs[0]);
lendian = *plendian;
nch = *pnch;

if ((lendian != 0) && (lendian != 1)) {
	mexErrMsgTxt("Swap_switch must equal 0 (don't swap bytes) or 1 (swap bytes)");}

/* Create a matrix for the return argument.  */
dims[0] = ntot;
dims[1] = 1;
plhs[0] = mxCreateNumericArray(2, dims, mxUINT8_CLASS, mxREAL);
if (plhs[0] == NULL) {
            mexErrMsgTxt("Could not create output Array #1.\n");
	} 
plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
if (plhs[1] == NULL) {
            mexErrMsgTxt("Could not create output Array #2.\n");
	} 
/* Assign pointers to output.  */
y = (char *) mxGetPr(plhs[0]);
pnes = mxGetPr(plhs[1]);

/*
 *  adapted from src/lib/libd/rdf/raw_compress.c
 *	Raw data compression routine - exploits smoothness of data
 *	to reduce # bits in a type of delta coding of 12 bit data
 *	in short words for multiplexed data.
 *
 *	Strictly canonical ordering won't work well with the coding,
 *	and it's slower given the binary weighting in a word. Packed
 *	in units of 4 bit nibbles so code is seen first from the high
 *	end of each portion.
 *
 *	Jonathan C. Hansen	UCSD ERPL	Mar, 1990
 */

#define	put4(n)	*rop = ( *rop << 4 ) | (n); \
		if ( nnib == 3 ) { rop++; nnib = 0; } else nnib++

#define	put8(n)	if ( nnib == 3 ) { \
			*rop = ( *rop << 4 ) | ( (n) >> 4 ); \
			*++rop = (n) & 0x000f;	nnib = 1; \
		} else { \
			*rop = ( *rop << 8 ) | (n); \
			if ( nnib == 2 ) { rop++;	nnib = 0; } \
			else	nnib += 2; \
		}

#define	put12(n) if ( nnib == 3 ) { *rop = ( *rop << 4 ) | ( (n) >> 8 ); \
			*++rop = (n) & 0x00ff;	nnib = 2; } \
		else if ( nnib == 2 ) { *rop = ( *rop << 8 ) | ( (n) >> 4 ); \
			*++rop = (n) & 0x000f; nnib = 1; } \
		else if ( nnib ) { *rop = ( *rop << 12 ) | (n);	rop++; \
			nnib = 0; } \
		else { *rop = (n); nnib = 3; }
#define	put16(n) if ( nnib == 3 ) { \
			*rop = ( *rop << 4 ) | ( ( (n) >> 12 ) & 0x000f ); \
			*++rop = (n) & 0x0fff; \
		} else if ( nnib == 2 ) { \
			*rop = ( *rop << 8 ) | ( ( (n) >> 8 ) & 0x00ff ); \
			*++rop = (n) & 0x00ff; \
		} else if ( nnib ) { \
			*rop = ( *rop << 12 ) | ( ( (n) >> 4 ) & 0x0fff ); \
			*++rop = (n) & 0x000f; \
		} else { *rop++ = (n); }

#define	putlast() if ( nnib == 3 ) { *rop <<= 4; rop++; } \
		else if ( nnib == 2 ) { *rop <<= 8; rop++; } \
		else if ( nnib ) { *rop <<= 12; rop++; } \
		nnib = 0;

/*
 *	compression algorithm - returns # shorts of encoded data.
 */

#define	C3MASK	0x0007
#define	C3	0x0000
#define	C6MASK	0x003f
#define	C6	0x0080
#define	C9MASK	0x01ff
#define	C9	0x0c00
#define	C12MASK	0x0fff
#define	C12	0xe000

	inptr = x;
	outptr = (short *) y;
	rop = outptr;
	nnib = 0;
	rlp = inptr;

	for ( rip = inptr, limp = inptr + nch; rip < limp; )
		*rop++ = ( *rip++ & C12MASK ) | C12;
	for ( limp = inptr + (ntot/2); rip < limp; rip++, rlp++ ) {
		delta = *rip - *rlp;
		if ( delta < 0 ) {
			if ( delta > -5 ) {
				delta = ( delta & C3MASK ) | C3;
				put4( delta );
			} else
			if ( delta > -33 ) {
				delta = ( delta & C6MASK ) | C6;
				put8( delta );
			} else
			if ( delta > -257 ) {
				delta = ( delta & C9MASK ) | C9;
				put12( delta );
			} else {
				delta = ( *rip & C12MASK ) | C12;
				put16( delta );
			}
		} else {
			if ( delta < 4 ) {
				delta |= C3;
				put4( delta );
			} else
			if ( delta < 32 ) {
				delta |= C6;
				put8( delta );
			} else
			if ( delta < 256 ) {
				delta |= C9;
				put12( delta );
			} else {
				delta = ( *rip & C12MASK ) | C12;
				put16( delta );
			}
		}
	}
	putlast();

/* Set length of compressed data output variable */
	*pnes = rop - outptr;
	
/* Perform byteswap on output compressed data matrix. */
if (lendian == 1) {
	for(i=y; i<(y+((rop - outptr)*2)); i+=2) {
    	    t = *i;
        	*i = *(i+1);
        	*(i+1) = t;
    }
}

	return;
}